@@ -30,7 +30,7 @@
 #define SCREEN_WIDTH 640
 #define SCREEN_HEIGHT 480 
 
-void Terminate (int ExitCode);
+void Terminate (int exit_code, int save_config);
 
 char* output_filename = NULL;
 char* input_filename = NULL;
@@ -71,8 +71,10 @@ Thanks a lot in advance, the Freedroid dev team.\n\n";
 int debug_level = 0;
 int vid_bpp;
 
-int offset_x_override = -30000; // something completely unlikely
-int offset_y_override = -30000; // something completely unlikely
+#define UNDEFINED_OFFSET -30000 // something completely unlikely
+
+int offset_x_override = UNDEFINED_OFFSET;
+int offset_y_override = UNDEFINED_OFFSET;
 
 //--------------------
 // We add this dummy, so that dialog editor and item editor
@@ -143,7 +145,7 @@ void *MyMalloc(long Mamount)
 
 	if ((Mptr = malloc ((size_t) Mamount)) == NULL) {
 		printf (" MyMalloc(%ld) did not succeed!\n", Mamount);
-		Terminate(EXIT_FAILURE);
+		Terminate(EXIT_FAILURE, TRUE);
 	}
 
 	return Mptr;
@@ -171,7 +173,7 @@ void DebugPrintf(int db_level, char *fmt, ...)
  * This function is used for terminating freedroid.  It will close
  * the SDL submodules and exit.
  * ---------------------------------------------------------------------- */
-void Terminate(int ExitCode)
+void Terminate(int exit_code, int save_config)
 {
 	DebugPrintf(2, "\nvoid Terminate(int ExitStatus) was called....");
 	DebugPrintf(1, "\n----------------------------------------------------------------------");
@@ -180,7 +182,7 @@ void Terminate(int ExitCode)
 	DebugPrintf(1,"Thank you for using the FreedroidRPG Croppy Tool.\n\n");
   
 	SDL_Quit();
-	exit (ExitCode);
+	exit (exit_code);
 } // void Terminate ( int ExitCode )
 
 /* -----------------------------------------------------------------
@@ -285,12 +287,13 @@ void ParseCommandLine(int argc, char *const argv[])
 
 	if (input_filename == NULL) {
 		DebugPrintf(-1, "\nERROR:  No input file specified... Terminating...\n");
-		Terminate(EXIT_FAILURE);
-    }
+		printf("\n%s", usage_string);
+		Terminate(EXIT_FAILURE, TRUE);
+  }
 
 	if (output_filename == NULL) {
 		output_filename = strdup(input_filename);
-    }
+  }
 
 } // ParseCommandLine 
 
@@ -303,7 +306,7 @@ void InitVideo (void)
 	/* Initialize the SDL library */
 	if (SDL_Init ( SDL_INIT_VIDEO ) == -1) {
 		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
-		Terminate(EXIT_FAILURE);
+		Terminate(EXIT_FAILURE, FALSE);
 	} else {
 		DebugPrintf(1, "\nSDL Video initialisation successful.\n");
 	}
@@ -311,7 +314,7 @@ void InitVideo (void)
 
 	if (SDL_InitSubSystem(SDL_INIT_TIMER) == -1) {
 		fprintf(stderr, "Couldn't initialize SDL Timer: %s\n",SDL_GetError());
-		Terminate(EXIT_FAILURE);
+		Terminate(EXIT_FAILURE, FALSE);
 	} else {
       DebugPrintf(1, "\nSDL Timer initialisation successful.\n");
 	}
@@ -575,10 +578,58 @@ int get_default_center(int *default_center_x, int *default_center_y)
 				*default_center_x, *default_center_y);
 		return TRUE;
 	}
-	
+
+	DebugPrintf(1, "\nCROPPY: Images size %dx%d unrecognized. Using %d/%d default origin. The offset value will be inaccurate.",
+		*default_center_x, *default_center_y);
 	return FALSE;
 }
 
+int read_offset_file(int *center_x, int *center_y)
+{
+	char filename[10000];
+	int offset_x = 0, offset_y = 0;
+	FILE *offset_file;
+
+	// Determine offset file name
+	strcpy(filename, input_filename);
+	if (strstr(filename, ".png"))
+		filename[strlen(filename) - 4] = '\0';
+	strcat(filename, ".offset");
+
+	// Read offset file
+	if ((offset_file = fopen(filename, "r")) == NULL)
+		return 0;
+
+	fseek(offset_file, 0, SEEK_END);
+	long file_size = ftell(offset_file);
+	fseek(offset_file, 0, SEEK_SET);
+
+	char *offset_data = malloc(file_size + 1);
+	size_t bytes_read = fread(offset_data, 1, file_size, offset_file);
+	fclose(offset_file);
+	offset_data[file_size] = '\0';
+
+	if (bytes_read != file_size) {
+		free(offset_data);
+		return 0;
+	}
+
+	char *offset_x_string = strstr(offset_data, OFFSET_FILE_OFFSETX_STRING);
+	char *offset_y_string = strstr(offset_data, OFFSET_FILE_OFFSETY_STRING);
+
+	if (offset_x_string)
+		sscanf(offset_x_string, OFFSET_FILE_OFFSETX_STRING "%d", &offset_x);
+	if (offset_y_string)
+		sscanf(offset_y_string, OFFSET_FILE_OFFSETY_STRING "%d", &offset_y);
+	free(offset_data);
+
+	fprintf(stderr, "Using offset %d/%d from file %s\n", offset_x, offset_y, filename);
+
+	*center_x = -offset_x;
+	*center_y = -offset_y;
+	return 1;
+}
+
 void write_offset_file(int default_center_x, int default_center_y) 
 {
 	char filename[10000];
@@ -592,9 +643,9 @@ void write_offset_file(int default_center_x, int default_center_y)
 	// the default origin settings for this image.  We take that possibility into
 	// account here.
 	//
-	if (offset_x_override != -30000)
+	if (offset_x_override != UNDEFINED_OFFSET)
 		default_center_x = offset_x_override;
-	if (offset_y_override != -30000)
+	if (offset_y_override != UNDEFINED_OFFSET)
 		default_center_y = offset_y_override;
 	
 	//--------------------
@@ -613,7 +664,7 @@ void write_offset_file(int default_center_x, int default_center_y)
 	//
 	if ((OffsetFile = fopen(filename, "w")) == NULL) {
 		DebugPrintf(-1, "\n\nError opening save game file for writing...\n\nTerminating...\n\n");
-		Terminate(EXIT_FAILURE);
+		Terminate(EXIT_FAILURE, TRUE);
 	}
 
 	fprintf(OffsetFile, "** Start of iso_image offset file **\n");
@@ -628,7 +679,7 @@ void write_offset_file(int default_center_x, int default_center_y)
 
 	if (fclose(OffsetFile) == EOF) {
 		DebugPrintf(-1, "\n\nClosing of .offset file failed...\n\nTerminating\n\n");
-		Terminate(EXIT_FAILURE);
+		Terminate(EXIT_FAILURE, TRUE);
 	}
 
 	DebugPrintf(1, "\nSaving of '.offset' file successful.\n");
@@ -643,7 +694,7 @@ void save_output_surface()
 {
 	if (png_save_surface(output_filename, output_surface) != 0) {
 		DebugPrintf(-1, "\nCROPPY ERROR: unable to save the cropped png");      
-		Terminate(EXIT_FAILURE);
+		Terminate(EXIT_FAILURE, TRUE);
 	}
 } // void copy_and_crop_input_file ( ) ;
 
@@ -673,14 +724,12 @@ int main(int argc, char *const argv[])
 	if (input_surface == NULL) {
 		DebugPrintf(-1, "\n\nERROR:  Unable to load input file... ");
 		DebugPrintf(-1, "\nFile name was : %s . ", input_filename);
-		Terminate(EXIT_FAILURE);
+		Terminate(EXIT_FAILURE, TRUE);
 	}
 
-	if (!get_default_center(&default_center_x, &default_center_y)) {
-		DebugPrintf(-1, "\nCROPPY ERROR:  Unrecognized image format received... terminating in order to prevent accidents...\n");
-		Terminate(EXIT_FAILURE);		
-	}
-	
+	get_default_center(&default_center_x, &default_center_y);
+	read_offset_file(&default_center_x, &default_center_y);
+
 	if (!no_graphics_output) {
 		SDL_BlitSurface(input_surface, NULL, Screen, NULL);
 		SDL_Flip(Screen);
@@ -706,7 +755,7 @@ int main(int argc, char *const argv[])
 		if (background_surface == NULL) {
 			DebugPrintf(-1, "\n\nERROR:  Unable to load background file... ");
 			DebugPrintf(-1, "\nFile name was : %s . ", background_filename);
-			Terminate(EXIT_FAILURE);
+			Terminate(EXIT_FAILURE, TRUE);
 		}
 		SDL_BlitSurface(background_surface, NULL, Screen, NULL);
 		fill_rect.x = 0;
