@@ -12,138 +12,144 @@
 #include "struct.h"
 #include "proto.h"
 #include "global.h"
-#include "SDL_rotozoom.h"
 
 /* Current font */
 static BFont_Info *CurrentFont;
 
+#define FIRST_FONT_CHAR '!'
+
 /**
- *
- *
+ * Initialize a font.
  */
-static void InitFont(BFont_Info * Font)
+static void find_character_positions(BFont_Info *font, SDL_Rect char_rect[MAX_CHARS_IN_FONT])
 {
-	unsigned int x = 0, i = 0;
-	Uint32 sentry;
-	SDL_Surface *tmp_char1;
+	unsigned int x = 0, i = 0, y = 0, max_h = 1;
+	SDL_Rect *rect;
+	SDL_Surface *font_surf = font->font_image.surface;
+	
+	i = FIRST_FONT_CHAR;
+	int sentry_horiz = SDL_MapRGB(font_surf->format, 255, 0, 255);
+	int sentry_vert = SDL_MapRGB(font_surf->format, 0, 255, 0);
 
-	Font->h = Font->Surface->h;
+	if (SDL_MUSTLOCK(font_surf))
+		SDL_LockSurface(font_surf);
 
-	i = '!';
-	sentry = SDL_MapRGB(Font->Surface->format, 255, 0, 255);
+	while (1) {
+		if (i == MAX_CHARS_IN_FONT)
+			break;
 
-	if (Font->Surface == NULL) {
-		fprintf(stderr, "BFont: The font has not been loaded!\n");
-		exit(1);
-	}
+		// Read this line of characters
+		while (x < font_surf->w - 1 && i < MAX_CHARS_IN_FONT) {
+			if (FdGetPixel(font_surf, x, y) != sentry_horiz) {
+				// Found a character
+				rect = &char_rect[i];
+				rect->x = x;
+				rect->y = y;
 
-	if (SDL_MUSTLOCK(Font->Surface))
-		SDL_LockSurface(Font->Surface);
+				// Compute character width
+				int x2 = x;
+				while (x2 < font_surf->w) {
+					if (FdGetPixel(font_surf, x2, y) == sentry_horiz)
+						break;
+					x2++;
+				}
+				rect->w = x2 - x;
 
-	x = 0;
-	while (x < (Font->Surface->w - 1) && i < MAX_CHARS_IN_FONT) {
-		if (FdGetPixel(Font->Surface, x, 0) != sentry) {
-			Font->Chars[i].x = x;
-			Font->Chars[i].y = 1;
-			Font->Chars[i].h = Font->Surface->h;
-			while (x < (Font->Surface->w)) {
-				if (FdGetPixel(Font->Surface, x, 0) == sentry)
+				if (x2 == font_surf->w)
 					break;
+
+				// Compute character height
+				int y2 = y;
+				while (y2 < font_surf->h) {
+					if (FdGetPixel(font_surf, x, y2) == sentry_horiz ||
+							FdGetPixel(font_surf, x, y2) == sentry_vert)
+						break;
+					y2++;
+				}
+				rect->h = y2 - y;
+
+				// Update maximal h
+				if (max_h < y2 - y)
+					max_h = y2 - y;
+
+				// Move on to the next character
+				i++;
+				x = x2;
+			} else {
+				// On a sentry? Move right.
 				x++;
 			}
-			Font->Chars[i].w = (x - Font->Chars[i].x);
-
-			Font->number_of_chars = i + 1;
-
-			tmp_char1 = SDL_CreateRGBSurface(0, CharWidth(Font, i), FontHeight(Font) - 1, 32, rmask, gmask, bmask, amask);
-			Font->char_iso_image[i].surface = SDL_DisplayFormatAlpha(tmp_char1);
-			Font->char_iso_image[i].texture_has_been_created = FALSE;
-
-			our_SDL_blit_surface_wrapper(Font->Surface, &(Font->Chars[i]), Font->char_iso_image[i].surface, NULL);
-			SDL_SetAlpha(Font->char_iso_image[i].surface, SDL_SRCALPHA, SDL_ALPHA_OPAQUE);
-			SDL_SetColorKey(Font->char_iso_image[i].surface, 0, 0);
-
-			flip_image_vertically(Font->char_iso_image[i].surface);
-
-			SDL_FreeSurface(tmp_char1);
-			// Now we can go on to the next char
-			//
-			i++;
-		} else {
-			x++;
 		}
 
+		// Find the next line of characters
+		y += max_h + 1;
+		max_h = 1;
+		x = 0;
+
+		if (y >= font_surf->h)
+			break;	
+
 	}
-	Font->Chars[' '].x = 0;
-	Font->Chars[' '].y = 0;
-	Font->Chars[' '].h = Font->Surface->h;
-	Font->Chars[' '].w = Font->Chars['!'].w;
-	Font->number_of_chars = i;
 
-#ifdef HAVE_LIBGL
-	if (use_open_gl)
-		Font->list_base = glGenLists(i);
-#endif
-	if (SDL_MUSTLOCK(Font->Surface))
-		SDL_UnlockSurface(Font->Surface);
+	if (SDL_MUSTLOCK(font_surf))
+		SDL_UnlockSurface(font_surf);
 
-	SDL_SetColorKey(Font->Surface, 0, 0);
-	SDL_SetAlpha(Font->Surface, SDL_SRCALPHA, SDL_ALPHA_OPAQUE);
-};				// void InitFont (BFont_Info * Font)
+	// Set "space" character width
+	char_rect[' '].w = char_rect['!'].w;
+	char_rect[' '].h = char_rect['!'].h;
+	
+	// We assume a constant font height
+	font->h = char_rect['!'].h;
+	font->number_of_chars = i;
+}
+
+
+/**	
+ * Prepare font for rendering: create struct image for each character in the font.
+ */
+static void prepare_font(BFont_Info *font, SDL_Rect char_rect[MAX_CHARS_IN_FONT])
+{
+	int i;
+
+	if (use_open_gl) {
+		make_texture_out_of_surface(&font->font_image);
+	}
+
+	// Create each char_image
+	for (i = FIRST_FONT_CHAR; i < font->number_of_chars; i++) {
+		struct image *img = &font->char_image[i];
+
+		create_subimage(&font->font_image, img, &char_rect[i]);
+	}
+	
+	// Space is a special case	
+	create_subimage(&font->font_image, &font->char_image[' '], &char_rect[' ']);
+	
+	// Delete font_image SDL surface
+	free_image_surface(&font->font_image);
+}
 
 /**
  * Load the font and stores it in the BFont_Info structure 
  */
-BFont_Info *LoadFont(const char *filename)
+BFont_Info *LoadFont(char *filename)
 {
-	int x;
-	BFont_Info *Font = NULL;
-	SDL_Surface *tmp = NULL;
+	BFont_Info *font = MyMalloc(sizeof(BFont_Info));
 
-	if (filename != NULL) {
-		Font = (BFont_Info *) MyMalloc(sizeof(BFont_Info));
-		if (Font != NULL) {
-			tmp = (SDL_Surface *) IMG_Load(filename);
+	// Load the font image
+	load_image_surface(&font->font_image, filename, FALSE);
 
-			if (tmp != NULL) {
-				Font->Surface = SDL_DisplayFormatAlpha(tmp);
-				SDL_FreeSurface(tmp);
-				for (x = 0; x < 256; x++) {
-					Font->Chars[x].x = 0;
-					Font->Chars[x].y = 0;
-					Font->Chars[x].h = 0;
-					Font->Chars[x].w = 0;
-				}
-				SDL_SetAlpha(Font->Surface, 0, 255);
-				SDL_SetColorKey(Font->Surface, 0, 0);
+	// Find character coordinates in the image
+	SDL_Rect char_rect[MAX_CHARS_IN_FONT];
+	memset(char_rect, 0, sizeof(char_rect));
+	find_character_positions(font, char_rect);
 
-				/* Init the font */
-				InitFont(Font);
-				/* Set the font as the current font */
-				SetCurrentFont(Font);
-
-			} else {
-				fprintf(stderr, "Loading font \"%s\":%s\n", filename, IMG_GetError());
-				/* free memory allocated for the BFont_Info structure */
-				free(Font);
-				Font = NULL;
-			}
-		}
-	}
-
-	return Font;
+	// Prepare the data structures according to the rendering mode
+	prepare_font(font, char_rect);	
+	
+	return font;
 }
 
-/** 
- *
- *
- */
-void FreeFont(BFont_Info * Font)
-{
-	SDL_FreeSurface(Font->Surface);
-	free(Font);
-};				// void FreeFont (BFont_Info * Font)
-
 /**
  * Set the current font 
  */
@@ -175,7 +181,7 @@ int CharWidth(BFont_Info * Font, unsigned char c)
 {
 	if (c < ' ' || c > Font->number_of_chars - 1)
 		c = '.';
-	return Font->Chars[c].w;
+	return Font->char_image[c].w;
 }
 
 /**
@@ -225,6 +231,8 @@ int PutCharFont(SDL_Surface * Surface, BFont_Info * Font, int x, int y, unsigned
 {
 	SDL_Rect dest;
 	SDL_Rect clipping_rect;
+	struct image *img;
+
 	dest.w = CharWidth(Font, ' ');
 	dest.h = FontHeight(Font);
 	dest.x = x;
@@ -232,74 +240,24 @@ int PutCharFont(SDL_Surface * Surface, BFont_Info * Font, int x, int y, unsigned
 
 	if (c < ' ' || c > Font->number_of_chars - 1)
 		c = '.';
+
+	img = &Font->char_image[c];
+
 	if ((c != ' ') && (c != '\n')) {
 		if (use_open_gl) {
 			SDL_GetClipRect(Surface, &clipping_rect);
 
 			if ((dest.x < clipping_rect.x + clipping_rect.w) && (dest.x >= clipping_rect.x)) {
-#ifdef HAVE_LIBGL
-				// We need to simulate the clipping as would be done by SDL.
-				// We simply do "all or none" clipping for left and right, but for the top/bottom of the screen,
-				// we use GL to do proper clipping in order not to see a whole line disappear at once.
-
-				GLdouble eqntop[4] = { 0.0, 1.0, 0.0, -clipping_rect.y };
-				GLdouble eqnbottom[4] = { 0.0, -1.0, 0.0, clipping_rect.y + clipping_rect.h };
-				glClipPlane(GL_CLIP_PLANE0, eqntop);
-				glEnable(GL_CLIP_PLANE0);
-				glClipPlane(GL_CLIP_PLANE1, eqnbottom);
-				glEnable(GL_CLIP_PLANE1);
-#endif
-
-				if (!Font->char_iso_image[c].texture_has_been_created) {
-					// If the character is not ready to be printed on screen
-					if (Font->char_iso_image[c].surface == NULL) {
-						ErrorMessage(__FUNCTION__, "Surface for character %d was NULL pointer!",
-							     PLEASE_INFORM, IS_FATAL, c);
-					}
-					make_texture_out_of_surface(&(Font->char_iso_image[c]));
-#ifdef HAVE_LIBGL
-					glNewList(Font->list_base + c, GL_COMPILE);
-
-					glBindTexture(GL_TEXTURE_2D, (Font->char_iso_image[c].texture));
-					glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-					glEnable(GL_BLEND);
-
-					glBegin(GL_QUADS);
-
-					glTexCoord2f(0.0, 1.0);
-					glVertex2i(0, 0);
-					glTexCoord2f(0.0, 0.0);
-					glVertex2i(0, Font->char_iso_image[c].texture_height);
-					glTexCoord2f(1.0, 0.0);
-					glVertex2i(Font->char_iso_image[c].texture_width, Font->char_iso_image[c].texture_height);
-					glTexCoord2f(1.0, 1.0);
-					glVertex2i(Font->char_iso_image[c].texture_width, 0);
-
-					glEnd();
-					glEndList();
-					glDisable(GL_BLEND);
-
-#endif
-				}
-#ifdef HAVE_LIBGL
-				glPushMatrix();
-				glMatrixMode(GL_MODELVIEW);
-				glTranslated(dest.x, dest.y, 0);
-				glCallList(Font->list_base + c);
-				glPopMatrix();
-				glDisable(GL_CLIP_PLANE0);
-				glDisable(GL_CLIP_PLANE1);
-#endif
-
+				display_image_on_screen(img, dest.x, dest.y, IMAGE_NO_TRANSFO);
 			}
 		} else {
-			our_SDL_blit_surface_wrapper(Font->Surface, &Font->Chars[c], Surface, &dest);
+			display_image_on_screen(img, dest.x, dest.y, IMAGE_NO_TRANSFO);
 		}
 	}
 
 	return CharWidth(Font, c);
 
-};				// int PutCharFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, int c)
+}
 
 /**
  *
@@ -308,6 +266,16 @@ void PutString(SDL_Surface *surface, int x, int y, char *text)
 {
 	int i = 0;
 
+	if (use_open_gl) {
+		// Set up clipping
+		SDL_Rect clip_rect;
+		SDL_GetClipRect(surface, &clip_rect);
+
+		set_gl_clip_rect(&clip_rect);
+	}
+
+	start_image_batch();
+
 	while (text[i] != '\0') {
 		int letter_spacing = get_letter_spacing(GetCurrentFont());
 		if (!handle_switch_font_char(text[i])) {
@@ -315,6 +283,15 @@ void PutString(SDL_Surface *surface, int x, int y, char *text)
 		}
 		i++;
 	}
+
+	end_image_batch();
+
+#ifdef HAVE_LIBGL
+	if (use_open_gl) {
+		unset_gl_clip_rect();
+	}
+#endif
+
 }
 
 /**
@@ -488,87 +465,6 @@ void LeftPutStringFont(SDL_Surface * Surface, BFont_Info * Font, int y, char *te
 	PutStringFont(Surface, Font, 0, y, text);
 }
 
-/******/
-
-void PrintString(SDL_Surface * Surface, int x, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-
-		PutStringFont(Surface, CurrentFont, x, y, temp);
-
-		free(temp);
-	}
-	va_end(args);
-}
-
-/*********************************************************************************************************/
-/*********************************************************************************************************/
-/*********************************************************************************************************/
-
-inline void PutPixel32(SDL_Surface * surface, int x, int y, Uint32 pixel)
-{
-	Uint8 *p;
-	p = (Uint8 *) surface->pixels + y * surface->pitch + x * 4;
-
-	*(Uint32 *) p = pixel;
-
-}				// void PutPixel ( ... )
-
-void PutPixel(SDL_Surface * surface, int x, int y, Uint32 pixel)
-{
-	int bpp = surface->format->BytesPerPixel;
-	Uint8 *p;
-
-	if (use_open_gl) {
-		if (surface == Screen) {
-			PutPixel_open_gl(x, y, pixel);
-			return;
-		}
-	}
-
-	// Here I add a security query against segfaults due to writing
-	// perhaps even far outside of the surface pixmap data.
-	//
-	if ((x < 0) || (y < 0) || (x >= surface->w) || (y >= surface->h))
-		return;
-
-	/* Here p is the address to the pixel we want to set */
-	p = (Uint8 *) surface->pixels + y * surface->pitch + x * bpp;
-
-	switch (bpp) {
-	case 1:
-		*p = pixel;
-		break;
-
-	case 2:
-		*(Uint16 *) p = pixel;
-		break;
-
-	case 3:
-		// pixel = pixel & 0x0ffffff ;
-		if (SDL_BYTEORDER == SDL_BIG_ENDIAN) {
-			p[0] = (pixel >> 16) & 0xff;
-			p[1] = (pixel >> 8) & 0xff;
-			p[2] = pixel & 0xff;
-		} else {
-			p[0] = pixel & 0xff;
-			p[1] = (pixel >> 8) & 0xff;
-			p[2] = (pixel >> 16) & 0xff;
-		}
-		break;
-
-	case 4:
-		*(Uint32 *) p = pixel;
-		break;
-	}
-
-};				// void PutPixel ( ... )
-
 /**
  * NOTE:  I THINK THE SURFACE MUST BE LOCKED FOR THIS!
  *
