@@ -64,7 +64,7 @@ static void find_dropable_position_near_chest(float *item_x, float *item_y, int
 	*item_y = Me.pos.y;
 
 	// Step 1: randomly choose one of 16 main 22.5° directions around the chest
-	float obs_diag = obstacle_map[obst_level->obstacle_list[obst_index].type].diaglength;
+	float obs_diag = get_obstacle_spec(obst_level->obstacle_list[obst_index].type)->diaglength;
 	offset_vector.x = obs_diag + 0.5;
 	offset_vector.y = 0.0;
 	RotateVectorByAngle(&offset_vector, (float)MyRandom(16) * 22.5);
@@ -182,10 +182,11 @@ static int mouse_cursor_is_on_that_obstacle(level *lvl, int obst_index)
 	if (obs_vpos.z == -1)
 		return FALSE;
 
-	if (mouse_cursor_is_on_that_iso_image(obs_vpos.x, obs_vpos.y, get_obstacle_image(lvl->obstacle_list[obst_index].type))) {
-		return (TRUE);
-	}
-	return (FALSE);
+	struct image *img = get_obstacle_image(lvl->obstacle_list[obst_index].type, lvl->obstacle_list[obst_index].frame_index);
+	if (mouse_cursor_is_on_that_image(obs_vpos.x, obs_vpos.y, img))
+		return TRUE;
+
+	return FALSE;
 }
 
 /**
@@ -200,65 +201,61 @@ static int mouse_cursor_is_on_that_obstacle(level *lvl, int obst_index)
  */
 int clickable_obstacle_below_mouse_cursor(level **obst_lvl)
 {
-	int x, y;
+#define HOVER_CHECK_DIST 2
+#define SIDE_LENGTH (HOVER_CHECK_DIST * 2 + 1)
 	finepoint MapPositionOfMouse;
-	int i;
-	int obst_index;
+	int i, j, x, y;
+	int return_index = -1, obst_index = 0;
+	float obst_normal = 0, max_normal = 0;
 
-	*obst_lvl = NULL;
+	if(obst_lvl)
+		*obst_lvl = NULL;
 
-	// Now if the cursor is not inside the user rectangle at all, then
-	// there can never be an obstacle under the mouse cursor...
-	//
+	// If the cursor is not inside the user rectangle, 
+	// there is no obstacle below it.
 	if (!MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y()))
-		return (-1);
+		return -1;
 
-	// We find the approximate position of the mouse cursor on the floor.
-	// We will use that as the basis for our scanning for obstacles under the
-	// current mouse cursor.
-	//
+	// We find the position of the mouse cursor on the floor.
 	MapPositionOfMouse.x = translate_pixel_to_map_location((float)input_axis.x, (float)input_axis.y, TRUE);
 	MapPositionOfMouse.y = translate_pixel_to_map_location((float)input_axis.x, (float)input_axis.y, FALSE);
 
-	// We will scan a small area around the mouse cursor, to ease object's selection
-	// Each scanned position could be on a neighbor of the current's level,
-	// so we should resolve all those virtual positions.
-	// But resolving a virtual position is a bit CPU costly, so in order
-	// to minimize cost, we only resolve the mouse cursor position.
-	//
 	gps mouse_target_vpos = { MapPositionOfMouse.x, MapPositionOfMouse.y, Me.pos.z };
 	gps mouse_target_pos;
 	if (!resolve_virtual_position(&mouse_target_pos, &mouse_target_vpos))
 		return -1;
 	if (!level_is_visible(mouse_target_pos.z))
 		return -1;
+
 	level *lvl = curShip.AllLevels[mouse_target_pos.z];
 
-	for (y = mouse_target_pos.y + 3; y > mouse_target_pos.y - 3; y--) {
-		for (x = mouse_target_pos.x + 3; x > mouse_target_pos.x - 3; x--) {
-			if (!pos_inside_level(x, y, lvl))
-				continue;
+	// Iterate through a square area of tiles with sides HOVER_CHECK_DIST * 2 + 1
+	// centered on the tile under the mouse
+	for (i = 0; i < pow(SIDE_LENGTH, 2); i++) {
+		y = i / SIDE_LENGTH - HOVER_CHECK_DIST - 1  + mouse_target_pos.y;
+		x = i % SIDE_LENGTH - HOVER_CHECK_DIST - 1  + mouse_target_pos.x;
 
-			// scan all obstacles of the level targeted by the mouse cursor 
-			for (i = 0; i < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; i++) {
-				obst_index = lvl->map[y][x].obstacles_glued_to_here[i];
-				if (obst_index == (-1))
-					continue;
-				if (mouse_cursor_is_on_that_obstacle(lvl, obst_index)) {
-					DebugPrintf(1, "\n%s(): obstacle under cursor identified.", __FUNCTION__);
-					if (obstacle_map[lvl->obstacle_list[obst_index].type].flags & IS_CLICKABLE) {
-						*obst_lvl = lvl;
-						return obst_index;
-					} else {
-						continue;
-					}
-				}
+		if (!pos_inside_level(x, y, lvl))
+			continue;
+
+		// Iterate through all candidate obstacles on this tile
+		for (j = 0; j < lvl->map[y][x].glued_obstacles.size; j++) {
+			obst_index = ((int *)(lvl->map[y][x].glued_obstacles.arr))[j];
+			obst_normal = lvl->obstacle_list[obst_index].pos.x + lvl->obstacle_list[obst_index].pos.y;
+
+			if (obst_normal > max_normal &&
+				get_obstacle_spec(lvl->obstacle_list[obst_index].type)->flags & IS_CLICKABLE &&
+				mouse_cursor_is_on_that_obstacle(lvl, obst_index)) {
+
+				max_normal = obst_normal;
+				return_index = obst_index;
+				if (obst_lvl)
+					*obst_lvl = lvl;
 			}
 		}
 	}
-
-	return (-1);
-}				// int obstacle_below_mouse_cursor (level **, int [], int)
+	return return_index;
+}
 
 /**
  * In order to perform a click-action, tux must first move to the obstacle.
@@ -268,18 +265,19 @@ int clickable_obstacle_below_mouse_cursor(level **obst_lvl)
  */
 static int reach_obstacle_from_any_direction(level *obst_lvl, int obst_index) {
 	gps obst_vpos;
+	obstacle_spec *obstacle_spec = get_obstacle_spec(obst_lvl->obstacle_list[obst_index].type);
+
 	update_virtual_position(&obst_vpos, &(obst_lvl->obstacle_list[obst_index].pos), Me.pos.z);
-	int obst_type = obst_lvl->obstacle_list[obst_index].type;
-    if (calc_distance(Me.pos.x, Me.pos.y, obst_vpos.x, obst_vpos.y)
-		<= (obstacle_map[obst_type].block_area_parm_1 * sqrt(2)) / 2.0 + 0.5) {
-    	// Maybe a combo_action has made us come here and open the chest.  Then of
-    	// course we can remove the combo action setting now...
-    	//
-    	Me.mouse_move_target_combo_action_type = NO_COMBO_ACTION_SET;
-    	Me.mouse_move_target_combo_action_parameter = -1;
-        return 1;
-    }	
-    
+	if (calc_distance(Me.pos.x, Me.pos.y, obst_vpos.x, obst_vpos.y)
+		<= (obstacle_spec->block_area_parm_1 * sqrt(2)) / 2.0 + 0.5) {
+		// Maybe a combo_action has made us come here and open the chest.  Then of
+		// course we can remove the combo action setting now...
+		//
+		Me.mouse_move_target_combo_action_type = NO_COMBO_ACTION_SET;
+		Me.mouse_move_target_combo_action_parameter = -1;
+		return 1;
+	}
+
 	// We set up a course, that will lead us directly to the barrel, that we are
 	// supposed to smash (upon arrival, later).
 	//
@@ -300,8 +298,8 @@ static int reach_obstacle_from_any_direction(level *obst_lvl, int obst_index) {
 	step_vector.y = Me.pos.y - obst_vpos.y;
 	vec_len = vect_len(step_vector);
 
-	step_vector.x *= ((obstacle_map[obst_type].block_area_parm_1 * sqrt(2)) / 2.0 + 0.05) / vec_len;
-	step_vector.y *= ((obstacle_map[obst_type].block_area_parm_1 * sqrt(2)) / 2.0 + 0.05) / vec_len;
+	step_vector.x *= ((obstacle_spec->block_area_parm_1 * sqrt(2)) / 2.0 + 0.05) / vec_len;
+	step_vector.y *= ((obstacle_spec->block_area_parm_1 * sqrt(2)) / 2.0 + 0.05) / vec_len;
 
 	for (i = 0; i < 8; i++) {
 		if (DirectLineColldet(Me.pos.x, Me.pos.y, obst_vpos.x, obst_vpos.y, Me.pos.z, &WalkablePassFilter)) {
@@ -345,8 +343,8 @@ static int reach_obstacle_from_any_direction(level *obst_lvl, int obst_index) {
 	moderately_finepoint point_near_obst, point_away_from_obst;
 
 	// half-size of the barrel
-	moderately_finepoint half_size = { (obstacle_map[obst_type].block_area_parm_1 * sqrt(2)) / 2.0,
-		(obstacle_map[obst_type].block_area_parm_2 * sqrt(2)) / 2.0
+	moderately_finepoint half_size = { (obstacle_spec->block_area_parm_1 * sqrt(2)) / 2.0,
+		(obstacle_spec->block_area_parm_2 * sqrt(2)) / 2.0
 	};
 
 	for (i = 0; i < 8; i++) {
@@ -403,7 +401,7 @@ static int reach_obstacle_from_specific_direction(level *obst_lvl, int obst_inde
         return 1;
     }
 
-	// So here we know, that we must set the course thowards the chest.  We
+	// So here we know, that we must set the course towards the chest.  We
 	// do so first.
 	// The target's position has to be defined relatively to the chest's level, so that we
 	// can retrieve the chest later (at the end the combo action)
@@ -418,18 +416,19 @@ static int reach_obstacle_from_specific_direction(level *obst_lvl, int obst_inde
 	Me.mouse_move_target_combo_action_parameter = obst_index;
 	int obst_type = obst_lvl->obstacle_list[obst_index].type;
 
+	obstacle_spec *spec = get_obstacle_spec(obst_type);
 	switch (direction) {
 	case EAST:
-		Me.mouse_move_target.x += obstacle_map[obst_type].block_area_parm_1;
+		Me.mouse_move_target.x += spec->block_area_parm_1;
 		break;
 	case SOUTH:
-		Me.mouse_move_target.y += obstacle_map[obst_type].block_area_parm_2;
+		Me.mouse_move_target.y += spec->block_area_parm_2;
 		break;
 	case WEST:
-		Me.mouse_move_target.x -= obstacle_map[obst_type].block_area_parm_1;
+		Me.mouse_move_target.x -= spec->block_area_parm_1;
 		break;
 	case NORTH:
-		Me.mouse_move_target.y -= obstacle_map[obst_type].block_area_parm_2;
+		Me.mouse_move_target.y -= spec->block_area_parm_2;
 		break;
 	default:
 		ErrorMessage(__FUNCTION__, "Invalid direction!!", PLEASE_INFORM, IS_FATAL);
@@ -463,15 +462,22 @@ static int approach(int type)
 			{ ISO_CONSOLE_S, NORTH },
 			{ ISO_CONSOLE_E, WEST },
 			{ ISO_CONSOLE_W, EAST },
+			{ ISO_WALL_TERMINAL_S, SOUTH},
+			{ ISO_WALL_TERMINAL_E, EAST},
+			{ ISO_WALL_TERMINAL_N, NORTH},
+			{ ISO_WALL_TERMINAL_W, WEST},
 			{ ISO_SIGN_1, EAST },
-			{ ISO_SIGN_3, EAST },
+			{ ISO_SIGN_1_FLASH, EAST },
 			{ ISO_SIGN_2, SOUTH },
+			{ ISO_SIGN_2_FLASH, SOUTH },
+			{ ISO_SIGN_3, EAST },
+			{ ISO_SIGN_3_FLASH, EAST },
 	};
 
 	for (i = 0; i < sizeof(lookup)/sizeof(lookup[0]); i++) {
 		if (lookup[i].type == type)
 			return lookup[i].direction;
-	}	
+	}
 
 
 	ErrorMessage(__FUNCTION__, "Tried to approach obstacle type %d, but this is not a known type.", PLEASE_INFORM, IS_WARNING_ONLY, type);
@@ -517,9 +523,10 @@ static void act_terminal(level *l, obstacle *o)
 	}
 
 	enemy dummy_enemy;
+	dummy_enemy.type = get_droid_type("TRM");
+	enemy_reset(&dummy_enemy);
 	dummy_enemy.dialog_section_name = dialog;
-	dummy_enemy.will_rush_tux = 0;
-	dummy_enemy.type = 35;
+	dummy_enemy.will_rush_tux = FALSE;
 	ChatWithFriendlyDroid(&dummy_enemy);
 }
 
@@ -530,7 +537,7 @@ static void act_sign(level *l, obstacle *o)
 		message = "There is nothing on this sign.";
 	}
 
-	append_new_game_message(message);
+	append_new_game_message("%s", message);
 }
 
 static void __obstacle_action(level *lvl, int index, enum interactive_obstacle_type act)
@@ -680,4 +687,33 @@ int check_for_items_to_pickup(level *item_lvl, int item_index)
 	return FALSE;
 }
 
+/**
+ * This function returns a pointer to the obstacle action function for the given action name.
+ * If action with the given name wasn't found it returns NULL.
+ */
+action_fptr get_action_by_name(const char *action_name)
+{
+	const struct {
+		const char *name;
+		action_fptr action;
+	} action_map[] = {
+		{ "barrel", barrel_action },
+		{ "chest", chest_open_action },
+		{ "terminal", terminal_connect_action },
+		{ "sign", sign_read_action }
+	};
+
+	if (!action_name)
+		return NULL;
+
+	int i;
+	for (i = 0; i < sizeof(action_map) / sizeof(action_map[0]); i++) {
+		if (!strcmp(action_name, action_map[i].name))
+			return action_map[i].action;
+	}
+
+	ErrorMessage(__FUNCTION__, "\nUnknown obstacle action '%s'.\n", PLEASE_INFORM, IS_FATAL, action_name);
+	return NULL;
+}
+
 #undef _action_c
