@@ -43,18 +43,15 @@
 //--------------------
 // Animation timelines.
 //
-static float teleport_timeline = 0.0;
-static float refresh_timeline = 0.0;
 static float autogun_timeline = 0.0;
 static float door_timeline = 0.0;
+static float animation_timeline = 0.0;
 
 void animation_timeline_reset()
 {
-	teleport_timeline = 0.0;	
-	refresh_timeline = 0.0;
 	autogun_timeline = 0.0;
 	door_timeline = 0.0;
-
+	animation_timeline = 0.0;
 }
 
 /*
@@ -62,14 +59,13 @@ void animation_timeline_reset()
  */
 void animation_timeline_advance()
 {
-	teleport_timeline += Frame_Time() * 10;	
-	refresh_timeline += Frame_Time() * 3;
 	autogun_timeline += Frame_Time();
 	if (autogun_timeline >= 0.3)
 		autogun_timeline = 0.0;
 	door_timeline += Frame_Time();
 	if (door_timeline >= 0.02)
 		door_timeline = 0.0;
+	animation_timeline += Frame_Time();
 }
 
 /*****************************************************************************
@@ -142,40 +138,26 @@ int animate_door(level* door_lvl, int door_idx)
 	Pos = &(door_lvl->obstacle_list[door_idx].type);
 
 	if (one_player_close_enough || some_bot_was_close_to_this_door) {
-		if ((*Pos != ISO_H_DOOR_100_OPEN) && (*Pos != ISO_V_DOOR_100_OPEN)
-			&& (*Pos != ISO_DH_DOOR_100_OPEN) && (*Pos != ISO_DV_DOOR_100_OPEN)
-			&& (*Pos != ISO_OUTER_DOOR_H_100) && (*Pos != ISO_OUTER_DOOR_V_100))
+		if ( ((*Pos >= ISO_H_DOOR_000_OPEN) && (*Pos < ISO_H_DOOR_100_OPEN)) ||
+		     ((*Pos >= ISO_V_DOOR_000_OPEN) && (*Pos < ISO_V_DOOR_100_OPEN)) ||
+		     ((*Pos >= ISO_DH_DOOR_000_OPEN) && (*Pos < ISO_DH_DOOR_100_OPEN)) ||
+		     ((*Pos >= ISO_DV_DOOR_000_OPEN) && (*Pos < ISO_DV_DOOR_100_OPEN)) ||
+		     ((*Pos >= ISO_OUTER_DOOR_H_00) && (*Pos < ISO_OUTER_DOOR_H_100)) ||
+		     ((*Pos >= ISO_OUTER_DOOR_V_00) && (*Pos < ISO_OUTER_DOOR_V_100)) )
 			*Pos += 1;
 	} else {
-		if ((*Pos != ISO_V_DOOR_000_OPEN) && (*Pos != ISO_H_DOOR_000_OPEN)
-			&& (*Pos != ISO_DV_DOOR_000_OPEN) && (*Pos != ISO_DH_DOOR_000_OPEN)
-			&& (*Pos != ISO_OUTER_DOOR_V_00) && (*Pos != ISO_OUTER_DOOR_H_00))
+		if ( ((*Pos > ISO_H_DOOR_000_OPEN) && (*Pos <= ISO_H_DOOR_100_OPEN)) ||
+		     ((*Pos > ISO_V_DOOR_000_OPEN) && (*Pos <= ISO_V_DOOR_100_OPEN)) ||
+		     ((*Pos > ISO_DH_DOOR_000_OPEN) && (*Pos <= ISO_DH_DOOR_100_OPEN)) ||
+		     ((*Pos > ISO_DV_DOOR_000_OPEN) && (*Pos <= ISO_DV_DOOR_100_OPEN)) ||
+		     ((*Pos > ISO_OUTER_DOOR_H_00) && (*Pos <= ISO_OUTER_DOOR_H_100)) ||
+		     ((*Pos > ISO_OUTER_DOOR_V_00) && (*Pos <= ISO_OUTER_DOOR_V_100)) )
 			*Pos -= 1;	
 	}
 
 	return TRUE;
 }
 
-/*
- * This functions continuously animates the iso image of a teleporter 
- */
-int animate_teleporter(level* teleporter_lvl, int teleporter_idx)
-{
-	teleporter_lvl->obstacle_list[teleporter_idx].type = (((int)rintf(teleport_timeline)) % 5) + ISO_TELEPORTER_1;
-
-	return TRUE;
-}
-
-/*
- * This function continuously animates the iso image of a "droid nest" 
- */
-int animate_refresh(level* refresh_lvl, int refresh_idx)
-{
-	refresh_lvl->obstacle_list[refresh_idx].type = (((int)rintf(refresh_timeline)) % 5) + ISO_REFRESH_1;
-
-	return TRUE;
-}
-
 /*
  * This function animates an autogun.
  * It fires a bullet regularly 
@@ -230,12 +212,7 @@ int animate_autogun(level* autogun_lvl, int autogun_idx)
 	CurBullet->faction = FACTION_BOTS;
 	CurBullet->owner = -3;
 	CurBullet->bullet_lifetime = ItemMap[weapon_item_type].item_gun_bullet_lifetime;
-	CurBullet->ignore_wall_collisions = ItemMap[weapon_item_type].item_gun_bullet_ignore_wall_collisions;
-	CurBullet->time_in_frames = 0;
 	CurBullet->time_in_seconds = 0;
-	CurBullet->was_reflected = FALSE;
-	CurBullet->reflect_other_bullets = ItemMap[weapon_item_type].item_gun_bullet_reflect_other_bullets;
-	CurBullet->pass_through_explosions = ItemMap[weapon_item_type].item_gun_bullet_pass_through_explosions;
 	CurBullet->pass_through_hit_bodies = ItemMap[weapon_item_type].item_gun_bullet_pass_through_hit_bodies;
 	CurBullet->freezing_level = 0;
 	CurBullet->poison_duration = 0;
@@ -248,6 +225,7 @@ int animate_autogun(level* autogun_lvl, int autogun_idx)
 	CurBullet->pos.x = autogunx;
 	CurBullet->pos.y = autoguny;
 	CurBullet->pos.z = autogun_lvl->levelnum;
+	CurBullet->height = 20;
 
 	switch (*AutogunType) {
 	case ISO_AUTOGUN_W:
@@ -282,3 +260,38 @@ is not really an autogun.  Instead it's something else.", PLEASE_INFORM, IS_FATA
 	
 	return TRUE;
 }
+
+int animate_obstacle(level *obstacle_lvl, int obstacle_idx)
+{
+	obstacle *obstacle = &obstacle_lvl->obstacle_list[obstacle_idx];
+	obstacle_spec *spec = get_obstacle_spec(obstacle->type);
+	obstacle->frame_index = (int)rintf(spec->animation_fps * animation_timeline) % spec->filenames.size;
+	return TRUE;
+}
+
+/**
+ * This functions returns a pointer to the obstacle animation function for the
+ * given animation name.
+ */
+animation_fptr get_animation_by_name(const char *animation_name)
+{
+	const struct {
+		const char *name;
+		animation_fptr animation;
+	} animation_map[] = {
+		{ "door", animate_door },
+		{ "autogun", animate_autogun }
+	};
+
+	if (!animation_name)
+		return NULL;
+
+	int i;
+	for (i = 0; i < sizeof(animation_map) / sizeof(animation_map[0]); i++) {
+		if (!strcmp(animation_name, animation_map[i].name))
+			return animation_map[i].animation;
+	}
+
+	ErrorMessage(__FUNCTION__, "\nUnknown obstacle animation '%s'.\n", PLEASE_INFORM, IS_FATAL, animation_name);
+	return NULL;
+}
