@@ -33,15 +33,18 @@
 #include "global.h"
 #include "proto.h"
 #include "map.h"
-#include "SDL_rotozoom.h"
 
 int AUTOMAP_TEXTURE_WIDTH = 2048;
 int AUTOMAP_TEXTURE_HEIGHT = 1024;
 
 #define AUTOMAP_SQUARE_SIZE 3
-#define AUTOMAP_COLOR 0x0FFFF
+#define WALL_COLOR 0x00, 0xFF, 0xFF
+#define EXIT_COLOR 0xFF, 0xFF, 0x00
+#define TUX_COLOR 0x00, 0x00, 0xFF
+#define FRIEND_COLOR 0x00, 0xFF, 0x00
+#define ENEMY_COLOR 0xFF, 0x00, 0x00
 
-static iso_image compass;
+static struct image compass;
 
 /**
  * This function clears out the Automap data.
@@ -78,6 +81,33 @@ static void round_automap_pos(float a, float b, int *ra, int *rb)
 	*rb = *ra + dist;
 }
 
+/**
+ * This function first erases the automap data on a square,
+ * then it tags each square touching it to redraw the automap.
+ *
+ */
+static void update_automap_square(int z, int x, int y)
+{
+	int a, b;
+	level *automap_level = curShip.AllLevels[z];
+
+	Me.Automap[z][y][x] &= ~UPDATE_SQUARE_BIT;
+	Me.Automap[z][y][x] &= ~EW_WALL_BIT;
+	Me.Automap[z][y][x] &= ~NS_WALL_BIT;
+
+	for (a = x - 1 ; a <=  x + 1; a++) {
+		if ((a < 0) || (a >= automap_level->xlen))
+			continue;
+			
+		for (b = y - 1; b <=  y + 1; b++) {
+			if ((b < 0) || (b >= automap_level->ylen))
+				continue;
+
+			Me.Automap[z][b][a] &= ~SQUARE_SEEN_AT_ALL_BIT;
+		}
+	}	
+}
+
 /**
  * This function collects the automap data and stores it in the Me data
  * structure.
@@ -86,16 +116,13 @@ void CollectAutomapData(void)
 {
 	int x, y;
 	int start_x, start_y, end_x, end_y;
-	gps ObjPos;
 	static int TimePassed;
 	Level automap_level = curShip.AllLevels[Me.pos.z];
 	int i;
 	obstacle *our_obstacle;
 	int level = Me.pos.z;
 
-	ObjPos.z = Me.pos.z;
-
-	// Checking the whole map for passablility will surely take a lot
+	// Checking the whole map for passability will surely take a lot
 	// of computation.  Therefore we only do this once every second of
 	// real time.
 	//
@@ -130,18 +157,28 @@ void CollectAutomapData(void)
 		start_y = 0;
 	if (end_y >= automap_level->ylen)
 		end_y = automap_level->ylen;
+	
+	int tstamp = next_glue_timestamp();
 
 	// Now we do the actual checking for visible wall components.
 	//
 	for (y = start_y; y < end_y; y++) {
 		for (x = start_x; x < end_x; x++) {
+			if (Me.Automap[level][y][x] & UPDATE_SQUARE_BIT)
+				update_automap_square(level, x, y);
+					
 			if (Me.Automap[level][y][x] & SQUARE_SEEN_AT_ALL_BIT)
 				continue;
 
-			for (i = 0; i < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; i++) {
-				if (automap_level->map[y][x].obstacles_glued_to_here[i] == (-1))
+			for (i = 0; i < automap_level->map[y][x].glued_obstacles.size; i++) {
+				int idx = ((int *)(automap_level->map[y][x].glued_obstacles.arr))[i];
+
+				our_obstacle = &(automap_level->obstacle_list[idx]);
+
+				if (our_obstacle->timestamp == tstamp)
 					continue;
-				our_obstacle = &(automap_level->obstacle_list[automap_level->map[y][x].obstacles_glued_to_here[i]]);
+
+				our_obstacle->timestamp = tstamp;
 
 				if ((our_obstacle->type >= ISO_H_DOOR_000_OPEN) && (our_obstacle->type <= ISO_V_DOOR_100_OPEN))
 					continue;
@@ -155,10 +192,11 @@ void CollectAutomapData(void)
 				int obstacle_start_y;
 				int obstacle_end_y;
 
-				round_automap_pos(our_obstacle->pos.x + obstacle_map[our_obstacle->type].left_border, our_obstacle->pos.x + obstacle_map[our_obstacle->type].right_border,
+				obstacle_spec *obstacle_spec = get_obstacle_spec(our_obstacle->type);
+				round_automap_pos(our_obstacle->pos.x + obstacle_spec->left_border, our_obstacle->pos.x + obstacle_spec->right_border,
 						&obstacle_start_x, &obstacle_end_x);
 
-				round_automap_pos(our_obstacle->pos.y + obstacle_map[our_obstacle->type].upper_border, our_obstacle->pos.y + obstacle_map[our_obstacle->type].lower_border,
+				round_automap_pos(our_obstacle->pos.y + obstacle_spec->upper_border, our_obstacle->pos.y + obstacle_spec->lower_border,
 						&obstacle_start_y, &obstacle_end_y);
 
 				if (obstacle_start_x < 0)
@@ -185,24 +223,20 @@ void CollectAutomapData(void)
 				for (a = obstacle_start_x; a <=  obstacle_end_x; a++) {
 					for (b = obstacle_start_y; b <=  obstacle_end_y; b++) {
 
-						if (obstacle_map[our_obstacle->type].block_area_type == COLLISION_TYPE_RECTANGLE) {
+						if (obstacle_spec->block_area_type == COLLISION_TYPE_RECTANGLE) {
 
-							if (obstacle_map[our_obstacle->type].block_area_parm_1 > 0.80) {
-								if (Me.pos.x < our_obstacle->pos.x)
-									Me.Automap[level][b][a] |= LEFT_WALL_BIT;
-								else
-									Me.Automap[level][b][a] |= RIGHT_WALL_BIT;
+							if (obstacle_spec->block_area_parm_1 > 0.80) {
+									Me.Automap[level][b][a] |= EW_WALL_BIT;
 							}
-							if (obstacle_map[our_obstacle->type].block_area_parm_2 > 0.80) {
-								if (Me.pos.y < our_obstacle->pos.y)
-									Me.Automap[level][b][a] |= UP_WALL_BIT;
-								else
-									Me.Automap[level][b][a] |= DOWN_WALL_BIT;
+							if (obstacle_spec->block_area_parm_2 > 0.80) {
+								Me.Automap[level][b][a] |= NS_WALL_BIT;
 							}
 						}
 					}
 				}
 			}
+			if (visible_event_at_location(x, y, level))
+				Me.Automap[level][y][x] |= VISIBLE_EVENT_BIT;
 
 			Me.Automap[level][y][x] = Me.Automap[level][y][x] | SQUARE_SEEN_AT_ALL_BIT;
 		}
@@ -210,19 +244,63 @@ void CollectAutomapData(void)
 
 };				// void CollectAutomapData ( void )
 
+/**
+ * Sets all of the squares overlapped as not seen, so they can
+ * be updated.
+ */
+void update_obstacle_automap(int z, obstacle *our_obstacle)
+{
+	level *automap_level = curShip.AllLevels[z];
+	int obstacle_start_x;
+	int obstacle_end_x;
+	int obstacle_start_y;
+	int obstacle_end_y;
+
+	obstacle_spec *obstacle_spec = get_obstacle_spec(our_obstacle->type);
+	round_automap_pos(our_obstacle->pos.x + obstacle_spec->left_border,
+			our_obstacle->pos.x + obstacle_spec->right_border, &obstacle_start_x, &obstacle_end_x);
+
+	round_automap_pos(our_obstacle->pos.y + obstacle_spec->upper_border,
+			our_obstacle->pos.y + obstacle_spec->lower_border, &obstacle_start_y, &obstacle_end_y);
+
+	if (obstacle_start_x < 0)
+		obstacle_start_x = 0;
+	if (obstacle_start_y < 0)
+		obstacle_start_y = 0;
+	if (obstacle_end_x < 0)
+		obstacle_end_x = 0;
+	if (obstacle_end_y < 0)
+		obstacle_end_y = 0;
+	if (obstacle_start_x >= automap_level->xlen)
+		obstacle_start_x = automap_level->xlen - 1;
+	if (obstacle_start_y >= automap_level->ylen)
+		obstacle_start_y = automap_level->ylen - 1;
+	if (obstacle_end_x >= automap_level->xlen)
+		obstacle_end_x = automap_level->xlen - 1;
+	if (obstacle_end_y >= automap_level->ylen)
+		obstacle_end_y = automap_level->ylen - 1;
+
+	int x, y;
+	for (x = obstacle_start_x; x <=  obstacle_end_x; x++) {
+		for (y = obstacle_start_y; y <=  obstacle_end_y; y++) {
+			Me.Automap[z][y][x] |= UPDATE_SQUARE_BIT;
+		}
+	}
+}
 
 /**
  * Use GL_POINTS primitive in OpenGL mode to render automap points for better performance.
  * In SDL mode, fallback to PutPixel.
  */
-static inline void PutPixel_automap_wrapper(SDL_Surface * abc, int x, int y, Uint32 pixel)
+static inline void PutPixel_automap_wrapper(SDL_Surface * abc, int x, int y, int r, int g, int b)
 {
 	if (!use_open_gl)
-		PutPixel(abc, x, y, pixel);
+		sdl_put_pixel(abc, x, y, r, g, b, 255);
 #ifdef HAVE_LIBGL
 	else {
-		glColor3ub(((pixel >> 16) & 0xff), (pixel >> 8) & 0xff, (pixel) & 0xff);
+		glColor3ub(r, g, b);
 		glVertex2i(x, y);
+		glColor3ub(255, 255, 255);
 	}
 #endif
 }
@@ -245,18 +323,11 @@ void toggle_automap(void)
 static void display_automap_compass()
 {
 	//load the compass if necessary
-	if (!iso_image_loaded(&compass)) {
-		load_iso_image(&compass, "compass.png", FALSE);
-	}
-
-	if (use_open_gl) {
-		draw_gl_textured_quad_at_screen_position(&compass, GameConfig.screen_width - compass.original_image_width - 10, 40);
-	} else {
-		SDL_Rect dr = {.x = GameConfig.screen_width - compass.original_image_width -10,.y = 40,.w = compass.original_image_width,.h =
-			    compass.original_image_height };
-		our_SDL_blit_surface_wrapper(compass.surface, NULL, Screen, &dr);
+	if (!image_loaded(&compass)) {
+		load_image(&compass, "compass.png", FALSE);
 	}
 
+	display_image_on_screen(&compass, GameConfig.screen_width - compass.w - 10, 40, IMAGE_NO_TRANSFO);
 }
 
 /**
@@ -270,9 +341,7 @@ static void display_automap_compass()
  */
 void display_automap(void)
 {
-	int x, y;
-	int TuxColor = SDL_MapRGB(Screen->format, 0, 0, 255);
-	int FriendColor = SDL_MapRGB(Screen->format, 0, 255, 0);
+	int x, y, i, j;
 	Level automap_level = curShip.AllLevels[Me.pos.z];
 	int level = Me.pos.z;
 
@@ -282,7 +351,8 @@ void display_automap(void)
 		return;
 
 	// Display the compass
-	display_automap_compass();
+	if (game_status != INSIDE_LVLEDITOR)
+		display_automap_compass();
 
 	// Also if there is no map-maker present in inventory, then we need not
 	// do a thing here...
@@ -301,22 +371,33 @@ void display_automap(void)
 	// obstacles on this level.
 	for (y = 0; y < automap_level->ylen; y++) {
 		for (x = 0; x < automap_level->xlen; x++) {
-			if (Me.Automap[level][y][x] & (RIGHT_WALL_BIT | LEFT_WALL_BIT)) {
+			if (Me.Automap[level][y][x] & EW_WALL_BIT) {
 				PutPixel_automap_wrapper(Screen,
 							 1 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * (automap_level->ylen - y),
-							 1 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, AUTOMAP_COLOR);
+							 1 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, WALL_COLOR);
 				PutPixel_automap_wrapper(Screen,
 							 2 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * (automap_level->ylen - y),
-							 2 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, AUTOMAP_COLOR);
+							 2 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, WALL_COLOR);
 			}
 
-			if (Me.Automap[level][y][x] & (UP_WALL_BIT | DOWN_WALL_BIT)) {
+			if (Me.Automap[level][y][x] & NS_WALL_BIT) {
 				PutPixel_automap_wrapper(Screen,
 							 1 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * (automap_level->ylen - y),
-							 2 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, AUTOMAP_COLOR);
+							 2 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, WALL_COLOR);
 				PutPixel_automap_wrapper(Screen,
 							 2 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * (automap_level->ylen - y),
-							 1 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, AUTOMAP_COLOR);
+							 1 + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, WALL_COLOR);
+			}
+
+			//Now we blit known event triggers
+			if (Me.Automap[level][y][x] & VISIBLE_EVENT_BIT) {
+				for (i = 0; i < AUTOMAP_SQUARE_SIZE; i++) {
+					for (j = 0; j < AUTOMAP_SQUARE_SIZE; j++) {
+						PutPixel_automap_wrapper(Screen,
+							i + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * (automap_level->ylen - y),
+							j + AUTOMAP_SQUARE_SIZE * x + AUTOMAP_SQUARE_SIZE * y, EXIT_COLOR);
+					}
+				}
 			}
 		}
 	}
@@ -334,7 +415,13 @@ void display_automap(void)
 								 AUTOMAP_SQUARE_SIZE * erot->pos.x +
 								 AUTOMAP_SQUARE_SIZE * (automap_level->ylen - erot->pos.y) + x,
 								 AUTOMAP_SQUARE_SIZE * erot->pos.x + AUTOMAP_SQUARE_SIZE * erot->pos.y + y,
-								 FriendColor);
+								 FRIEND_COLOR);
+				} else if (Me.nmap_duration > 0) {
+					PutPixel_automap_wrapper(Screen,
+								 AUTOMAP_SQUARE_SIZE * erot->pos.x +
+								 AUTOMAP_SQUARE_SIZE * (automap_level->ylen - erot->pos.y) + x,
+								 AUTOMAP_SQUARE_SIZE * erot->pos.x + AUTOMAP_SQUARE_SIZE * erot->pos.y + y,
+								 ENEMY_COLOR);
 				}
 			}
 		}
@@ -345,7 +432,7 @@ void display_automap(void)
 		for (y = 0; y < AUTOMAP_SQUARE_SIZE; y++) {
 			PutPixel_automap_wrapper(Screen,
 						 AUTOMAP_SQUARE_SIZE * Me.pos.x + AUTOMAP_SQUARE_SIZE * (automap_level->ylen - Me.pos.y) +
-						 x, AUTOMAP_SQUARE_SIZE * Me.pos.x + AUTOMAP_SQUARE_SIZE * Me.pos.y + y, TuxColor);
+						 x, AUTOMAP_SQUARE_SIZE * Me.pos.x + AUTOMAP_SQUARE_SIZE * Me.pos.y + y, TUX_COLOR);
 		}
 	}
 
