@@ -30,6 +30,7 @@
 #include "struct.h"
 #include "global.h"
 #include "proto.h"
+#include "lvledit/lvledit_validator.h"
 
 static int start_stamp;
 static int stop_stamp;
@@ -45,7 +46,7 @@ static void timer_stop()
 }
 
 /* Text rendering performance measurement */
-static void text_bench()
+static int text_bench()
 {
     char *str = "abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz";
 
@@ -59,27 +60,33 @@ static void text_bench()
 	while (nb--) {
 		PutString(Screen, 0, 0, str);
 	}
-
-	our_SDL_update_rect_wrapper(Screen, 0, 0, Screen->w, Screen->h);
+		
+	our_SDL_flip_wrapper();
 	timer_stop();
+
+	return 0;
 }
 
 /* Dialog validator (not an actual benchmark) */
-static void dialog_test()
+static int dialog_test()
 {
+	int failed;
+
 	timer_start();
-	validate_dialogs();
+	failed = validate_dialogs();
 	timer_stop();
+
+	return failed;
 }
 
 /* LoadShip (level loading) performance test */
-static void loadship_bench()
+static int loadship_bench()
 {
 	int loop = 10;
 
 	// Find a ship file to load
 	char fp[2048];
-	find_file("freedroid.levels", MAP_DIR, fp, 0);
+	find_file("levels.dat", MAP_DIR, fp, 0);
 
 	// Load it many times
 	timer_start();
@@ -87,15 +94,18 @@ static void loadship_bench()
 		LoadShip(fp, 0);
 	}
 	timer_stop();
+
+	return 0;
 }
 
 /* LoadGame (savegame loading) performance test */
-static void loadgame_bench()
+static int loadgame_bench()
 {
 	int loop = 3;
 
 	// Use MapEd.savegame
-	sprintf(Me.character_name, "MapEd");
+	free(Me.character_name);
+	Me.character_name = strdup("MapEd");
 
 	// Load it many times
 	timer_start();
@@ -103,10 +113,12 @@ static void loadgame_bench()
 		LoadGame();
 	}
 	timer_stop();
+
+	return 0;
 }
 
 /* Test of dynamic arrays */
-static void dynarray_test()
+static int dynarray_test()
 {
 	int loop = 10;
 	item dummy;
@@ -128,40 +140,59 @@ static void dynarray_test()
 			}
 		}
 		dynarray_free(dyn);
+		free(dyn);
+		dyn = NULL;
 	}
 
 	timer_stop();
+
+	return 0;
 }
 
-static void mapgen_bench()
+static int mapgen_bench()
 {
 	int loop = 100;
-	int i;
-	level l;
-	l.xlen = 90;
-	l.ylen = 90;
-	l.random_dungeon = 2;
-	l.teleport_pair = 0;
-	l.Levelname = "mapgen test level";
-	for (i = 0; i < l.ylen; i++)
-		l.map[i] = (map_tile*)MyMalloc(sizeof(map_tile) * l.xlen);
-	dynarray_init(&l.obstacle_extensions, 0, sizeof(struct obstacle_extension));
-	dynarray_init(&l.map_labels, 0, sizeof(struct map_label));
-	dynarray_init(&l.waypoints, 0, sizeof(struct waypoint));
-
+	extern void CreateNewMapLevel(int);
+	extern int delete_map_level(int);
 	timer_start();
 	while (loop--) {
-		set_dungeon_output(&l);
-		generate_dungeon(l.xlen, l.xlen, l.random_dungeon, l.teleport_pair);
+		CreateNewMapLevel(0);
+		level *l = curShip.AllLevels[0];
+		l->xlen = 90;
+		l->ylen = 90;
+		l->random_dungeon = 2;
+		l->teleport_pair = 0;
+		set_dungeon_output(l);
+		generate_dungeon(l->xlen, l->ylen, l->random_dungeon, l->teleport_pair);
+		delete_map_level(0);
 	}
 	timer_stop();
+
+	return 0;
 }
 
-void benchmark()
+/* Levels validator (not an actual benchmark) */
+static int level_test()
+{
+	int failed;
+
+	// Load default ship
+	char fp[2048];
+	find_file("levels.dat", MAP_DIR, fp, 0);
+	LoadShip(fp, 0);
+
+	timer_start();
+	failed = level_validation_on_console_only();
+	timer_stop();
+
+	return failed;
+}
+
+int benchmark()
 {
 	struct {
 		char *name;
-		void (*func)();
+		int (*func)();
 	} benchs[] = {
 			{ "text", text_bench },
 			{ "dialog", dialog_test },
@@ -169,22 +200,24 @@ void benchmark()
 			{ "loadgame", loadgame_bench },
 			{ "dynarray", dynarray_test },
 			{ "mapgen", mapgen_bench },
+			{ "leveltest", level_test },
 	};
 
 	int i;
 	char str[1024];
+	int failed = 0;
 
-	ClearGraphMem();
+	clear_screen();
 	sprintf(str, "Testing \"%s\"...", do_benchmark);
 	PutString(Screen, 10, 100, str);
-	our_SDL_update_rect_wrapper(Screen, 0, 0, Screen->w, Screen->h);
+	our_SDL_flip_wrapper();
 
 
 	for (i = 0; i < sizeof(benchs)/sizeof(benchs[0]); i++) {
 		if (!strcmp(do_benchmark, benchs[i].name)) {
-			benchs[i].func();
+			failed |= benchs[i].func();
 			printf("Running test %s took %d milliseconds.\n", do_benchmark, stop_stamp - start_stamp);
-			return;
+			return failed;
 		}
 	}
 	
@@ -192,7 +225,7 @@ void benchmark()
 	for (i = 0; i < sizeof(benchs)/sizeof(benchs[0]); i++) {
 		fprintf(stderr, "\t%s\n", benchs[i].name);
 	}
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
 
