@@ -39,16 +39,6 @@
 #define DROIDHITDIST  (0.25)
 #define DROIDHITDIST2 (DROIDHITDIST*DROIDHITDIST)
 
-/**
- * Hit/miss rules depend on the level of the enemy, among other things.
- * This function computes the multiplier used in those rules.
- */
-static float compute_hit_multiplier(int level)
-{
-	if (level < 1 || level > 9) {  ErrorMessage(__FUNCTION__, "Requested non existing level %d.\n", PLEASE_INFORM, IS_WARNING_ONLY, level); return 1.0; }
-	return  HitBotMultiplierTable[level - 1];
-}
-
 /**
  *
  *
@@ -143,21 +133,24 @@ void delete_melee_shot(melee_shot * t)
 void DoMeleeDamage(void)
 {
 	int i;
+	float latest_frame_time = Frame_Time();
 	melee_shot *CurMelS;
 
 	/* Browse all melee shots */
 	for (i = 0; i < MAX_MELEE_SHOTS; i++) {
 		CurMelS = &AllMeleeShots[i];
 
-		if (CurMelS->attack_target_type == ATTACK_TARGET_IS_NOTHING)
+		if (CurMelS->attack_target_type == ATTACK_TARGET_IS_NOTHING || CurMelS->time_to_hit > 0) {
+			CurMelS->time_to_hit -= latest_frame_time;
 			continue;
+		}
 
 		if (CurMelS->attack_target_type == ATTACK_TARGET_IS_ENEMY) {
 			/* Attack an enemy */
 			enemy *tg = enemy_resolve_address(CurMelS->bot_target_n, &CurMelS->bot_target_addr);
 			if (!tg) {
 				ErrorMessage(__FUNCTION__,
-					     "Melee shot was set to ATTACK_TARGET_IS_ENEMY but had no targetted enemy. Deleting.\n",
+					     "Melee shot was set to ATTACK_TARGET_IS_ENEMY but had no targeted enemy. Deleting.\n",
 					     NO_NEED_TO_INFORM, IS_WARNING_ONLY);
 				delete_melee_shot(CurMelS);
 				continue;
@@ -169,7 +162,7 @@ void DoMeleeDamage(void)
 				continue;
 			}
 
-			if (MyRandom(100) < compute_hit_multiplier(Druidmap[tg->type].monster_level) * CurMelS->to_hit ) {
+			if (MyRandom(100) < CurMelS->to_hit) {
 				hit_enemy(tg, CurMelS->damage, CurMelS->mine ? 1 : 0, CurMelS->owner, CurMelS->mine ? 1 : 0);
 			}
 
@@ -229,7 +222,6 @@ void MoveBullets(void)
 			DeleteBullet(i, FALSE);
 			continue;
 		}
-		CurBullet->time_in_frames++;
 		CurBullet->time_in_seconds += Frame_Time();
 
 	}			/* for */
@@ -248,14 +240,11 @@ void DeleteBullet(int Bulletnumber, int ShallWeStartABlast)
 	// cause later, after the bullet is deleted, it will be hard to know
 	// the correct location ;)
 	//
-	int type = (CurBullet->type == 4) ? OWNBLAST : BULLETBLAST;
-
 	if (ShallWeStartABlast)
-		StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, type, (type == OWNBLAST) ? (CurBullet->damage * 1) : 2.0);
+		StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, Bulletmap[CurBullet->type].blast_type, CurBullet->damage, CurBullet->faction);
 
 	CurBullet->type = INFOUT;
 	CurBullet->time_in_seconds = 0;
-	CurBullet->time_in_frames = 0;
 	CurBullet->mine = FALSE;
 	CurBullet->owner = -100;
 	CurBullet->phase = 0;
@@ -263,8 +252,7 @@ void DeleteBullet(int Bulletnumber, int ShallWeStartABlast)
 	CurBullet->pos.y = 0;
 	CurBullet->pos.z = -1;
 	CurBullet->angle = 0;
-
-};				// void DeleteBullet( int Bulletnumber , int StartBlast )
+}
 
 /**
  * This function starts a blast (i.e. an explosion) at the given location
@@ -276,10 +264,10 @@ void DeleteBullet(int Bulletnumber, int ShallWeStartABlast)
  * OWNBLAST          (not implemented)
  *
  */
-void StartBlast(float x, float y, int level, int type, int dmg)
+void StartBlast(float x, float y, int level, int type, int dmg, int faction)
 {
 	int i;
-	Blast NewBlast;
+	blast *NewBlast;
 
 	// Resolve blast real position, if possible
 	//
@@ -292,9 +280,9 @@ void StartBlast(float x, float y, int level, int type, int dmg)
 A BLAST VIRTUAL POSITION WAS FOUND TO BE INCONSISTENT.\n\
 \n\
 However, the error is not fatal and will be silently compensated for now.\n\
-When reporting a problem to the Freedroid developers, please note if this\n\
+When reporting a problem to the FreedroidRPG developers, please note if this\n\
 warning message was created prior to the error in your report.\n\
-However, it should NOT cause any serious trouble for Freedroid.", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
+However, it should NOT cause any serious trouble for FreedroidRPG.", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
 		return;
 	}
 	// If the blast is not on a visible level, we do not take care of it
@@ -326,18 +314,12 @@ However, it should NOT cause any serious trouble for Freedroid.", NO_NEED_TO_INF
 	NewBlast->type = type;
 	NewBlast->phase = 0;
 
-	NewBlast->MessageWasDone = 0;
 	NewBlast->damage_per_second = dmg;
 
-	if (type == DROIDBLAST) {
-		DruidBlastSound();
-	}
+	NewBlast->faction = faction;
 
-	if (type == OWNBLAST) {
-		ExterminatorBlastSound();
-	}
-
-}				// void StartBlast( ... )
+	play_blast_sound(type, &NewBlast->pos);
+}
 
 /**
  * This function advances the different phases of an explosion according
@@ -346,7 +328,7 @@ However, it should NOT cause any serious trouble for Freedroid.", NO_NEED_TO_INF
 void animate_blasts(void)
 {
 	int i;
-	Blast CurBlast = AllBlasts;
+	blast *CurBlast = AllBlasts;
 
 	for (i = 0; i < MAXBLASTS; i++, CurBlast++) {
 		if (CurBlast->type != INFOUT) {
@@ -358,35 +340,32 @@ void animate_blasts(void)
 			if (!pos_inside_level(CurBlast->pos.x, CurBlast->pos.y, curShip.AllLevels[CurBlast->pos.z])) {
 				ErrorMessage(__FUNCTION__, "\
 A BLAST WAS FOUND TO EXIST OUTSIDE THE BOUNDS OF THE MAP.\n\
-This is an idication for an inconsistency in Freedroid.\n\
+This is an indication of an inconsistency in FreedroidRPG.\n\
 \n\
 However, the error is not fatal and will be silently compensated for now.\n\
-When reporting a problem to the Freedroid developers, please note if this\n\
+When reporting a problem to the FreedroidRPG developers, please note if this\n\
 warning message was created prior to the error in your report.\n\
-However, it should NOT cause any serious trouble for Freedroid.", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
+However, it should NOT cause any serious trouble for FreedroidRPG.", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
 				CurBlast->pos.x = 0;
 				CurBlast->pos.y = 0;
 				CurBlast->pos.z = 0;
 				DeleteBlast(i);
 				continue;
 			}
-			// Druid blasts are dangerous, so we check if someone gets
-			// hurt by this particular droid explosion
-			//
-			if (CurBlast->type == DROIDBLAST || CurBlast->type == OWNBLAST)
+			
+			if (Blastmap[CurBlast->type].do_damage)
 				CheckBlastCollisions(i);
 
 			// And now we advance the phase of the blast according to the
 			// time that has passed since the last frame (approximately)
 			//
-			// CurBlast->phase += Frame_Time () * Blastmap[ CurBlast->type ].phases / Blastmap[ CurBlast->type ].total_animation_time;
-			CurBlast->phase += Frame_Time() * PHASES_OF_EACH_BLAST / Blastmap[CurBlast->type].total_animation_time;
+			CurBlast->phase += Frame_Time() * Blastmap[CurBlast->type].phases / Blastmap[CurBlast->type].total_animation_time;
 
 			// Maybe the blast has lived over his normal lifetime already.
 			// Then of course it's time to delete the blast, which is done
 			// here.
 			//
-			if ((int)floorf(CurBlast->phase) >= PHASES_OF_EACH_BLAST)
+			if ((int)floorf(CurBlast->phase) >= Blastmap[CurBlast->type].phases)
 				DeleteBlast(i);
 		}		/* if */
 	}
@@ -606,15 +585,11 @@ I seem to have run out of free bullet entries.  This can't normally happen.  -->
 void check_bullet_background_collisions(bullet * CurBullet, int num)
 {
 	// Check for collision with background
-	if (!SinglePointColldet(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, &FlyablePassFilter)) {
-		if (CurBullet->ignore_wall_collisions) {
-			StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, BULLETBLAST, 0);
-		} else {
-			DeleteBullet(num, TRUE);	// we want a bullet-explosion
-			return;
-		}
+	struct colldet_filter filter = { &FlyablePassFilterCallback, NULL, 0.05, NULL };
+	if (!SinglePointColldet(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, &filter)) {
+			DeleteBullet(num, TRUE);
 	}
-};				// void check_bullet_background_collisions ( CurBullet , num )
+}
 
 /**
  *
@@ -716,7 +691,7 @@ void check_bullet_enemy_collisions(bullet * CurBullet, int num)
 		// be completely deleted of course, with the same small explosion as well
 		//
 		if (CurBullet->pass_through_hit_bodies)
-			StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, BULLETBLAST, 0);
+			StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, BULLETBLAST, 0, CurBullet->faction);
 		else
 			DeleteBullet(num, TRUE);	// we want a bullet-explosion
 
@@ -724,61 +699,6 @@ void check_bullet_enemy_collisions(bullet * CurBullet, int num)
 	}
 };				// void check_bullet_enemy_collisions ( CurBullet , num )
 
-/**
- *
- *
- */
-void check_bullet_bullet_collisions(bullet * CurBullet, int num)
-{
-	int i;
-
-	// check for collisions with other bullets
-	for (i = 0; i < MAXBULLETS; i++) {
-		if (i == num)
-			continue;	// never check for collision with youself.. ;)
-		if (AllBullets[i].type == INFOUT)
-			continue;	// never check for collisions with dead bullets.. 
-
-		// to check the distance between the 2 bullets, we need compatible gps positions, if possible
-		gps other_bullet_pos;
-		update_virtual_position(&other_bullet_pos, &AllBullets[i].pos, CurBullet->pos.z);
-		if (other_bullet_pos.z == -1)
-			continue;
-
-		if (fabsf(other_bullet_pos.x - CurBullet->pos.x) > BULLET_BULLET_COLLISION_DIST)
-			continue;
-		if (fabsf(other_bullet_pos.y - CurBullet->pos.y) > BULLET_BULLET_COLLISION_DIST)
-			continue;
-
-		// it seems like we have a collision of two bullets!
-		// both will be deleted and replaced by blasts..
-		DebugPrintf(1, "\nBullet-Bullet-Collision detected...");
-
-		if (CurBullet->reflect_other_bullets) {
-			if (AllBullets[i].was_reflected) {
-				// well, if it has been reflected once, we don't do any more
-				// reflections after that...
-			} else {
-				AllBullets[i].speed.x = -AllBullets[i].speed.x;
-				AllBullets[i].speed.y = -AllBullets[i].speed.y;
-				AllBullets[i].was_reflected = TRUE;
-			}
-		}
-
-		if (AllBullets[i].reflect_other_bullets) {
-			if (CurBullet->was_reflected) {
-				// well, if it has been reflected once, we don't do any more
-				// reflections after that...
-			} else {
-				CurBullet->speed.x = -CurBullet->speed.x;
-				CurBullet->speed.y = -CurBullet->speed.y;
-				CurBullet->was_reflected = TRUE;
-			}
-		}
-
-	}
-};				// void check_bullet_bullet_collisions ( CurBullet , num )
-
 /**
  * This function checks if there are some collisions of the one bullet
  * with number num with anything else in the game, like blasts, walls,
@@ -847,12 +767,6 @@ void CheckBulletCollisions(int num)
 		CurBullet->pos.y = bullet_actual_pos.y;
 		CurBullet->pos.z = bullet_actual_pos.z;
 
-		// Bullet/bullet collision code is level-independent
-
-		check_bullet_bullet_collisions(CurBullet, num);
-		if (CurBullet->type == INFOUT)
-			return;
-
 		break;
 	}			// switch ( Bullet-Type )
 };				// CheckBulletCollisions( ... )
@@ -867,33 +781,9 @@ void CheckBulletCollisions(int num)
  */
 void CheckBlastCollisions(int num)
 {
-	int i;
-	Blast CurBlast = &(AllBlasts[num]);
+	blast *CurBlast = &(AllBlasts[num]);
 	static const float Blast_Radius = 1.5;
 
-	// At first, we check for collisions of this blast with all bullets 
-	//
-	if (CurBlast->phase <= 4) {
-		for (i = 0; i < MAXBULLETS; i++) {
-			if (AllBullets[i].type == INFOUT)
-				continue;
-			if (AllBullets[i].pass_through_explosions)
-				continue;
-
-			// Compatible gps positions are needed to compute distance between bullet and blast
-			gps bullet_vpos;
-			update_virtual_position(&bullet_vpos, &AllBullets[i].pos, CurBlast->pos.z);
-			if (bullet_vpos.z == -1)
-				continue;
-
-			if (abs(bullet_vpos.x - CurBlast->pos.x) >= Blast_Radius)
-				continue;
-			if (abs(bullet_vpos.y - CurBlast->pos.y) >= Blast_Radius)
-				continue;
-
-			DeleteBullet(i, TRUE);	// we want a bullet-explosion
-		}
-	}
 	// Now we check for enemys, that might have stepped into this
 	// one blasts area of effect...
 	//
@@ -922,25 +812,65 @@ void CheckBlastCollisions(int num)
 			if (fabsf(erot->pos.y - blast_vpos.y) >= Blast_Radius)
 				continue;
 
-			/* we have no support for blast ownership yet, so we give no XP *and* don't know who killed the guy */
-			hit_enemy(erot, CurBlast->damage_per_second * Frame_Time(), 0, -1, 0);
+			if (is_friendly(CurBlast->faction, erot->faction))
+				continue;
+
+			hit_enemy(erot, CurBlast->damage_per_second * Frame_Time(), 0, -1, CurBlast->faction == FACTION_SELF ? 1 : 0);
 		}
 	}
 
 	// Now we check, if perhaps the influencer has stepped into the area
 	// of effect of this one blast.  Then he'll get burnt ;)
-	// 
-	update_virtual_position(&blast_vpos, &CurBlast->pos, Me.pos.z);
-	if (blast_vpos.z != -1) {
-		if ((Me.energy > 0) && (fabsf(Me.pos.x - blast_vpos.x) < Blast_Radius) && (fabsf(Me.pos.y - blast_vpos.y) < Blast_Radius)) {
-			hit_tux(CurBlast->damage_per_second * Frame_Time(), -100);
-			// So the influencer got some damage from the hot blast
-			// Now most likely, he then will also say so :)
-			if (!CurBlast->MessageWasDone) {
-				CurBlast->MessageWasDone = TRUE;
+	//
+	if (!is_friendly(CurBlast->faction, FACTION_SELF)) {	
+		update_virtual_position(&blast_vpos, &CurBlast->pos, Me.pos.z);
+		if (blast_vpos.z != -1) {
+			if ((fabsf(Me.pos.x - blast_vpos.x) < Blast_Radius) && (fabsf(Me.pos.y - blast_vpos.y) < Blast_Radius)) {
+				float real_damage = CurBlast->damage_per_second * Frame_Time() * get_player_damage_factor();
+				hit_tux(real_damage, -100);
 			}
 		}
 	}
-};				// CheckBlastCollisions( ... )
+}
+
+/**
+ * This function returns the bullet number of a specified bullet string.
+ * Bullet strings are defined in map/bullet_archtypes.dat
+ *
+ * Defaults to 0
+ */
+int GetBulletByName(const char *bullet_name)
+{
+	int i;
+	for (i=0; i < Number_Of_Bullet_Types; i++) {
+		if (!Bulletmap[i].name)
+			return 0;
+
+		if (!strcmp(bullet_name, Bulletmap[i].name))
+			return i;
+	}
+	ErrorMessage(__FUNCTION__, "\
+The bullet name \"%s\" lacks a definition.", PLEASE_INFORM, IS_WARNING_ONLY, bullet_name);
+	return 0;
+}
+
+/**
+ * This functions resolves the blast name to blast type.
+ * If blast type wasn't find for the given name the default blast type is
+ * returned.
+ */
+int get_blast_type_by_name(const char *name)
+{
+	if (!name)
+		return 0;
+
+	int i;
+	for (i = 0; i < sizeof(Blastmap) / sizeof(Blastmap[0]); i++) {
+		if (!strcmp(name, Blastmap[i].name))
+			return i;
+	}
+
+	return 0;
+}
 
 #undef _bullet_c
