@@ -36,11 +36,13 @@
 #include "global.h"
 #include "proto.h"
 
-#define Energy_Gain_Per_Vit_Point 2;
-#define Maxtemp_Gain_Per_CPU_Point 4;
+#define BASE_EXP_REQUIRED 500
+#define BASE_HPMAX 20
+#define HPMAX_GAIN_PER_LEVEL 5
+#define HPMAX_GAIN_PER_PHY_POINT 2
+#define Maxtemp_Gain_Per_CPU_Point 4
 
 #define RECHARGE_SPEED_PERCENT_PER_DEX_POINT 0
-#define TOHIT_PERCENT_PER_DEX_POINT (1.0)
 
 //--------------------
 // At first we state some geometry constants for where to insert
@@ -75,47 +77,6 @@
 #define RANGED_SKILL_Y 268
 #define SPELLCASTING_SKILL_Y 306
 
-/**
- * This function displays all the buttons that open up the character
- * screen and the inventory screen
- */
-void DisplayButtons(void)
-{
-
-	if (GameConfig.screen_height == 480 && (GameConfig.Inventory_Visible || GameConfig.skill_explanation_screen_visible))
-		goto display_overlay_chascr;
-
-	if (Me.quest_browser_changed)
-		ShowGenericButtonFromList(LOG_SCREEN_TOGGLE_BUTTON_RED);
-
-	if (MouseCursorIsOnButton(INV_SCREEN_TOGGLE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		ShowGenericButtonFromList(INV_SCREEN_TOGGLE_BUTTON_YELLOW);
-	} else if (MouseCursorIsOnButton(LOG_SCREEN_TOGGLE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		ShowGenericButtonFromList(LOG_SCREEN_TOGGLE_BUTTON_YELLOW);
-	}
-
- display_overlay_chascr:
-	if (GameConfig.screen_height == 480 && (GameConfig.SkillScreen_Visible || GameConfig.CharacterScreen_Visible))
-		return;
-
-	// When the Tux has some extra skill points, that can be distributed
-	// to some character stats or saved for later training with some trainer
-	// character in the city, we mark the character screen toggle button as
-	// red to indicate the available points.
-	//
-	if (Me.points_to_distribute > 0) {
-		// blit_special_background ( MOUSE_BUTTON_PLUS_BACKGROUND_PICTURE_CODE );
-		ShowGenericButtonFromList(CHA_SCREEN_TOGGLE_BUTTON_RED);
-	}
-
-	if (MouseCursorIsOnButton(CHA_SCREEN_TOGGLE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		ShowGenericButtonFromList(CHA_SCREEN_TOGGLE_BUTTON_YELLOW);
-	} else if (MouseCursorIsOnButton(SKI_SCREEN_TOGGLE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		ShowGenericButtonFromList(SKI_SCREEN_TOGGLE_BUTTON_YELLOW);
-	}
-
-};				// void DisplayButtons( void )
-
 /**
  * This function adds any bonuses that might be on the influencers things
  * concerning ONLY PRIMARY STATS, NOT SECONDARY STATS!
@@ -128,10 +89,10 @@ static void AddInfluencerItemAttributeBonus(item * BonusItem)
 		return;
 
 	// Apply the bonuses to the stats of the character.
-	Me.Strength += BonusItem->bonus_to_str + BonusItem->bonus_to_all_attributes;
-	Me.Dexterity += BonusItem->bonus_to_dex + BonusItem->bonus_to_all_attributes;
-	Me.Magic += BonusItem->bonus_to_cooling + BonusItem->bonus_to_all_attributes;
-	Me.Vitality += BonusItem->bonus_to_physique + BonusItem->bonus_to_all_attributes;
+	Me.strength += BonusItem->bonus_to_str + BonusItem->bonus_to_all_attributes;
+	Me.dexterity += BonusItem->bonus_to_dex + BonusItem->bonus_to_all_attributes;
+	Me.cooling += BonusItem->bonus_to_cooling + BonusItem->bonus_to_all_attributes;
+	Me.physique += BonusItem->bonus_to_physique + BonusItem->bonus_to_all_attributes;
 
 };				// void AddInfluencerItemAttributeBonus( item* BonusItem )
 
@@ -151,8 +112,6 @@ static void AddInfluencerItemSecondaryBonus(item * BonusItem)
 	Me.maxenergy += BonusItem->bonus_to_health_points;
 	Me.health_recovery_rate += BonusItem->bonus_to_health_recovery;
 	Me.cooling_rate += BonusItem->bonus_to_cooling_rate;
-	Me.resist_fire += BonusItem->bonus_to_resist_fire;
-	Me.resist_electricity += BonusItem->bonus_to_resist_electricity;
 
 	// Apply bonuses to the special abilities of the character.
 	Me.slowing_melee_targets += BonusItem->bonus_to_slow_enemy;
@@ -162,41 +121,27 @@ static void AddInfluencerItemSecondaryBonus(item * BonusItem)
 }
 
 /**
- * Maybe the influencer has reached a new experience level?
- * Let's check this...
+ * \brief Get the experience required for next level.
+ * \param level Current level.
  */
-void check_for_new_experience_level_reached()
+int get_experience_required(int level)
 {
-	int BaseExpRequired = 500;
+	if (level < 0)
+		return 0;
+	return BASE_EXP_REQUIRED * (level * level) + BASE_EXP_REQUIRED;
+}
 
-	if (Me.exp_level >= 24)
-		return;
+/**
+ * \brief Check if Tux has reached a new experience level.
+ */
+static void check_for_new_experience_level_reached()
+{
+	int exp_required = get_experience_required(Me.exp_level);
 
-	Me.ExpRequired = BaseExpRequired * (exp((Me.exp_level - 1) * log(2)));
-
-	// For display reasons in the experience graph, we also state the experience 
-	// needed for the previous level inside the tux struct.  Therefore all exp/level
-	// calculations are found in this function.
-	//
-	if (Me.exp_level > 1) {
-		Me.ExpRequired_previously = BaseExpRequired * (exp((Me.exp_level - 2) * log(2)));
-	} else
-		Me.ExpRequired_previously = 0;
-
-	if (Me.Experience > Me.ExpRequired) {
+	if (Me.Experience >= exp_required) {
 		Me.exp_level++;
 		Me.points_to_distribute += 5;
 
-		if (Me.exp_level >= 24) {
-			SetNewBigScreenMessage(_("Max level reached!"));
-			return;
-		}
-
-		// Like in the Gothic 1 game, maximum life force will now automatically
-		// be increased upon reaching a new character level.
-		//
-		Me.base_vitality += 3;
-
 		// When a droid reaches a new experience level, all health and 
 		// force are restored to full this one time no longer.  Gothic
 		// rulez more than Diablo rulez.
@@ -213,8 +158,7 @@ void check_for_new_experience_level_reached()
 };				// void check_for_new_experience_level_reached ( )
 
 /**
- *
- *
+ * Update all primary stats.
  */
 void update_all_primary_stats()
 {
@@ -222,12 +166,10 @@ void update_all_primary_stats()
 
 	// Now we base PRIMARY stats
 	//
-	Me.Strength = Me.base_strength;
-	Me.Dexterity = Me.base_dexterity;
-	Me.Magic = Me.base_magic;
-	Me.Vitality = Me.base_vitality;
-
-	Me.double_ranged_damage = FALSE;
+	Me.strength = Me.base_strength;
+	Me.dexterity = Me.base_dexterity;
+	Me.cooling = Me.base_cooling;
+	Me.physique = Me.base_physique;
 
 	// Now we add all bonuses to the influencers PRIMARY stats
 	//
@@ -241,9 +183,9 @@ void update_all_primary_stats()
 	// now...
 	//
 	if (Me.dexterity_bonus_end_date > Me.current_game_date)
-		Me.Dexterity += Me.current_dexterity_bonus;
+		Me.dexterity += Me.current_dexterity_bonus;
 	if (Me.power_bonus_end_date > Me.current_game_date)
-		Me.Strength += Me.current_power_bonus;
+		Me.strength += Me.current_power_bonus;
 
 	// Unequip items whose requirements aren't met anymore.
 	// This needs to be done last so that temporary bonuses, for example from
@@ -258,6 +200,16 @@ void update_all_primary_stats()
 			give_item(equipment[i]);
 		}
 	}
+
+	// Ensure the effective primary stats never drops below 1
+	if (Me.strength < 1)
+		Me.strength = 1;
+	if (Me.dexterity < 1)
+		Me.dexterity = 1;
+	if (Me.cooling < 1)
+		Me.cooling = 1;
+	if (Me.physique < 1)
+		Me.physique = 1;
 }
 
 /**
@@ -266,24 +218,25 @@ void update_all_primary_stats()
  * current 'magic' modifiers from equipped items will be applied somewhere
  * else.
  */
-void update_secondary_stats_from_primary_stats()
+static void update_secondary_stats_from_primary_stats()
 {
 	// The chance that this player character will score a hit on an enemy
 	//
-	Me.to_hit = 60 + (Me.Dexterity) * TOHIT_PERCENT_PER_DEX_POINT;
+	Me.to_hit = 100 - 40 * exp(- 0.03 * Me.dexterity);
 
 	// How many life points can this character aquire currently
 	//
-	Me.maxenergy = (Me.Vitality) * Energy_Gain_Per_Vit_Point;
+	Me.maxenergy = BASE_HPMAX + (Me.exp_level) * HPMAX_GAIN_PER_LEVEL
+                                  + (Me.physique) * HPMAX_GAIN_PER_PHY_POINT;
 
 	// The maximum mana value computed from the primary stats
 	//
-	Me.max_temperature = (Me.Magic) * Maxtemp_Gain_Per_CPU_Point;
+	Me.max_temperature = (Me.cooling) * Maxtemp_Gain_Per_CPU_Point;
 
 	// How long can this character run until he must take a break and
 	// walk a bit
 	//
-	Me.max_running_power = (Me.Strength) + (Me.Dexterity) + (Me.Vitality) + Me.running_power_bonus;
+	Me.max_running_power = (Me.strength) + (Me.dexterity) + (Me.physique) + Me.running_power_bonus;
 
 	// base regeneration speed set to 0.2 points per second
 	Me.health_recovery_rate = 0.2;
@@ -296,16 +249,16 @@ void update_secondary_stats_from_primary_stats()
  * character is using.  And depending on the weapon type (melee or
  * ranged weapon) some additional attributes will also play a role.
  */
-void update_damage_tux_can_do()
+static void update_damage_tux_can_do()
 {
 	if (Me.weapon_item.type != (-1)) {
 		if (ItemMap[Me.weapon_item.type].item_weapon_is_melee != 0) {
 			// Damage modifier in case of MELEE WEAPON is computed:  
-			// weapon's modifier * (100+Strength)%
+			// weapon's modifier * (100+strength)%
 			//
-			Me.base_damage = Me.weapon_item.damage * (Me.Strength + 100.0) / 100.0;
+			Me.base_damage = Me.weapon_item.damage * (Me.strength + 100.0) / 100.0;
 
-			Me.damage_modifier = Me.weapon_item.damage_modifier * (Me.Strength + 100.0) / 100.0;
+			Me.damage_modifier = Me.weapon_item.damage_modifier * (Me.strength + 100.0) / 100.0;
 
 			// Damage AND damage modifier a modified by additional melee weapon
 			// skill:  A multiplier is applied!
@@ -314,24 +267,16 @@ void update_damage_tux_can_do()
 			Me.base_damage *= MeleeDamageMultiplierTable[Me.melee_weapon_skill];
 		} else {
 			// Damage modifier in case of RANGED WEAPON is computed:  
-			// weapon's modifier * (100+Dexterity)%
+			// weapon's modifier * (100+dexterity)%
 			//
-			Me.base_damage = Me.weapon_item.damage * (Me.Dexterity + 100.0) / 100.0;
-			Me.damage_modifier = Me.weapon_item.damage_modifier * (Me.Dexterity + 100.0) / 100.0;
+			Me.base_damage = Me.weapon_item.damage * (Me.dexterity + 100.0) / 100.0;
+			Me.damage_modifier = Me.weapon_item.damage_modifier * (Me.dexterity + 100.0) / 100.0;
 
 			// Damage AND damage modifier a modified by additional ranged weapon
 			// skill:  A multiplier is applied!
 			//
 			Me.damage_modifier *= RangedDamageMultiplierTable[Me.ranged_weapon_skill];
 			Me.base_damage *= RangedDamageMultiplierTable[Me.ranged_weapon_skill];
-
-			// Maybe there is a plugin for double damage with ranged
-			// weapons present?
-			//
-			if (Me.double_ranged_damage != 0) {
-				Me.base_damage *= 2;
-				Me.damage_modifier *= 2;
-			}
 		}
 	} else {
 		// In case of no weapon equipped at all, we initialize
@@ -401,11 +346,13 @@ void UpdateAllCharacterStats()
 		Me.light_bonus_from_tux += 5 * ratio;
 	}
 
-	// Check player's health and temperature
+	// Check player's health, temperature and stamina
 	if (Me.energy > Me.maxenergy)
 		Me.energy = Me.maxenergy;
 	if (Me.temperature < 0)
 		Me.temperature = 0;
+	if (Me.running_power > Me.max_running_power)
+		Me.running_power = Me.max_running_power;
 
 }
 
@@ -413,7 +360,7 @@ void UpdateAllCharacterStats()
  * Now we print out the current skill levels in hacking skill, 
  * spellcasting, melee combat, ranged weapon combat and repairing things
  */
-void show_character_screen_skills()
+static void show_character_screen_skills()
 {
 
 	// We add some security against skill values out of allowed
@@ -425,10 +372,10 @@ void show_character_screen_skills()
 Error: melee weapon skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
 	}
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Melee"), RIGHT_TXT_X + CharacterRect.x, MELEE_SKILL_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Melee"), RIGHT_TXT_X + CharacterRect.x, MELEE_SKILL_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
-	DisplayText(_(AllSkillTexts[Me.melee_weapon_skill]),
-		    SKILLS_VALUE_X + CharacterRect.x, MELEE_SKILL_Y + 17 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_(AllSkillTexts[Me.melee_weapon_skill]),
+		    SKILLS_VALUE_X + CharacterRect.x, MELEE_SKILL_Y + 17 + CharacterRect.y, &CharacterRect);
 
 	// We add some security against skill values out of allowed
 	// bounds.
@@ -439,10 +386,10 @@ Error: melee weapon skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
 Error: ranged weapon skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
 	}
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Ranged"), RIGHT_TXT_X + CharacterRect.x, RANGED_SKILL_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Ranged"), RIGHT_TXT_X + CharacterRect.x, RANGED_SKILL_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
-	DisplayText(_(AllSkillTexts[Me.ranged_weapon_skill]),
-		    SKILLS_VALUE_X + CharacterRect.x, RANGED_SKILL_Y + 17 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_(AllSkillTexts[Me.ranged_weapon_skill]),
+		    SKILLS_VALUE_X + CharacterRect.x, RANGED_SKILL_Y + 17 + CharacterRect.y, &CharacterRect);
 
 	// We add some security against skill values out of allowed
 	// bounds.
@@ -453,10 +400,10 @@ Error: ranged weapon skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
 Error: Programming_Skill skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
 	}
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Programming"), RIGHT_TXT_X + CharacterRect.x, SPELLCASTING_SKILL_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Programming"), RIGHT_TXT_X + CharacterRect.x, SPELLCASTING_SKILL_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
-	DisplayText(_(AllSkillTexts[Me.spellcasting_skill]),
-		    SKILLS_VALUE_X + CharacterRect.x, SPELLCASTING_SKILL_Y + 17 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_(AllSkillTexts[Me.spellcasting_skill]),
+		    SKILLS_VALUE_X + CharacterRect.x, SPELLCASTING_SKILL_Y + 17 + CharacterRect.y, &CharacterRect);
 
 };				// void show_character_screen_skills ( )
 
@@ -466,7 +413,6 @@ Error: Programming_Skill skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
 void ShowCharacterScreen()
 {
 	char CharText[1000];
-	point CurPos;
 
 	DebugPrintf(2, "\n%s(): Function call confirmed.", __FUNCTION__);
 
@@ -478,12 +424,7 @@ void ShowCharacterScreen()
 		return;
 
 	SetCurrentFont(Messagestat_BFont);
-
-	// We will need the current mouse position on several spots...
-	//
-	CurPos.x = GetMousePos_x();
-	CurPos.y = GetMousePos_y();
-
+	
 	// We define the right side of the user screen as the rectangle
 	// for our inventory screen.
 	//
@@ -492,102 +433,102 @@ void ShowCharacterScreen()
 	CharacterRect.w = CHARACTERRECT_W;
 	CharacterRect.h = CHARACTERRECT_H;
 
-	blit_special_background(CHARACTER_SCREEN_BACKGROUND_CODE);
+	blit_background("character.png");
 
 	SetCurrentFont(Blue_BFont);
 
 	SetCurrentFont(Menu_BFont);
-	DisplayText(Me.character_name, 20 + CharacterRect.x, 12 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(Me.character_name, 20 + CharacterRect.x, 12 + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Level"), LEFT_TXT_X + CharacterRect.x, 73 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Level"), LEFT_TXT_X + CharacterRect.x, 73 + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d", Me.exp_level);
-	DisplayText(CharText, LEVEL_NR_X + CharacterRect.x, 73 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 73 + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Experience"), LEFT_TXT_X + CharacterRect.x, 89 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Experience"), LEFT_TXT_X + CharacterRect.x, 89 + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%u", Me.Experience);
-	DisplayText(CharText, LEVEL_NR_X + CharacterRect.x, 89 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 89 + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Next level"), LEFT_TXT_X + CharacterRect.x, 107 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Next level"), LEFT_TXT_X + CharacterRect.x, 107 + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%u", Me.ExpRequired);
-	DisplayText(CharText, LEVEL_NR_X + CharacterRect.x, 107 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	sprintf(CharText, "%u", get_experience_required(Me.exp_level));
+	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 107 + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Money"), RIGHT_TXT_X + CharacterRect.x, 71 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Circuits"), RIGHT_TXT_X + CharacterRect.x, 71 + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%6d", Me.Gold);
-	DisplayText(CharText, 240 + CharacterRect.x, 71 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, 240 + CharacterRect.x, 71 + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Strength"), LEFT_TXT_X + CharacterRect.x, STR_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Strength"), LEFT_TXT_X + CharacterRect.x, STR_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%d", Me.Strength);
-	if (Me.Strength != Me.base_strength)
-		sprintf(CharText + strlen(CharText), " (%+d)", Me.Strength - Me.base_strength);
-	DisplayText(CharText, STR_X + CharacterRect.x, STR_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	sprintf(CharText, "%d", Me.strength);
+	if (Me.strength != Me.base_strength)
+		sprintf(CharText + strlen(CharText), " (%+d)", Me.strength - Me.base_strength);
+	display_text(CharText, STR_X + CharacterRect.x, STR_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Dexterity"), LEFT_TXT_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Dexterity"), LEFT_TXT_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%d", Me.Dexterity);
-	if (Me.Dexterity != Me.base_dexterity)
-		sprintf(CharText + strlen(CharText), " (%+d)", Me.Dexterity - Me.base_dexterity);
-	DisplayText(CharText, STR_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	sprintf(CharText, "%d", Me.dexterity);
+	if (Me.dexterity != Me.base_dexterity)
+		sprintf(CharText + strlen(CharText), " (%+d)", Me.dexterity - Me.base_dexterity);
+	display_text(CharText, STR_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Physique"), LEFT_TXT_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Physique"), LEFT_TXT_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%d", Me.Vitality);
-	if (Me.Vitality != Me.base_vitality)
-		sprintf(CharText + strlen(CharText), " (%+d)", Me.Vitality - Me.base_vitality);
-	DisplayText(CharText, STR_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	sprintf(CharText, "%d", Me.physique);
+	if (Me.physique != Me.base_physique)
+		sprintf(CharText + strlen(CharText), " (%+d)", Me.physique - Me.base_physique);
+	display_text(CharText, STR_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Cooling"), LEFT_TXT_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Cooling"), LEFT_TXT_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%d", Me.Magic);
-	if (Me.Magic != Me.base_magic)
-		sprintf(CharText + strlen(CharText), " (%+d)", Me.Magic - Me.base_magic);
-	DisplayText(CharText, STR_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	sprintf(CharText, "%d", Me.cooling);
+	if (Me.cooling != Me.base_cooling)
+		sprintf(CharText + strlen(CharText), " (%+d)", Me.cooling - Me.base_cooling);
+	display_text(CharText, STR_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect);
 
 	SDL_Rect tmprect = CharacterRect;
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Training points"), LEFT_TXT_X + CharacterRect.x, POINTS_Y + CharacterRect.y, &tmprect, TEXT_STRETCH);
+	display_text(_("Training points"), LEFT_TXT_X + CharacterRect.x, POINTS_Y + CharacterRect.y, &tmprect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d", Me.points_to_distribute);
-	DisplayText(CharText, 155 + CharacterRect.x, POINTS_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, 155 + CharacterRect.x, POINTS_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Health"), LEFT_TXT_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &tmprect, TEXT_STRETCH);
+	display_text(_("Health"), LEFT_TXT_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &tmprect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d/%d", (int)Me.energy, (int)Me.maxenergy);
-	DisplayText(CharText, HEALTH_NR_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, HEALTH_NR_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Temperature"), LEFT_TXT_X + CharacterRect.x, TEMP_STAT_Y + CharacterRect.y, &tmprect, TEXT_STRETCH);
+	display_text(_("Temperature"), LEFT_TXT_X + CharacterRect.x, TEMP_STAT_Y + CharacterRect.y, &tmprect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d/%d", (int)Me.temperature, (int)Me.max_temperature);
-	DisplayText(CharText, HEALTH_NR_X + CharacterRect.x, TEMP_STAT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, HEALTH_NR_X + CharacterRect.x, TEMP_STAT_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Stamina"), LEFT_TXT_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &tmprect, TEXT_STRETCH);
+	display_text(_("Stamina"), LEFT_TXT_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &tmprect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d/%d", (int)Me.running_power, (int)Me.max_running_power);
-	DisplayText(CharText, HEALTH_NR_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, HEALTH_NR_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Attack"), RIGHT_TXT_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Attack"), RIGHT_TXT_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d%%", (int)Me.to_hit);
-	DisplayText(CharText, ARMOR_NR_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, ARMOR_NR_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Damage"), RIGHT_TXT_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Damage"), RIGHT_TXT_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
 
 	// Display range of damage, or a single value if there is no range
@@ -596,13 +537,13 @@ void ShowCharacterScreen()
 	else
 		sprintf(CharText, "%d", (int)Me.base_damage);
 
-	DisplayText(CharText, ARMOR_NR_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, ARMOR_NR_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Armor"), RIGHT_TXT_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(_("Armor"), RIGHT_TXT_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d", (int)Me.armor_class);
-	DisplayText(CharText, ARMOR_NR_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	display_text(CharText, ARMOR_NR_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect);
 
 	// Now we print out the current skill levels in hacking skill, 
 	// spellcasting, melee combat, ranged weapon combat and repairing things
@@ -635,11 +576,11 @@ void HandleCharacterScreen(void)
 			Me.points_to_distribute--;
 		}
 		if (MouseCursorIsOnButton(MORE_MAG_BUTTON, GetMousePos_x(), GetMousePos_y()) && MouseLeftClicked()) {
-			Me.base_magic++;
+			Me.base_cooling++;
 			Me.points_to_distribute--;
 		}
 		if (MouseCursorIsOnButton(MORE_VIT_BUTTON, GetMousePos_x(), GetMousePos_y()) && MouseLeftClicked()) {
-			Me.base_vitality++;
+			Me.base_physique++;
 			Me.points_to_distribute--;
 		}
 
