@@ -38,14 +38,19 @@
 #include "struct.h"
 #include "proto.h"
 #include "global.h"
-#include "SDL_rotozoom.h"
+
+#include "widgets/widgets.h"
 
 #define PUSH_ROSTER 2
 #define POP_ROSTER 3
+#define CHAT_TOPIC_STACK_SIZE 10
 
-char *chat_initialization_code;	//first time with a character-code
-char *chat_startup_code;	//every time we start this dialog-code
-EXTERN char *PrefixToFilename[ENEMY_ROTATION_MODELS_AVAILABLE];
+static char *chat_initialization_code;	//first time with a character-code
+static char *chat_startup_code;	//every time we start this dialog-code
+
+// Topic stack
+static char *topic_stack[CHAT_TOPIC_STACK_SIZE];
+static unsigned int topic_stack_slot;
 
 static void run_chat(enemy *ChatDroid, int is_subdialog);
 
@@ -55,8 +60,9 @@ static void run_chat(enemy *ChatDroid, int is_subdialog);
  */
 static void clear_dialog_option(dialogue_option * d)
 {
+	d->topic = "";
 	d->option_text = "";
-	d->option_sample_file_name = "";
+	d->no_text = 0;
 	d->lua_code = NULL;
 	d->exists = 0;
 }
@@ -67,13 +73,12 @@ static void clear_dialog_option(dialogue_option * d)
 static void delete_one_dialog_option(int i, int FirstInitialisation)
 {
 	// If this is not the first initialisation, we have to free the allocated
-	// strings first, or we'll be leaking memory otherwise...
-	//
+	// strings first, or we'll be leaking memory...
 	if (!FirstInitialisation) {
+		if (strlen(ChatRoster[i].topic))
+			free(ChatRoster[i].topic);
 		if (strlen(ChatRoster[i].option_text))
 			free(ChatRoster[i].option_text);
-		if (strlen(ChatRoster[i].option_sample_file_name))
-			free(ChatRoster[i].option_sample_file_name);
 	}
 
 	if (ChatRoster[i].lua_code)
@@ -125,82 +130,35 @@ There was an unrecognized parameter handled to this function.", PLEASE_INFORM, I
 
 };				// push_or_pop_chat_roster ( int push_or_pop )
 
-/**
- * This function plants a cookie, i.e. sets a new text string with the
- * purpose of serving as a flag.  These flags can be set/unset from the dialog
- * file and used from within there and they get stored and loaded with
- * every gave via the tux_t structure.
- */
-void PlantCookie(const char *CookieString)
-{
-	int i;
-
-	// Check if the cookie has already been set. 
-	//
-	for (i = 0; i < MAX_COOKIES; i++) {
-		if (Me.cookie_list[i]) {
-			if (!strcmp(Me.cookie_list[i], CookieString)) {
-				DebugPrintf(0, "\n\nTHAT COOKIE WAS ALREADY SET... DOING NOTHING...\n\n");
-				return;
-			}
-		}
-	}
-
-	// Now we find a good new and free position for our new cookie...
-	//
-	for (i = 0; i < MAX_COOKIES; i++) {
-		if (!Me.cookie_list[i]) {
-			break;
-		}
-	}
-
-	// Maybe the position we have found is the last one.  That would mean too
-	// many cookies, a case that should never occur in FreedroidRPG and that is
-	// a considered a fatal error...
-	if (i >= MAX_COOKIES) {
-		fprintf(stderr, "\n\nCookieString: %s\n", CookieString);
-		ErrorMessage(__FUNCTION__, "\
-There were no more free positions available to store this cookie.\n\
-This should not be possible without a severe bug in FreedroidRPG.", PLEASE_INFORM, IS_FATAL);
-	}
-	
-	// Now that we know that we have found a good position for storing our
-	// new cookie, we can do it.
-	Me.cookie_list[i] = MyMalloc(strlen(CookieString)+1);
-	strcpy(Me.cookie_list[i], CookieString);
-	DebugPrintf(0, "\n\nNEW COOKIE STORED:  Position=%d Text='%s'.\n\n", i, CookieString);
-
-};
 
 /**
- * This function deletes planted cookie, i.e. delete a text string with the
- * purpose of serving as a flag.  These flags can be set/unset from the dialog
- * file and used from within there and they get stored and loaded with
- * every game via the tux_t structure.
+ * \brief Push a new topic in the topic stack.
+ * \param topic The topic name.
  */
-void DeleteCookie(const char *CookieString)
+void chat_push_topic(const char *topic)
 {
-	int i;
-	for (i = 0; i < MAX_COOKIES; i++) {
-		if (!Me.cookie_list[i])
-			continue;
-
-		if (!strcmp(Me.cookie_list[i], CookieString))
-			break;
-	}
-
-	if (i == MAX_COOKIES) {
-		ErrorMessage(__FUNCTION__, "The specified cookie \"%s\" was not found.", PLEASE_INFORM, IS_WARNING_ONLY, CookieString);
+	if (topic_stack_slot < CHAT_TOPIC_STACK_SIZE - 1) {
+		topic_stack[++topic_stack_slot] = strdup(topic);
 	} else {
-		free(Me.cookie_list[i]);
-		for (i = i+1; i < MAX_COOKIES; i++) {
-			Me.cookie_list[i - 1] = Me.cookie_list[i];
-		}
-		Me.cookie_list[MAX_COOKIES - 1] = NULL;
-
-		DebugPrintf(1, "Cookie deleted.");
+		ErrorMessage(__FUNCTION__,
+			     "The maximum depth of %hd topics has been maxed out.",
+			     PLEASE_INFORM, IS_WARNING_ONLY, CHAT_TOPIC_STACK_SIZE);
 	}
-};				// void DeleteCookie ( char* CookieString )
+}
+
+/**
+ * \brief Pop the top topic in the topic stack.
+ */
+void chat_pop_topic() {
+	if (topic_stack_slot > 0) {
+		free(topic_stack[topic_stack_slot]);
+		topic_stack_slot--;
+	} else {
+		ErrorMessage(__FUNCTION__,
+			     "Root topic can't be popped.",
+			     PLEASE_INFORM, IS_WARNING_ONLY);
+	}
+}
 
 /**
  * This function should load new chat dialogue information from the 
@@ -219,7 +177,7 @@ static void load_dialog(const char *fpath)
 #define CHAT_CHARACTER_BEGIN_STRING "Beginning of new chat dialog for character=\""
 #define CHAT_CHARACTER_END_STRING "End of chat dialog for character"
 #define NEW_OPTION_BEGIN_STRING "\nNr="
-#define NO_WAIT_OPTION_STRING "\nOptionSample=\"NO_SAMPLE_HERE_AND_DONT_WAIT_EITHER\""
+#define NO_TEXT_OPTION_STRING "NO_TEXT"
 
 	// Read the whole chat file information into memory
 	ChatData = ReadAndMallocAndTerminateFile(fpath, CHAT_CHARACTER_END_STRING);
@@ -231,19 +189,18 @@ static void load_dialog(const char *fpath)
 		free(chat_initialization_code);
 		chat_initialization_code = NULL;
 	}
-	chat_initialization_code = ReadAndMallocStringFromDataOptional(SectionPointer, "FirstTime LuaCode={", "}");
+	chat_initialization_code = ReadAndMallocStringFromDataOptional(SectionPointer, "<FirstTime LuaCode>", "</LuaCode>");
 
 	if (chat_startup_code) {
 		free(chat_startup_code);
 		chat_startup_code = NULL;
 	}
-	chat_startup_code = ReadAndMallocStringFromDataOptional(SectionPointer, "EveryTime LuaCode={", "}");
+	chat_startup_code = ReadAndMallocStringFromDataOptional(SectionPointer, "<EveryTime LuaCode>", "</LuaCode>");
 
 	// At first we go take a look on how many options we have
 	// to decode from this section.
 	//
 	NumberOfOptionsInSection = CountStringOccurences(SectionPointer, NEW_OPTION_BEGIN_STRING);
-	DebugPrintf(CHAT_DEBUG_LEVEL, "\nWe have counted %d Option entries in this section.", NumberOfOptionsInSection);
 
 	// Now we see which option index is assigned to this option.
 	// It may happen, that some numbers are OMITTED here!  This
@@ -255,7 +212,6 @@ static void load_dialog(const char *fpath)
 		ReadValueFromString(SectionPointer, NEW_OPTION_BEGIN_STRING, "%d",
 				    &OptionIndex, SectionPointer + strlen(NEW_OPTION_BEGIN_STRING) + 50);
 
-		DebugPrintf(CHAT_DEBUG_LEVEL, "\nFound New Option entry.  Index found is: %d. ", OptionIndex);
 		SectionPointer++;
 
 		// Find the end of this dialog option
@@ -272,18 +228,16 @@ static void load_dialog(const char *fpath)
 			ChatRoster[OptionIndex].option_text = ReadAndMallocStringFromData(SectionPointer, "Text=_\"", "\"");
 		}
 
-		DebugPrintf(CHAT_DEBUG_LEVEL, "\nText found : \"%s\".", ChatRoster[OptionIndex].option_text);
+		ChatRoster[OptionIndex].topic = ReadAndMallocStringFromDataOptional(SectionPointer, "Topic=\"", "\"");
 
-		ChatRoster[OptionIndex].option_sample_file_name =
-		    ReadAndMallocStringFromDataOptional(SectionPointer, "OptionSample=\"", "\"");
+		if (!ChatRoster[OptionIndex].topic) {
+			ChatRoster[OptionIndex].topic = "";
+		}
 
-		if (!ChatRoster[OptionIndex].option_sample_file_name)
-			ChatRoster[OptionIndex].option_sample_file_name = strdup("Sorry_No_Voice_Sample_Yet_0.wav");
-
-		DebugPrintf(CHAT_DEBUG_LEVEL, "\nOptionSample found : \"%s\".", ChatRoster[OptionIndex].option_sample_file_name);
+		ChatRoster[OptionIndex].no_text = strstr(SectionPointer, NO_TEXT_OPTION_STRING) > 0;
 
 		if (strstr(SectionPointer, "LuaCode")) {
-			ChatRoster[OptionIndex].lua_code = ReadAndMallocStringFromData(SectionPointer, "LuaCode={", "}");
+			ChatRoster[OptionIndex].lua_code = ReadAndMallocStringFromData(SectionPointer, "<LuaCode>", "</LuaCode>");
 		} else
 			ChatRoster[OptionIndex].lua_code = NULL;
 
@@ -297,86 +251,11 @@ static void load_dialog(const char *fpath)
 	// We can now free the loaded file again.  Upon a new character dialogue
 	// being initiated, we'll just reload the file.  This is very convenient,
 	// for it allows making and testing changes to the dialogues without even
-	// having to restart Freedroid!  Very cool!
+	// having to restart FreedroidRPG!  Very cool!
 	//
 	free(ChatData);
 }
 
-/**
- *
- *
- */
-void make_sure_chat_portraits_loaded_for_this_droid(Enemy this_droid)
-{
-	SDL_Surface *Small_Droid;
-	SDL_Surface *Large_Droid;
-	char fpath[2048];
-	char fname[500];
-	int i;
-	int model_number;
-	static int first_call = TRUE;
-	static int this_type_has_been_loaded[ENEMY_ROTATION_MODELS_AVAILABLE];
-
-	// We make sure we only load the portrait files once and not
-	// every time...
-	//
-	if (first_call) {
-		for (i = 0; i < ENEMY_ROTATION_MODELS_AVAILABLE; i++)
-			this_type_has_been_loaded[i] = FALSE;
-	}
-	first_call = FALSE;
-
-	// We look up the model number for this chat partner.
-	//
-	model_number = this_droid->type;
-
-	// We should make sure, that we don't double-load images that we have loaded
-	// already, thereby wasting more resources, including OpenGL texture positions.
-	//
-	if (this_type_has_been_loaded[model_number])
-		return;
-	this_type_has_been_loaded[model_number] = TRUE;
-
-	// At first we try to load the image, that is named after this
-	// chat section.  If that succeeds, perfect.  If not, we'll revert
-	// to a default image.
-	//
-	strcpy(fname, "droids/");
-	strcat(fname, PrefixToFilename[model_number]);
-	strcat(fname, "/portrait.png");
-	find_file(fname, GRAPHICS_DIR, fpath, 0);
-	DebugPrintf(2, "\nFilename used for portrait: %s.", fpath);
-
-	Small_Droid = our_IMG_load_wrapper(fpath);
-	if (Small_Droid == NULL) {
-		strcpy(fname, "droids/");
-		strcat(fname, "DefaultPortrait.png");
-		find_file(fname, GRAPHICS_DIR, fpath, 0);
-		Small_Droid = our_IMG_load_wrapper(fpath);
-	}
-	if (Small_Droid == NULL) {
-		fprintf(stderr, "\n\nfpath: %s \n", fpath);
-		ErrorMessage(__FUNCTION__, "\
-It wanted to load a small portrait file in order to display it in the \n\
-chat interface of Freedroid.  But:  Loading this file has ALSO failed.", PLEASE_INFORM, IS_FATAL);
-	}
-
-	Large_Droid = zoomSurface(Small_Droid, (float)Droid_Image_Window.w / (float)Small_Droid->w,
-				  (float)Droid_Image_Window.w / (float)Small_Droid->w, 0);
-
-	SDL_FreeSurface(Small_Droid);
-
-	if (use_open_gl) {
-		chat_portrait_of_droid[model_number].surface =
-		    SDL_CreateRGBSurface(0, Large_Droid->w, Large_Droid->h, 32, rmask, gmask, bmask, amask);
-		SDL_SetAlpha(Large_Droid, 0, SDL_ALPHA_OPAQUE);
-		our_SDL_blit_surface_wrapper(Large_Droid, NULL, chat_portrait_of_droid[model_number].surface, NULL);
-		SDL_FreeSurface(Large_Droid);
-	} else
-		chat_portrait_of_droid[model_number].surface = Large_Droid;
-
-}
-
 static void show_chat_up_button(void)
 {
 	ShowGenericButtonFromList(CHAT_LOG_SCROLL_UP_BUTTON);
@@ -395,46 +274,75 @@ static void show_chat_down_button(void)
  */
 void show_chat_log(enemy *chat_enemy)
 {
-	blit_special_background(CHAT_DIALOG_BACKGROUND_PICTURE_CODE);
-	our_SDL_blit_surface_wrapper(chat_portrait_of_droid[chat_enemy->type].surface, NULL, Screen, &Droid_Image_Window);
+	struct image *img = get_droid_portrait_image(chat_enemy->type);
+	float scale;
+	moderately_finepoint pos;
+
+	blit_background("conversation.png");
+
+	// Compute the maximum uniform scale to apply to the bot image so that it fills
+	// the droid portrait image, and center the image.
+	scale = min((float)Droid_Image_Window.w / (float)img->w, (float)Droid_Image_Window.h / (float)img->h);
+	pos.x = (float)Droid_Image_Window.x + ((float)Droid_Image_Window.w - (float)img->w * scale) / 2.0;
+	pos.y = (float)Droid_Image_Window.y + ((float)Droid_Image_Window.h - (float)img->h * scale) / 2.0;
+
+	display_image_on_screen(img, pos.x, pos.y, IMAGE_SCALE_TRANSFO(scale));
 
 	chat_log.content_above_func = show_chat_up_button;
 	chat_log.content_below_func = show_chat_down_button;
 	ShowGenericButtonFromList(CHAT_LOG_SCROLL_OFF_BUTTON);
 	ShowGenericButtonFromList(CHAT_LOG_SCROLL_OFF2_BUTTON);
 
-	show_text_widget(&chat_log);
+	widget_text_display(WIDGET(&chat_log));
 }
 
 /**
- * Wait for a mouse click before continuing.
+ * Wait for a mouse click before continuing, updating the screen while waiting.
  */
-static void wait_for_click()
+static void wait_for_click(enemy *chat_droid)
 {
 	SDL_Event event;
 
+	// Rectangle for the player response area
+	SDL_Rect clip;
+	clip.x = UNIVERSAL_COORD_W(37);
+	clip.y = UNIVERSAL_COORD_H(336);
+	clip.w = UNIVERSAL_COORD_W(640 - 70);
+	clip.h = UNIVERSAL_COORD_H(118);
+
 	while (1) {
+		show_chat_log(chat_droid);
+		SetCurrentFont(Menu_BFont);
+		display_text(_("\1Click anywhere to continue..."), clip.x, clip.y, &clip);
+		blit_mouse_cursor();
+		our_SDL_flip_wrapper();
+		SDL_Delay(1);
+
 		SDL_WaitEvent(&event);
 
 		if (event.type == SDL_QUIT) {
-			Terminate(0);
+			Terminate(EXIT_SUCCESS, TRUE);
 		}
 
 		if (event.type == SDL_MOUSEBUTTONDOWN && event.button.button == 1)
-			return;
+			goto wait_for_click_return;
 
 		else if (event.type == SDL_KEYDOWN) {
 			switch (event.key.keysym.sym) {
 			case SDLK_SPACE:
 			case SDLK_RETURN:
 			case SDLK_ESCAPE:
-				return;
-				break;
+				goto wait_for_click_return;
 			default:
 				break;
 			}
 		}
 	}
+	 	
+wait_for_click_return:
+	while (EscapePressed() || MouseLeftPressed() || SpacePressed())
+		;
+	return;
 }
 
 /**
@@ -442,34 +350,25 @@ static void wait_for_click()
  * sample.  It is not very sophisticated or complicated, but nevertheless
  * important, because this combination does indeed occur so often.
  */
-void GiveSubtitleNSample(const char *SubtitleText, const char *SampleFilename, enemy * ChatDroid, int with_update)
+void chat_add_response(const char *response, int no_wait, enemy *chat_droid)
 {
-	int do_display = 1;
-	int do_wait = 1;
-
-	autostr_append(chat_log.text, SubtitleText);
+	autostr_append(chat_log.text, "%s", response);
 	chat_log.scroll_offset = 0;
 
-	if (!strcmp(SampleFilename, "NO_WAIT")) {
-		do_wait = 0;
-	}
+	show_chat_log(chat_droid);
+	blit_mouse_cursor();
+	our_SDL_flip_wrapper();
 
-	if (do_display) {
-		show_chat_log(ChatDroid);
-		if (with_update)
-			our_SDL_flip_wrapper();
-	}
-
-	if (do_wait)
-		wait_for_click();
+	if (!no_wait)
+		wait_for_click(chat_droid);
 }
 
 void run_subdialog(const char *tmp_filename)
 {
 	char fpath[2048];
 	char finaldir[50];
-	unsigned char dummyflags[MAX_ANSWERS_PER_PERSON];
-	unsigned char *old_chat_flags = chat_control_chat_flags;
+	uint8_t dummyflags[MAX_ANSWERS_PER_PERSON];
+	uint8_t *old_chat_flags = chat_control_chat_flags;
 
 	memset(dummyflags, 0, MAX_ANSWERS_PER_PERSON);
 	chat_control_chat_flags = &dummyflags[0];
@@ -488,9 +387,6 @@ void run_subdialog(const char *tmp_filename)
 		dummyflags[i] = 0;
 	}
 
-	if (chat_initialization_code)
-		run_lua(chat_initialization_code);
-
 	run_chat(chat_control_chat_droid, TRUE);
 
 	push_or_pop_chat_roster(POP_ROSTER);
@@ -502,30 +398,26 @@ void run_subdialog(const char *tmp_filename)
 }
 
 /**
- *
- *
+ * Process a chat option:
+ * - Echo the option text of the chosen option, unless NO_TEXT is set.
+ * - Run the associated LUA code.
  */
-static void ProcessThisChatOption(int MenuSelection, enemy *ChatDroid)
+static void process_this_chat_option(int menu_selection, enemy *chat_droid)
 {
-	//reset chat control variables for this option
+	// Reset chat control variables for this option
 	chat_control_end_dialog = 0;
 	chat_control_next_node = -1;
 
-	// Now a menu section has been made.  We do the reaction:
-	// say the samples and the replies, later we'll set the new option values
-	//
-	// But it might be the case that this option is more technical and not accompanied
-	// by any reply.  This case must also be caught.
-	//
-	if (strcmp(ChatRoster[MenuSelection].option_sample_file_name, "NO_SAMPLE_HERE_AND_DONT_WAIT_EITHER")) {
+	// Echo option text
+	if (!ChatRoster[menu_selection].no_text) {
 		autostr_append(chat_log.text, "\1- ");
-		GiveSubtitleNSample(L_(ChatRoster[MenuSelection].option_text),
-				    ChatRoster[MenuSelection].option_sample_file_name, ChatDroid, TRUE);
+		chat_add_response(L_(ChatRoster[menu_selection].option_text), FALSE, chat_droid);
 		autostr_append(chat_log.text, "\n");
 	}
-	if (ChatRoster[MenuSelection].lua_code) {
-		run_lua(ChatRoster[MenuSelection].lua_code);
-	}
+
+	// Run LUA associated with this selection
+	if (ChatRoster[menu_selection].lua_code)
+		run_lua(LUA_DIALOG, ChatRoster[menu_selection].lua_code);
 }
 
 /**
@@ -544,19 +436,23 @@ static void run_chat(enemy *ChatDroid, int is_subdialog)
 	chat_control_end_dialog = 0;
 	chat_control_next_node = -1;
 
+	// Reset topic stack variables
+	for(i = 1; i <= topic_stack_slot ; i++) {
+		free(topic_stack[i]);
+	}
+	topic_stack_slot = 0;
+	topic_stack[0] = "";
+
 	/* Initialize the chat log widget. */
 	if (!is_subdialog)
-		init_text_widget(&chat_log, _("\3--- Start of Dialog ---\n"));
+		widget_text_init(&chat_log, _("\3--- Start of Dialog ---\n"));
 
-	chat_log.rect.x = CHAT_SUBDIALOG_WINDOW_X;
-	chat_log.rect.y = CHAT_SUBDIALOG_WINDOW_Y;
-	chat_log.rect.w = CHAT_SUBDIALOG_WINDOW_W;
-	chat_log.rect.h = CHAT_SUBDIALOG_WINDOW_H;
+	widget_set_rect(WIDGET(&chat_log), CHAT_SUBDIALOG_WINDOW_X, CHAT_SUBDIALOG_WINDOW_Y, CHAT_SUBDIALOG_WINDOW_W, CHAT_SUBDIALOG_WINDOW_H);
 	chat_log.font = FPS_Display_BFont;
-	chat_log.text_stretch = TEXT_STRETCH;
+	chat_log.line_height_factor = LINE_HEIGHT_FACTOR;
 
-	show_chat_log(ChatDroid);
-	our_SDL_flip_wrapper();
+	if (chat_initialization_code)
+		run_lua(LUA_DIALOG, chat_initialization_code);
 
 	// We load the option texts into the dialog options variable..
 	//
@@ -570,7 +466,7 @@ static void run_chat(enemy *ChatDroid, int is_subdialog)
 	while (1) {
 		// Now we run the startup code
 		if (chat_startup_code && chat_control_next_node == -1) {
-			run_lua(chat_startup_code);
+			run_lua(LUA_DIALOG, chat_startup_code);
 			free(chat_startup_code);
 			chat_startup_code = NULL;	//and free it immediately so as not to run it again in this session
 
@@ -580,8 +476,8 @@ static void run_chat(enemy *ChatDroid, int is_subdialog)
 
 		if (chat_control_next_node == -1) {
 			chat_control_next_node =
-			    ChatDoMenuSelectionFlagged(_("What will you say?"), DialogMenuTexts, 1, -1,
-						       FPS_Display_BFont, ChatDroid);
+				chat_do_menu_selection_filtered(DialogMenuTexts, ChatDroid, 
+								topic_stack[topic_stack_slot]);
 			// We do some correction of the menu selection variable:
 			// The first entry of the menu will give a 1 and so on and therefore
 			// we need to correct this to more C style.
@@ -594,7 +490,7 @@ static void run_chat(enemy *ChatDroid, int is_subdialog)
 			chat_control_next_node = END_ANSWER;
 		}
 
-		ProcessThisChatOption(chat_control_next_node, ChatDroid);
+		process_this_chat_option(chat_control_next_node, ChatDroid);
 			
 		if (chat_control_end_dialog)
 			goto wait_click_and_out;
@@ -604,157 +500,6 @@ wait_click_and_out:
 	while (EscapePressed() || MouseLeftPressed() || SpacePressed());
 }
 
-/**
- * When the Tux (or rather the player :) ) clicks on a friendly droid,
- * a chat menu will be invoked to do the communication with that friendly
- * character.  However, before the chat menu even starts up, there is a
- * certain time frame still spent in the isometric viewpoint where the
- * two characters (Tux and the chat partner) should turn to each other,
- * so the scene looks a bit more personal and realistic.  This function
- * handles that time interval and the two characters turning to each
- * other.
- */
-void DialogPartnersTurnToEachOther(Enemy ChatDroid)
-{
-	int TurningDone = FALSE;
-	float AngleInBetween;
-	float WaitBeforeTurningTime = 0.00001;
-	float WaitAfterTurningTime = 0.0001;
-	int TurningStartTime;
-	float OldAngle;
-	float RightAngle;
-	float TurningDirection;
-
-#define TURN_SPEED 900.0
-
-	// We reset the mouse cursor shape and abort any other
-	// mouse global mode operation.
-	//
-	global_ingame_mode = GLOBAL_INGAME_MODE_NORMAL;
-
-	Activate_Conservative_Frame_Computation();
-
-	// We make sure the one droid in question is in the standing and not
-	// in the middle of the walking motion when turning to the chat partner...
-	//
-	// Calling AnimatEnemies() ONCE for this task seems justified...
-	//
-	ChatDroid->speed.x = 0;
-	ChatDroid->speed.y = 0;
-
-	// At first do some waiting before the turning around starts...
-	//
-	TurningStartTime = SDL_GetTicks();
-	TurningDone = FALSE;
-	while (!TurningDone) {
-		StartTakingTimeForFPSCalculation();
-
-		AssembleCombatPicture(SHOW_ITEMS | USE_OWN_MOUSE_CURSOR);
-
-		our_SDL_flip_wrapper();
-
-		if ((SDL_GetTicks() - TurningStartTime) >= 1000.0 * WaitBeforeTurningTime)
-			TurningDone = TRUE;
-
-		ComputeFPSForThisFrame();
-	}
-
-	// Now we find out what the final target direction of facing should
-	// be.
-	//
-	// For this we use the atan2, which gives angles from -pi to +pi.
-	// 
-	// Attention must be paid, since 'y' in our coordinates ascends when
-	// moving down and descends when moving 'up' on the screen.  So that
-	// one sign must be corrected, so that everything is right again.
-	//
-	RightAngle = (atan2(-(Me.pos.y - ChatDroid->pos.y), +(Me.pos.x - ChatDroid->pos.x)) * 180.0 / M_PI);
-	//
-	// Another thing there is, that must also be corrected:  '0' begins
-	// with facing 'down' in the current rotation models.  Therefore angle
-	// 0 corresponds to that.  We need to shift again...
-	//
-	RightAngle += 90;
-
-	// Now it's time do determine which direction to move, i.e. if to 
-	// turn to the left or to turn to the right...  For this purpose
-	// we convert the current angle, which is between 270 and -90 degrees
-	// to one between -180 and +180 degrees...
-	//
-	if (RightAngle > 180.0)
-		RightAngle -= 360.0;
-
-	// DebugPrintf ( 0 , "\nRightAngle: %f." , RightAngle );
-	// DebugPrintf ( 0 , "\nCurrent angle: %f." , ChatDroid -> current_angle );
-
-	// Having done these preparations, it's now easy to determine the right
-	// direction of rotation...
-	//
-	AngleInBetween = RightAngle - ChatDroid->current_angle;
-	if (AngleInBetween > 180)
-		AngleInBetween -= 360;
-	if (AngleInBetween <= -180)
-		AngleInBetween += 360;
-
-	if (AngleInBetween > 0)
-		TurningDirection = +1;
-	else
-		TurningDirection = -1;
-
-	// Now we turn and show the image until both chat partners are
-	// facing each other, mostly the chat partner is facing the Tux,
-	// since the Tux may still turn around to somewhere else all the 
-	// while, if the chose so
-	//
-	TurningStartTime = SDL_GetTicks();
-	TurningDone = FALSE;
-	while (!TurningDone) {
-		StartTakingTimeForFPSCalculation();
-
-		AssembleCombatPicture(SHOW_ITEMS | USE_OWN_MOUSE_CURSOR);
-		our_SDL_flip_wrapper();
-
-		OldAngle = ChatDroid->current_angle;
-
-		ChatDroid->current_angle = OldAngle + TurningDirection * Frame_Time() * TURN_SPEED;
-
-		// In case of positive turning direction, we wait, till our angle is greater
-		// than the right angle.
-		// Otherwise we wait till our angle is lower than the right angle.
-		//
-		AngleInBetween = RightAngle - ChatDroid->current_angle;
-		if (AngleInBetween > 180)
-			AngleInBetween -= 360;
-		if (AngleInBetween <= -180)
-			AngleInBetween += 360;
-
-		if ((TurningDirection > 0) && (AngleInBetween < 0))
-			TurningDone = TRUE;
-		if ((TurningDirection < 0) && (AngleInBetween > 0))
-			TurningDone = TRUE;
-
-		ComputeFPSForThisFrame();
-	}
-
-	// Now that turning around is basically done, we still wait a few frames
-	// until we start the dialog...
-	//
-	TurningStartTime = SDL_GetTicks();
-	TurningDone = FALSE;
-	while (!TurningDone) {
-		StartTakingTimeForFPSCalculation();
-
-		AssembleCombatPicture(SHOW_ITEMS | USE_OWN_MOUSE_CURSOR);
-		our_SDL_flip_wrapper();
-
-		if ((SDL_GetTicks() - TurningStartTime) >= 1000.0 * WaitAfterTurningTime)
-			TurningDone = TRUE;
-
-		ComputeFPSForThisFrame();
-	}
-
-};				// void DialogPartnersTurnToEachOther ( Enemy ChatDroid )
-
 /**
  * This is more or less the 'main' function of the chat with friendly 
  * droids and characters.  It is invoked directly from the user interface
@@ -763,27 +508,14 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
  */
 void ChatWithFriendlyDroid(enemy * ChatDroid)
 {
-	int i;
-	char *DialogMenuTexts[MAX_ANSWERS_PER_PERSON];
 	char fpath[2048];
 	char tmp_filename[5000];
 	struct npc *npc;
 
 	chat_control_chat_droid = ChatDroid;
 
-	DialogPartnersTurnToEachOther(ChatDroid);
-
 	Activate_Conservative_Frame_Computation();
 
-	make_sure_chat_portraits_loaded_for_this_droid(ChatDroid);
-
-	// First we empty the array of possible answers in the
-	// chat interface.
-	//
-	for (i = 0; i < MAX_ANSWERS_PER_PERSON; i++) {
-		DialogMenuTexts[i] = "";
-	}
-
 	// We clean out the chat roster from any previous use
 	//
 	InitChatRosterForNewDialogue();
@@ -806,14 +538,15 @@ void ChatWithFriendlyDroid(enemy * ChatDroid)
 		for (i = 0; i < MAX_ANSWERS_PER_PERSON; i++) {
 			chat_control_chat_flags[i] = 0;
 		}
-
-		if (chat_initialization_code)
-			run_lua(chat_initialization_code);
-
-		npc->chat_character_initialized = 1;
+	} else {
+		free(chat_initialization_code);
+		chat_initialization_code = NULL;
 	}
 
 	run_chat(ChatDroid, FALSE);
+
+	if (!npc->chat_character_initialized)
+		npc->chat_character_initialized = TRUE;
 }
 
 /**
@@ -827,45 +560,60 @@ void ChatWithFriendlyDroid(enemy * ChatDroid)
  * As a result, the fact that the validator finds no error does not imply there are no errors in dialogs. 
  * Syntax is checked fully, but runtime validation cannot check all of the code.
  */
-void validate_dialogs()
+int validate_dialogs()
 {
-	int j;
+	int j, k;
 	const char *basename;
 	char filename[1024];
 	char fpath[2048];
 	enemy *dummy_partner;
 	struct npc *n;
-	
+	int error_caught = FALSE;
+
 	skip_initial_menus = 1;
 
-	find_file("freedroid.levels", MAP_DIR, fpath, 0);
+	find_file("levels.dat", MAP_DIR, fpath, 0);
 	LoadShip(fpath, 0);
 	PrepareStartOfNewCharacter("NewTuxStartGameSquare");
-	clear_player_inventory_and_stats();
+
+	/* Disable sound to speed up validation. */
+	int old_sound_on = sound_on;
+	sound_on = FALSE;
+
+	/* Temporarily disable screen fadings to speed up validation. */
+	GameConfig.do_fadings = FALSE;
 
 	/* _says functions are not run by the validator, as they display
 	   text on screen and wait for clicks */
-	run_lua("function npc_says(a)\nend\n");
-	run_lua("function tux_says(a)\nend\n");
-	run_lua("function cli_says(a)\nend\n");
+	run_lua(LUA_DIALOG, "function npc_says(a)\nend\n");
+	run_lua(LUA_DIALOG, "function tux_says(a)\nend\n");
+	run_lua(LUA_DIALOG, "function cli_says(a)\nend\n");
 
-	/* Subdialogs will be tested anyway, no need to run them from Lua */
-	run_lua("function run_subdialog(a)\nend\n");
+	/* Subdialogs currently call run_chat and we cannot do that when validating dialogs */
+	run_lua(LUA_DIALOG, "function run_subdialog(a)\nend\n");
 
 	/* Shops must not be run (display + wait for clicks) */
-	run_lua("function trade_with(a)\nend\n");
+	run_lua(LUA_DIALOG, "function trade_with(a)\nend\n");
 	
 	/* drop_dead cannot be tested because it means we would try to kill our dummy bot
 	 several times, which is not allowed by the engine */
-	run_lua("function drop_dead(a)\nend\n");
+	run_lua(LUA_DIALOG, "function drop_dead(a)\nend\n");
 
-	run_lua("function user_input_string(a)\nreturn \"dummy\";\nend\n");
+	run_lua(LUA_DIALOG, "function user_input_string(a)\nreturn \"dummy\";\nend\n");
 
-	run_lua("function upgrade_items(a)\nend\n");
-	run_lua("function craft_addons(a)\nend\n");
+	run_lua(LUA_DIALOG, "function upgrade_items(a)\nend\n");
+	run_lua(LUA_DIALOG, "function craft_addons(a)\nend\n");
 	/* set_bot_destination cannot be tested because this may be invoked when the bot is
-	 diffrent level then when the bot starts */
-	run_lua("function set_bot_destination(a)\nend\n");
+	 on a different level than where the bot starts */
+	run_lua(LUA_DIALOG, "function set_bot_destination(a)\nend\n");
+
+	/* takeover requires user input - hardcode it to win */
+	run_lua(LUA_DIALOG, "function takeover(a)\nreturn true\nend\n");
+
+	/* push_topic() and pop_topic() cannot be tested because the validator doesn't follow 
+	 the logical order of nodes */
+	run_lua(LUA_DIALOG, "function topic(a)\nend\n");
+	run_lua(LUA_DIALOG, "function pop_topic(a)\nend\n");
 
 	/* This dummy will be used to test break_off_and_attack() functions and such. */
 	BROWSE_ALIVE_BOTS(dummy_partner) {
@@ -888,24 +636,43 @@ void validate_dialogs()
 
 		if (chat_initialization_code) {
 			printf("\tinit code\n");
-			run_lua(chat_initialization_code);
+			run_lua(LUA_DIALOG, chat_initialization_code);
 		}
 
 
 		if (chat_startup_code) {
 			printf("\tstartup code\n");
-			run_lua(chat_startup_code);
+			run_lua(LUA_DIALOG, chat_startup_code);
 		}
-
+		k = 0;
 		for (j = 0; j < MAX_DIALOGUE_OPTIONS_IN_ROSTER; j++) {
 			if (ChatRoster[j].lua_code) {
-				printf("\tnode %d\n", j);
-				run_lua(ChatRoster[j].lua_code);
+				if (!(k%5)) {
+					printf("\n");
+				}
+				printf("|node %2d|\t", j);
+				run_lua(LUA_DIALOG, ChatRoster[j].lua_code);
+				k++;
 			}
 		}
+		if(!ChatRoster[END_ANSWER].lua_code){
+			printf("\n\nERROR: \"%s\" Dialog missing END NODE (node %d).\n\n", n->dialog_basename, END_ANSWER);
+			error_caught = TRUE;
+			break;
+			//ErrorMessage(__FUNCTION__, " NO_NEED_TO_INFORM, IS_FATAL,
+			//		
+		}
 
-		printf("... dialog OK\n");
+		printf("\n... dialog OK\n");
 	}
+
+	/* Re-enable sound as needed. */
+	sound_on = old_sound_on;
+
+	/* Re-enable screen fadings. */
+	GameConfig.do_fadings = TRUE;
+
+	return error_caught;
 }
 
 #undef _chat_c
