@@ -104,7 +104,7 @@ static inline float calc_squared_distance_seg_point_normalized(float x1, float y
  */
 int WalkablePassFilterCallback(colldet_filter * this, obstacle * obs, int obs_idx)
 {
-	if (obstacle_map[obs->type].flags & IS_WALKABLE)
+	if (get_obstacle_spec(obs->type)->flags & IS_WALKABLE)
 		return TRUE;
 
 	if (this->next)
@@ -121,7 +121,7 @@ colldet_filter WalkableWithMarginPassFilter = { WalkablePassFilterCallback, NULL
  */
 int FlyablePassFilterCallback(colldet_filter * this, obstacle * obs, int obs_idx)
 {
-	if (obstacle_map[obs->type].flags & GROUND_LEVEL)
+	if (get_obstacle_spec(obs->type)->flags & GROUND_LEVEL)
 		return TRUE;
 
 	if (this->next)
@@ -138,7 +138,7 @@ colldet_filter FlyablePassFilter = { FlyablePassFilterCallback, NULL, 0, NULL };
  */
 int VisiblePassFilterCallback(colldet_filter * this, obstacle * obs, int obs_idx)
 {
-	if (!(obstacle_map[obs->type].flags & BLOCKS_VISION_TOO))
+	if (!(get_obstacle_spec(obs->type)->flags & BLOCKS_VISION_TOO))
 		return TRUE;
 
 	if (this->next)
@@ -166,16 +166,43 @@ colldet_filter ObstacleByIdPassFilter = { ObstacleByIdPassFilterCallback, NULL,
 colldet_filter WalkableExceptIdPassFilter = { ObstacleByIdPassFilterCallback, NULL, 0, &WalkablePassFilter };
 colldet_filter FlyableExceptIdPassFilter = { ObstacleByIdPassFilterCallback, NULL, 0, &FlyablePassFilter };
 
+static const float Druid_Radius = 0.5;
+
+/**
+ * This function checks if there are any droids at a given location.
+ */
+int location_free_of_droids(float x, float y, int levelnum, freeway_context *ctx)
+{
+	if (ctx->check_tux) {
+		float dist = (Me.pos.x - x)*(Me.pos.x - x) + (Me.pos.y - y)*(Me.pos.y - y);
+		if (dist < Druid_Radius*Druid_Radius)
+			return FALSE;
+	}
+
+	enemy *this_enemy;
+	BROWSE_LEVEL_BOTS(this_enemy, levelnum) {
+		if ((this_enemy->pure_wait > 0) ||
+		    (ctx->except_bots[0] != NULL && ctx->except_bots[0] == this_enemy) ||
+		    (ctx->except_bots[1] != NULL && ctx->except_bots[1] == this_enemy))
+			continue;
+
+		float dist = (this_enemy->pos.x - x)*(this_enemy->pos.x - x) + (this_enemy->pos.y - y)*(this_enemy->pos.y - y);
+		if (dist < Druid_Radius*Druid_Radius)
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 /**
  * This function checks if the connection between two points is free of
- * droids.  
+ * droids.
  *
  * OBSTACLES ARE NOT TAKEN INTO CONSIDERATION, ONLY DROIDS!!!
  *
  */
-int CheckIfWayIsFreeOfDroids(float x1, float y1, float x2, float y2, int OurLevel, freeway_context * ctx)
+int way_free_of_droids(float x1, float y1, float x2, float y2, int levelnum, freeway_context * ctx)
 {
-	const float Druid_Radius = 0.5;
 	float minx = min(x1, x2) - 0.5;
 	float miny = min(y1, y2) - 0.5;
 	float maxx = max(x1, x2) + 0.5;
@@ -192,7 +219,7 @@ int CheckIfWayIsFreeOfDroids(float x1, float y1, float x2, float y2, int OurLeve
 	}
 
 	enemy *this_enemy;
-	BROWSE_LEVEL_BOTS(this_enemy, OurLevel) {
+	BROWSE_LEVEL_BOTS(this_enemy, levelnum) {
 		if ((this_enemy->pure_wait > 0) ||
 		    (ctx->except_bots[0] != NULL && ctx->except_bots[0] == this_enemy) ||
 		    (ctx->except_bots[1] != NULL && ctx->except_bots[1] == this_enemy))
@@ -208,8 +235,7 @@ int CheckIfWayIsFreeOfDroids(float x1, float y1, float x2, float y2, int OurLeve
 	}
 
 	return TRUE;
-
-};				// CheckIfWayIsFreeOfDroids ( float x1 , float y1 , float x2 , float y2 , int OurLevel , freeway_ctx* ctx )
+}
 
 enum {
 	RECT_IN = 0,
@@ -264,6 +290,7 @@ static int dlc_on_one_level(int x_tile_start, int x_tile_end, int y_tile_start,
 			    gps * p1, gps * p2, level * lvl, colldet_filter * filter)
 {
 	int x_tile, y_tile;
+	int tstamp = next_glue_timestamp();
 
 	char ispoint = ((p1->x == p2->x) && (p1->y == p2->y));
 
@@ -272,18 +299,26 @@ static int dlc_on_one_level(int x_tile_start, int x_tile_end, int y_tile_start,
 			// We can list all obstacles here  
 			int glue_index;
 
-			for (glue_index = 0; glue_index < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; glue_index++) {
-				int obstacle_index = lvl->map[y_tile][x_tile].obstacles_glued_to_here[glue_index];
+			for (glue_index = 0; glue_index < lvl->map[y_tile][x_tile].glued_obstacles.size; glue_index++) {
+				int *glued_obstacles = lvl->map[y_tile][x_tile].glued_obstacles.arr;
+				int obstacle_index = glued_obstacles[glue_index];
 
 				if (obstacle_index == (-1))
 					break;
 
 				obstacle *our_obs = &(lvl->obstacle_list[obstacle_index]);
 
+				if (our_obs->timestamp == tstamp) {
+					continue;
+				}
+
+				our_obs->timestamp = tstamp;
+
 				// If the obstacle doesn't even have a collision rectangle, then
-				// of course it's easy, cause then there can't be any collsision
+				// of course it's easy, cause then there can't be any collision
 				//
-				if (obstacle_map[our_obs->type].block_area_type == COLLISION_TYPE_NONE)
+				obstacle_spec *obstacle_spec = get_obstacle_spec(our_obs->type);
+				if (obstacle_spec->block_area_type == COLLISION_TYPE_NONE)
 					continue;
 
 				// Filter out some obstacles, if asked
@@ -293,11 +328,11 @@ static int dlc_on_one_level(int x_tile_start, int x_tile_end, int y_tile_start,
 				// So we have our obstacle 
 
 				// Check the flags of both points against the rectangle of the object
-				moderately_finepoint rect1 = { our_obs->pos.x + obstacle_map[our_obs->type].left_border,
-					our_obs->pos.y + obstacle_map[our_obs->type].upper_border
+				moderately_finepoint rect1 = { our_obs->pos.x + obstacle_spec->left_border,
+					our_obs->pos.y + obstacle_spec->upper_border
 				};
-				moderately_finepoint rect2 = { our_obs->pos.x + obstacle_map[our_obs->type].right_border,
-					our_obs->pos.y + obstacle_map[our_obs->type].lower_border
+				moderately_finepoint rect2 = { our_obs->pos.x + obstacle_spec->right_border,
+					our_obs->pos.y + obstacle_spec->lower_border
 				};
 
 				// When DLC is called by the pathfinder, we grow a bit the obstacle's size.
@@ -414,11 +449,18 @@ int DirectLineColldet(float x1, float y1, float x2, float y2, int z, colldet_fil
 	gps vp1, vp2;		// segment's virtual endpoints
 
 	// Segment's bbox.
-	// We add 2 tiles around the segment's bbox, because some obstacles covers several tiles.
-	int x_tile_start = floor(min(x1, x2)) - 2;
-	int y_tile_start = floor(min(y1, y2)) - 2;
-	int x_tile_end = ceil(max(x1, x2)) - 1 + 2;
-	int y_tile_end = ceil(max(y1, y2)) - 1 + 2;
+	int x_tile_start = floor(min(x1, x2));
+	int y_tile_start = floor(min(y1, y2));
+	int x_tile_end = ceil(max(x1, x2)) - 1;
+	int y_tile_end = ceil(max(y1, y2)) - 1;
+
+	// Special case: if x1==x2 and x1 is an integral value, the previous
+	// definition ends up with x_tile_end = x_tile_start - 1, which is definitely
+	// not intended (same remark with Y values)
+	if (x_tile_end < x_tile_start)
+		x_tile_end = x_tile_start;
+	if (y_tile_end < y_tile_start)
+		y_tile_end = y_tile_start;
 
 	int x_start, y_start, x_end, y_end;	// intersection between the bbox and one level's limits
 	struct neighbor_data_cell *ngb_cell = NULL;
@@ -519,14 +561,15 @@ int MoveOutOfObstacle(float *posX, float *posY, int posZ, obstacle * ThisObstacl
 	enum { RIGHT, DOWN, LEFT, TOP };
 	unsigned int i;
 	moderately_finepoint new_pos = { 0.5, 0.5 };
+	obstacle_spec *obstacle_spec = get_obstacle_spec(ThisObstacle->type);
 
 	// Construct a sorted list of distance between character's position and the rectangle's edges
 	//
-	moderately_finepoint rect1 = { ThisObstacle->pos.x + obstacle_map[ThisObstacle->type].left_border,
-		ThisObstacle->pos.y + obstacle_map[ThisObstacle->type].upper_border
+	moderately_finepoint rect1 = { ThisObstacle->pos.x + obstacle_spec->left_border,
+		ThisObstacle->pos.y + obstacle_spec->upper_border
 	};
-	moderately_finepoint rect2 = { ThisObstacle->pos.x + obstacle_map[ThisObstacle->type].right_border,
-		ThisObstacle->pos.y + obstacle_map[ThisObstacle->type].lower_border
+	moderately_finepoint rect2 = { ThisObstacle->pos.x + obstacle_spec->right_border,
+		ThisObstacle->pos.y + obstacle_spec->lower_border
 	};
 
 	struct dist_elt dist_arr[] = { {rect2.x - (*posX), RIGHT},
@@ -607,11 +650,10 @@ int EscapeFromObstacle(float *posX, float *posY, int posZ, colldet_filter * filt
 
 	for (y = start_y; y < end_y; ++y) {
 		for (x = start_x; x < end_x; ++x) {
-			for (i = 0; i < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; ++i) {
-				if (ThisLevel->map[y][x].obstacles_glued_to_here[i] == (-1))
-					break;
+			for (i = 0; i < ThisLevel->map[y][x].glued_obstacles.size; i++) {
+				int *glued_obstacles = ThisLevel->map[y][x].glued_obstacles.arr;
 
-				obst_index = ThisLevel->map[y][x].obstacles_glued_to_here[i];
+				obst_index = glued_obstacles[i];
 
 				obstacle *our_obs = &(ThisLevel->obstacle_list[obst_index]);
 
@@ -619,18 +661,19 @@ int EscapeFromObstacle(float *posX, float *posY, int posZ, colldet_filter * filt
 					continue;
 
 				// If the obstacle doesn't even have a collision rectangle, then
-				// of course it's easy, cause then there can't be any collsision
+				// of course it's easy, cause then there can't be any collision
 				//
-				if (obstacle_map[our_obs->type].block_area_type == COLLISION_TYPE_NONE)
+				obstacle_spec *obstacle_spec = get_obstacle_spec(our_obs->type);
+				if (obstacle_spec->block_area_type == COLLISION_TYPE_NONE)
 					continue;
 
 				// Now if the position lies inside the collision rectangle, then there's
 				// a collision.
 				//
-				if ((*posX > our_obs->pos.x + obstacle_map[our_obs->type].left_border) &&
-				    (*posX < our_obs->pos.x + obstacle_map[our_obs->type].right_border) &&
-				    (*posY > our_obs->pos.y + obstacle_map[our_obs->type].upper_border) &&
-				    (*posY < our_obs->pos.y + obstacle_map[our_obs->type].lower_border)) {
+				if ((*posX > our_obs->pos.x + obstacle_spec->left_border) &&
+				    (*posX < our_obs->pos.x + obstacle_spec->right_border) &&
+				    (*posY > our_obs->pos.y + obstacle_spec->upper_border) &&
+				    (*posY < our_obs->pos.y + obstacle_spec->lower_border)) {
 					// Find a new position for the character
 					return MoveOutOfObstacle(posX, posY, posZ, our_obs, filter);
 				}
