@@ -26,11 +26,6 @@
 /* ----------------------------------------------------------------------
  * This file contains all the define-constants and macros
  * ---------------------------------------------------------------------- */
-/*
- * This file has been checked for remnants of german documentation.
- * If you still find some german documentation anywhere in here, please
- * let me know.  Thanks a lot.
- */
 #ifndef _defs_h
 #define _defs_h
 
@@ -88,12 +83,11 @@ enum {
 	PART_GROUP_WEAPONARM = 5,
 	ALL_PART_GROUPS = 6
 };
-#define NOT_LOADED_MARKER "nothing_loaded"
 
-#define ENEMY_ROTATION_MODELS_AVAILABLE 40
+#define ENEMY_ROTATION_MODELS_AVAILABLE 42
 #define ROTATION_ANGLES_PER_ROTATION_MODEL 8
 
-#define MAX_ENEMY_MOVEMENT_PHASES 50
+#define MAX_ENEMY_MOVEMENT_PHASES 70
 #define WALK_ANIMATION 113
 #define ATTACK_ANIMATION 114
 #define GETHIT_ANIMATION 115
@@ -108,10 +102,9 @@ enum {
 #define MAX_OBSTACLE_DESCRIPTIONS_PER_LEVEL 100
 
 #define FLOOR_TILES_VISIBLE_AROUND_TUX ((GameConfig . screen_width >= 1024 ? 13 : GameConfig . screen_width >= 800 ? 9 : 7))
-#define MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE 5
 #define MAX_ITEMS_PER_LEVEL 300
 #define MAX_ITEMS_IN_INVENTORY 100
-#define MAX_ITEMS_IN_NPC_INVENTORY 20
+#define MAX_ITEMS_IN_NPC_SHOPLIST 50
 #define INVENTORY_GRID_WIDTH 10
 #define INVENTORY_GRID_HEIGHT 6
 #define INV_SUBSQUARE_WIDTH (31)
@@ -134,7 +127,6 @@ enum {
 #define MAX_BIG_SCREEN_MESSAGES 10
 
 #define NUMBER_OF_SKILL_LEVELS 10
-#define NUMBER_OF_SKILL_PAGES 8
 #define NUMBER_OF_SKILLS_PER_SKILL_PAGE 5
 #define MAX_ACTIVE_SPELLS 100
 #define MAX_NUMBER_OF_PROGRAMS 50
@@ -152,13 +144,13 @@ enum skill_forms {
 	PROGRAM_FORM_SELF,	//instant hit player
 };
 
-enum _global_ingame_modes {
-	GLOBAL_INGAME_MODE_NORMAL = 7124,
-	GLOBAL_INGAME_MODE_SCROLL_UP = 7127,
-	GLOBAL_INGAME_MODE_SCROLL_DOWN = 7128,
-	GLOBAL_INGAME_MODE_REPAIR = 7129,
-	GLOBAL_INGAME_MODE_SELECT_TOOL = 7130,
-	GLOBAL_INGAME_MODE_DRAGDROP_TOOL = 7131,
+enum _game_mouse_cursors {
+	MOUSE_CURSOR_NORMAL = 7124,
+	MOUSE_CURSOR_SCROLL_UP = 7127,
+	MOUSE_CURSOR_SCROLL_DOWN = 7128,
+	MOUSE_CURSOR_REPAIR = 7129,
+	MOUSE_CURSOR_SELECT_TOOL = 7130,
+	MOUSE_CURSOR_DRAGDROP_TOOL = 7131,
 };
 
 enum _game_config_screen_visible_parameters {
@@ -181,12 +173,12 @@ enum _game_config_screen_visible_parameters {
 #define UserCenter_x (User_Rect.x+User_Rect.w/2)
 #define UserCenter_y (User_Rect.y+User_Rect.h/2)
 
-#define MAX_CHARACTER_NAME_LENGTH (25)
+#define MAX_CHARACTER_NAME_LENGTH 13
 
 // The flags for AssembleCombatWindow are:
-enum { ONLY_SHOW_MAP = 1, DO_SCREEN_UPDATE = 2, ONLY_SHOW_MAP_AND_TEXT = 4, SHOW_GRID = 8, SHOW_TUX_AND_ENEMIES = 16, SHOW_TEXT =
-	    32, SHOW_ITEMS = 64, OMIT_OBSTACLES = 128, OMIT_TUX = 256, OMIT_ENEMIES = 512, ZOOM_OUT = 1024, OMIT_BLASTS = 2048,
-		 USE_OWN_MOUSE_CURSOR = 4096, SKIP_LIGHT_RADIUS = 8192, NO_CURSOR = 16384, OMIT_ITEMS_LABEL = 32768 };
+enum { ONLY_SHOW_MAP = 1, DO_SCREEN_UPDATE = 2, ONLY_SHOW_MAP_AND_TEXT = 4,
+		SHOW_ITEMS = 8, OMIT_OBSTACLES = 16, OMIT_TUX = 32, OMIT_ENEMIES = 64, ZOOM_OUT = 128, OMIT_BLASTS = 256,
+		SKIP_LIGHT_RADIUS = 512, NO_CURSOR = 1024, OMIT_ITEMS_LABEL = 2048 };
 
 // Flags for obstacles
 enum {
@@ -196,6 +188,10 @@ enum {
 	/* Walls */
 	IS_HORIZONTAL = 2,
 	IS_VERTICAL = 4,
+	CORNER_NE = 2048,
+	CORNER_NW = 4096,
+	CORNER_SE = 8192,
+	CORNER_SW = 16384,
 
 	/* Obstacle specs */
 	BLOCKS_VISION_TOO = 8,	/* Light will not pass through this obstacle */
@@ -204,14 +200,12 @@ enum {
 	NEEDS_PRE_PUT = 64,
 	GROUND_LEVEL = 128,	/* This obstacle has a small height */
 	IS_WALKABLE = 256,	/* A bot can traverse this obstacle */
-    IS_CLICKABLE = 512  /* Player can click on this obstacle */
+	IS_CLICKABLE = 512,	/* Player can click on this obstacle */
+	IS_VOLATILE = 1024	/* This obstacle vanishes when level is respawned. */
 };
 
 #define AUTOMAP_ZOOM_OUT_FACT 8.0
 
-// The color filters are:
-enum { FILTER_BLUE = 1, FILTER_GREEN = 2, FILTER_RED = 3, FILTER_HIGHLIGHT = 4 };
-
 //--------------------
 // Constants for Paths and names of Data-files
 // the root "FD_DATADIR" should be defined in the Makefile as $(pkgdatadir)
@@ -227,39 +221,9 @@ enum { FILTER_BLUE = 1, FILTER_GREEN = 2, FILTER_RED = 3, FILTER_HIGHLIGHT = 4 }
 #define TITLES_DIR		"map/titles/"
 #define DIALOG_DIR		"dialogs/"
 
-#define SKILL_LEVEL_BUTTON_FILE "skill_buttons.png"
-
-#define INVENTORY_SCREEN_BACKGROUND_CODE 0
-#define CHARACTER_SCREEN_BACKGROUND_CODE 1
-#define SKILL_SCREEN_BACKGROUND_CODE 2
-#define SKILL_SCREEN_EXPLANATION_BACKGROUND_CODE 3
-#define NE_TITLE_PIC_BACKGROUND_CODE 4
-#define SHOP_BACKGROUND_IMAGE_CODE 6
-#define ITEM_BROWSER_BG_PIC_BACKGROUND_CODE 7
-#define ITEM_BROWSER_SHOP_BACKGROUND_CODE 8
-#define LEVEL_EDITOR_BANNER_CODE 14
-// #define LEVEL_EDITOR_BANNER_CODE2 15
-// #define LEVEL_EDITOR_BANNER_CODE3 16 
-// #define LEVEL_EDITOR_BANNER_CODE4 17 
-// #define LEVEL_EDITOR_BANNER_CODE5 18 
-// #define LEVEL_EDITOR_BANNER_CODE6 19 
-// #define LEVEL_EDITOR_BANNER_CODE7 20 
-#define FREEDROID_LOADING_PICTURE_CODE 21
-#define MOUSE_BUTTON_PLUS_BACKGROUND_PICTURE_CODE 25
-#define CHAT_DIALOG_BACKGROUND_PICTURE_CODE 26
-#define CHAT_DIALOG_BACKGROUND_EXCERPT_CODE 27
-#define TAKEOVER_BACKGROUND_CODE 28
-#define QUEST_BROWSER_BACKGROUND_CODE 29
-#define NUMBER_SELECTOR_BACKGROUND_CODE 30
-#define HUD_BACKGROUND_CODE 32
-#define ITEM_UPGRADE_BACKGROUND_CODE 33
-#define ADDON_CRAFTING_BACKGROUND_CODE 34
-
 #define ICON_FILE		"paraicon.bmp"
 
 //--------------------
-// Constants about the size (and mem?) of pixmaps
-//
 #define SKILL_LEVEL_BUTTON_HEIGHT 32
 #define SKILL_LEVEL_BUTTON_WIDTH 30
 
@@ -273,7 +237,7 @@ enum { FILTER_BLUE = 1, FILTER_GREEN = 2, FILTER_RED = 3, FILTER_HIGHLIGHT = 4 }
 #define TABCHAR 'W'
 #define TABWIDTH 8
 //--------------------
-// Startpos + dimesnions of Banner-Texts 
+// Startpos + dimensions of Banner-Texts 
 //
 #define LEFT_INFO_X 	(13*2)
 #define LEFT_INFO_Y	(10*2)
@@ -281,15 +245,10 @@ enum { FILTER_BLUE = 1, FILTER_GREEN = 2, FILTER_RED = 3, FILTER_HIGHLIGHT = 4 }
 #define RIGHT_INFO_Y	(8*2)
 #define LEFT_TEXT_LEN 20
 #define RIGHT_TEXT_LEN 6
-#define TEXT_STRETCH (6.0/5.0)
+#define LINE_HEIGHT_FACTOR (6.0/5.0)
 
 #define BACKGROUND_TEXT_RECT_ALPHA 110
-
-//--------------------
-// Definitions for the menu inside the in-game console
-//
-#define CONS_MENU_HEIGHT 		256
-#define CONS_MENU_LENGTH 		100
+#define TEXT_BANNER_HORIZONTAL_MARGIN 4
 
 //--------------------
 // dimensions of the droids as required for functions NotPassable() and CheckEnemyCollsion() )
@@ -311,90 +270,11 @@ enum _sounds {
 // for these periodic happenings...
 #define CRY_SOUND_INTERVAL 2
 
-/* Those are the codes used internally to 
- * store the state of the dialogs with 
- * those various characters. When adding 
- * a character, you are going to break existing
- * savegames unless it is added at the end.
- * Replace a PERSON_PLACEHOLDER with your new 
- * name to minimize damage on existing savegames.
- */
-enum _non_player_characters {
-	PERSON_CHA = 0,
-	PERSON_SORENSON,
-	PERSON_RMS,
-	PERSON_STONE,
-	PERSON_DIXON,
-	PERSON_DUNCAN,
-	PERSON_PENDRAGON,
-	PERSON_614,
-	PERSON_614_CRYO,
-	PERSON_MER,
-	PERSON_TUTORIALTOM,
-	PERSON_FRANCIS,
-	PERSON_FIRMWARE_SERVER,
-	PERSON_BENJAMIN,
-	PERSON_BENDER,
-	PERSON_SPENCER,
-	PERSON_BUTCH,
-	PERSON_DARWIN,
-	PERSON_DOC_MOORE,
-	PERSON_MELFIS,
-	PERSON_MICHELANGELO,
-	PERSON_SKIPPY,
-	PERSON_STANDARD_OLD_TOWN_GATE_GUARD,
-	PERSON_OLD_TOWN_GATE_GUARD_LEADER,
-	PERSON_STANDARD_MS_FACILITY_GATE_GUARD,
-	PERSON_MS_FACILITY_GATE_GUARD_LEADER,
-	PERSON_STANDARD_BOT_AFTER_TAKEOVER,
-	PERSON_BRUCE,
-	PERSON_SUBDIALOG_DUMMY,
-	PERSON_TYBALT,
-	PERSON_EWALD,
-	PERSON_KEVINS_GUARD,
-	PERSON_KEVIN,
-	PERSON_LUKAS,
-	PERSON_JASMINE,
-	PERSON_SADD,
-	PERSON_SACD,
-	PERSON_TANIA,
-	PERSON_KOAN,
-	PERSON_BORIS,
-	PERSON_LINA,
-	PERSON_SERGE,
-	PERSON_TEST_DROID,
-	PERSON_MSCD,
-
-	PERSON_CNET,
-	PERSON_IRIS,
-	PERSON_PLACEHOLDER3,
-	PERSON_PLACEHOLDER4,
-	PERSON_PLACEHOLDER5,
-	PERSON_PLACEHOLDER6,
-	PERSON_PLACEHOLDER7,
-	PERSON_PLACEHOLDER8,
-	PERSON_PLACEHOLDER9,
-	PERSON_PLACEHOLDER10,
-
-	LAST_PERSON
-};
-#define MAX_PERSONS             (LAST_PERSON)
-
 #define MAX_DIALOGUE_OPTIONS_IN_ROSTER 100
 #define MAX_ANSWERS_PER_PERSON MAX_DIALOGUE_OPTIONS_IN_ROSTER
 #define END_ANSWER (MAX_ANSWERS_PER_PERSON-1)
 
 enum {
-	LOG_SCREEN_TOGGLE_BUTTON,
-	CHA_SCREEN_TOGGLE_BUTTON,
-	INV_SCREEN_TOGGLE_BUTTON,
-	SKI_SCREEN_TOGGLE_BUTTON,
-	CHA_SCREEN_TOGGLE_BUTTON_RED,
-	LOG_SCREEN_TOGGLE_BUTTON_YELLOW,
-	LOG_SCREEN_TOGGLE_BUTTON_RED,
-	CHA_SCREEN_TOGGLE_BUTTON_YELLOW,
-	INV_SCREEN_TOGGLE_BUTTON_YELLOW,
-	SKI_SCREEN_TOGGLE_BUTTON_YELLOW,
 	UP_BUTTON,
 	DOWN_BUTTON,
 	ITEM_BROWSER_LEFT_BUTTON,
@@ -429,13 +309,17 @@ enum {
 
 	LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON,
 	LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON_PUSHED,
-	LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON,
-	LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON_PUSHED,
+	LEVEL_EDITOR_NEXT_OBJECT_BUTTON,
+	LEVEL_EDITOR_NEXT_OBJECT_BUTTON_PUSHED,
 
 	LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON,
 	LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON_PUSHED,
 	LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON_OFF,
 	LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON_OFF_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_ENEMY_BUTTON,
+	LEVEL_EDITOR_TYPESELECT_ENEMY_BUTTON_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_ENEMY_BUTTON_OFF,
+	LEVEL_EDITOR_TYPESELECT_ENEMY_BUTTON_OFF_PUSHED,
 	LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON,
 	LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON_PUSHED,
 	LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON_OFF,
@@ -449,6 +333,11 @@ enum {
 	LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON_OFF,
 	LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON_OFF_PUSHED,
 
+	LEVEL_EDITOR_TYPESELECT_MAP_LABEL_BUTTON,
+	LEVEL_EDITOR_TYPESELECT_MAP_LABEL_BUTTON_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_MAP_LABEL_BUTTON_OFF,
+	LEVEL_EDITOR_TYPESELECT_MAP_LABEL_BUTTON_OFF_PUSHED,
+
 	LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON,
 	LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON_PUSHED,
 	LEVEL_EDITOR_ZOOM_IN_BUTTON,
@@ -457,12 +346,8 @@ enum {
 	LEVEL_EDITOR_ZOOM_OUT_BUTTON_PUSHED,
 	LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON,
 	LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON_PUSHED,
-	LEVEL_EDITOR_NEW_MAP_LABEL_BUTTON,
-	LEVEL_EDITOR_NEW_MAP_LABEL_BUTTON_PUSHED,
 	LEVEL_EDITOR_EDIT_CHEST_BUTTON,
 	LEVEL_EDITOR_EDIT_CHEST_BUTTON_PUSHED,
-	LEVEL_EDITOR_ESC_BUTTON,
-	LEVEL_EDITOR_ESC_BUTTON_PUSHED,
 	LEVEL_EDITOR_QUIT_BUTTON,
 	LEVEL_EDITOR_QUIT_BUTTON_PUSHED,
 	LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON,
@@ -487,6 +372,14 @@ enum {
 	LEVEL_EDITOR_TOGGLE_GRID_BUTTON_PUSHED,
 	LEVEL_EDITOR_TOGGLE_GRID_BUTTON_FULL,
 	LEVEL_EDITOR_TOGGLE_GRID_BUTTON_FULL_PUSHED,
+	LEVEL_EDITOR_TOGGLE_WAYPOINT_CONNECTIONS_BUTTON,
+	LEVEL_EDITOR_TOGGLE_WAYPOINT_CONNECTIONS_BUTTON_PUSHED,
+	LEVEL_EDITOR_TOGGLE_WAYPOINT_CONNECTIONS_BUTTON_OFF,
+	LEVEL_EDITOR_TOGGLE_WAYPOINT_CONNECTIONS_BUTTON_OFF_PUSHED,
+	LEVEL_EDITOR_ALL_FLOOR_LAYERS_BUTTON,
+	LEVEL_EDITOR_ALL_FLOOR_LAYERS_BUTTON_PUSHED,
+	LEVEL_EDITOR_SINGLE_FLOOR_LAYER_BUTTON,
+	LEVEL_EDITOR_SINGLE_FLOOR_LAYER_BUTTON_PUSHED,
 
 	LEVEL_EDITOR_NEXT_ITEM_GROUP_BUTTON,
 	LEVEL_EDITOR_PREV_ITEM_GROUP_BUTTON,
@@ -523,15 +416,6 @@ enum {
 	CHAT_LOG_SCROLL_OFF_BUTTON,
 	CHAT_LOG_SCROLL_OFF2_BUTTON,
 
-	QUEST_BROWSER_EXIT_BUTTON,
-	QUEST_BROWSER_OPEN_QUESTS_BUTTON,
-	QUEST_BROWSER_OPEN_QUESTS_OFF_BUTTON,
-	QUEST_BROWSER_DONE_QUESTS_BUTTON,
-	QUEST_BROWSER_DONE_QUESTS_OFF_BUTTON,
-	QUEST_BROWSER_NOTES_BUTTON,
-	QUEST_BROWSER_NOTES_OFF_BUTTON,
-	QUEST_BROWSER_SCROLL_UP_BUTTON,
-	QUEST_BROWSER_SCROLL_DOWN_BUTTON,
 	QUEST_BROWSER_ITEM_SHORT_BUTTON,
 	QUEST_BROWSER_ITEM_LONG_BUTTON,
 
@@ -544,6 +428,9 @@ enum {
 	ITEM_UPGRADE_APPLY_BUTTON_DISABLED,
 	ITEM_UPGRADE_CLOSE_BUTTON,
 
+	ADDON_CRAFTING_APPLY_BUTTON,
+	ADDON_CRAFTING_APPLY_BUTTON_DISABLED,
+	ADDON_CRAFTING_CLOSE_BUTTON,
 	ADDON_CRAFTING_SCROLL_UP_BUTTON,
 	ADDON_CRAFTING_SCROLL_DOWN_BUTTON,
 	ADDON_CRAFTING_SCROLL_DESC_UP_BUTTON,
@@ -562,7 +449,7 @@ enum {
 #define CHAT_SUBDIALOG_WINDOW_W ((337)*GameConfig . screen_width/640.0)
 #define CHAT_SUBDIALOG_WINDOW_H ((225)*GameConfig . screen_height/480.0)
 
-#define STR_X 95
+#define STR_X 90
 #define STR_Y 140
 #define DEX_Y 165
 #define VIT_Y 190
@@ -571,7 +458,9 @@ enum {
 #define BUTTON_WIDTH 35
 #define BUTTON_HEIGHT 19
 
-#define CHARACTERRECT_X (GameConfig . screen_width-320)
+#define DEFAULT_SCREEN_WIDTH  (800)
+#define DEFAULT_SCREEN_HEIGHT (600)
+#define CHARACTERRECT_X (GameConfig.screen_width - 320)
 #define CHARACTERRECT_W (320)
 #define CHARACTERRECT_H (480)
 
@@ -580,7 +469,7 @@ enum {
 #define ITEM_UPGRADE_RECT_W 320
 #define ITEM_UPGRADE_RECT_H 480
 
-#define ADDON_CRAFTING_RECT_X 320
+#define ADDON_CRAFTING_RECT_X 0
 #define ADDON_CRAFTING_RECT_Y 0
 #define ADDON_CRAFTING_RECT_W 320
 #define ADDON_CRAFTING_RECT_H 480
@@ -603,7 +492,7 @@ enum {
 				    // this variable describes the amount of time in SECONDS 
 #define WAIT_BEFORE_ROTATE 0.7 // Wait a bit to avoid jittering between two rotation angles
 
-#define ALLBLASTTYPES		3	/* number of different exposions */
+#define ALLBLASTTYPES		3	/* number of different explosions */
 
 #define MAX_MELEE_SHOTS		100
 #define MAXBULLETS		100	/* maximum possible Bullets in the air */
@@ -620,27 +509,23 @@ enum {
 
 #define MIN_MAP_LINES		30
 #define MAX_MAP_LINES 		100
-#define MAX_COOKIES 200
 #define MAX_INTERMEDIATE_WAYPOINTS_FOR_TUX 100
 
-#define RIGHT_WALL_BIT 1
-#define LEFT_WALL_BIT 2
-#define UP_WALL_BIT 4
-#define DOWN_WALL_BIT 8
-#define SQUARE_SEEN_AT_ALL_BIT 16
+#define EW_WALL_BIT 1
+#define NS_WALL_BIT 2
+#define VISIBLE_EVENT_BIT 4
+#define SQUARE_SEEN_AT_ALL_BIT 8
+#define UPDATE_SQUARE_BIT 16
 
 #define MAX_SAVED_CHARACTERS_ON_DISK 7	// Not actually amount of chars, but amount "per page"
 
 #define MAX_INFLU_POSITION_HISTORY 500
-#define MAX_MISSIONS_IN_GAME 50	// how many missions can there be at most in Freedroid
+#define MAX_MISSIONS_IN_GAME 50	// how many missions can there be at most in FreedroidRPG
 #define MAX_MISSION_DESCRIPTION_TEXTS 25
 
 #define MAX_LEVELS		100	// how many map levels are allowed in one ship
 
-#define MAX_EVENT_TRIGGERS      500	// how many event triggers at most to allow
-
 #define MAX_PHASES_IN_A_BULLET 12
-#define PHASES_OF_EACH_BLAST 20
 
 #define UNIVERSAL_COORD_W(W) (int)((float)(W) * ((float)(GameConfig . screen_width) / 640.0))
 #define UNIVERSAL_COORD_H(H) (int)((float)(H) * ((float)(GameConfig . screen_height) / 480.0))
@@ -648,12 +533,6 @@ enum {
 //--------------------
 // Rectangle definitions
 
-// Game message window
-#define MESSAGE_TEXT_WIDGET_X UNIVERSAL_COORD_W(120)
-#define MESSAGE_TEXT_WIDGET_Y UNIVERSAL_COORD_H(389)
-#define MESSAGE_TEXT_WIDGET_W UNIVERSAL_COORD_W(415)
-#define MESSAGE_TEXT_WIDGET_H UNIVERSAL_COORD_H(44)
-
 // the inventory rect is the rectangle inside the inventory screen,
 // where all the items "in the backpack" including the quick inventory
 // are displayed.  it's relative to the top left corner, as always.
@@ -662,37 +541,8 @@ enum {
 #define INVENTORY_RECT_Y 260
 #define INVENTORY_RECT_X 8
 
-#define CURRENT_SKILL_RECT_X 537
-#define CURRENT_SKILL_RECT_Y (GameConfig . screen_height - 80 * GameConfig . screen_height / 480)
-#define CURRENT_SKILL_RECT_W 64
-#define CURRENT_SKILL_RECT_H 64
-
 // 388 is the maximum y size to fit items up to 6 inventory squares high
 // 398 is the maximum y size to fit items up to 5 inventory squares high
-#define CURRENT_WEAPON_RECT_X 35
-#define CURRENT_WEAPON_RECT_Y 398
-#define CURRENT_WEAPON_RECT_W 64
-#define CURRENT_WEAPON_RECT_H 64
-
-#define WHOLE_HEALTH_RECT_Y (480-85)
-#define WHOLE_HEALTH_RECT_W 6
-#define WHOLE_HEALTH_RECT_H 50
-#define WHOLE_HEALTH_RECT_X 614
-
-#define WHOLE_RUNNING_POWER_RECT_X 6
-#define WHOLE_RUNNING_POWER_RECT_Y (480-85)
-#define WHOLE_RUNNING_POWER_RECT_W 6
-#define WHOLE_RUNNING_POWER_RECT_H 50
-
-#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_X 19
-#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_Y WHOLE_RUNNING_POWER_RECT_Y
-#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_W WHOLE_RUNNING_POWER_RECT_W
-#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_H WHOLE_RUNNING_POWER_RECT_H
-
-#define WHOLE_FORCE_RECT_Y ( WHOLE_HEALTH_RECT_Y )
-#define WHOLE_FORCE_RECT_W ( WHOLE_HEALTH_RECT_W )
-#define WHOLE_FORCE_RECT_H ( WHOLE_HEALTH_RECT_H )
-#define WHOLE_FORCE_RECT_X ( 640-12)
 
 //--------------------
 // The weapon rectangle is of course for the weapon the Tux is
@@ -707,7 +557,7 @@ enum {
 #define WEAPON_RECT_Y 117
 
 //--------------------
-// The armour rectangle is where you can place some armour
+// The armor rectangle is where you can place some armor
 // for the body of the Tux.
 //
 #define ARMOUR_RECT_WIDTH 66
@@ -747,50 +597,24 @@ enum {
 #define PUT_NO_THROWN_ITEMS 4
 
 //--------------------
-// bullet types
-//
-// Important Note : this enum HAS to be in sync with bullet_identifiers[]
-// defined in iso_load_bullet_surfaces(), and so also with the
-// 'bullet_image_type' values in freedroid.item_archetypes
-//
-enum _bullets {
-	PULSE = 0,
-	SINGLE_PULSE,
-	MILITARY,
-	FLASH, // Not used
-	EXTERMINATOR,
-	LASER_RIFLE,
-	SINGLE_LASER,
-	PLASMA_PISTOL,
-	LASER_SWORD_1,
-	LASER_AXE,
-	LASER_SWORD_2,
-	GREEN_BULLET,
-	BLUE_BULLET,
-	MAGENTA_BULLET,
-	WHITE_BULLET
-	    //--------------------
-	    // If you add some more bullet types here, don't forget to
-	    //   1.  Also add a specification in the freedroid ruleset
-	    //   2.  Also add an entry for the number of phases in the config.theme file!!
-	    //   3.  Specify the size of the bullet image in the blocks.c loading function
-	    //
+// item qualities
+enum item_quality {
+	NORMAL_QUALITY = 0,
+	BAD_QUALITY = 1,
+	GOOD_QUALITY = 2
 };
 
+#define MAX_BULLET_DEFS 16
+
 //--------------------
 // explosion types
 //
 enum _explosions {
 	BULLETBLAST = 0,
 	DROIDBLAST,
-	OWNBLAST
+	EXTERMINATORBLAST,
 };
 
-#define BLINKENERGY 25
-
-//--------------------
-// possible states of the influencer as displayed in the top status banner
-//
 enum _status {
 	INFOUT = -30,
 };
@@ -844,9 +668,7 @@ enum {
 };
 enum {
 	TRANSPARENCY_NONE = 0,
-	TRANSPARENCY_FOR_WALLS = 10,
-	TRANSPARENCY_FOR_SEE_THROUGH_OBJECTS = 11,
-	TRANSPARENCY_CUROBJECT = 12
+	TRANSPARENCY_FOR_WALLS = 10
 };
 
 //--------------------
@@ -876,9 +698,6 @@ enum obstacle_types {
 
 	ISO_TREE_4 = 0,
 
-	//--------------------
-	// These obstacles originate from wall_tiles.blend.
-	//
 	ISO_V_WALL = 1,
 	ISO_H_WALL = 2,
 	ISO_V_WALL_WITH_DOT = 3,
@@ -886,9 +705,6 @@ enum obstacle_types {
 
 	ISO_TREE_5 = 5,
 
-	//--------------------
-	// These obstacles originate from door_tiles.blend.
-	//
 	ISO_H_DOOR_000_OPEN = 6,
 	ISO_H_DOOR_025_OPEN = 7,
 	ISO_H_DOOR_050_OPEN = 8,
@@ -900,21 +716,9 @@ enum obstacle_types {
 	ISO_V_DOOR_075_OPEN = 14,
 	ISO_V_DOOR_100_OPEN = 15,
 
-	//--------------------
-	// These obstacles are generated from 'iso_refresh_and_teleport.blend' file
-	// in the cvs rep.
-	//
 	ISO_TELEPORTER_1 = 16,
-	ISO_TELEPORTER_2 = 17,
-	ISO_TELEPORTER_3 = 18,
-	ISO_TELEPORTER_4 = 19,
-	ISO_TELEPORTER_5 = 20,
 
 	ISO_REFRESH_1 = 21,
-	ISO_REFRESH_2 = 22,
-	ISO_REFRESH_3 = 23,
-	ISO_REFRESH_4 = 24,
-	ISO_REFRESH_5 = 25,
 
 	ISO_H_DOOR_LOCKED = 26,
 	ISO_V_DOOR_LOCKED = 27,
@@ -928,11 +732,8 @@ enum obstacle_types {
 	ISO_AUTOGUN_N = 33,
 	ISO_AUTOGUN_E = 34,
 	ISO_AUTOGUN_S = 35,
-//see at the end for disabled autoguns
-	//--------------------
-	// These obstacles are generated from 'iso_cave_walls.blend' file
-	// in the cvs rep.
-	//
+	//see at the end for disabled autoguns
+
 	ISO_CAVE_WALL_H = 36,
 	ISO_CAVE_WALL_V = 37,
 	ISO_CAVE_CORNER_NE = 38,
@@ -947,10 +748,6 @@ enum obstacle_types {
 	ISO_CONSOLE_N = 45,
 	ISO_CONSOLE_W = 46,
 
-	//--------------------
-	// These obstacles are generated from 'iso_pillars_and_barrels.blend' file
-	// in the cvs rep.
-	//
 	ISO_PILLAR_TALL = 47,
 	ISO_PILLAR_SHORT = 48,
 	ISO_TV_PILLAR_W = 49,
@@ -967,11 +764,6 @@ enum obstacle_types {
 	ISO_ENHANCER_RD = 58,
 	ISO_ENHANCER_LD = 55,
 
-	//--------------------
-	// These obstacles are generated from 'commercial_models_for_fences.blend' file
-	// which is currently not in the cvs rep. for licensing issues, but I'll send a
-	// version to any Freedroid co-worker involved with the graphics
-	//
 	ISO_V_WOOD_FENCE = 59,
 	ISO_H_WOOD_FENCE = 60,
 	ISO_V_DENSE_FENCE = 61,
@@ -984,12 +776,12 @@ enum obstacle_types {
 	//--------------------
 	// These obstacles are generated from 'commercial_models_for_furniture.blend' file
 	// which is currently not in the cvs rep. for licensing issues, but I'll send a
-	// version to any Freedroid co-worker involved with the graphics
+	// version to any FreedroidRPG co-worker involved with the graphics
 	//
 	ISO_N_TOILET_SMALL = 67,
 	ISO_E_TOILET_SMALL,
-	ISO_S_TOILET_SMALL,
-	ISO_W_TOILET_SMALL,
+	ISO_S_TOILET_WHITE_SMALL,
+	ISO_W_TOILET_WHITE_SMALL,
 	ISO_N_TOILET_BIG,
 	ISO_E_TOILET_BIG,
 	ISO_S_TOILET_BIG,
@@ -1010,7 +802,7 @@ enum obstacle_types {
 	//--------------------
 	// These obstacles are generated from 'commercial_models_for_furniture2.blend' file
 	// which is currently not in the cvs rep. for licensing issues, but I'll send a
-	// version to any Freedroid co-worker involved with the graphics
+	// version to any FreedroidRPG co-worker involved with the graphics
 	//
 	ISO_N_BED = 87,
 	ISO_E_BED,
@@ -1035,7 +827,7 @@ enum obstacle_types {
 	//--------------------
 	// These obstacles are generated from 'commercial_models_for_furniture3.blend' file
 	// which is currently not in the cvs rep. for licensing issues, but I'll send a
-	// version to any Freedroid co-worker involved with the graphics
+	// version to any FreedroidRPG co-worker involved with the graphics
 	//
 	ISO_H_BATHTUB = 103,
 	ISO_V_BATHTUB,
@@ -1080,10 +872,10 @@ enum obstacle_types {
 	ISO_BRICK_WALL_V,
 	ISO_BRICK_WALL_END,
 
-	ISO_BRICK_WALL_CORNER_1,	// = 137 , 
-	ISO_BRICK_WALL_CORNER_2,
-	ISO_BRICK_WALL_CORNER_3,
-	ISO_BRICK_WALL_CORNER_4,
+	ISO_BRICK_WALL_CORNER_NE,	// = 137 , 
+	ISO_BRICK_WALL_CORNER_SW,
+	ISO_BRICK_WALL_CORNER_NW,
+	ISO_BRICK_WALL_CORNER_SE,
 
 	ISO_BLOOD_1,		// = 141 ,
 	ISO_BLOOD_2,
@@ -1127,10 +919,10 @@ enum obstacle_types {
 	ISO_OUTER_WALL_W2,
 	ISO_OUTER_WALL_W3,
 
-	ISO_OUTER_WALL_CORNER_1,	// = 177 ,
-	ISO_OUTER_WALL_CORNER_2,
-	ISO_OUTER_WALL_CORNER_3,
-	ISO_OUTER_WALL_CORNER_4,
+	ISO_OUTER_WALL_CORNER_NW,	// = 177 ,
+	ISO_OUTER_WALL_CORNER_SW,
+	ISO_OUTER_WALL_CORNER_SE,
+	ISO_OUTER_WALL_CORNER_NE,
 
 	ISO_OUTER_DOOR_V_00,	// = 181 ,
 	ISO_OUTER_DOOR_V_25,
@@ -1313,10 +1105,10 @@ enum obstacle_types {
 	ISO_BRICK_WALL_CABLES_H,
 	ISO_BRICK_WALL_CABLES_V,
 
-	ISO_BRICK_WALL_CABLES_CORNER_1,
-	ISO_BRICK_WALL_CABLES_CORNER_2,
-	ISO_BRICK_WALL_CABLES_CORNER_3,
-	ISO_BRICK_WALL_CABLES_CORNER_4,
+	ISO_BRICK_WALL_CABLES_CORNER_NE,
+	ISO_BRICK_WALL_CABLES_CORNER_SW,
+	ISO_BRICK_WALL_CABLES_CORNER_NW,
+	ISO_BRICK_WALL_CABLES_CORNER_SE,
 
 	ISO_RESTAURANT_DESK_1,
 	ISO_RESTAURANT_DESK_2,
@@ -1389,60 +1181,73 @@ enum obstacle_types {
 	ISO_BOTLINE_02_N,
 	ISO_BOTLINE_02_W,
 	ISO_BOTLINE_02_S,
-	ISO_BOTLINE_03_N,
-	ISO_BOTLINE_03_E,
-	ISO_BOTLINE_04_N,
-	ISO_BOTLINE_04_E,
-	ISO_BOTLINE_05_N,
-	ISO_BOTLINE_05_E,
-	NUMBER_OF_OBSTACLE_TYPES
+	ISO_BOTLINE_06_N,
+	ISO_BOTLINE_06_W,
+	ISO_BOTLINE_06_S,
+	ISO_BOTLINE_06_E,
+	ISO_BOTLINE_07_N,
+	ISO_BOTLINE_07_W,
+	ISO_FREIGHTER_RAILWAY_01_N,
+	ISO_FREIGHTER_RAILWAY_01_E,
+	ISO_FREIGHTER_RAILWAY_02_S,
+	ISO_FREIGHTER_RAILWAY_02_E,
+	ISO_FREIGHTER_RAILWAY_02_N,
+	ISO_FREIGHTER_RAILWAY_02_W,
+	ISO_SOLAR_PANEL_BROKEN,
+	ISO_BARREL_5,
+	ISO_REACTOR_S,
+	ISO_REACTOR_E,
+	ISO_REACTOR_N,
+	ISO_REACTOR_W,
+	ISO_WALL_TERMINAL_S,
+	ISO_WALL_TERMINAL_E,
+	ISO_WALL_TERMINAL_N,
+	ISO_WALL_TERMINAL_W,
+	ISO_TURBINES_SMALL_W,
+	ISO_TURBINES_SMALL_N,
+	ISO_TURBINES_SMALL_E,
+	ISO_TURBINES_SMALL_S,
+	ISO_WEAPON_CRATE,
+	ISO_DEFAULT_DEAD_BODY_0_1,
+	ISO_DEFAULT_DEAD_BODY_1_1,
+	ISO_DEFAULT_DEAD_BODY_2_1,
+	ISO_DEFAULT_DEAD_BODY_3_1,
+	ISO_DEFAULT_DEAD_BODY_4_1,
+	ISO_DEFAULT_DEAD_BODY_5_1,
+	ISO_DEFAULT_DEAD_BODY_6_1,
+	ISO_DEFAULT_DEAD_BODY_7_1,
+	ISO_DEFAULT_DEAD_BODY_0_2,
+	ISO_LADDER_1,
+	ISO_LADDER_2,
+	ISO_WRECKED_CAR_1,
+	ISO_WRECKED_CAR_2,
+	ISO_WRECKED_CAR_3,
+	ISO_WRECKED_CAR_4,
+	ISO_N_TOILET_WHITE_SMALL,
+	ISO_E_TOILET_WHITE_SMALL,
+	ISO_BOTLINE_07_S,
+	ISO_BOTLINE_07_E,
+	ISO_BOTLINE_08_N,
+	ISO_BOTLINE_08_W,
+	ISO_BOTLINE_08_S,
+	ISO_BOTLINE_08_E,
+	ISO_CRUSHED_476,
+	ISO_BROKEN_GLASS_WALL_2,
+	ISO_SIGN_1_FLASH,
+	ISO_SIGN_2_FLASH,
+	ISO_SIGN_3_FLASH,
+	ISO_BARREL_RADIOACTIVE,
 };
 
 enum {
-	ISO_FLOOR_ERROR_TILE = 0,
-	ISO_FLOOR_STONE_FLOOR = 1,
-	ISO_FLOOR_STONE_FLOOR_WITH_DOT = 2,
-	ISO_FLOOR_STONE_FLOOR_WITH_GRATE = 3,
-	ISO_FLOOR_SAND = 4,
-	ISO_FLOOR_HOUSE_FLOOR = 5,
+	ISO_FLOOR_EMPTY,
+	ISO_FLOOR_ERROR_TILE,
+	ISO_FLOOR_STONE_FLOOR,
+	ISO_FLOOR_STONE_FLOOR_WITH_DOT,
+	ISO_FLOOR_STONE_FLOOR_WITH_GRATE,
+	ISO_FLOOR_HOUSE_FLOOR,
 
-	ISO_FLOOR_SAND_WITH_GRASS_1,
-	ISO_FLOOR_SAND_WITH_GRASS_2,
-	ISO_FLOOR_SAND_WITH_GRASS_3,
-	ISO_FLOOR_SAND_WITH_GRASS_4,
-	ISO_FLOOR_SAND_WITH_GRASS_5,
-	ISO_FLOOR_SAND_WITH_GRASS_6,
-	ISO_FLOOR_SAND_WITH_GRASS_7,
-	ISO_FLOOR_SAND_WITH_GRASS_8,
-	ISO_FLOOR_SAND_WITH_GRASS_9,
-	ISO_FLOOR_SAND_WITH_GRASS_10,
-
-	ISO_FLOOR_SAND_WITH_GRASS_11,
-	ISO_FLOOR_SAND_WITH_GRASS_12,
-	ISO_FLOOR_SAND_WITH_GRASS_13,
-
-	ISO_FLOOR_SAND_WITH_GRASS_14,
-	ISO_FLOOR_SAND_WITH_GRASS_15,
-	ISO_FLOOR_SAND_WITH_GRASS_16,
-	ISO_FLOOR_SAND_WITH_GRASS_17,
-
-	ISO_FLOOR_SAND_WITH_GRASS_18,
-	ISO_FLOOR_SAND_WITH_GRASS_19,
-	ISO_FLOOR_SAND_WITH_GRASS_20,
-	ISO_FLOOR_SAND_WITH_GRASS_21,
-
-	ISO_FLOOR_SAND_WITH_GRASS_22,
-	ISO_FLOOR_SAND_WITH_GRASS_23,
-	ISO_FLOOR_SAND_WITH_GRASS_24,
-
-	ISO_WATER,
-	ISO_COMPLETELY_DARK,
 	ISO_RED_WAREHOUSE_FLOOR,
-	ISO_FLOOR_SAND_WITH_GRASS_25,
-	ISO_FLOOR_SAND_WITH_GRASS_26,
-	ISO_FLOOR_SAND_WITH_GRASS_27,
-	ISO_FLOOR_SAND_WITH_GRASS_28,
-	ISO_FLOOR_SAND_WITH_GRASS_29,
 
 	ISO_MISCELLANEOUS_FLOOR_9,
 	ISO_MISCELLANEOUS_FLOOR_10,
@@ -1458,7 +1263,9 @@ enum {
 	ISO_MISCELLANEOUS_FLOOR_20,
 	ISO_MISCELLANEOUS_FLOOR_21,
 	ISO_MISCELLANEOUS_FLOOR_22,
+	ISO_MISCELLANEOUS_FLOOR_23,
 
+	ISO_SIDEWALK_0,
 	ISO_SIDEWALK_1,
 	ISO_SIDEWALK_2,
 	ISO_SIDEWALK_3,
@@ -1479,9 +1286,12 @@ enum {
 	ISO_SIDEWALK_18,
 	ISO_SIDEWALK_19,
 	ISO_SIDEWALK_20,
+	ISO_SIDEWALK_21,
+	ISO_SIDEWALK_22,
+	ISO_SIDEWALK_23,
+	ISO_SIDEWALK_24,
 
-	ISO_MISCELLANEOUS_FLOOR_23,
-
+	ISO_FLOOR_SAND,
 	ISO_SAND_FLOOR_1,
 	ISO_SAND_FLOOR_2,
 	ISO_SAND_FLOOR_3,
@@ -1489,24 +1299,25 @@ enum {
 	ISO_SAND_FLOOR_5,
 	ISO_SAND_FLOOR_6,
 
-        ISO_WATER_EDGE_1,
-        ISO_WATER_EDGE_2,
-        ISO_WATER_EDGE_3,
-        ISO_WATER_EDGE_4,
-        ISO_WATER_EDGE_5,
-        ISO_WATER_EDGE_6,
-        ISO_WATER_EDGE_7,
-        ISO_WATER_EDGE_8,
-        ISO_WATER_EDGE_9,
-        ISO_WATER_EDGE_10,
-        ISO_WATER_EDGE_11,
-        ISO_WATER_EDGE_12,
-
-        ISO_SIDEWALK_21,
-        ISO_SIDEWALK_22,
-        ISO_SIDEWALK_23,
-        ISO_SIDEWALK_24,
-	ISO_SIDEWALK_0,
+	ISO_WATER,
+	ISO_WATER_EDGE_1,
+	ISO_WATER_EDGE_2,
+	ISO_WATER_EDGE_3,
+	ISO_WATER_EDGE_4,
+	ISO_WATER_EDGE_5,
+	ISO_WATER_EDGE_6,
+	ISO_WATER_EDGE_7,
+	ISO_WATER_EDGE_8,
+	ISO_WATER_EDGE_9,
+	ISO_WATER_EDGE_10,
+	ISO_WATER_EDGE_11,
+	ISO_WATER_EDGE_12,
+	ISO_WATER_EDGE_13,
+	ISO_WATER_EDGE_14,
+	ISO_WATER_SIDEWALK_01,
+	ISO_WATER_SIDEWALK_02,
+	ISO_WATER_SIDEWALK_03,
+	ISO_WATER_SIDEWALK_04,
 
 	ISO_CARPET_TILE_0001,
 	ISO_CARPET_TILE_0002,
@@ -1550,14 +1361,82 @@ enum {
 
 	ISO_TWOSQUARE_0001,
 	ISO_TWOSQUARE_0002,
-	ISO_TWOSQUARE_0003,
-	ISO_WATER_SIDEWALK_01,
-	ISO_WATER_SIDEWALK_02,
-	ISO_WATER_SIDEWALK_03,
-	ISO_WATER_SIDEWALK_04,
-	ALL_ISOMETRIC_FLOOR_TILES
+	ISO_TWOSQUARE_0003
 };
 
+#define MAX_UNDERLAY_FLOOR_TILES 500
+#define MAX_OVERLAY_FLOOR_TILES  500
+#define MAX_FLOOR_TILES (MAX_UNDERLAY_FLOOR_TILES + MAX_OVERLAY_FLOOR_TILES)
+
+/*
+ * Underlay and overlay floor tiles share the same namespace.
+ * Underlay floor tiles are numbered from 0 to MAX_UNDERLAY_FLOOR_TILES - 1.
+ * Overlay floor tiles are numbered from MAX_UNDERLAY_FLOOR_TILES
+ * to MAX_FLOOR_TILES - 1.
+ */
+enum {
+	ISO_OVERLAY_GRASS_01 = MAX_UNDERLAY_FLOOR_TILES,
+	ISO_OVERLAY_GRASS_02,
+	ISO_OVERLAY_GRASS_03,
+	ISO_OVERLAY_GRASS_04,
+	ISO_OVERLAY_GRASS_05,
+	ISO_OVERLAY_GRASS_06,
+	ISO_OVERLAY_GRASS_07,
+	ISO_OVERLAY_GRASS_08,
+	ISO_OVERLAY_GRASS_09,
+	ISO_OVERLAY_GRASS_10,
+	ISO_OVERLAY_GRASS_11,
+	ISO_OVERLAY_GRASS_12,
+	ISO_OVERLAY_GRASS_13,
+	ISO_OVERLAY_GRASS_14,
+	ISO_OVERLAY_GRASS_15,
+	ISO_OVERLAY_GRASS_16,
+	ISO_OVERLAY_GRASS_17,
+	ISO_OVERLAY_GRASS_18,
+	ISO_OVERLAY_GRASS_19,
+	ISO_OVERLAY_GRASS_20,
+	ISO_OVERLAY_GRASS_21,
+
+	ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_1,
+	ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_2,
+	ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_3,
+	ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_4,
+	ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_5,
+
+	ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_25,
+	ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_26,
+	ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_27,
+
+	ISO_GROUNDMARKER_BLUE_1,
+	ISO_GROUNDMARKER_BLUE_2,
+	ISO_GROUNDMARKER_BLUE_3,
+	ISO_GROUNDMARKER_BLUE_4,
+	ISO_GROUNDMARKER_BLUE_5,
+	ISO_GROUNDMARKER_BLUE_6,
+	ISO_GROUNDMARKER_BLUE_7,
+	ISO_GROUNDMARKER_BLUE_8,
+	ISO_GROUNDMARKER_RED_1,
+	ISO_GROUNDMARKER_RED_2,
+	ISO_GROUNDMARKER_RED_3,
+	ISO_GROUNDMARKER_RED_4,
+	ISO_GROUNDMARKER_RED_5,
+	ISO_GROUNDMARKER_RED_6,
+	ISO_GROUNDMARKER_RED_7,
+	ISO_GROUNDMARKER_RED_8,
+	ISO_GROUNDMARKER_BLUE_Y_1,
+	ISO_GROUNDMARKER_BLUE_Y_2,
+	ISO_GROUNDMARKER_BLUE_Y_3,
+	ISO_GROUNDMARKER_BLUE_Y_4,
+	ISO_GROUNDMARKER_BLUE_X_1,
+	ISO_GROUNDMARKER_RED_Y_1,
+	ISO_GROUNDMARKER_RED_Y_2,
+	ISO_GROUNDMARKER_RED_Y_3,
+	ISO_GROUNDMARKER_RED_Y_4,
+	ISO_GROUNDMARKER_RED_X_1
+};
+
+#define MAX_FLOOR_LAYERS 2
+
 // Maximum distance for both taking and dropping items
 #define ITEM_TAKE_DIST (0.7)
 // Maximum and minimum distance for a melee fight
@@ -1598,6 +1477,7 @@ enum faction_id {
 	FACTION_CIVILIAN,
 	FACTION_CRAZY,
 	FACTION_SINGULARITY,
+	FACTION_NEUTRAL,
 
 	FACTION_NUMBER_OF_FACTIONS, //keep this last
 };
@@ -1616,4 +1496,10 @@ enum obstacle_extension_type {
 	OBSTACLE_EXTENSION_DIALOGFILE, // This extension indicates the dialog filename associated to an interactive obstacle. Contains the dialog basename.
 	OBSTACLE_EXTENSION_SIGNMESSAGE = OBSTACLE_EXTENSION_DIALOGFILE, // This extension contains the message written on a signpost. It is exclusive with a dialogfile specification
 };
+
+enum lua_target {
+	LUA_CONFIG,
+	LUA_DIALOG
+};
+
 #endif
