@@ -101,7 +101,7 @@ static int enemy_find_closest_waypoint(struct enemy *this_bot)
 	}
 
 	if (best_waypoint == -1)
-		ErrorMessage(__FUNCTION__, "Found no closest waypoint on level %d.", PLEASE_INFORM, IS_FATAL, lvl->levelnum);
+		ErrorMessage(__FUNCTION__, "Found no closest waypoint on level %d.", PLEASE_INFORM, IS_WARNING_ONLY, lvl->levelnum);
 
 	return best_waypoint;
 }
@@ -118,8 +118,13 @@ int teleport_to_closest_waypoint(struct enemy *ThisRobot)
 	// Find the closest waypoint
 	int best_waypoint = enemy_find_closest_waypoint(ThisRobot);
 
-	// Teleport the robot to the best waypoint
-	teleport_to_waypoint(ThisRobot, lvl, best_waypoint);
+	if (best_waypoint >= 0) {
+		// Teleport the robot to the best waypoint
+		teleport_to_waypoint(ThisRobot, lvl, best_waypoint);
+	} else {
+		// No waypoint found, make this bot wander
+		ThisRobot->combat_state = WAYPOINTLESS_WANDERING;
+	}
 
 	return best_waypoint;
 }
@@ -166,7 +171,7 @@ int teleport_to_random_waypoint(enemy *erot, level *this_level, char *wp_used)
 			found_wp = start_wp;
 		} else {
 			ErrorMessage(__FUNCTION__, "There was no free waypoint found on level %d to place another random bot.\n",
-			                           PLEASE_INFORM, IS_WARNING_ONLY, this_level->levelnum);
+			                            NO_NEED_TO_INFORM, IS_WARNING_ONLY, this_level->levelnum);
 			found_wp = last_checked_wp;
 		}
 	}
@@ -211,9 +216,15 @@ void teleport_enemy(enemy *robot, int z, float x, float y)
 
 	// Reset the bot's waypoints
 	int best_waypoint = enemy_find_closest_waypoint(robot);
-	robot->homewaypoint = best_waypoint;
-	robot->lastwaypoint = best_waypoint;
-	robot->nextwaypoint = best_waypoint;
+
+	if (best_waypoint >= 0) {
+		robot->homewaypoint = best_waypoint;
+		robot->lastwaypoint = best_waypoint;
+		robot->nextwaypoint = best_waypoint;
+	} else {
+		// No waypoint found, make this bot wander
+		robot->combat_state = WAYPOINTLESS_WANDERING;
+	}
 }
 
 /**
@@ -303,7 +314,7 @@ enemy *enemy_new(int type)
 	this_enemy->type = type;
 
 	// Init 'identity' attributes.
-	this_enemy->SpecialForce = 0;
+	this_enemy->SpecialForce = FALSE;
 	this_enemy->marker = 0;
 	this_enemy->max_distance_to_home = 0;
 	this_enemy->dialog_section_name = NULL;
@@ -517,11 +528,15 @@ static void move_enemy_to_spot(Enemy ThisRobot, moderately_finepoint next_target
 		// The waypoints used by the bot have no sense on this new level. They have
 		// to be re-initialized. The closest available waypoint is chosen.
 		int best_waypoint = enemy_find_closest_waypoint(ThisRobot);
-		
-		ThisRobot->homewaypoint = best_waypoint;
-		ThisRobot->lastwaypoint = best_waypoint;
-		ThisRobot->nextwaypoint = best_waypoint;
-		
+
+		if (best_waypoint >= 0) {
+			ThisRobot->homewaypoint = best_waypoint;
+			ThisRobot->lastwaypoint = best_waypoint;
+			ThisRobot->nextwaypoint = best_waypoint;
+		} else {
+			// No waypoint found, make this bot wander
+			ThisRobot->combat_state = WAYPOINTLESS_WANDERING;
+		}
 		// Move the bot to the appropriate level list
 		list_move(&ThisRobot->level_list, &(level_bots_head[ThisRobot->pos.z]));
 	}
@@ -533,7 +548,7 @@ static void move_enemy_to_spot(Enemy ThisRobot, moderately_finepoint next_target
  * This function moves one robot towards his next waypoint.  If already
  * there, the function does nothing more.
  */
-static void MoveThisRobotThowardsHisCurrentTarget(enemy * ThisRobot)
+static void MoveThisRobotTowardsHisCurrentTarget(enemy *ThisRobot)
 {
 	if (ThisRobot->animation_type == ATTACK_ANIMATION)
 		return;
@@ -568,8 +583,7 @@ static void MoveThisRobotThowardsHisCurrentTarget(enemy * ThisRobot)
 		ThisRobot->PrivatePathway[0].x = ThisRobot->pos.x;
 		ThisRobot->PrivatePathway[0].y = ThisRobot->pos.y;
 	}
-
-};				// void MoveThisRobotThowardsHisCurrentTarget ( int EnemyNum )
+}
 
 /**
  * This function sets a new waypoint to a bot.
@@ -651,7 +665,7 @@ static int set_new_random_waypoint(enemy *this_robot)
 	for (i = 0; i < num_conn; i++) {
 		waypoint *w = &wpts[connections[i]];
 
-		int is_free = CheckIfWayIsFreeOfDroids(current_waypoint->x + 0.5, current_waypoint->y + 0.5,
+		int is_free = way_free_of_droids(current_waypoint->x + 0.5, current_waypoint->y + 0.5,
 				w->x + 0.5, w->y + 0.5, this_robot->pos.z, &frw_ctx);
 		if (is_free) {
 			if (connections[i] != this_robot->lastwaypoint) {
@@ -731,16 +745,6 @@ static void set_new_waypointless_walk_target(enemy * ThisRobot, moderately_finep
 	}
 };				// void set_new_waypointless_walk_target ( enemy* ThisRobot )
 
-/**
- * This function tells if a given level is active in the sense that there
- * is one ore more player character on the level, so that need exists to
- * move all the enemies on this level etc.
- */
-int IsActiveLevel(int levelnum)
-{
-	return (Me.energy > 0 && Me.pos.z == levelnum);
-};				// int IsActiveLevel ( int levelnum ) 
-
 /**
  * When a (hostile) robot is defeated and explodes, it will drop some 
  * treasure, i.e. stuff it had or parts that it consisted of or similar
@@ -789,9 +793,8 @@ void DropEnemyTreasure(Enemy ThisRobot)
 		// We make sure the item created is of a reasonable type
 		//
 		if ((ThisRobot->on_death_drop_item_code <= 0) || (ThisRobot->on_death_drop_item_code >= Number_Of_Item_Types)) {
-			DebugPrintf(-1000, "\n%s(): item type found: %d.", ThisRobot->on_death_drop_item_code);
-			ErrorMessage(__FUNCTION__, "\
-Item to be dropped (forced for this bot) is of illegal type!", PLEASE_INFORM, IS_FATAL);
+			ErrorMessage(__FUNCTION__, "Bot at %f %f (level %d, dialog %s) is specified to drop an item that doesn't exist (item type %d).", PLEASE_INFORM, IS_WARNING_ONLY, ThisRobot->pos.x, ThisRobot->pos.y, ThisRobot->pos.z, ThisRobot->dialog_section_name, ThisRobot->on_death_drop_item_code);
+			return;
 		}
 
 		DropItemAt(ThisRobot->on_death_drop_item_code, ThisRobot->pos.z, ThisRobot->virt_pos.x, ThisRobot->virt_pos.y, 1);
@@ -801,9 +804,8 @@ Item to be dropped (forced for this bot) is of illegal type!", PLEASE_INFORM, IS
 	// there is still some chance, that the enemy will have (and drop) some other
 	// valuables, that the Tux can then collect afterwards.
 	//
-	DropRandomItem(ThisRobot->pos.z, ThisRobot->virt_pos.x, ThisRobot->virt_pos.y, Druidmap[ThisRobot->type].monster_level, FALSE);
-
-};				// void DropEnemyTreasure ( Enemy ThisRobot )
+	DropRandomItem(ThisRobot->pos.z, ThisRobot->virt_pos.x, ThisRobot->virt_pos.y, Druidmap[ThisRobot->type].drop_class, FALSE);
+}
 
 /**
  *
@@ -819,7 +821,7 @@ int MakeSureEnemyIsInsideHisLevel(Enemy ThisRobot)
 There was a droid found outside the bounds of this level (when dying).\n\
 This is an error and should not occur, but most likely it does since\n\
 the bots are allowed some motion without respect to existing waypoints\n\
-in Freedroid RPG.\n", PLEASE_INFORM, IS_FATAL);
+in FreedroidRPG.\n", PLEASE_INFORM, IS_FATAL);
 		return (FALSE);
 	}
 
@@ -840,7 +842,6 @@ in Freedroid RPG.\n", PLEASE_INFORM, IS_FATAL);
  */
 static void enemy_spray_blood(enemy * CurEnemy)
 {
-	extern obstacle *action_create_obstacle(level *, double, double, int);
 	moderately_finepoint target_pos = { 1.0, 0 };
 
 	RotateVectorByAngle(&target_pos, MyRandom(360));
@@ -848,12 +849,21 @@ static void enemy_spray_blood(enemy * CurEnemy)
 	target_pos.x += CurEnemy->virt_pos.x;
 	target_pos.y += CurEnemy->virt_pos.y;
 
-	if (Druidmap[CurEnemy->type].is_human)
-		action_create_obstacle(curShip.AllLevels[CurEnemy->pos.z], target_pos.x, target_pos.y, ISO_BLOOD_1 + MyRandom(7));
-	else
-		action_create_obstacle(curShip.AllLevels[CurEnemy->pos.z], target_pos.x, target_pos.y, ISO_OIL_STAINS_1 + MyRandom(7));
+	struct obstacle_group *blood_group = NULL;
 
-};				// void enemy_spray_blood ( Enemy CurEnemy ) 
+	if (Druidmap[CurEnemy->type].is_human)
+		blood_group = get_obstacle_group_by_name("blood");
+	else
+		blood_group = get_obstacle_group_by_name("oil stains");
+
+	if (!blood_group) {
+		ErrorMessage(__FUNCTION__, "Could not find obstacle group for blood.", PLEASE_INFORM, IS_WARNING_ONLY);
+		return;
+	}
+
+	int *random_blood_type = dynarray_member(&blood_group->members, MyRandom(blood_group->members.size - 1), sizeof(int));
+	add_obstacle(curShip.AllLevels[CurEnemy->pos.z], target_pos.x, target_pos.y, *random_blood_type);
+}
 
 /**
  * When a robot has reached energy <= 1, then this robot will explode and
@@ -918,10 +928,10 @@ static int kill_enemy(enemy * target, char givexp, int killertype)
 		//        phase is reached and so serious bugs other than that, so I think it 
 		//        will be tolerable this way.
 		//
-		target->animation_phase = ((float)first_death_animation_image[target->type]) - 1 + 0.1;
+		target->animation_phase = ((float)first_death_animation_image[Druidmap[target->type].individual_shape_nr]) - 1 + 0.1;
 		target->animation_type = DEATH_ANIMATION;
 		play_death_sound_for_bot(target);
-		DebugPrintf(1, "\n%s(): playing death sound because bot of type %d really died.", __FUNCTION__, target->type);
+		DebugPrintf(1, "\n%s(): playing death sound because bot of type %d really died.", __FUNCTION__, Druidmap[target->type].individual_shape_nr);
 
 		if (MyRandom(15) == 1)
 			enemy_spray_blood(target);
@@ -930,6 +940,8 @@ static int kill_enemy(enemy * target, char givexp, int killertype)
 	list_move(&(target->global_list), &dead_bots_head);	// bot is dead? move it to dead list
 	list_del(&(target->level_list));	// bot is dead? remove it from level list
 
+	event_enemy_died(target);
+
 	return 0;
 };
 
@@ -942,11 +954,11 @@ static void start_gethit_animation(enemy * ThisRobot)
 	// Maybe this robot is fully animated.  In this case, after getting
 	// hit, the gethit animation should be displayed, which we'll initiate here.
 	//
-	if ((last_gethit_animation_image[ThisRobot->type] - first_gethit_animation_image[ThisRobot->type] > 0)) {
+	if ((last_gethit_animation_image[Druidmap[ThisRobot->type].individual_shape_nr] - first_gethit_animation_image[Druidmap[ThisRobot->type].individual_shape_nr] > 0)) {
 		if ((ThisRobot->animation_type == DEATH_ANIMATION)) {
 			DebugPrintf(-4, "\n%s(): WARNING: animation phase reset for INFOUT bot... ", __FUNCTION__);
 		}
-		ThisRobot->animation_phase = ((float)first_gethit_animation_image[ThisRobot->type]) + 0.1;
+		ThisRobot->animation_phase = ((float)first_gethit_animation_image[Druidmap[ThisRobot->type].individual_shape_nr]) + 0.1;
 		ThisRobot->animation_type = GETHIT_ANIMATION;
 	}
 
@@ -993,8 +1005,6 @@ void hit_enemy(enemy * target, float hit, char givexp, short int killertype, cha
 		}
 	}
 
-	EnemyHitByBulletText(target);
-
 	PlayEnemyGotHitSound(Druidmap[target->type].got_hit_sound_type);
 
 	target->energy -= hit;
@@ -1018,7 +1028,7 @@ static void MoveThisEnemy(enemy * ThisRobot)
 
 	gps oldpos = { ThisRobot->pos.x, ThisRobot->pos.y, ThisRobot->pos.z };
 
-	MoveThisRobotThowardsHisCurrentTarget(ThisRobot);
+	MoveThisRobotTowardsHisCurrentTarget(ThisRobot);
 
 	if (CheckEnemyEnemyCollision(ThisRobot)) {
 		ThisRobot->pos.x = oldpos.x;
@@ -1414,13 +1424,17 @@ static void state_machine_situational_transitions(enemy * ThisRobot)
 	/* The various situations are listed in increasing priority order (ie. they may override each other, so the least priority comes first. */
 	/* In an ideal world, this would not exist and be done for each state. But we're in reality and have to limit code duplication. */
 
-	/* Rush Tux when he's close */
+	// Rush Tux when he's close & visible
 	update_virtual_position(&ThisRobot->virt_pos, &ThisRobot->pos, Me.pos.z);
-	if (ThisRobot->will_rush_tux && ThisRobot->virt_pos.z != -1
+	if (ThisRobot->will_rush_tux && ThisRobot->virt_pos.z != -1 && (Me.invisible_duration <= 0)
 	    && (powf(Me.pos.x - ThisRobot->virt_pos.x, 2) + powf(Me.pos.y - ThisRobot->virt_pos.y, 2)) < 16) {
 		ThisRobot->combat_state = RUSH_TUX_AND_OPEN_TALK;
 	}
 
+	//Transition away from Rush Tux gracefully if it is unset
+	if (!ThisRobot->will_rush_tux && (ThisRobot->combat_state == RUSH_TUX_AND_OPEN_TALK))
+		ThisRobot->combat_state = UNDEFINED_STATE;
+
 	/* Return home if we're too far away */
 	if (ThisRobot->max_distance_to_home != 0 &&
 	    sqrt(powf((wpts[ThisRobot->homewaypoint].x + 0.5) - ThisRobot->pos.x, 2) +
@@ -1464,7 +1478,7 @@ static void state_machine_situational_transitions(enemy * ThisRobot)
 		ThisRobot->combat_state = STOP_AND_EYE_TARGET;
 	}
 
-	/* If combat_state is always UNDEFINED_STATE, go to SELECT_NEW_WAYPOINT state */
+	/* If combat_state is UNDEFINED_STATE, go to SELECT_NEW_WAYPOINT state */
 	if (ThisRobot->combat_state == UNDEFINED_STATE) {
 		ThisRobot->combat_state = SELECT_NEW_WAYPOINT;
 	}
@@ -1494,10 +1508,10 @@ static void state_machine_stop_and_eye_target(enemy * ThisRobot, moderately_fine
 	TurnABitTowardsPosition(ThisRobot, target_vpos.x, target_vpos.y, 120);
 
 	/* Do greet sound if not already done */
-	if (ThisRobot->has_greeted_influencer == FALSE) {
+	if (!ThisRobot->has_greeted_influencer) {
 		ThisRobot->has_greeted_influencer = TRUE;
 		if (Druidmap[ThisRobot->type].greeting_sound_type != (-1)) {
-			PlayGreetingSound(Druidmap[ThisRobot->type].greeting_sound_type);
+			play_greeting_sound(ThisRobot);
 		}
 	}
 
@@ -1517,7 +1531,7 @@ static void state_machine_stop_and_eye_target(enemy * ThisRobot, moderately_fine
 		ThisRobot->combat_state = ATTACK;
 		ThisRobot->last_combat_step = ATTACK_MOVE_RATE + 1.0;	// So that attack will start immediately
 		if (Druidmap[ThisRobot->type].greeting_sound_type != (-1)) {
-			play_enter_attack_run_state_sound(Druidmap[ThisRobot->type].greeting_sound_type);
+			play_enter_attack_run_state_sound(ThisRobot);
 		}
 	}
 }
@@ -2020,7 +2034,7 @@ void update_enemy(enemy * ThisRobot)
  * and attack behavior.
  *
  * Note that no enemy must be killed by the logic function. It's a technical limitation
- * and a requirement in freedroidRPG.
+ * and a requirement in FreedroidRPG.
  */
 void move_enemies(void)
 {
@@ -2049,7 +2063,7 @@ void move_enemies(void)
 
 /**
  * When an enemy is firing a shot, the newly created bullet must be 
- * assigned a speed, that would lead the bullet thowards the intended
+ * assigned a speed, that would lead the bullet towards the intended
  * target, which is done here.
  */
 void set_bullet_speed_to_target_direction(bullet * NewBullet, float bullet_speed, float xdist, float ydist)
@@ -2102,11 +2116,10 @@ static void RawStartEnemysShot(enemy * ThisRobot, float xdist, float ydist)
 		bullet_index = find_free_bullet_index();
 		NewBullet = &(AllBullets[bullet_index]);
 
-		// We send the bullet onto it's way thowards the given target
-		//
+		// We send the bullet onto it's way towards the given target
 		set_bullet_speed_to_target_direction(NewBullet, bullet_speed, xdist, ydist);
 
-		// Newly, also enemys have to respect the angle modifier in their weapons...
+		// Newly, also enemies have to respect the angle modifier in their weapons...
 		//
 		RotateVectorByAngle(&(NewBullet->speed), ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_start_angle_modifier);
 		NewBullet->angle = -(90 + 45 + 180 * atan2(NewBullet->speed.y, NewBullet->speed.x) / M_PI);
@@ -2120,6 +2133,7 @@ static void RawStartEnemysShot(enemy * ThisRobot, float xdist, float ydist)
 		NewBullet->pos.x = ThisRobot->virt_pos.x;
 		NewBullet->pos.y = ThisRobot->virt_pos.y;
 		NewBullet->pos.z = ThisRobot->virt_pos.z;
+		NewBullet->height = Druidmap[ThisRobot->type].gun_muzzle_height;
 
 		// fire bullets so, that they don't hit the shooter...
 		NewBullet->pos.x += (NewBullet->speed.x) / (bullet_speed) * 0.5;
@@ -2133,7 +2147,6 @@ static void RawStartEnemysShot(enemy * ThisRobot, float xdist, float ydist)
 		    MyRandom(ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_damage_modifier);
 
 		NewBullet->time_in_seconds = 0;
-		NewBullet->time_in_frames = 0;
 
 		// Most enemy shots will not have any special 'magic' property...
 		//
@@ -2145,13 +2158,6 @@ static void RawStartEnemysShot(enemy * ThisRobot, float xdist, float ydist)
 		NewBullet->bullet_lifetime = ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_bullet_lifetime;
 
 		NewBullet->owner = ThisRobot->id;
-		NewBullet->ignore_wall_collisions =
-		    ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_bullet_ignore_wall_collisions;
-		NewBullet->was_reflected = FALSE;
-		NewBullet->pass_through_explosions =
-		    ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_bullet_pass_through_explosions;
-		NewBullet->reflect_other_bullets =
-		    ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_bullet_reflect_other_bullets;
 		NewBullet->pass_through_hit_bodies =
 		    ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_bullet_pass_through_hit_bodies;
 		NewBullet->faction = ThisRobot->faction;
@@ -2161,7 +2167,7 @@ static void RawStartEnemysShot(enemy * ThisRobot, float xdist, float ydist)
 		melee_shot *NewShot = &(AllMeleeShots[shot_index]);
 
 		NewShot->attack_target_type = ThisRobot->attack_target_type;
-		NewShot->mine = 0;	/* shot comes from a bot not tux */
+		NewShot->mine = FALSE;	/* shot comes from a bot not tux */
 
 		if (ThisRobot->attack_target_type == ATTACK_TARGET_IS_ENEMY) {
 			NewShot->bot_target_n = ThisRobot->bot_target_n;
@@ -2190,13 +2196,13 @@ static void RawStartEnemysShot(enemy * ThisRobot, float xdist, float ydist)
 	if (ThisRobot->firewait < ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_recharging_time)
 		ThisRobot->firewait = ItemMap[Druidmap[ThisRobot->type].weapon_item.type].item_gun_recharging_time;
 
-	if (last_attack_animation_image[ThisRobot->type] - first_attack_animation_image[ThisRobot->type] > 1) {
-		ThisRobot->animation_phase = ((float)first_attack_animation_image[ThisRobot->type]) + 0.1;
+	if (last_attack_animation_image[Druidmap[ThisRobot->type].individual_shape_nr] - first_attack_animation_image[Druidmap[ThisRobot->type].individual_shape_nr] > 1) {
+		ThisRobot->animation_phase = ((float)first_attack_animation_image[Druidmap[ThisRobot->type].individual_shape_nr]) + 0.1;
 		ThisRobot->animation_type = ATTACK_ANIMATION;
 		ThisRobot->current_angle = -(-90 + 180 * atan2(ydist, xdist) / M_PI);
 	}
 
-	Fire_Bullet_Sound(guntype);
+	fire_bullet_sound(guntype, &ThisRobot->pos);
 };				// void RawStartEnemysShot( enemy* ThisRobot , float xdist , float ydist )
 
 /**
@@ -2214,7 +2220,7 @@ static int ConsideredMoveIsFeasible(Enemy ThisRobot, moderately_finepoint StepVe
 	if ((DirectLineColldet(ThisRobot->pos.x, ThisRobot->pos.y, ThisRobot->pos.x + StepVector.x,
 			       ThisRobot->pos.y + StepVector.y,
 			       ThisRobot->pos.z, NULL)) &&
-	    (CheckIfWayIsFreeOfDroids(ThisRobot->pos.x, ThisRobot->pos.y,
+	    (way_free_of_droids(ThisRobot->pos.x, ThisRobot->pos.y,
 				      ThisRobot->pos.x + StepVector.x, ThisRobot->pos.y + StepVector.y, ThisRobot->pos.z, &frw_ctx))) {
 		return TRUE;
 	}
@@ -2266,7 +2272,7 @@ static void MoveToMeleeCombat(enemy *ThisRobot, gps *target_pos, moderately_fine
 		checked_pos.x = floorf(checked_pos.x * 10.0) / 10.0;
 		checked_pos.y = floorf(checked_pos.y * 10.0) / 10.0;
 
-		if (CheckIfWayIsFreeOfDroids(target_pos->x, target_pos->y, checked_pos.x, checked_pos.y, target_pos->z, &frw_ctx)) {
+		if (way_free_of_droids(target_pos->x, target_pos->y, checked_pos.x, checked_pos.y, target_pos->z, &frw_ctx)) {
 			// If the checked_pos is free, also check that the target is reachable
 			target_reachable =
 			    DirectLineColldet(checked_pos.x, checked_pos.y, target_pos->x, target_pos->y, target_pos->z,
@@ -2454,8 +2460,7 @@ static int TurnABitTowardsPosition(enemy *ThisRobot, float x, float y, float Tur
 		return (TRUE);
 
 	return (FALSE);
-
-};				// int TurnABitThowardsTux ( Enemy ThisRobot , float TurnSpeed )
+}
 
 /**
  * Enemies act as groups.  If one is hit, all will attack and the like.
@@ -2534,12 +2539,12 @@ void animate_enemy(enemy *our_enemy)
 	switch (our_enemy->animation_type) {
 
 	case WALK_ANIMATION:
-		our_enemy->animation_phase += Frame_Time() * droid_walk_animation_speed_factor[our_enemy->type];
+		our_enemy->animation_phase += Frame_Time() * droid_walk_animation_speed_factor[Druidmap[our_enemy->type].individual_shape_nr];
 
 		// While we're in the walk animation cycle, we have the walk animation
 		// images cycle.
 		//
-		if (our_enemy->animation_phase >= last_walk_animation_image[our_enemy->type]) {
+		if (our_enemy->animation_phase >= last_walk_animation_image[Druidmap[our_enemy->type].individual_shape_nr]) {
 			our_enemy->animation_phase = 0;
 			our_enemy->animation_type = WALK_ANIMATION;
 		}
@@ -2547,42 +2552,42 @@ void animate_enemy(enemy *our_enemy)
 		// to the standing cycle...
 		//
 		if ((fabsf(our_enemy->speed.x) < 0.1) && (fabsf(our_enemy->speed.y) < 0.1)) {
-			our_enemy->animation_phase = first_stand_animation_image[our_enemy->type] - 1;
+			our_enemy->animation_phase = first_stand_animation_image[Druidmap[our_enemy->type].individual_shape_nr] - 1;
 			our_enemy->animation_type = STAND_ANIMATION;
 		}
 		break;
 	case ATTACK_ANIMATION:
-		our_enemy->animation_phase += Frame_Time() * droid_attack_animation_speed_factor[our_enemy->type];
+		our_enemy->animation_phase += Frame_Time() * droid_attack_animation_speed_factor[Druidmap[our_enemy->type].individual_shape_nr];
 
-		if (our_enemy->animation_phase >= last_attack_animation_image[our_enemy->type]) {
+		if (our_enemy->animation_phase >= last_attack_animation_image[Druidmap[our_enemy->type].individual_shape_nr]) {
 			our_enemy->animation_phase = 0;
 			our_enemy->animation_type = WALK_ANIMATION;
 		}
 		break;
 	case GETHIT_ANIMATION:
-		our_enemy->animation_phase += Frame_Time() * droid_gethit_animation_speed_factor[our_enemy->type];
+		our_enemy->animation_phase += Frame_Time() * droid_gethit_animation_speed_factor[Druidmap[our_enemy->type].individual_shape_nr];
 
-		if (our_enemy->animation_phase >= last_gethit_animation_image[our_enemy->type]) {
+		if (our_enemy->animation_phase >= last_gethit_animation_image[Druidmap[our_enemy->type].individual_shape_nr]) {
 			our_enemy->animation_phase = 0;
 			our_enemy->animation_type = WALK_ANIMATION;
 		}
 		break;
 	case DEATH_ANIMATION:
-		our_enemy->animation_phase += Frame_Time() * droid_death_animation_speed_factor[our_enemy->type];
+		our_enemy->animation_phase += Frame_Time() * droid_death_animation_speed_factor[Druidmap[our_enemy->type].individual_shape_nr];
 
-		if (our_enemy->animation_phase >= last_death_animation_image[our_enemy->type] - 1) {
-			our_enemy->animation_phase = last_death_animation_image[our_enemy->type] - 1;
+		if (our_enemy->animation_phase >= last_death_animation_image[Druidmap[our_enemy->type].individual_shape_nr] - 1) {
+			our_enemy->animation_phase = last_death_animation_image[Druidmap[our_enemy->type].individual_shape_nr] - 1;
 			our_enemy->animation_type = DEAD_ANIMATION;
 		}
 		break;
 	case DEAD_ANIMATION:
-		our_enemy->animation_phase = last_death_animation_image[our_enemy->type] - 1;
+		our_enemy->animation_phase = last_death_animation_image[Druidmap[our_enemy->type].individual_shape_nr] - 1;
 		break;
 	case STAND_ANIMATION:
-		our_enemy->animation_phase += Frame_Time() * droid_stand_animation_speed_factor[our_enemy->type];
+		our_enemy->animation_phase += Frame_Time() * droid_stand_animation_speed_factor[Druidmap[our_enemy->type].individual_shape_nr];
 
-		if (our_enemy->animation_phase >= last_stand_animation_image[our_enemy->type] - 1) {
-			our_enemy->animation_phase = first_stand_animation_image[our_enemy->type] - 1;
+		if (our_enemy->animation_phase >= last_stand_animation_image[Druidmap[our_enemy->type].individual_shape_nr] - 1) {
+			our_enemy->animation_phase = first_stand_animation_image[Druidmap[our_enemy->type].individual_shape_nr] - 1;
 			our_enemy->animation_type = STAND_ANIMATION;
 		}
 		break;
@@ -2734,7 +2739,6 @@ enemy *get_enemy_with_dialog(const char *dialog)
 		}
 	}
 
-	ErrorMessage(__FUNCTION__, _("Could not find a droid corresponding to the dialog \"%s\"."), PLEASE_INFORM, IS_FATAL, dialog);
 	return NULL;
 }
 #undef _enemy_c
