@@ -36,8 +36,7 @@
 #else
 #define EXTERN extern
 
-EXTERN char *floor_tile_filenames[ALL_ISOMETRIC_FLOOR_TILES];
-EXTERN obstacle_spec obstacle_map[NUMBER_OF_OBSTACLE_TYPES];
+EXTERN struct dynarray obstacle_map;
 EXTERN float FPSover1;
 EXTERN float FPSover10;
 EXTERN float FPSover100;
@@ -48,7 +47,6 @@ EXTERN char font_switchto_neon[2];
 EXTERN char font_switchto_msgstat[2];
 EXTERN char font_switchto_msgvar[2];
 
-EXTERN float HitBotMultiplierTable[];
 EXTERN int SpellHitPercentageTable[];
 EXTERN float MeleeDamageMultiplierTable[];
 EXTERN float MeleeRechargeMultiplierTable[];
@@ -75,6 +73,7 @@ extern char *our_config_dir;
 EXTERN int Number_Of_Droid_Types;
 EXTERN int QuitProgram;
 EXTERN int GameOver;
+EXTERN int quest_browser_activated;
 
 extern list_head_t alive_bots_head;
 extern list_head_t dead_bots_head;
@@ -92,8 +91,12 @@ EXTERN int debug_level;		// 0=no debug 1=some debug messages 2=...etc
 				// (currently only 0 or !=0 is implemented) 
 EXTERN int show_all_droids;	// display enemies regardless of IsVisible() 
 EXTERN int draw_collision_rectangles;	// to better debug collision rectangles
-EXTERN int draw_grid;		// grid to see where objects will be positioned
 EXTERN int start_editor;
+EXTERN int load_saved;
+EXTERN char *saved_game_name;
+
+EXTERN FPSmanager SDL_FPSmanager;
+
 #undef EXTERN
 #ifdef _misc_c
 #define EXTERN
@@ -103,15 +106,11 @@ EXTERN int start_editor;
 
 EXTERN float timeout_from_item_drop;
 EXTERN int use_open_gl;
-EXTERN int command_line_override_for_screen_resolution;
-EXTERN char *character_descriptions[MAX_PERSONS];
 
-EXTERN char previous_part_strings[ALL_PART_GROUPS][50];
 EXTERN mouse_press_button AllMousePressButtons[MAX_MOUSE_PRESS_BUTTONS];
 EXTERN item *item_held_in_hand;
 
-EXTERN iso_image MouseCursorImageList[NUMBER_OF_MOUSE_CURSOR_PICTURES];
-EXTERN iso_image SpellLevelButtonImageList[NUMBER_OF_SKILL_PAGES];
+EXTERN struct image MouseCursorImageList[NUMBER_OF_MOUSE_CURSOR_PICTURES];
 
 EXTERN int Number_Of_Droids_On_Ship;
 EXTERN configuration_for_freedroid GameConfig;
@@ -125,15 +124,13 @@ EXTERN BFont_Info *Red_BFont;
 EXTERN float Overall_Average;
 EXTERN int SkipAFewFrames;
 
-EXTERN int global_ingame_mode;
-
 #undef EXTERN
 #ifdef _view_c
 #define EXTERN
 #else
 #define EXTERN extern
 #endif
-EXTERN iso_image light_radius_chunk[NUMBER_OF_SHADOW_IMAGES];
+EXTERN struct image light_radius_chunk[NUMBER_OF_SHADOW_IMAGES];
 EXTERN SDL_Rect InventoryRect;
 EXTERN SDL_Rect CharacterRect;
 #ifdef HAVE_LIBGL
@@ -168,11 +165,8 @@ EXTERN int gps_transform_map_dirty_flag;
 #define NEIGHBOR_ID_S(lvl)  NEIGHBOR_ID(lvl, 1, 2)
 #define NEIGHBOR_ID_SE(lvl) NEIGHBOR_ID(lvl, 2, 2)
 
-typedef struct tux_part_render_set *tux_part_render_motionclass[MAX_TUX_DIRECTIONS];
-EXTERN struct tux_rendering_s {
-	struct dynarray motion_class_names;
-	tux_part_render_motionclass *render_order;
-} tux_rendering;
+EXTERN struct tux_rendering tux_rendering;
+EXTERN int clickable_obstacle_under_cursor;
 
 #undef EXTERN
 #ifdef _light_c
@@ -182,14 +176,6 @@ EXTERN struct tux_rendering_s {
 #endif
 EXTERN light_radius_config LightRadiusConfig;
 
-#undef EXTERN
-#ifdef _event_c
-#define EXTERN
-#else
-#define EXTERN extern
-#endif
-EXTERN event_trigger AllEventTriggers[MAX_EVENT_TRIGGERS];
-
 #undef EXTERN
 #ifdef _chat_c
 #define EXTERN
@@ -199,10 +185,10 @@ EXTERN event_trigger AllEventTriggers[MAX_EVENT_TRIGGERS];
 int chat_control_next_node;	//what is the next node to use?
 int chat_control_end_dialog;	//end current dialog?
 int chat_control_partner_started;	//the dialog partner is the one who started the talk
-unsigned char *chat_control_chat_flags; //current chat flags
+uint8_t *chat_control_chat_flags; //current chat flags
 enemy *chat_control_chat_droid;	//droid we are chatting with
 dialogue_option ChatRoster[MAX_DIALOGUE_OPTIONS_IN_ROSTER];
-EXTERN text_widget chat_log;
+EXTERN struct widget_text chat_log;
 
 #undef EXTERN
 #ifdef _sound_c
@@ -227,24 +213,17 @@ EXTERN text_widget chat_log;
 EXTERN int Number_Of_Bullet_Types;
 EXTERN SDL_Surface *Screen;
 EXTERN SDL_Surface *StoredMenuBackground[2];
+EXTERN int mouse_cursor;
 #ifdef HAVE_LIBGL
 EXTERN GLuint StoredMenuBackgroundTex[2];
 #endif
-EXTERN int current_mouse_cursor_shape;
 
 //--------------------
 // Now the iso-image pointers for the new individually shaped
 // isometric enemy images
 //
-// EXTERN SDL_Surface *EnemyRotationSurfacePointer[ ENEMY_ROTATION_MODELS_AVAILABLE ] [ ROTATION_ANGLES_PER_ROTATION_MODEL ];
-EXTERN iso_image enemy_iso_images[ENEMY_ROTATION_MODELS_AVAILABLE][ROTATION_ANGLES_PER_ROTATION_MODEL][MAX_ENEMY_MOVEMENT_PHASES];
-EXTERN iso_image
-    BlueEnemyRotationSurfacePointer[ENEMY_ROTATION_MODELS_AVAILABLE][ROTATION_ANGLES_PER_ROTATION_MODEL][MAX_ENEMY_MOVEMENT_PHASES];
-EXTERN iso_image
-    RedEnemyRotationSurfacePointer[ENEMY_ROTATION_MODELS_AVAILABLE][ROTATION_ANGLES_PER_ROTATION_MODEL][MAX_ENEMY_MOVEMENT_PHASES];
-EXTERN iso_image
-    GreenEnemyRotationSurfacePointer[ENEMY_ROTATION_MODELS_AVAILABLE][ROTATION_ANGLES_PER_ROTATION_MODEL][MAX_ENEMY_MOVEMENT_PHASES];
-EXTERN iso_image chat_portrait_of_droid[ENEMY_ROTATION_MODELS_AVAILABLE];
+EXTERN struct image enemy_images[ENEMY_ROTATION_MODELS_AVAILABLE][ROTATION_ANGLES_PER_ROTATION_MODEL][MAX_ENEMY_MOVEMENT_PHASES];
+EXTERN struct image chat_portrait_of_droid[ENEMY_ROTATION_MODELS_AVAILABLE];
 
 // EXTERN int phases_in_enemy_animation [ ENEMY_ROTATION_MODELS_AVAILABLE ];
 EXTERN int first_walk_animation_image[ENEMY_ROTATION_MODELS_AVAILABLE];
@@ -257,17 +236,16 @@ EXTERN int first_death_animation_image[ENEMY_ROTATION_MODELS_AVAILABLE];
 EXTERN int last_death_animation_image[ENEMY_ROTATION_MODELS_AVAILABLE];
 EXTERN int first_stand_animation_image[ENEMY_ROTATION_MODELS_AVAILABLE];
 EXTERN int last_stand_animation_image[ENEMY_ROTATION_MODELS_AVAILABLE];
-EXTERN int use_default_attack_image[ENEMY_ROTATION_MODELS_AVAILABLE];
-EXTERN int use_default_gethit_image[ENEMY_ROTATION_MODELS_AVAILABLE];
-EXTERN int use_default_death_image[ENEMY_ROTATION_MODELS_AVAILABLE];
-EXTERN int use_default_stand_image[ENEMY_ROTATION_MODELS_AVAILABLE];
 EXTERN int droid_walk_animation_speed_factor[ENEMY_ROTATION_MODELS_AVAILABLE];
 EXTERN int droid_attack_animation_speed_factor[ENEMY_ROTATION_MODELS_AVAILABLE];
 EXTERN int droid_gethit_animation_speed_factor[ENEMY_ROTATION_MODELS_AVAILABLE];
 EXTERN int droid_death_animation_speed_factor[ENEMY_ROTATION_MODELS_AVAILABLE];
 EXTERN int droid_stand_animation_speed_factor[ENEMY_ROTATION_MODELS_AVAILABLE];
 
-EXTERN iso_image floor_iso_images[ALL_ISOMETRIC_FLOOR_TILES];
+EXTERN struct tux_motion_class_images *tux_images;
+EXTERN struct dynarray underlay_floor_images;
+EXTERN struct dynarray overlay_floor_images;
+EXTERN struct dynarray obstacle_images;
 
 EXTERN float iso_floor_tile_width;
 EXTERN float iso_floor_tile_height;
@@ -281,7 +259,7 @@ EXTERN int vid_bpp;		// bits per pixel
 #else
 #define EXTERN extern
 #endif
-EXTERN text_widget message_log;
+EXTERN struct widget_text *message_log;
 
 #undef EXTERN
 #ifdef _blocks_c
@@ -289,7 +267,10 @@ EXTERN text_widget message_log;
 #else
 #define EXTERN extern
 #endif
-EXTERN obstacle_spec obstacle_map[NUMBER_OF_OBSTACLE_TYPES];
+EXTERN struct dynarray obstacle_map;
+EXTERN struct dynarray underlay_floor_tile_filenames;
+EXTERN struct dynarray overlay_floor_tile_filenames;
+EXTERN char *PrefixToFilename[ENEMY_ROTATION_MODELS_AVAILABLE];
 
 #undef EXTERN
 #ifdef _text_c
@@ -297,7 +278,6 @@ EXTERN obstacle_spec obstacle_map[NUMBER_OF_OBSTACLE_TYPES];
 #else
 #define EXTERN extern
 #endif
-EXTERN int display_char_disabled;
 EXTERN SDL_Rect Droid_Image_Window;
 
 #undef EXTERN
@@ -356,6 +336,15 @@ enum {
 	ROOT_IS_LVLEDIT,
 };
 
+#undef EXTERN
+#ifdef _obstacle_c
+#define EXTERN
+#else
+#define EXTERN extern
+#endif
+
+EXTERN struct dynarray obstacle_groups;
+
 #undef EXTERN
 #ifdef _saveloadgame_c
 #define EXTERN
@@ -363,6 +352,7 @@ enum {
 #define EXTERN extern
 #endif
 EXTERN struct auto_string *savestruct_autostr;
+EXTERN jmp_buf saveload_jmpbuf;
 
 #undef EXTERN
 #ifdef _benchmark_c
@@ -381,6 +371,14 @@ EXTERN char *do_benchmark;
 
 EXTERN list_head_t npc_head;
 
+#undef EXTERN
+#ifdef _game_ui_c
+#define EXTERN
+#else
+#define EXTERN extern
+#endif
+EXTERN struct widget_button *game_map;
+
 #undef EXTERN
 #ifdef _influ_c
 #define EXTERN
@@ -395,5 +393,8 @@ EXTERN struct {
 	struct distancebased_animation run;
 } tux_anim;
 
+#define IMAGE_SCALE_RGB_TRANSFO(SCALE, R, G, B) set_image_transformation(SCALE, SCALE, R, G, B, 1.0, 0)
+#define IMAGE_SCALE_TRANSFO(SCALE) IMAGE_SCALE_RGB_TRANSFO(SCALE, 1.0, 1.0, 1.0)
+#define IMAGE_NO_TRANSFO IMAGE_SCALE_TRANSFO(1.0)
 #endif				// _global_h
 
