@@ -25,7 +25,7 @@
  */
 
 /**
- * This file contains all features, movement, fireing, collision and 
+ * This file contains all features, movement, firing, collision and
  * extras of the influencer.
  */
 
@@ -37,6 +37,7 @@
 #include "struct.h"
 #include "global.h"
 #include "proto.h"
+#include "widgets/widgets.h"
 
 #define BEST_MELEE_DISTANCE (0.8)
 #define BEST_CHAT_DISTANCE (BEST_MELEE_DISTANCE+0.2)
@@ -67,34 +68,57 @@ float vect_len(moderately_finepoint our_vector)
 	return (sqrt(powf(our_vector.x, 2) + powf(our_vector.y, 2)));
 };				// float vect_len ( moderately_finepoint our_vector )
 
+static float get_tux_running_speed(void)
+{
+	if (GameConfig.cheat_double_speed)
+		return 2 * TUX_RUNNING_SPEED;
+
+	return TUX_RUNNING_SPEED;
+}
+
 /**
  * This function adapts the influencers current speed to the maximal speed
- * possible for the influencer (determined by the currely used drive type).
+ * possible for the influencer.
+ *
+ * This function also stops Tux from moving while fighting.
  */
-static void limit_tux_speed_to_a_maximum()
+static void limit_tux_speed()
 {
-	double maxspeed = TUX_RUNNING_SPEED;
-	const double speed = sqrt(Me.speed.x * Me.speed.x + Me.speed.y * Me.speed.y);
-	if (speed > maxspeed) {
-		const double ratio = maxspeed / speed;
+	/* Stop all movement if Tux is currently inside attack animation.  This is
+	 * to stop Tux from moving without his legs animating.
+	 *
+	 * NOTE REGARDING MELEE ATTACK.  Tux starts to swing before he has reached
+	 * BEST_MELEE_DISTANCE.  If we stop before reaching this distance, Tux will
+	 * continue to try to move to this distance between swings, which causes a
+	 * jerking motion towards the enemy.  To stop this from happening, we force
+	 * Tux NOT to stand still when he is first charging a new target.  He stands
+	 * still ONLY when he changes target during a swing. */
+	static enemy *previous_target;
+	enemy *current_target = enemy_resolve_address(Me.current_enemy_target_n,
+												  &Me.current_enemy_target_addr);
+
+	if (Me.weapon_item.type >= 0) {
+		int has_melee = ItemMap[Me.weapon_item.type].item_weapon_is_melee;
+		if (Me.weapon_swing_time != -1 && (!has_melee
+		   || (has_melee && (previous_target != current_target || current_target == NULL))))
+		{
+			Me.speed.x = 0;
+			Me.speed.y = 0;
+			return;
+		} else {
+			previous_target = current_target;
+		}
+	}
+
+	/* Limit the speed when Tux is not attacking. */
+	float speed = Me.speed.x * Me.speed.x + Me.speed.y * Me.speed.y;
+	float running_speed = get_tux_running_speed();
+	if (speed > running_speed * running_speed) {
+		float ratio = (running_speed * running_speed) / speed;
 		Me.speed.x *= ratio;
 		Me.speed.y *= ratio;
 	}
-};				// void limit_tux_speed_to_a_maximum ( ) 
-
-/**
- * This function reduces the influencers speed as long as no direction 
- * key of any form is pressed and also no mouse move target is set.
- */
-static void tux_friction_with_air()
-{
-	if (Me.next_intermediate_point[0].x != (-1))
-		return;
-
-	Me.speed.x = 0;
-	Me.speed.y = 0;
-
-};				// tux_friction_with_air ( )
+}
 
 /**
  * When the player has requested an attack motion, we start the 
@@ -175,7 +199,7 @@ void correct_tux_position_according_to_jump()
 	old_mouse_move_target.y = Me.mouse_move_target.y;
 	old_mouse_move_target.z = Me.mouse_move_target.z;
 
-	Teleport(newpos.z, newpos.x, newpos.y, FALSE);
+	Teleport(newpos.z, newpos.x, newpos.y, FALSE, FALSE);
 
 	Me.mouse_move_target.x = old_mouse_move_target.x;
 	Me.mouse_move_target.y = old_mouse_move_target.y;
@@ -302,13 +326,10 @@ float GetInfluPositionHistoryZ(int HowLongPast)
  */
 void CheckIfCharacterIsStillOk()
 {
-	if (Me.AllMissions[GetMissionIndexByName("Propagating a faulty firmware")].MissionIsComplete) {
+	if (Me.AllMissions[GetMissionIndexByName("Propagating a faulty firmware update")].MissionIsComplete) {
 		ThouHastWon();
 	}
 
-	if (Me.god_mode)
-		Me.energy = Me.maxenergy;
-
 	// Now we check if the main character is really still ok.
 	//
 	if (Me.energy <= 0) {
@@ -333,8 +354,8 @@ static void CheckForTuxOutOfMap()
 		fprintf(stderr, "\n\nplayer's last position: X=%f, Y=%f, Z=%d.\n", Me.pos.x, Me.pos.y, Me.pos.z);
 		ErrorMessage(__FUNCTION__, "\
 A player's Tux was found outside the map.\n\
-This indicates either a bug in the Freedroid RPG code or\n\
-a bug in the currently used map system of Freedroid RPG.", PLEASE_INFORM, IS_FATAL);
+This indicates either a bug in the FreedroidRPG code or\n\
+a bug in the currently used map system of FreedroidRPG.", PLEASE_INFORM, IS_FATAL);
 	}
 };				// void CheckForTuxOutOfMap ( )
 
@@ -406,39 +427,30 @@ void tux_get_move_target_and_attack(gps * movetgt)
 }				// void tux_get_move_target_and_attack( )
 
 /**
- * Self-explanatory.
- * This function also takes into account any conveyor belts the Tux might
- * be standing on.
+ * Actually move Tux towards the target.
  */
-static void MoveTuxAccordingToHisSpeed()
+static void move_tux_according_to_his_speed()
 {
 	float planned_step_x;
 	float planned_step_y;
 
 	// Now we move influence according to current speed.  But there has been a problem
-	// reported from people, that the influencer would (*very* rarely) jump throught walls
-	// and even out of the ship.  This has *never* occured on my fast machine.  Therefore
-	// I assume that the problem is related to sometimes very low framerates on these machines.
+	// reported from people, that the influencer would (*very* rarely) jump through walls
+	// and even out of the ship.  This has *never* occurred on my fast machine.  Therefore
+	// I assume that the problem is related to sometimes very low frame rates on these machines.
 	// So, we do a sanity check not to make steps too big.
 	//
-	// And on machines with FPS << 20, it will certainly alter the game behaviour, so people
-	// should really start using a pentium or better machine.
+	// And on machines with FPS << 20, it will certainly alter the game behavior, so people
+	// should really start using a Pentium or better machine.
 	//
 	planned_step_x = Me.speed.x * Frame_Time();
 	planned_step_y = Me.speed.y * Frame_Time();
 
-	// Maybe the Tux is just executing a weapon strike.  In this case, there should
-	// be no movement at all, so in this case we'll just not go anywhere...
-	//
-/*XXX  if ( Me . weapon_swing_time > 0 )
-    {
-      planned_step_x = 0 ;
-      planned_step_y = 0 ;
-    }*/
-
 	Me.pos.x += planned_step_x;
 	Me.pos.y += planned_step_y;
 
+	trigger_position_events();
+
 	// If the Tux got stuck, i.e. if he got no speed at all and still is 
 	// currently not in a 'passable' position, the fallback handling needs
 	// to be applied to move the Tux out of the offending obstacle (i.e. 
@@ -487,17 +499,14 @@ static void MoveTuxAccordingToHisSpeed()
 			}
 		}
 	}
-
-};				// void MoveTuxAccordingToHisSpeed ( )
+}
 
 /**
- * This function contains the final dumb movement code, that, without
- * any checks and any refinement, just moves the tux thowards the given
- * target position.
- *
- * The return value indicates, if the target has been sufficiently 
- * approximated (TRUE) already or not (FALSE) .
+ * This function contains dumb movement code that, without any checks nor
+ * refinement, calculates the direction and speed in which Tux has to move to
+ * reach the target position.
  *
+ * Returns TRUE if the target has been sufficiently approximated.
  */
 static int move_tux_towards_raw_position(float x, float y)
 {
@@ -539,8 +548,9 @@ static int move_tux_towards_raw_position(float x, float y)
 
 	if ((LeftCtrlPressed() || GameConfig.autorun_activated) &&
 		!(LeftCtrlPressed() && GameConfig.autorun_activated) && (!Me.running_must_rest)) {
-		planned_step.x = RemainingWay.x * TUX_RUNNING_SPEED / length;
-		planned_step.y = RemainingWay.y * TUX_RUNNING_SPEED / length;
+		float running_speed = get_tux_running_speed();
+		planned_step.x = RemainingWay.x * running_speed / length;
+		planned_step.y = RemainingWay.y * running_speed / length;
 	} else {
 		planned_step.x = RemainingWay.x * TUX_WALKING_SPEED / length;
 		planned_step.y = RemainingWay.y * TUX_WALKING_SPEED / length;
@@ -578,31 +588,27 @@ static void move_tux_towards_intermediate_point(void)
 {
 	int i;
 
-	// If there is no intermediate course, we don't need to do anything
-	// in this function.
-	//
-	if (Me.next_intermediate_point[0].x == (-1)) {
-		// The fact that there is no more intermediate course can mean, that
-		// there never has been any intermediate course or we have now arrived
-		// at the end of the previous intermediate course.
-		//
-		// But that maybe means, that it is now time for the combo_action, that
-		// can occur on the end of any intermediate course, like e.g. open a
-		// chest or pick up some item.
-		//
-		// DebugPrintf ( 2 , "\nAm I now at the last intermediate point???" );
+	/* If we have no intermediate point, Tux has arrived at the target. */
+	if (Me.next_intermediate_point[0].x == -1) {
+		/* First, stop Tux. */
+		Me.speed.x = 0;
+		Me.speed.y = 0;
+
+		/* We might have a combo_action, that can occur on the end of any
+		 * course, like e.g. open a chest or pick up some item. */
+		level *lvl = curShip.AllLevels[Me.mouse_move_target.z];
 		switch (Me.mouse_move_target_combo_action_type) {
 		case NO_COMBO_ACTION_SET:
 			break;
 		case COMBO_ACTION_OBSTACLE:
-			obstacle_map[curShip.AllLevels[Me.mouse_move_target.z]->obstacle_list[Me.mouse_move_target_combo_action_parameter].type].action(
-                    curShip.AllLevels[Me.mouse_move_target.z], 
+			get_obstacle_spec(lvl->obstacle_list[Me.mouse_move_target_combo_action_parameter].type)->action_fn(
+                    lvl,
                     Me.mouse_move_target_combo_action_parameter);
 			break;
 		case COMBO_ACTION_PICK_UP_ITEM:
 			// If Tux arrived at destination, pick up the item and give it to the player
-			if (check_for_items_to_pickup(curShip.AllLevels[Me.mouse_move_target.z], Me.mouse_move_target_combo_action_parameter)) {
-				item *it = &curShip.AllLevels[Me.mouse_move_target.z]->ItemList[Me.mouse_move_target_combo_action_parameter];
+			if (check_for_items_to_pickup(lvl, Me.mouse_move_target_combo_action_parameter)) {
+				item *it = &lvl->ItemList[Me.mouse_move_target_combo_action_parameter];
 
 				if (GameConfig.Inventory_Visible) {
 					// Special case: when the inventory screen is open, and there
@@ -671,15 +677,12 @@ void move_tux()
 	// We store the influencers position for the history record and so that others
 	// can follow his trail.
 	//
-	static int i = 0;
-	i++;
-	if (i & 1) {
-		Me.current_zero_ring_index++;
-		Me.current_zero_ring_index %= MAX_INFLU_POSITION_HISTORY;
-		Me.Position_History_Ring_Buffer[Me.current_zero_ring_index].x = Me.pos.x;
-		Me.Position_History_Ring_Buffer[Me.current_zero_ring_index].y = Me.pos.y;
-		Me.Position_History_Ring_Buffer[Me.current_zero_ring_index].z = Me.pos.z;
-	}
+	Me.current_zero_ring_index++;
+	Me.current_zero_ring_index %= MAX_INFLU_POSITION_HISTORY;
+	Me.Position_History_Ring_Buffer[Me.current_zero_ring_index].x = Me.pos.x;
+	Me.Position_History_Ring_Buffer[Me.current_zero_ring_index].y = Me.pos.y;
+	Me.Position_History_Ring_Buffer[Me.current_zero_ring_index].z = Me.pos.z;
+
 
 	if (Me.paralyze_duration) {
 		Me.speed.x = 0;
@@ -731,8 +734,13 @@ void move_tux()
 
 	// Maybe we need to fire a bullet or set a new mouse move target
 	// for the new move-to location
-	//
-	AnalyzePlayersMouseClick();
+	// There are currently two different input systms in use - event based and state based.
+	// In order to maintain compatibility between the two, a game_map widget is added on the
+	// game main widget to detect how user input should be handled. Therefore, when the mouse
+	// is over the game_map widget (the widget is not in its DEFAULT state), no further event handling
+	// is done by the widget system and AnalyzePlayersMouseClick is called.
+	if (game_map->state != DEFAULT)
+		AnalyzePlayersMouseClick();
 
 	if (MouseLeftPressed())
 		no_left_button_press_in_previous_analyze_mouse_click = FALSE;
@@ -748,24 +756,26 @@ void move_tux()
 		enemy_set_reference(&Me.current_enemy_target_n, &Me.current_enemy_target_addr, NULL);
 		return;
 	}
-	// The influ should lose some of his speed when no key is pressed and
-	// also no mouse move target is set.
-	//
-	tux_friction_with_air();
 
-	limit_tux_speed_to_a_maximum();
+	limit_tux_speed();
 
-	MoveTuxAccordingToHisSpeed();
+	move_tux_according_to_his_speed();
 
-	animate_tux();		// move the "phase" of influencers rotation
-};				// void move_tux( );
+	animate_tux();
+}
 
 /**
- * This function decrements Tux's health and increments the relevent statistic 
+ * This function decrements Tux's health and increments the relevant statistic
  * variable.
  */
 void hit_tux(float damage, int owner)
 {
+	if (Me.god_mode)
+		return;
+
+	if (Me.energy < 0)
+		return;
+
 	Me.energy -= damage;
 
 	if (damage > Me.energy / 10)
@@ -799,7 +809,15 @@ void animate_tux()
 #define STEP_TIME (0.28)
 	static float step_countdown = 0;
 
-	// First we handle the case of just getting hit
+	// If Tux is paralyzed, show him as standing still.
+	if (Me.paralyze_duration) {
+		Me.walk_cycle_phase = 0.0;
+		Me.phase = tux_anim.standing_keyframe;
+
+		return;
+	}
+
+	// Handle the case of Tux just getting hit
 	//
 	// Note: We do not yet have keyframes for such an animation, so those
 	// commented lines are only a place-holder for future extension.
@@ -868,7 +886,7 @@ void animate_tux()
 		anim_spec = &(tux_anim.run);
 		step_countdown += Frame_Time();
 		if (step_countdown > STEP_TIME) {
-			play_sample_using_WAV_cache_v("effects/tux_footstep.ogg", FALSE, FALSE, 0.2);
+			play_sound_cached_v("effects/tux_footstep.ogg", 0.2);
 			step_countdown -= STEP_TIME;
 		}
 	}
@@ -894,7 +912,7 @@ void animate_tux()
 }
 
 /**
- * This function creates several exprosions around the location where the
+ * This function creates several explosions around the location where the
  * influencer is (was) positioned.  It is used after the influencers 
  * death to make his death more spectacular.
  */
@@ -925,9 +943,67 @@ void start_tux_death_explosions(void)
 
 };				// void start_tux_death_explosions ( void )
 
+/** 
+ * This function opens a menu when tux dies, asking the
+ * player if he/she wants to load latest or backup game, 
+ * quit to main menu or quit the game.
+ */
+void do_death_menu()
+{
+        char *MenuTexts[100];
+        int done = FALSE;
+        int MenuPosition = 1;
+        int i;
+
+        game_status = INSIDE_MENU;
+
+        input_handle();
+
+        enum {
+                LOAD_LATEST_POSITION = 1,
+                LOAD_BACKUP_POSITION,
+                QUIT_TO_MAIN_POSITION,
+                QUIT_POSITION
+        };
+
+        while (!done) {
+                i = 0;
+                MenuTexts[i++] = "Load Latest";
+                MenuTexts[i++] = "Load Backup";
+                MenuTexts[i++] = "Quit to Main Menu";
+                MenuTexts[i++] = "Exit FreedroidRPG";
+                MenuTexts[i++] = "";
+
+                MenuPosition = DoMenuSelection("", MenuTexts, 1, "--GAME_BACKGROUND--", Menu_BFont);
+
+                switch (MenuPosition) {
+
+                case LOAD_LATEST_POSITION:
+                        LoadGame();
+                        done = !done;
+                        break;
+                case LOAD_BACKUP_POSITION:
+                        LoadBackupGame();
+                        done = !done;
+                        break;
+                case QUIT_TO_MAIN_POSITION:
+                        if (game_root_mode == ROOT_IS_GAME) {
+                                GameOver = TRUE;
+                        }
+                        done = !done;
+                        break;
+                case QUIT_POSITION:
+                        Terminate(EXIT_SUCCESS, TRUE);
+                        break;
+                default:
+                        break;
+                }
+	}
+}
+
 /**
  * This function checks if the influencer is currently colliding with an
- * enemys and throws him back in that case.
+ * enemy and throws him back in that case.
  */
 void check_tux_enemy_collision(void)
 {
@@ -985,9 +1061,13 @@ enemy *GetLivingDroidBelowMouseCursor()
 {
 	gps mouse_vpos, mouse_pos;
 	int RotationModel, RotationIndex;
-	iso_image *our_iso_image;
+	struct image *our_image;
 	enemy *this_bot;
 
+	// no interaction with the game when the world is frozen
+	if (world_frozen())
+		return NULL;
+
 	mouse_vpos.x = translate_pixel_to_map_location((float)input_axis.x, (float)input_axis.y, TRUE);
 	mouse_vpos.y = translate_pixel_to_map_location((float)input_axis.x, (float)input_axis.y, FALSE);
 	mouse_vpos.z = Me.pos.z;
@@ -1012,10 +1092,10 @@ enemy *GetLivingDroidBelowMouseCursor()
 		//
 		RotationModel = set_rotation_model_for_this_robot(this_bot);
 
-		our_iso_image = &(enemy_iso_images[RotationModel][RotationIndex][(int)this_bot->animation_phase]);
+		our_image = &(enemy_images[RotationModel][RotationIndex][(int)this_bot->animation_phase]);
 
 		update_virtual_position(&this_bot->virt_pos, &this_bot->pos, Me.pos.z);
-		if (mouse_cursor_is_on_that_iso_image(this_bot->virt_pos.x, this_bot->virt_pos.y, our_iso_image)) {
+		if (mouse_cursor_is_on_that_image(this_bot->virt_pos.x, this_bot->virt_pos.y, our_image)) {
 			return this_bot;
 		}
 	}
@@ -1032,23 +1112,19 @@ void FillInDefaultBulletStruct(bullet * CurBullet, int bullet_image_type, short
 	CurBullet->pos.x = Me.pos.x;
 	CurBullet->pos.y = Me.pos.y;
 	CurBullet->pos.z = Me.pos.z;
+	CurBullet->height = 0;
 	CurBullet->type = bullet_image_type;
 
 	// Previously, we had the damage done only dependant upon the weapon used.  Now
 	// the damage value is taken directly from the character stats, and the UpdateAll...stats
-	// has to do the right computation and updating of this value.  hehe. very conventient.
+	// has to do the right computation and updating of this value.  hehe. very convenient.
 	CurBullet->damage = Me.base_damage + MyRandom(Me.damage_modifier);
 	CurBullet->mine = TRUE;
 	CurBullet->owner = -1;
 	CurBullet->time_to_hide_still = 0.3;
 	CurBullet->bullet_lifetime = ItemMap[weapon_item_type].item_gun_bullet_lifetime;
-	CurBullet->ignore_wall_collisions = ItemMap[weapon_item_type].item_gun_bullet_ignore_wall_collisions;
-	CurBullet->was_reflected = FALSE;
-	CurBullet->reflect_other_bullets = ItemMap[weapon_item_type].item_gun_bullet_reflect_other_bullets;
-	CurBullet->pass_through_explosions = ItemMap[weapon_item_type].item_gun_bullet_pass_through_explosions;
 	CurBullet->pass_through_hit_bodies = ItemMap[weapon_item_type].item_gun_bullet_pass_through_hit_bodies;
 
-	CurBullet->time_in_frames = 0;
 	CurBullet->time_in_seconds = 0;
 
 	CurBullet->freezing_level = 0;
@@ -1070,9 +1146,8 @@ void FireTuxRangedWeaponRaw(short int weapon_item_type, int bullet_image_type, b
 	float BulletSpeed = ItemMap[weapon_item_type].item_gun_speed;
 	double speed_norm;
 	moderately_finepoint speed;
-	float OffsetFactor;
+	float OffsetFactor = 0.5;
 	moderately_finepoint offset;
-	OffsetFactor = 0.25;
 #	define FIRE_TUX_RANGED_WEAPON_RAW_DEBUG 1
 
 	DebugPrintf(1, "\n%s(): target location: x=%f, y=%f.", __FUNCTION__, target_location.x, target_location.y);
@@ -1122,7 +1197,6 @@ void FireTuxRangedWeaponRaw(short int weapon_item_type, int bullet_image_type, b
 	//
 	offset.x = OffsetFactor * (CurBullet->speed.x / BulletSpeed);
 	offset.y = OffsetFactor * (CurBullet->speed.y / BulletSpeed);
-	RotateVectorByAngle(&(offset), -60);
 
 	// And now we re-do it all!  But this time with the extra offset
 	// applied to the SHOT TARGET POINT!
@@ -1151,11 +1225,11 @@ void FireTuxRangedWeaponRaw(short int weapon_item_type, int bullet_image_type, b
 	//
 	offset.x = OffsetFactor * (CurBullet->speed.x / BulletSpeed);
 	offset.y = OffsetFactor * (CurBullet->speed.y / BulletSpeed);
-	RotateVectorByAngle(&(offset), -60);
 
 	CurBullet->pos.x += offset.x;
 	CurBullet->pos.y += offset.y;
 	CurBullet->pos.z = Me.pos.z;
+	CurBullet->height = tux_rendering.gun_muzzle_height;
 
 	CurBullet->faction = FACTION_SELF;
 
@@ -1189,7 +1263,7 @@ int ButtonPressWasNotMeantAsFire()
 	if (MouseLeftPressed() &&
 	    (GameConfig.Inventory_Visible || GameConfig.CharacterScreen_Visible || GameConfig.SkillScreen_Visible
 	     || GameConfig.skill_explanation_screen_visible)
-	    && !MouseCursorIsInUserRect(User_Rect.x + User_Rect.w / 2 + input_axis.x, User_Rect.y + User_Rect.h / 2 + input_axis.y)) {
+	    && !MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y())) {
 		DebugPrintf(0, "\nCursor outside user-rect:\n  User_Rect.x=%d, User_Rect.w=%d, User_Rect.y=%d, User_Rect.h=%d.",
 			    User_Rect.x, User_Rect.w, User_Rect.y, User_Rect.h);
 		DebugPrintf(0, "\nCursor position: X=%d, Y=%d.", input_axis.x, input_axis.y);
@@ -1200,61 +1274,6 @@ int ButtonPressWasNotMeantAsFire()
 
 };				// int ButtonPressWasNotMeantAsFire ( )
 
-/**
- * When the user clicks on a button in the HUD, launch the related action.
- * Return TRUE if the mouse is pressed inside the HUD's area.
- */
-static int handle_click_in_hud()
-{
-	if (MouseCursorIsOnButton(INV_SCREEN_TOGGLE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		if (MouseLeftClicked()) {
-			toggle_game_config_screen_visibility(GAME_CONFIG_SCREEN_VISIBLE_INVENTORY);
-		}
-		return TRUE;
-	}
-
-	if (MouseCursorIsOnButton(CHA_SCREEN_TOGGLE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		if (MouseLeftClicked()) {
-			toggle_game_config_screen_visibility(GAME_CONFIG_SCREEN_VISIBLE_CHARACTER);
-		}
-		return TRUE;
-	}
-
-	if (MouseCursorIsOnButton(SKI_SCREEN_TOGGLE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		if (MouseLeftClicked()) {
-			toggle_game_config_screen_visibility(GAME_CONFIG_SCREEN_VISIBLE_SKILLS);
-		}
-		return TRUE;
-	}
-
-	if (MouseCursorIsOnButton(LOG_SCREEN_TOGGLE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		if (MouseLeftClicked()) {
-			quest_browser_interface();
-		}
-		return TRUE;
-	}
-
-	if (MouseCursorIsOnButton(WEAPON_MODE_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		if (MouseLeftClicked()) {
-			TuxReloadWeapon();
-		}
-		return TRUE;
-	}
-
-	if (MouseCursorIsOnButton(SKI_ICON_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-		if (MouseLeftClicked()) {
-			toggle_game_config_screen_visibility(GAME_CONFIG_SCREEN_VISIBLE_SKILLS);
-		}
-		return TRUE;
-	}
-	// Finally, protect all the other parts of the hud from user's clicks
-	if (GetMousePos_y() >= UNIVERSAL_COORD_H(MESSAGE_TEXT_WIDGET_Y))
-		return TRUE;
-
-	// The mouse is outside the hud.
-	return FALSE;
-}
-
 /**
  * At some point in the analysis of the users mouse click, we'll be 
  * certain, that a fireing/weapon swing was meant with the click.  Once
@@ -1290,7 +1309,7 @@ int PerformTuxAttackRaw(int use_mouse_cursor_for_targeting)
 	//
 	Me.weapon_swing_time = 0;
 
-	// Now that an attack is being made, the Tux must turn thowards the direction
+	// Now that an attack is being made, the Tux must turn towards the direction
 	// of the attack, no matter what.
 	//
 	// Note:  This is just some default value.  The real fine-tuning of the direction
@@ -1362,7 +1381,7 @@ int PerformTuxAttackRaw(int use_mouse_cursor_for_targeting)
 		// Since a melee weapon is swung, which may be only influencers fists,
 		// we calculate where the point
 		// of the weapon should be finally hitting and do some damage
-		// to all the enemys in that area.
+		// to all the enemies in that area.
 		//
 		// However, simply using the pixel-to-map-location code with the current
 		// mouse pointer is not the way, since the droid (e.g. 302) might be high
@@ -1393,7 +1412,7 @@ int PerformTuxAttackRaw(int use_mouse_cursor_for_targeting)
 			melee_shot *NewShot = &(AllMeleeShots[shot_index]);
 
 			NewShot->attack_target_type = ATTACK_TARGET_IS_ENEMY;
-			NewShot->mine = 1;
+			NewShot->mine = TRUE;
 
 			NewShot->bot_target_n = erot->id;
 			NewShot->bot_target_addr = erot;
@@ -1401,6 +1420,7 @@ int PerformTuxAttackRaw(int use_mouse_cursor_for_targeting)
 			NewShot->to_hit = Me.to_hit;
 			NewShot->damage = Me.base_damage + MyRandom(Me.damage_modifier);
 			NewShot->owner = -1;	//no "bot class number" owner
+			NewShot->time_to_hit = tux_anim.attack.duration / 2;
 
 			melee_weapon_hit_something = TRUE;
 
@@ -1412,7 +1432,7 @@ int PerformTuxAttackRaw(int use_mouse_cursor_for_targeting)
 
 		// Also, we should check if there was perhaps a chest or box
 		// or something that can be smashed up, cause in this case, we
-		// must open pendoras box now.
+		// must open Pandora's box now.
 		//
 		// SmashBox ( Weapon_Target_Vector.x , Weapon_Target_Vector.y );
 		if (smash_obstacle(Weapon_Target_Vector.x, Weapon_Target_Vector.y, Me.pos.z))
@@ -1437,15 +1457,15 @@ int PerformTuxAttackRaw(int use_mouse_cursor_for_targeting)
 		if (melee_weapon_hit_something)
 			play_melee_weapon_hit_something_sound();
 		else
-			play_melee_weapon_missed_sound();
+			play_melee_weapon_missed_sound(&Me.pos);
 
 		return 0;
 	}
 	//
 	if (Me.weapon_item.type != (-1))
-		Fire_Bullet_Sound(guntype);
+		fire_bullet_sound(guntype, &Me.pos);
 	else
-		Fire_Bullet_Sound(LASER_SWORD_1);
+		fire_bullet_sound(GetBulletByName("laser_sword"), &Me.pos);
 
 	FireTuxRangedWeaponRaw(Me.weapon_item.type, guntype, NULL, target_location);
 	return 0;
@@ -1503,7 +1523,14 @@ void check_for_droids_to_attack_or_talk_with()
 	 * of the found droid to be the bot's position relatively to Tux current level
 	 */
 	enemy *droid_below_mouse_cursor = GetLivingDroidBelowMouseCursor();
-	if (droid_below_mouse_cursor == NULL && (!APressed())) {
+
+	// Set a move action unless there's a droid below the cursor,
+	// 'A' is pressed,
+	// or the player has clicked to pickup an item and hasn't released the LMB
+	if (droid_below_mouse_cursor == NULL && (!APressed()) &&
+		(no_left_button_press_in_previous_analyze_mouse_click ||
+		Me.mouse_move_target_combo_action_type != COMBO_ACTION_PICK_UP_ITEM)) {
+
 		Me.mouse_move_target.x = translate_pixel_to_map_location(input_axis.x, input_axis.y, TRUE);
 		Me.mouse_move_target.y = translate_pixel_to_map_location(input_axis.x, input_axis.y, FALSE);
 		Me.mouse_move_target.z = Me.pos.z;
@@ -1570,14 +1597,6 @@ static void AnalyzePlayersMouseClick()
 {
 	int tmp;
 
-	/* Handle the main message log. */
-	if (MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y())) {
-		if (widget_handle_mouse(&message_log))
-			return;
-	} else {
-		global_ingame_mode = GLOBAL_INGAME_MODE_NORMAL;
-	}
-
 	// This flag avoids the mouse_move_target to change while the user presses
 	// LMB to start a combo action.
 	static int wait_mouseleft_release = FALSE;
@@ -1589,56 +1608,218 @@ static void AnalyzePlayersMouseClick()
 		wait_mouseleft_release = FALSE;
 		return;
 	}
-	// The action associated to MouseLeftPress depends on the global game state
-	//
-	switch (global_ingame_mode) {
-	case GLOBAL_INGAME_MODE_NORMAL:
-		if (ButtonPressWasNotMeantAsFire())
-			return;
-		if (handle_click_in_hud())
-			return;
-		if (no_left_button_press_in_previous_analyze_mouse_click) {
-			level *obj_lvl = NULL;
 
-			Me.mouse_move_target_combo_action_type = NO_COMBO_ACTION_SET;
+	if (ButtonPressWasNotMeantAsFire())
+		return;
+	if (no_left_button_press_in_previous_analyze_mouse_click) {
+		level *obj_lvl = NULL;
 
-			if ((tmp = clickable_obstacle_below_mouse_cursor(&obj_lvl)) != -1) {
-				obstacle_map[obj_lvl->obstacle_list[tmp].type].action(obj_lvl, tmp);
-				if (Me.mouse_move_target_combo_action_type != NO_COMBO_ACTION_SET)
+		Me.mouse_move_target_combo_action_type = NO_COMBO_ACTION_SET;
+
+		if ((tmp = clickable_obstacle_below_mouse_cursor(&obj_lvl)) != -1) {
+			get_obstacle_spec(obj_lvl->obstacle_list[tmp].type)->action_fn(obj_lvl, tmp);
+			if (Me.mouse_move_target_combo_action_type != NO_COMBO_ACTION_SET)
+				wait_mouseleft_release = TRUE;
+			return;
+		}
+
+		// If the inventory screen is open, let it manage any possibly picked item.
+		// Else, if the player left-clicked on an item, check if the item can be
+		// picked up. If so, get it and give it to the player.
+		// Note: if the item is too far away from Tux, check_for_items_to_pickup()
+		// creates a combo action to reach the item.
+		if (GameConfig.Inventory_Visible == FALSE) {
+			if ((tmp = get_floor_item_index_under_mouse_cursor(&obj_lvl)) != -1) {
+				if (check_for_items_to_pickup(obj_lvl, tmp)) {
+					// The item can be picked up immediately , so give it to the player
+					give_item(&obj_lvl->ItemList[tmp]);
 					wait_mouseleft_release = TRUE;
-			    break;
-			}
-
-			// If the inventory screen is open, let it manage any possibly picked item.
-			// Else, if the player left-clicked on an item, check if the item can be
-			// picked up. If so, get it and give it to the player.
-			// Note: if the item is too far away from Tux, check_for_items_to_pickup()
-			// creates a combo action to reach the item.
-			if (GameConfig.Inventory_Visible == FALSE) {
-				if ((tmp = get_floor_item_index_under_mouse_cursor(&obj_lvl)) != -1) {
-					if (check_for_items_to_pickup(obj_lvl, tmp)) {
-						// The item can be picked up immediately , so give it to the player
-						give_item(&obj_lvl->ItemList[tmp]);
-						wait_mouseleft_release = TRUE;
-					}
-					break;
 				}
+				return;
 			}
 		}
-		// Just after the beginning of a combo action, and while LMB is
-		// always pressed, mouse_move_target must not be changed (so that
-		// the player's character will actually move to the combo action's target)
-
-		if (!wait_mouseleft_release)
-			check_for_droids_to_attack_or_talk_with();
-
-		break;
-
-	default:
-		DebugPrintf(-4, "\n%s(): global_ingame_mode: %d.", __FUNCTION__, global_ingame_mode);
-		ErrorMessage(__FUNCTION__, "Illegal global ingame mode encountered!", PLEASE_INFORM, IS_FATAL);
-		break;
 	}
+	// Just after the beginning of a combo action, and while LMB is
+	// always pressed, mouse_move_target must not be changed (so that
+	// the player's character will actually move to the combo action's target)
+
+	if (!wait_mouseleft_release)
+		check_for_droids_to_attack_or_talk_with();
 }
 
+
+/**
+ *  Handles arrow key based movements
+ */
+void set_movement_with_keys(int move_x, int move_y)
+{
+	float floor_center = 0.5;
+	int move_amplitude = 1 + GameConfig.autorun_activated;
+
+	// Center the target waypoint, or establish a new target waypoint if there isn't one already
+	if (Me.mouse_move_target.z != -1) {
+		Me.mouse_move_target.x = floor(Me.mouse_move_target.x) + floor_center;
+		Me.mouse_move_target.y = floor(Me.mouse_move_target.y) + floor_center;
+	} else {
+		Me.mouse_move_target.z = Me.pos.z;
+		Me.mouse_move_target.x = floor(Me.pos.x) + floor_center;
+		Me.mouse_move_target.y = floor(Me.pos.y) + floor_center;
+	}
+
+	// Determine move amount
+	Me.mouse_move_target.x += move_x * move_amplitude;
+	Me.mouse_move_target.y += move_y * move_amplitude;
+}
+
+static void free_tux()
+{
+	int i;
+
+	free(Me.character_name);
+	Me.character_name = NULL;
+	free(Me.savegame_version_string);
+	Me.savegame_version_string = NULL;
+
+	clear_tux_mission_info();
+
+	// We mark all the big screen messages for this character
+	// as out of date, so they can be overwritten with new 
+	// messages...
+	//
+	Me.BigScreenMessageIndex = 0;
+	for (i = 0; i < MAX_BIG_SCREEN_MESSAGES; i++) {
+		if (Me.BigScreenMessage[i]) {
+			free(Me.BigScreenMessage[i]);
+			Me.BigScreenMessage[i] = NULL;
+		}
+	}
+
+}
+
+/**
+ * Reset the data in struct tux, in order to start a new game/load a game.
+ */
+void init_tux()
+{
+	int i;
+
+	free_tux();
+
+	memset(&Me, 0, sizeof(struct tux));
+
+	Me.current_game_date = 0.0;
+	Me.power_bonus_end_date = -1;	// negative dates are always in the past...
+	Me.dexterity_bonus_end_date = -1;
+
+	Me.speed.x = 0;
+	Me.speed.y = 0;
+
+	Me.pos.x = 0;
+	Me.pos.y = 0;
+	Me.pos.z = -1;
+	
+	Me.mouse_move_target.x = -1;
+	Me.mouse_move_target.y = -1;
+	Me.mouse_move_target.z = -1;
+	
+	Me.teleport_anchor.x = 0;
+	Me.teleport_anchor.y = 0;
+	Me.teleport_anchor.z = 0;
+	
+	enemy_set_reference(&Me.current_enemy_target_n, &Me.current_enemy_target_addr, NULL);
+	
+	Me.god_mode = FALSE;
+	
+	Me.mouse_move_target_combo_action_type = NO_COMBO_ACTION_SET;
+	Me.mouse_move_target_combo_action_parameter = -1;
+	
+	Me.map_maker_is_present = FALSE;
+
+	Me.temperature = 0.0;
+
+	Me.health_recovery_rate = 0.2;
+	Me.cooling_rate = 0.2;
+	
+	Me.busy_time = 0;
+	Me.busy_type = NONE;
+
+	Me.phase = 0;
+	Me.angle = 180;
+	Me.walk_cycle_phase = 0;
+	Me.weapon_swing_time = -1;
+	Me.MissionTimeElapsed = 0;
+	Me.got_hit_time = -1;
+
+	Me.points_to_distribute = 0;
+	
+	// reset statistics	
+	Me.meters_traveled = 0;
+	for (i = 0; i < Number_Of_Droid_Types + 1; i++) {
+		Me.destroyed_bots[i]    = 0;
+		Me.damage_dealt[i]      = 0;
+		Me.TakeoverSuccesses[i] = 0;
+		Me.TakeoverFailures[i]  = 0;
+	}
+
+	for (i = 0; i < MAX_LEVELS; i++) {
+		Me.HaveBeenToLevel[i] = FALSE;
+		Me.time_since_last_visit_or_respawn[i] = -1;
+	}
+
+	Me.Experience = 0;
+	Me.exp_level = 0;
+	Me.Gold = 0;
+	
+	Me.readied_skill = 0;
+	for (i = 0; i < number_of_skills; i++) {
+		Me.skill_level[i] = SpellSkillMap[i].present_at_startup;
+	}
+
+	GameConfig.spell_level_visible = 0;
+
+	Me.melee_weapon_skill = 0;
+	Me.ranged_weapon_skill = 0;
+	Me.spellcasting_skill = 0;
+
+	Me.running_power_bonus = 0;
+
+	for (i = 0; i < 10; i++) {
+		Me.program_shortcuts[i] = -1;
+	}
+
+	Me.paralyze_duration = 0;
+	Me.invisible_duration = 0;
+	Me.nmap_duration = 0;
+
+	Me.readied_skill = 0;
+
+	Me.quest_browser_changed = 0;
+
+	for (i = 0; i < MAX_ITEMS_IN_INVENTORY; i++) {
+		init_item(&Me.Inventory[i]);
+	}
+	init_item(&Me.weapon_item);
+	init_item(&Me.armour_item);
+	init_item(&Me.shield_item);
+	init_item(&Me.special_item);
+	init_item(&Me.drive_item);
+	item_held_in_hand = NULL;
+
+	clear_out_intermediate_points(&Me.pos, Me.next_intermediate_point, MAX_INTERMEDIATE_WAYPOINTS_FOR_TUX);
+	Me.next_intermediate_point[0].x = -1;
+	Me.next_intermediate_point[0].y = -1;
+	
+	Me.TextToBeDisplayed = "";
+	Me.TextVisibleTime = 0;
+
+	Me.base_physique = 20;
+	Me.base_strength = 10;
+	Me.base_dexterity = 15;
+	Me.base_cooling = 25;
+	
+	UpdateAllCharacterStats();
+	
+	Me.energy = Me.maxenergy;
+	Me.running_power = Me.max_running_power;
+}
 #undef _influ_c
