@@ -24,6 +24,7 @@
 #include "struct.h"
 #include "global.h"
 #include "proto.h"
+#include "widgets/widgets.h"
 
 #define ADDON_ITEMS_MAX 4
 
@@ -62,7 +63,7 @@ struct upgrade_ui_rects {
 };
 
 // Global variables.
-static iso_image images[IMAGE_MAX];
+static struct image images[IMAGE_MAX];
 static int ui_visible = FALSE;
 static int images_loaded = FALSE;
 static struct upgrade_ui ui;
@@ -94,7 +95,6 @@ static const struct upgrade_ui_rects rects = {
 static void load_images()
 {
 	int i;
-	char fpath[2048];
 	const char *fnames[IMAGE_MAX] = {
 		"item_upgrade/socket_none.png",
 		"item_upgrade/socket_mechanical.png",
@@ -105,19 +105,26 @@ static void load_images()
 	};
 
 	for (i = 0; i < IMAGE_MAX; i++) {
-		find_file(fnames[i], GRAPHICS_DIR, fpath, FALSE);
-		memset(&images[i], 0, sizeof(iso_image));
-		get_iso_image_from_file_and_path(fpath, &images[i], FALSE);
+		memset(&images[i], 0, sizeof(struct image));
+		load_image(&images[i], fnames[i], FALSE);
 	}
 }
 
+/**
+ * Returns the item upgrade interface's status.
+ */
+int item_upgrade_ui_visible()
+{
+	return ui_visible;
+}
+
 /**
  * \brief Gets the tooltip text the item upgrade UI would like to display.
  * \param cursor Cursor position.
  * \param result Return location for the tooltip text.
  * \return TRUE if the tooltip was set, FALSE if the cursor isn't on the upgrade UI.
  */
-int get_item_upgrade_ui_tooltip(const point *cursor, char *result)
+int append_item_upgrade_ui_tooltip(const point *cursor, struct auto_string *str)
 {
 	int i;
 
@@ -131,7 +138,7 @@ int get_item_upgrade_ui_tooltip(const point *cursor, char *result)
 		for (i = 0; i < ADDON_ITEMS_MAX; i++) {
 			if (MouseCursorIsInRect(&rects.socket_slots[i], cursor->x, cursor->y)) {
 				if (ui.addon_items[i].type != -1) {
-					give_item_description(result, &ui.addon_items[i]);
+					append_item_description(str, &ui.addon_items[i]);
 					return TRUE;
 				}
 			}
@@ -141,7 +148,7 @@ int get_item_upgrade_ui_tooltip(const point *cursor, char *result)
 	// Check if a tooltip for the customized item needs to be displayed.
 	if (MouseCursorIsInRect(&rects.custom_slot, cursor->x, cursor->y)) {
 		if (ui.custom_item.type != -1) {
-			give_item_description(result, &ui.custom_item);
+			append_item_description(str, &ui.custom_item);
 			return TRUE;
 		}
 	}
@@ -150,7 +157,6 @@ int get_item_upgrade_ui_tooltip(const point *cursor, char *result)
 	// caller that no tooltip should be shown. The tooltips of NPCs and other
 	// such things behind the upgrade UI would be displayed without this.
 	if (MouseCursorIsInRect(&rects.main, cursor->x, cursor->y)) {
-		strcpy(result, "");
 		return TRUE;
 	}
 
@@ -171,10 +177,10 @@ static void show_socket_creation_menu()
 
 	// Draw the menu title and the column headers.
 	SetCurrentFont(Blue_BFont);
-	DisplayText(_("Add socket"), rects.prompt.x, rects.prompt.y, NULL, TEXT_STRETCH);
+	display_text(_("Add socket"), rects.prompt.x, rects.prompt.y, NULL);
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Type"), rects.table[0][0].x, rects.table[0][0].y, NULL, TEXT_STRETCH);
-	DisplayText(_("Cost"), rects.table[1][0].x, rects.table[1][0].y, NULL, TEXT_STRETCH);
+	display_text(_("Type"), rects.table[0][0].x, rects.table[0][0].y, NULL);
+	display_text(_("Cost"), rects.table[1][0].x, rects.table[1][0].y, NULL);
 	SetCurrentFont(Messagevar_BFont);
 
 	// Draw the selectable rows whose contents were given as parameters.
@@ -185,12 +191,12 @@ static void show_socket_creation_menu()
 		}
 
 		// Draw the name column.
-		DisplayText(names[i], rects.table[0][i + 1].x, rects.table[0][i + 1].y, NULL, TEXT_STRETCH);
+		display_text(names[i], rects.table[0][i + 1].x, rects.table[0][i + 1].y, NULL);
 
 		// Draw the cost column if the row has a cost assigned to it.
 		if (costs[i] != -1) {
 			sprintf(buffer, "%d", costs[i]);
-			DisplayText(buffer, rects.table[1][i + 1].x, rects.table[1][i + 1].y, NULL, TEXT_STRETCH);
+			display_text(buffer, rects.table[1][i + 1].x, rects.table[1][i + 1].y, NULL);
 		}
 	}
 }
@@ -215,9 +221,8 @@ void show_item_upgrade_ui()
 {
 	int i;
 	char buffer[512];
-	iso_image *image;
+	struct image *image;
 	SDL_Rect rect;
-	SDL_Surface *surface;
 	struct upgrade_socket_dynarray *sockets = &ui.custom_item.upgrade_sockets;
 
 	// We're being called every time the UI of the game is drawn so we need to
@@ -227,7 +232,7 @@ void show_item_upgrade_ui()
 	}
 
 	// Draw the background image.
-	blit_special_background(ITEM_UPGRADE_BACKGROUND_CODE);
+	blit_background("item_upgrade.png");
 
 	// Draw the apply and close buttons.
 	if (ui.apply_button_active) {
@@ -242,32 +247,32 @@ void show_item_upgrade_ui()
 		ShowItemPicture(rects.custom_slot.x, rects.custom_slot.y, ui.custom_item.type);
 	} else {
 		SetCurrentFont(Messagestat_BFont);
-		DisplayText(_("Place an item here"), rects.custom_slot.x,
-		            rects.custom_slot.y + rects.custom_slot.h / 2, NULL, TEXT_STRETCH);
+		display_text(_("Place an item here"), rects.custom_slot.x,
+		            rects.custom_slot.y + rects.custom_slot.h / 2, NULL);
 	}
 
 	// Draw the title.
 	SetCurrentFont(Menu_BFont);
-	DisplayText(_("Upgrade Items"), rects.title_text.x, rects.title_text.y, NULL, TEXT_STRETCH);
+	display_text(_("Upgrade Items"), rects.title_text.x, rects.title_text.y, NULL);
 
 	// Draw the text labels.
 	SetCurrentFont(Messagevar_BFont);
-	sprintf(buffer, _("Money: %d"), Me.Gold);
-	DisplayText(buffer, rects.money_text.x, rects.money_text.y, NULL, TEXT_STRETCH);
+	sprintf(buffer, _("Circuits: %d"), Me.Gold);
+	display_text(buffer, rects.money_text.x, rects.money_text.y, NULL);
 	if (ui.cost > Me.Gold) {
 		SetCurrentFont(Messagestat_BFont);
 	} else {
 		SetCurrentFont(Messagevar_BFont);
 	}
 	sprintf(buffer, _("Cost: %d"), ui.cost);
-	DisplayText(buffer, rects.cost_text.x, rects.cost_text.y, NULL, TEXT_STRETCH);
+	display_text(buffer, rects.cost_text.x, rects.cost_text.y, NULL);
 	SetCurrentFont(Blue_BFont);
-	DisplayText(_("Bonuses"), rects.bonus_text.x, rects.bonus_text.y, NULL, TEXT_STRETCH);
+	display_text(_("Bonuses"), rects.bonus_text.x, rects.bonus_text.y, NULL);
 
 	// Draw the item bonus string.
 	SetCurrentFont(Messagevar_BFont);
-	DisplayText(ui.bonus_text->value, rects.bonus_list.x, rects.bonus_list.y,
-	            &rects.bonus_list, TEXT_STRETCH);
+	display_text(ui.bonus_text->value, rects.bonus_list.x, rects.bonus_list.y,
+	            &rects.bonus_list);
 
 	if (ui.create_socket_active) {
 
@@ -298,28 +303,33 @@ void show_item_upgrade_ui()
 			} else {
 				image = &images[IMAGE_SOCKET_NONE];
 			}
+
+			if (!image)
+				continue;
+
 			rect = rects.socket_slots[i];
-			rect.x += (rect.w - image->original_image_width) / 2;
-			rect.y += (rect.h - image->original_image_height) / 2;
-			blit_iso_image_to_screen_position(image, rect.x, rect.y);
+			rect.x += (rect.w - image->w) / 2;
+			rect.y += (rect.h - image->h) / 2;
+			display_image_on_screen(image, rect.x, rect.y, IMAGE_NO_TRANSFO);
 			if (i < sockets->size && ui.custom_item.upgrade_sockets.arr[i].addon) {
 				image = &images[IMAGE_SOCKET_INSTALLED];
 				rect = rects.socket_slots[i];
-				rect.x += (rect.w - image->original_image_width) / 2;
-				rect.y += (rect.h - image->original_image_height) / 2;
-				blit_iso_image_to_screen_position(image, rect.x, rect.y);
+				rect.x += (rect.w - image->w) / 2;
+				rect.y += (rect.h - image->h) / 2;
+				display_image_on_screen(image, rect.x, rect.y, IMAGE_NO_TRANSFO);
 			}
 		}
 
 		// Draw the items in the sockets.
 		for (i = 0; i < ADDON_ITEMS_MAX; i++) {
 			if (ui.addon_items[i].type != -1) {
-				surface = ItemMap[ui.addon_items[i].type].inv_image.Surface;
-				if (surface) {
+				struct image *img = get_item_inventory_image(ui.addon_items[i].type);
+
+				if (img) {
 					rect = rects.socket_slots[i];
-					rect.x += (rect.w - surface->w) / 2;
-					rect.y += (rect.h - surface->h) / 2;
-					our_SDL_blit_surface_wrapper(surface, NULL, Screen, &rect);
+					rect.x += (rect.w - img->w) / 2;
+					rect.y += (rect.h - img->h) / 2;
+					display_image_on_screen(img, rect.x, rect.y, IMAGE_NO_TRANSFO);
 				}
 			}
 		}
@@ -327,7 +337,7 @@ void show_item_upgrade_ui()
 	}
 }
 
-static void clear_addon_items()
+static void clear_addon_items(void)
 {
 	int i;
 	struct upgrade_socket *socket;
@@ -348,7 +358,7 @@ static void clear_addon_items()
 	}
 }
 
-static void calculate_cost_and_bonuses()
+static void calculate_cost_and_bonuses(void)
 {
 	int i;
 	int changed = FALSE;
@@ -382,7 +392,7 @@ static void calculate_cost_and_bonuses()
 
 	// Rebuild the item bonuses using a temporary item that has all the
 	// yet to be installed add-ons installed.
-	CopyItem(&ui.custom_item, &temp, FALSE);
+	CopyItem(&ui.custom_item, &temp);
 	for (i = 0; i < ADDON_ITEMS_MAX; i++) {
 		if (i < temp.upgrade_sockets.size) {
 			addon = &ui.addon_items[i];
@@ -399,7 +409,7 @@ static void calculate_cost_and_bonuses()
 	DeleteItem(&temp);
 }
 
-static int grab_customized_item()
+static int grab_customized_item(void)
 {
 	// If the slot is empty, do nothing and return FALSE to inform
 	// the caller that no drag could be initiated.
@@ -410,17 +420,17 @@ static int grab_customized_item()
 	// If the player dragged some add-ons to the sockets but didn't
 	// press the apply button, return the add-on items to the
 	// inventory so that the player doesn't lose them.
-	clear_addon_items(ui);
+	clear_addon_items();
 
 	// Mark the item as being dragged. We move it to a temporary
 	// field so that it doesn't get cleared when we alter the UI. 
 	MoveItem(&ui.custom_item, &ui.dragged_item);
 	item_held_in_hand = &ui.dragged_item;
-	play_item_sound(item_held_in_hand->type);
+	play_item_sound(item_held_in_hand->type, &Me.pos);
 
 	// Reset the cost field to zero and disable the apply button
 	// since there's no item to customize.
-	calculate_cost_and_bonuses(ui);
+	calculate_cost_and_bonuses();
 
 	// Close the socket creation prompt if it was open. We don't
 	// want it to be visible when there's no item to customize.
@@ -443,7 +453,7 @@ static int grab_addon_item(int index)
 	// field so that it doesn't get cleared when we alter the UI. 
 	MoveItem(&ui.addon_items[index], &ui.dragged_item);
 	item_held_in_hand = &ui.dragged_item;
-	play_item_sound(item_held_in_hand->type);
+	play_item_sound(item_held_in_hand->type, &Me.pos);
 
 	// Remove the corresponding string from the upgrade socket array. We
 	// use the NULL pointer to identify sockets that have been modified
@@ -458,23 +468,23 @@ static int grab_addon_item(int index)
 
 	// Removing a yet to be installed item reduces the cost and modifies
 	// bonuses so we need to recalculate them.
-	calculate_cost_and_bonuses(ui);
+	calculate_cost_and_bonuses();
 
 	return TRUE;
 }
 
-static void clear_customized_item()
+static void clear_customized_item(void)
 {
 	// If an item was being customized, drop it and any add-ons not yet
 	// installed to it to the inventory so that the player doesn't lose them.
 	if (ui.custom_item.type != -1) {
-		clear_addon_items(ui);
+		clear_addon_items();
 		give_item(&ui.custom_item);
 	}
 
 	// Reset the cost to zero and disable the apply button since
 	// there's no item to customize present in the interface anymore.
-	calculate_cost_and_bonuses(ui);
+	calculate_cost_and_bonuses();
 }
 
 static int set_customized_item(item *it)
@@ -489,10 +499,10 @@ static int set_customized_item(item *it)
 
 	// If there are any items in the upgrade UI, drop them
 	// to the inventory so that they aren't overwritten and lost.
-	clear_customized_item(ui);
+	clear_customized_item();
 
 	// Move the item to the customization slot.
-	play_item_sound(it->type);
+	play_item_sound(it->type, &Me.pos);
 	MoveItem(it, &ui.custom_item);
 
 	// Create add-on items from the strings in the upgrade socket array
@@ -505,7 +515,7 @@ static int set_customized_item(item *it)
 	}
 
 	// Update the bonus string since the item might have existing bonuses.
-	calculate_cost_and_bonuses(ui);
+	calculate_cost_and_bonuses();
 
 	return TRUE;
 }
@@ -523,17 +533,17 @@ static int set_addon_item(item *it, int index)
 	// Move the item to the socket slot. The upgrade socket array of the 
 	// customized item remains unchanged; the player needs to press the
 	// apply button before any modifications to the item take place.
-	play_item_sound(it->type);
+	play_item_sound(it->type, &Me.pos);
 	MoveItem(it, &ui.addon_items[index]);
 
 	// Recalculate the cost and the bonus string and enable the apply button
 	// if the player has enough gold to pay for the upgrade.
-	calculate_cost_and_bonuses(ui);
+	calculate_cost_and_bonuses();
 
 	return TRUE;
 }
 
-static void apply_customization()
+static void apply_customization(void)
 {
 	int i;
 	item *addon;
@@ -561,7 +571,7 @@ static void apply_customization()
 
 	// Reset the cost to zero and disable the apply button since we
 	// just did all the changes the player has requested.
-	calculate_cost_and_bonuses(ui);
+	calculate_cost_and_bonuses();
 }
 
 static void handle_buy_socket(point *cursor, int row, int type, int cost)
@@ -651,7 +661,7 @@ static int handle_ui()
 	if (MouseCursorIsInRect(&rects.custom_slot, cursor.x, cursor.y))
 	{
 		if (!item_held_in_hand) {
-			grab_customized_item(ui);
+			grab_customized_item();
 		} else if (set_customized_item(item_held_in_hand)) {
 			item_held_in_hand = NULL;
 		}
@@ -662,7 +672,7 @@ static int handle_ui()
 	if (MouseCursorIsOnButton(ITEM_UPGRADE_APPLY_BUTTON, cursor.x, cursor.y))
 	{
 		if (ui.apply_button_active) {
-			apply_customization(ui);
+			apply_customization();
 		}
 		return TRUE;
 	}
@@ -690,7 +700,7 @@ void item_upgrade_ui()
 {
 	int i;
 	int old_game_status = game_status;
-	game_status = INSIDE_MENU;
+	game_status = INSIDE_GAME;
 
 	// Load all static images if not already loaded.
 	if (!images_loaded) {
@@ -708,38 +718,38 @@ void item_upgrade_ui()
 	}
 
 	// Setup the correct screen state.
-	make_sure_system_mouse_cursor_is_turned_off();
 	GameConfig.Inventory_Visible = TRUE;
 	GameConfig.CharacterScreen_Visible = FALSE;
 	GameConfig.SkillScreen_Visible = FALSE;
 	GameConfig.skill_explanation_screen_visible = FALSE;
 	ui_visible = TRUE;
 
+	// Prevent keyboard input.
+	input_hold_keyboard();
+
 	// Loop until the player clicks the close button of the UI or presses and
 	// releases escape. We need to ensure that the escape key is released so
 	// that it doesn't interfere with the dialog that opened the upgrade UI.
 	while (!ui.quit || EscapePressed()) {
-		StartTakingTimeForFPSCalculation();
-
 		// Handle input. Since the UI makes use of the inventory, we need to let
 		// the inventory handle events that fall outside of the upgrade UI.
 		save_mouse_state();
+		update_widgets();
 		input_handle();
 		if (!handle_ui())
 			HandleInventoryScreen();
 
 		// Draw the UI. AssembleCombatPicture will take care of calling our
 		// drawing function in the right place.
-		AssembleCombatPicture(DO_SCREEN_UPDATE | SHOW_ITEMS | USE_OWN_MOUSE_CURSOR);
-
-		// Limit framerate if configured to do so.
-		limit_fps();
-		ComputeFPSForThisFrame();
+		AssembleCombatPicture(DO_SCREEN_UPDATE | SHOW_ITEMS);
 	}
 
+	// Release keyboard input.
+	input_release_keyboard();
+
 	// If the player left any items to the upgrade interface,
 	// drop them to the inventory or to the ground.
-	clear_customized_item(ui);
+	clear_customized_item();
 	if (ui.dragged_item.type != -1) {
 		item_held_in_hand = NULL;
 		give_item(&ui.dragged_item);
