@@ -16,7 +16,7 @@
  */
 
 /**
- * @file input.c
+ * @file keyboard.c
  *
  * @brief Handles all the keybindings and input.
  */
@@ -30,21 +30,26 @@
 
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_actions.h"
-#include "lvledit/lvledit_grass_actions.h"
+#include "lvledit/lvledit_beautify_actions.h"
 #include "lvledit/lvledit_tools.h"
 #include "lvledit/lvledit_widgets.h"
+#include "lvledit/lvledit_validator.h"
 
 #define KEY_PRESS    ( 1.) /**< Key is pressed. */
 #define KEY_RELEASE  (-1.) /**< Key is released. */
+#define ISKEYPAD(k) (k >= SDLK_KP0 && k <= SDLK_KP_EQUALS)
+
+static int input_keyboard_locked; // Block the execution of the keybindings.
 
 /* name of each keybinding */
 const char *keybindNames[] = {
 	/* General */
-	"keychart", "fullscreen", "quit", "wall_transparency", "grab_input",
+	"keychart", "fullscreen", "quit", "wall_transparency", "grab_input", "take_screenshot",
 
 	/* Ingame */
 	"inventory", "skill", "character", "quests",
 	"reload", "autorun",
+	"move_north", "move_south", "move_east", "move_west",
 	"quicksave", "quickload", "pause",
 	"automap",
 	"show_item_labels",
@@ -60,12 +65,14 @@ const char *keybindNames[] = {
 	"place_obstacle_kp1", "place_obstacle_kp2", "place_obstacle_kp3",
 	"place_obstacle_kp4", "place_obstacle_kp5", "place_obstacle_kp6",
 	"place_obstacle_kp7", "place_obstacle_kp8", "place_obstacle_kp9",
-	"change_map_label", "zoom_out",
-	"cycle_marked_obstacle",
+	"change_obstacle_label", "change_map_label", "zoom_out",
+	"cycle_marked_object",
 	"cut", "copy", "paste",
-	"next_tab", "undo", "redo", "beautify_grass",
-	"connect_waypoint",
+	"next_selection_type", "previous_selection_type",
+	"next_tab", "previous_tab", "undo", "redo", "beautify_grass", "beautify_water",
+	"toggle_waypoint", "toggle_waypoint_randomspawn", "connect_waypoint",
 	"toolbar_scroll_left", "toolbar_scroll_right", "toolbar_step_left", "toolbar_step_right",
+	"run_map_validator",
 
 	/* Cheat keys */
 	"cheat_xp+_1k", "cheat_xp*_2",
@@ -73,34 +80,38 @@ const char *keybindNames[] = {
 	"cheat_drop_random_item", "cheat_drop_random_magical_item",
 	"cheat_respawn_level",
 	"cheat_menu", "cheat_level_editor",
-	"end"
-};				/* must terminate in "end" */
+	"cheat_reload_graphics",
+	NULL
+};				/* must terminate with NULL */
 
 /**
- * @fn void input_init (void)
+ * @fn void input_keyboard_init (void)
  *
  * @brief Initializes the input subsystem (does not set keys).
  */
 void input_keyboard_init(void)
 {
 	int i;
-	for (i = 0; strcmp(keybindNames[i], "end"); i++) ;	/* gets number of bindings */
 
-	GameConfig.input_keybinds[0].name = "end";
+	GameConfig.input_keybinds[0].name = NULL;
 
-	if (i > sizeof(GameConfig.input_keybinds) / sizeof(GameConfig.input_keybinds[0]))
+	if (sizeof(keybindNames) / sizeof(keybindNames[0]) > sizeof(GameConfig.input_keybinds) / sizeof(GameConfig.input_keybinds[0]))
 		ErrorMessage(__FUNCTION__,
-			     "There are %d keyboard commands defined in keyboard.c, but GameConfig structure only supports %d.\n",
-			     PLEASE_INFORM, IS_FATAL, i, sizeof(GameConfig.input_keybinds) / sizeof(GameConfig.input_keybinds[0]));
+			     "There are %zu keyboard commands defined in keyboard.c, but GameConfig structure only supports %zu.\n",
+			     PLEASE_INFORM, IS_FATAL,
+			     sizeof(keybindNames) / sizeof(keybindNames[0]),
+			     sizeof(GameConfig.input_keybinds) / sizeof(GameConfig.input_keybinds[0]));
 
 	/* creates a null keybinding for each */
-	for (i = 0; strcmp(keybindNames[i], "end"); i++) {
+	for (i = 0; keybindNames[i] != NULL; i++) {
 		GameConfig.input_keybinds[i].name = (char *)keybindNames[i];
 		GameConfig.input_keybinds[i].key = SDLK_UNKNOWN;
 		GameConfig.input_keybinds[i].mod = KMOD_NONE;
 	}
 
-	GameConfig.input_keybinds[i].name = "end";
+	GameConfig.input_keybinds[i].name = NULL;
+
+	input_keyboard_locked = 0;
 }
 
 /**
@@ -123,7 +134,7 @@ void input_exit(void)
 void input_set_keybind(char *keybind, SDLKey key, SDLMod mod)
 {
 	int i;
-	for (i = 0; strcmp(keybindNames[i], "end"); i++)
+	for (i = 0; keybindNames[i] != NULL; i++)
 		if (strcmp(keybind, GameConfig.input_keybinds[i].name) == 0) {
 			GameConfig.input_keybinds[i].key = key;
 			GameConfig.input_keybinds[i].mod = mod;
@@ -133,10 +144,10 @@ void input_set_keybind(char *keybind, SDLKey key, SDLMod mod)
 		     keybind);
 }
 
-void input_get_keybind(char *cmdname, SDLKey * key, SDLMod * mod)
+void input_get_keybind(const char *cmdname, SDLKey * key, SDLMod * mod)
 {
 	int i;
-	for (i = 0; strcmp(keybindNames[i], "end"); i++) {
+	for (i = 0; keybindNames[i] != NULL; i++) {
 		if (!strcmp(cmdname, GameConfig.input_keybinds[i].name)) {
 			if (key)
 				*key = GameConfig.input_keybinds[i].key;
@@ -145,10 +156,41 @@ void input_get_keybind(char *cmdname, SDLKey * key, SDLMod * mod)
 			return;
 		}
 	}
-	ErrorMessage(__FUNCTION__, "Unable to get keybind for command '%s', that command does not exist.\n", PLEASE_INFORM, IS_FATAL,
+	ErrorMessage(__FUNCTION__, "Unable to get keybinding for command '%s', that command does not exist.\n", PLEASE_INFORM, IS_FATAL,
 		     cmdname);
+
+	/* This dead code writes "key" and "mod" to shut up GCC */
+	*key = GameConfig.input_keybinds[0].key;
+	*mod = GameConfig.input_keybinds[0].mod;
 }
 
+/**
+ * Print the key mapping of a given command into a string.
+ * @param cmd name of the command
+ * @param out pointer to a sufficiently large buffer to hold the key string
+ */
+void input_get_keybind_string(const char *cmd, char *out)
+{
+	SDLKey key;
+	SDLMod mod;
+
+	input_get_keybind(cmd, &key, &mod);
+
+	const char *ctrl_modstr = "";
+	const char *alt_modstr = "";
+	const char *shift_modstr = "";
+
+	if (mod & KMOD_LCTRL || mod & KMOD_RCTRL)
+		ctrl_modstr = "C-";
+	if (mod & KMOD_LALT || mod & KMOD_RALT)
+		alt_modstr = "A-";
+	if (mod & KMOD_LSHIFT || mod & KMOD_RSHIFT)
+		shift_modstr = "S-";
+
+	sprintf(out, "%s%s%s%s", ctrl_modstr, alt_modstr, shift_modstr, SDL_GetKeyName(key));
+}
+
+
 /**
  * @fn void input_setDefault (void)
  *
@@ -168,6 +210,7 @@ void input_set_default(void)
 	input_set_keybind("grab_input", SDLK_g, KMOD_LCTRL);
 	input_set_keybind("quit", SDLK_q, KMOD_LCTRL);
 	input_set_keybind("wall_transparency", SDLK_t, KMOD_NONE);
+	input_set_keybind("take_screenshot", SDLK_PRINT, KMOD_NONE);
 
 	/* Game */
 	input_set_keybind("inventory", SDLK_i, KMOD_NONE);
@@ -176,6 +219,12 @@ void input_set_default(void)
 	input_set_keybind("quests", SDLK_q, KMOD_NONE);
 	input_set_keybind("reload", SDLK_r, KMOD_NONE);
 	input_set_keybind("autorun", SDLK_u, KMOD_NONE);
+
+	input_set_keybind("move_north", SDLK_UP, KMOD_NONE);
+	input_set_keybind("move_south", SDLK_DOWN, KMOD_NONE);
+	input_set_keybind("move_east", SDLK_RIGHT, KMOD_NONE);
+	input_set_keybind("move_west", SDLK_LEFT, KMOD_NONE);
+
 	input_set_keybind("quicksave", SDLK_F3, KMOD_NONE);
 	input_set_keybind("quickload", SDLK_F4, KMOD_NONE);
 	input_set_keybind("pause", SDLK_p, KMOD_NONE);
@@ -217,13 +266,18 @@ void input_set_default(void)
 	input_set_keybind("place_obstacle_kp8", SDLK_KP8, KMOD_NONE);
 	input_set_keybind("place_obstacle_kp9", SDLK_KP9, KMOD_NONE);
 	input_set_keybind("place_obstacle_kp9", SDLK_KP9, KMOD_NONE);
+	input_set_keybind("change_obstacle_label", SDLK_h, KMOD_NONE);
 	input_set_keybind("change_map_label", SDLK_m, KMOD_NONE);
 	input_set_keybind("zoom_out", SDLK_o, KMOD_NONE);
-	input_set_keybind("cycle_marked_obstacle", SDLK_n, KMOD_NONE);
+	input_set_keybind("cycle_marked_object", SDLK_n, KMOD_NONE);
 	input_set_keybind("next_tab", SDLK_f, KMOD_NONE);
+	input_set_keybind("previous_tab", SDLK_f, KMOD_LSHIFT);
 	input_set_keybind("undo", SDLK_z, KMOD_NONE);
 	input_set_keybind("redo", SDLK_y, KMOD_NONE);
 	input_set_keybind("beautify_grass", SDLK_b, KMOD_LCTRL);
+	input_set_keybind("beautify_water", SDLK_w, KMOD_LCTRL);
+	input_set_keybind("toggle_waypoint", SDLK_w, KMOD_NONE);
+	input_set_keybind("toggle_waypoint_randomspawn", SDLK_w, KMOD_LSHIFT);
 	input_set_keybind("connect_waypoint", SDLK_c, KMOD_NONE);
 	input_set_keybind("toolbar_scroll_left", SDLK_PAGEUP, KMOD_NONE);
 	input_set_keybind("toolbar_scroll_right", SDLK_PAGEDOWN, KMOD_NONE);
@@ -232,6 +286,9 @@ void input_set_default(void)
 	input_set_keybind("cut", SDLK_x, KMOD_LCTRL);
 	input_set_keybind("copy", SDLK_c, KMOD_LCTRL);
 	input_set_keybind("paste", SDLK_v, KMOD_LCTRL);
+	input_set_keybind("next_selection_type", SDLK_TAB, KMOD_NONE);
+	input_set_keybind("previous_selection_type", SDLK_TAB, KMOD_LSHIFT);
+	input_set_keybind("run_map_validator", SDLK_e, KMOD_LCTRL);
 
 	/* Cheat */
 	input_set_keybind("cheat_xp+_1k", SDLK_KP1, KMOD_NONE);
@@ -247,12 +304,13 @@ void input_set_default(void)
 	input_set_keybind("cheat_respawn_level", SDLK_r, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT);
 	input_set_keybind("cheat_level_editor", SDLK_e, KMOD_LCTRL | KMOD_LALT);
 	input_set_keybind("cheat_menu", SDLK_c, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT);
+	input_set_keybind("cheat_reload_graphics", SDLK_g, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT);
 
 }
 
 #define KEYCHART_RECT_X (GameConfig.screen_width / 250)
 #define KEYCHART_RECT_Y (GameConfig.screen_height / 10)
-#define KEYCHART_RECT_W (0.8 * GameConfig.screen_width)
+#define KEYCHART_RECT_W (0.99 * GameConfig.screen_width)
 #define KEYCHART_RECT_H (0.8 * GameConfig.screen_height)
 
 /**
@@ -268,7 +326,7 @@ static int display_keychart(unsigned int startidx, unsigned int cursor, int high
 	int ypos = keychart_rect.y;
 	int xpos = keychart_rect.x;
 
-	blit_special_background(SHOP_BACKGROUND_IMAGE_CODE);
+	blit_background("shoppe.jpg");
 
 	ShadowingRectangle(Screen, keychart_rect);
 
@@ -277,24 +335,18 @@ static int display_keychart(unsigned int startidx, unsigned int cursor, int high
 	if (startidx >= sizeof(keybindNames) / sizeof(keybindNames[0]))
 		return -1;
 
-	ypos = keychart_rect.y;
-
-	for (i = startidx; strcmp(keybindNames[i], "end"); i++) {
-		char modstring[10] = "";
+	for (i = startidx; keybindNames[i] != NULL; i++) {
+		char keystr[100] = "";
 		const char *font_str = font_switchto_neon;
-		if (GameConfig.input_keybinds[i].mod & KMOD_LCTRL || GameConfig.input_keybinds[i].mod & KMOD_RCTRL)
-			strcat(modstring, "C-");
-		if (GameConfig.input_keybinds[i].mod & KMOD_LALT || GameConfig.input_keybinds[i].mod & KMOD_RALT)
-			strcat(modstring, "A-");
-		if (GameConfig.input_keybinds[i].mod & KMOD_LSHIFT || GameConfig.input_keybinds[i].mod & KMOD_RSHIFT)
-			strcat(modstring, "S-");
-
+		
 		if (i == cursor && highlight) {
 			font_str = font_switchto_red;
 		}
 
-		sprintf(txt, "%s%s%s - %s%s\n", font_str, (i == cursor) ? "** " : "   ",
-			GameConfig.input_keybinds[i].name, modstring, SDL_GetKeyName(GameConfig.input_keybinds[i].key));
+		input_get_keybind_string(keybindNames[i], &keystr[0]);
+
+		sprintf(txt, "%s%s%s - %s\n", font_str, (i == cursor) ? "** " : "   ",
+			GameConfig.input_keybinds[i].name, keystr);
 		PutString(Screen, xpos, ypos, txt);
 
 		ypos += FontHeight(GetCurrentFont());
@@ -357,7 +409,7 @@ void keychart()
 		while (SDL_PollEvent(&event)) {
 
 			if (event.type == SDL_QUIT) {
-				Terminate(0);
+				Terminate(EXIT_SUCCESS, TRUE);
 			}
 
 			endpos = display_keychart(startpos, cursor, FALSE);
@@ -414,7 +466,7 @@ void keychart()
 					newmod &= ~(KMOD_CAPS | KMOD_NUM | KMOD_MODE);	/* We want to ignore "global" modifiers. */
 					GameConfig.input_keybinds[cursor].mod = newmod;
 
-					for (i = 0; strcmp(keybindNames[i], "end"); i++) {
+					for (i = 0; keybindNames[i] != NULL; i++) {
 						if (i == cursor)
 							continue;
 
@@ -434,6 +486,34 @@ void keychart()
 	SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL);
 }
 
+/**
+ * Hold the keyboard input handler.
+ *
+ * Some UI panels have their own keyboard input handler, or in some cases
+ * we do not want the main keyboard input handler to react to key events.
+ * Some keybindings could however be 'un-interruptibles'.
+ * The opposite function is input_release_keyboard().
+ */
+void input_hold_keyboard(void)
+{
+	// Several UI elements may want to hold the keyboard handler, and
+	// thus call this function and its release companion.
+	// We thus need to count how many times this function is called, and
+	// how many times the release function is called.
+	input_keyboard_locked++;
+}
+
+/**
+ * Release the keyboard input handler.
+ *
+ * See input_hold_keyboard().
+ */
+void input_release_keyboard(void)
+{
+	if (input_keyboard_locked > 0)
+		input_keyboard_locked--;
+}
+
 /**
  * @fn static void input_key( int keynum, int value)
  *
@@ -448,6 +528,16 @@ void keychart()
 #define INLVLEDIT()	(game_status == INSIDE_LVLEDITOR)
 static int input_key(int keynum, int value)
 {
+	/* Non lockable keybindings */
+ 	if (KEYPRESS("quit")) {
+		Terminate(EXIT_SUCCESS, TRUE);
+		return 0;
+	}
+
+	/* Do not execute the other key actions, if the the keyboard input is 'held' */
+	if (input_keyboard_locked)
+		return -1;
+
 	/* Cheat keys - those all fall through to normal game keys! */
 	if (GameConfig.enable_cheatkeys && INGAME()) {
 		if (KEYPRESS("cheat_xp+_1k")) {
@@ -478,6 +568,8 @@ static int input_key(int keynum, int value)
 			DropRandomItem(Me.pos.z, Me.pos.x, Me.pos.y, 3, TRUE);
 		} else if (KEYPRESS("cheat_respawn_level")) {
 			respawn_level(Me.pos.z);
+		} else if (KEYPRESS("cheat_reload_graphics")) {
+			reload_graphics();
 		}
 	}
 
@@ -493,11 +585,23 @@ static int input_key(int keynum, int value)
 			toggle_game_config_screen_visibility(GAME_CONFIG_SCREEN_VISIBLE_CHARACTER);
 			return 0;
 		} else if (KEYPRESS("quests")) {
-			quest_browser_interface();
+			toggle_quest_browser();
 			return 0;
 		} else if (KEYPRESS("autorun")) {
 			GameConfig.autorun_activated = !GameConfig.autorun_activated;
 			return 0;
+		} else if (KEYPRESS("move_north")) {
+			set_movement_with_keys(0, -1);
+			return 0;
+		} else if (KEYPRESS("move_south")) {
+			set_movement_with_keys(0, 1);
+			return 0;
+		} else if (KEYPRESS("move_east")) {
+			set_movement_with_keys(1, 0);
+			return 0;
+		} else if (KEYPRESS("move_west")) {
+			set_movement_with_keys(-1, 0);
+			return 0;
 		} else if (KEYPRESS("reload")) {
 			TuxReloadWeapon();
 			return 0;
@@ -579,17 +683,30 @@ static int input_key(int keynum, int value)
 		if (KEYPRESS("drop_item")) {
 			ItemDropFromLevelEditor();
 			return 0;
+		} else if (KEYPRESS("change_obstacle_label")) {
+			if (single_tile_selection(OBJECT_OBSTACLE)) {
+				action_change_obstacle_label_user(EditLevel(), single_tile_selection(OBJECT_OBSTACLE));
+			}
 		} else if (KEYPRESS("change_map_label")) {
-			level_editor_action_change_map_label_user(CURLEVEL());
+			level_editor_action_change_map_label_user(CURLEVEL(), Me.pos.x, Me.pos.y);
 			return 0;
 		} else if (KEYPRESS("zoom_out")) {
 			GameConfig.zoom_is_on = !GameConfig.zoom_is_on;
 			return 0;
-		} else if (KEYPRESS("cycle_marked_obstacle")) {
-			level_editor_cycle_marked_obstacle();
+		} else if (KEYPRESS("cycle_marked_object")) {
+			level_editor_cycle_marked_object();
 			return 0;
 		} else if (KEYPRESS("next_tab")) {
-			alert_window("Reimplement that");
+			lvledit_categoryselect_switch(1);
+			return 0;
+		} else if (KEYPRESS("previous_tab")) {
+			lvledit_categoryselect_switch(-1);
+			return 0;
+		} else if (KEYPRESS("next_selection_type")) {
+			level_editor_switch_selection_type(1);
+			return 0;
+		} else if (KEYPRESS("previous_selection_type")) {
+			level_editor_switch_selection_type(-1);
 			return 0;
 		} else if (KEYPRESS("undo")) {
 			level_editor_action_undo();
@@ -600,17 +717,23 @@ static int input_key(int keynum, int value)
 		} else if (KEYPRESS("beautify_grass")) {
 			level_editor_beautify_grass_tiles(EditLevel());
 			return 0;
+		} else if (KEYPRESS("beautify_water")) {
+			level_editor_beautify_water_tiles(EditLevel());
+			return 0;
 		} else if (KEYPRESS("toolbar_scroll_left")) {
-			leveleditor_toolbar_scroll_left();
+			widget_lvledit_toolbar_scroll_left();
 			return 0;
 		} else if (KEYPRESS("toolbar_scroll_right")) {
-			leveleditor_toolbar_scroll_right();
+			widget_lvledit_toolbar_scroll_right();
 			return 0;
 		} else if (KEYPRESS("toolbar_step_left")) {
-			leveleditor_toolbar_left();
+			widget_lvledit_toolbar_left();
 			return 0;
 		} else if (KEYPRESS("toolbar_step_right")) {
-			leveleditor_toolbar_right();
+			widget_lvledit_toolbar_right();
+			return 0;
+		} else if (KEYPRESS("run_map_validator")) {
+			level_validation();	
 			return 0;
 		} else if (!strncmp(GameConfig.input_keybinds[keynum].name, "place_obstacle_kp", strlen("place_obstacle_kp"))
 			   && value == KEY_PRESS) {
@@ -622,6 +745,12 @@ static int input_key(int keynum, int value)
 			}
 			level_editor_place_aligned_object(number);
 			return 0;
+		} else if (KEYPRESS("toggle_waypoint")) {
+			lvledit_action_toggle_waypoint(FALSE);
+			return 0;
+		} else if (KEYPRESS("toggle_waypoint_randomspawn")) {
+			lvledit_action_toggle_waypoint(TRUE);
+			return 0;
 		} else if (KEYPRESS("connect_waypoint")) {
 			level_editor_action_toggle_waypoint_connection_user(CURLEVEL(), EditX(), EditY());
 			return 0;
@@ -637,7 +766,7 @@ static int input_key(int keynum, int value)
 		}
 	}
 
-	/* Global options */
+	/* Global keybindings */
 	if (KEYPRESS("fullscreen")) {
 		/* This doesn't seem to work so well on Windows */
 #ifndef __WIN32__
@@ -653,15 +782,19 @@ static int input_key(int keynum, int value)
 			mode = SDL_GRAB_OFF;
 
 		SDL_WM_GrabInput(mode);
+	} else if (KEYPRESS("take_screenshot")) {
+		char filename[1000];
+		char relative_filename[100];
+		sprintf(relative_filename, "%s.screenshot-%d.bmp", Me.character_name, SDL_GetTicks()/1000);
+		sprintf(filename, "%s/%s", our_config_dir, relative_filename);
+		save_screenshot(filename, FALSE);
+		alert_window(_("Screenshot saved to \"%s\" in your .freedroid_rpg/ directory."), relative_filename);
 	} else if (KEYPRESS("wall_transparency")) {
 		GameConfig.transparency = !GameConfig.transparency;
 		return 0;
 	} else if (KEYPRESS("keychart")) {
 		keychart();
 		return 0;
-	} else if (KEYPRESS("quit")) {
-		Terminate(0);
-		return 0;
 	}
 
 	return -1;
@@ -673,7 +806,7 @@ static int input_key_event(SDLKey key, SDLMod mod, int value)
 	int noteaten = -1;
 	mod &= ~(KMOD_CAPS | KMOD_NUM | KMOD_MODE);	/* We want to ignore "global" modifiers. */
 
-	for (i = 0; strcmp(keybindNames[i], "end"); i++)
+	for (i = 0; keybindNames[i] != NULL; i++)
 		if ((GameConfig.input_keybinds[i].key == key) && (GameConfig.input_keybinds[i].mod == mod)) {
 			if (!(noteaten = input_key(i, value)))
 				break;
@@ -683,18 +816,29 @@ static int input_key_event(SDLKey key, SDLMod mod, int value)
 
 int input_key_press(SDL_Event * event)
 {
-	int noteaten;
-	noteaten = input_key_event(event->key.keysym.sym, event->key.keysym.mod, KEY_PRESS);
-
-	if (!noteaten)
-		return 0;
-	else if (INLVLEDIT()) {
-		leveleditor_input_keybevent(event);
-	}
+	input_key_event(event->key.keysym.sym, event->key.keysym.mod, KEY_PRESS);
 
 	return 0;
 }
 
+/**
+ * @fn int kptoprint (int)
+ *
+ * @brief Returns printable keycodes when given a printable keypad code.
+ */
+static int kptoprint(int key) {
+	switch (key) {
+		case SDLK_KP_PERIOD: key = SDLK_PERIOD; break;
+		case SDLK_KP_DIVIDE: key = SDLK_SLASH; break;
+		case SDLK_KP_MULTIPLY: key = SDLK_ASTERISK; break;
+		case SDLK_KP_MINUS: key = SDLK_MINUS; break;
+		case SDLK_KP_PLUS: key = SDLK_PLUS; break;
+		case SDLK_KP_EQUALS: key = SDLK_EQUALS; break;
+		default: break;	
+	}	
+	return key;
+}
+
 /**
  * This function reads a character from the keyboard
  * and returns the SDLKey that was pressed.
@@ -708,7 +852,7 @@ int getchar_raw(int *mod)
 		SDL_WaitEvent(&event);	/* wait for next event */
 
 		if (event.type == SDL_QUIT) {
-			Terminate(0);
+			Terminate(EXIT_SUCCESS, TRUE);
 		}
 
 		if (event.type == SDL_KEYDOWN) {
@@ -732,6 +876,9 @@ int getchar_raw(int *mod)
 			 * "The keyboard syms have been cleverly chosen to map to ASCII"
 			 * ... I hope that this design feature is portable, and durable ;)
 			 */
+			if(ISKEYPAD(Returnkey)) {
+				Returnkey = kptoprint(Returnkey);
+			}
 			return (Returnkey);
 		}
  next:		;
