@@ -26,7 +26,7 @@
 
 /**
  * This file contains all the functions managing the lighting inside
- * the freedroidRPG game window, i.e. the 'light radius' of the Tux and
+ * the FreedroidRPG game window, i.e. the 'light radius' of the Tux and
  * of other light emanating objects.
  */
 
@@ -39,11 +39,10 @@
 #include "global.h"
 #include "map.h"
 #include "proto.h"
-#include "SDL_rotozoom.h"
 
 //--------------------
 // 
-#define MAX_NUMBER_OF_LIGHT_SOURCES 100
+#define MAX_NUMBER_OF_LIGHT_SOURCES 200
 
 /* Position and strength of a light source */
 struct light_source {
@@ -69,7 +68,7 @@ int *light_strength_buffer = NULL;
  * ------------------------------------------------
  * 
  * Some data (such as minimum_light_value) used to create the 'darkness map'
- * are level dependents. If several levels are displayed (i.e. when Tux is
+ * are level dependants. If several levels are displayed (i.e. when Tux is
  * near a level's border), we want to avoid abrupt light changes at level
  * boundaries. The call to soften_light_distribution() is not enough to smooth
  * those changes in a correct way. A better method is to interpolate the level
@@ -428,6 +427,10 @@ void LightRadiusInit()
 	LightRadiusConfig.cells_w = round((float)GameConfig.screen_width / (float)LIGHT_RADIUS_TEXTURE_DIVISOR);
 	LightRadiusConfig.cells_h = round((float)GameConfig.screen_height / (float)LIGHT_RADIUS_TEXTURE_DIVISOR);
 
+	// This will prevent a SIGFPE from happening below
+	if(LightRadiusConfig.cells_w <= 1) LightRadiusConfig.cells_w = 2;
+	if(LightRadiusConfig.cells_h <= 1) LightRadiusConfig.cells_h = 2;
+
 	// Find the the nearest power-of-two greater than of equal to the number of cells
 	// and limit the texture's size
 	int pot_w = pot_gte(LightRadiusConfig.cells_w);
@@ -491,7 +494,7 @@ void LightRadiusInit()
 	LightRadiusConfig.translate_y = (int)(UNIVERSAL_TUX_HEIGHT / unit_screen_height);
 
 	// Allocate the light_radius buffer
-	light_strength_buffer = malloc(LightRadiusConfig.cells_w * LightRadiusConfig.cells_h * sizeof(int));
+	light_strength_buffer = MyMalloc(LightRadiusConfig.cells_w * LightRadiusConfig.cells_h * sizeof(int));
 
 }				// void LightRadiusInit();
 
@@ -619,18 +622,22 @@ WARNING!  End of light sources array reached!", NO_NEED_TO_INFORM, IS_WARNING_ON
 		intersection_end.y = min(intersection_end.y, curr_lvl->ylen - 1);
 
 		// scan all obstacles inside the intersected area
+		int tstamp = next_glue_timestamp();
 		for (map_y = intersection_start.y; map_y < intersection_end.y; map_y++) {
 			for (map_x = intersection_start.x; map_x < intersection_end.x; map_x++) {
-				for (glue_index = 0; glue_index < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; glue_index++) {
-					// end of obstacles glued to here?  great->next square
-					//
-					if (curr_lvl->map[map_y][map_x].obstacles_glued_to_here[glue_index] == (-1))
-						break;
+				for (glue_index = 0; glue_index < curr_lvl->map[map_y][map_x].glued_obstacles.size; glue_index++) {
 
-					obs_index = curr_lvl->map[map_y][map_x].obstacles_glued_to_here[glue_index];
+					obs_index = ((int *)(curr_lvl->map[map_y][map_x].glued_obstacles.arr))[glue_index];
 					emitter = &(curr_lvl->obstacle_list[obs_index]);
 
-					if (obstacle_map[emitter->type].emitted_light_strength == 0)
+					if (emitter->timestamp == tstamp)
+						continue;
+
+					emitter->timestamp = tstamp;
+
+					struct dynarray *animated_light_strengths = &(get_obstacle_spec(emitter->type)->emitted_light_strength);
+					int emitted_light_strength = *(int *)dynarray_member(animated_light_strengths, emitter->frame_index % animated_light_strengths->size, sizeof(int));
+					if (!emitted_light_strength)
 						continue;
 
 					// Now we know that this one needs to be inserted!
@@ -643,7 +650,7 @@ WARNING!  End of light sources array reached!", NO_NEED_TO_INFORM, IS_WARNING_ON
 					if (light_sources[next_light_emitter_index].vpos.x == -1)
 						continue;
 
-					light_sources[next_light_emitter_index].strength = obstacle_map[emitter->type].emitted_light_strength;
+					light_sources[next_light_emitter_index].strength = emitted_light_strength;
 					next_light_emitter_index++;
 
 					// We must not write beyond the bounds of our light sources array!
@@ -790,7 +797,7 @@ static int calculate_light_strength(gps *cell_vpos)
 		// the light source into account. It means that we do not accumulate
 		// light sources.
 		if ( absolute_intensity - final_light_strength < 0 )
-			break;
+			continue;
 
 		// Some pre-computations
 		// First transform light source's position into virtual position, related to Tux's current level
@@ -842,7 +849,7 @@ static void soften_light_distribution(void)
 	uint32_t x, y;
 
 	// Now that the light buffer has been set up properly, we can start to
-	// smoothen it out a bit.  We do so in the direction of more light.
+	// smooth it out a bit.  We do so in the direction of more light.
 	// Propagate from top-left to bottom-right
 	//
 	for (y = 0; y < (LightRadiusConfig.cells_h - 1); y++) {
@@ -973,20 +980,6 @@ int get_light_strength_screen(int x, int y)
 
 }				// int get_light_screen_strength ( moderately_finepoint target_pos )
 
-/**
- * This function is used to find the light intensity at any given point
- * on the map.
- */
-int get_light_strength(moderately_finepoint target_pos)
-{
-	int x, y;
-
-	x = translate_map_point_to_screen_pixel_x(target_pos.x, target_pos.y);
-	y = translate_map_point_to_screen_pixel_y(target_pos.x, target_pos.y);
-
-	return get_light_strength_screen(x, y);
-}
-
 /**
  * This function should blit the shadows on the floor, that are used to
  * generate the impression of a 'light radius' around the players 
@@ -1031,17 +1024,11 @@ void blit_classic_SDL_light_radius(int decay_x, int decay_y)
 		first_call = FALSE;
 
 		int i;
-		char fpath[2048];
 		char constructed_file_name[2000];
-		SDL_Surface *tmp;
 
 		for (i = 0; i < NUMBER_OF_SHADOW_IMAGES; i++) {
 			sprintf(constructed_file_name, "light_radius_chunks/iso_light_radius_darkness_%04d.png", i);
-			find_file(constructed_file_name, GRAPHICS_DIR, fpath, 0);
-			get_iso_image_from_file_and_path(fpath, &(light_radius_chunk[i]), TRUE);
-			tmp = light_radius_chunk[i].surface;
-			light_radius_chunk[i].surface = SDL_DisplayFormatAlpha(light_radius_chunk[i].surface);
-			SDL_FreeSurface(tmp);
+			load_image(&light_radius_chunk[i], constructed_file_name, FALSE);
 		}
 
 		lrc_nb_columns = (int)ceilf((float)GameConfig.screen_width / (float)(LRC_ISO_WIDTH + LRC_ISO_GAP_X)) + 1;
@@ -1069,7 +1056,7 @@ void blit_classic_SDL_light_radius(int decay_x, int decay_y)
 			int light_strength;
 			light_strength = get_light_strength_screen((uint32_t) center_x, (uint32_t) center_y);
 			if (light_strength < (NUMBER_OF_SHADOW_IMAGES-1))
-				our_SDL_blit_surface_wrapper(light_radius_chunk[light_strength].surface, NULL, Screen, &target_rectangle);
+				SDL_BlitSurface(light_radius_chunk[light_strength].surface, NULL, Screen, &target_rectangle);
 
 			// Next tile along X
 			center_x += (LRC_ISO_WIDTH + LRC_ISO_GAP_X);
@@ -1089,7 +1076,7 @@ void blit_classic_SDL_light_radius(int decay_x, int decay_y)
 			int light_strength;
 			light_strength = get_light_strength_screen((uint32_t) center_x, (uint32_t) center_y);
 			if (light_strength < (NUMBER_OF_SHADOW_IMAGES-1))
-				our_SDL_blit_surface_wrapper(light_radius_chunk[light_strength].surface, NULL, Screen, &target_rectangle);
+				SDL_BlitSurface(light_radius_chunk[light_strength].surface, NULL, Screen, &target_rectangle);
 
 			// Next tile along X
 			center_x += LRC_ISO_WIDTH + LRC_ISO_GAP_X;
