@@ -32,17 +32,29 @@
 #include "struct.h"
 #include "global.h"
 #include "proto.h"
-#include "savestruct.h"
 
 #include "lvledit/lvledit_actions.h"
 #include "lvledit/lvledit_map.h"
 
-#include "../lua/lua.h"
-#include "../lua/lauxlib.h"
-#include "../lua/lualib.h"
+#include "widgets/widgets.h"
 
-/* Our Lua state for event execution */
-lua_State *global_lua_state;
+#include "lua.h"
+#include "lauxlib.h"
+#include "lualib.h"
+
+/* Lua state for dialog execution */
+static lua_State *dialog_lua_state;
+
+/* Lua state for config execution */
+lua_State *config_lua_state;
+
+lua_State *get_lua_state(enum lua_target target)
+{
+	if (target == LUA_CONFIG)
+		return config_lua_state;
+
+	return dialog_lua_state;
+}
 
 /** Helper to retrieve the enemy 
   * a Lua function must act upon.
@@ -50,23 +62,64 @@ lua_State *global_lua_state;
   * by default the function will act upon
   * the current chat droid.
   */
-static enemy *get_enemy_arg(lua_State *L, int param_number) 
+static enemy *get_enemy_opt(lua_State *L, int param_number, int optional)
 {
 	const char *dialog = luaL_optstring(L, param_number, NULL);
 
 	if (!dialog)
 		return chat_control_chat_droid;
 
-	return get_enemy_with_dialog(dialog);
+	enemy *en = get_enemy_with_dialog(dialog);
+	if (!optional && !en)
+		ErrorMessage(__FUNCTION__, _("Could not find a droid corresponding to the dialog \"%s\"."), PLEASE_INFORM, IS_FATAL, dialog);
+	return en;
+}
+
+static enemy *get_enemy_arg(lua_State *L, int param_number)
+{
+	return get_enemy_opt(L, param_number, FALSE);
+}
+
+/** Helper to modify the enemy state
+  * with a constant set of names.
+  */
+static void set_bot_state(enemy *en, const char *cmd) 
+{
+	if (!strcmp(cmd, "follow_tux")) {
+		en->follow_tux = TRUE;
+		en->CompletelyFixed = FALSE;
+	} else if (!strcmp(cmd, "fixed")) {
+		en->follow_tux = FALSE;
+		en->CompletelyFixed = TRUE;
+	} else if (!strcmp(cmd, "free")) {
+		en->follow_tux = FALSE;
+		en->CompletelyFixed = FALSE;
+	} else if (!strcmp(cmd, "home")) {
+		en->follow_tux = FALSE;
+		en->CompletelyFixed = FALSE;
+		en->combat_state = RETURNING_HOME;
+	} else if (!strcmp(cmd, "patrol")) {
+		en->follow_tux = FALSE;
+		en->CompletelyFixed = FALSE;
+		en->combat_state = SELECT_NEW_WAYPOINT;
+	} else {
+		ErrorMessage(__FUNCTION__,
+		     "I was called with an invalid state named %s. Accepted values are \"follow_tux\", \"fixed\", \"free\", \"home\", and \"patrol\".\n",
+		     PLEASE_INFORM, IS_WARNING_ONLY, cmd);
+	}
 }
 
 static int lua_event_teleport(lua_State * L)
 {
+	gps stop_pos = { -1, -1, -1 };
 	const char *label = luaL_checkstring(L, 1);
 	location TempLocation;
 	ResolveMapLabelOnShip(label, &TempLocation);
 	reset_visible_levels();
-	Teleport(TempLocation.level, TempLocation.x + 0.5, TempLocation.y + 0.5, TRUE);
+	Teleport(TempLocation.level, TempLocation.x + 0.5, TempLocation.y + 0.5, TRUE, TRUE);
+	Me.speed.x = 0.0;
+	Me.speed.y = 0.0;
+	clear_out_intermediate_points(&stop_pos, Me.next_intermediate_point, MAX_INTERMEDIATE_WAYPOINTS_FOR_TUX);
 	clear_active_bullets();
 	return 0;
 }
@@ -91,33 +144,21 @@ static int lua_event_display_big_message(lua_State * L)
 static int lua_event_display_console_message(lua_State * L)
 {
 	const char *msg = luaL_checkstring(L, 1);
-	append_new_game_message(msg);
+	append_new_game_message("%s", msg);
 	return 0;
 }
 
-static void event_modify_trigger(const char *name, int state)
-{
-	event_trigger *target_event = NULL;
-	int i;
-	for (i = 0; i < MAX_EVENT_TRIGGERS; i++) {
-		if (!strcmp(AllEventTriggers[i].name, name))
-			target_event = &AllEventTriggers[i];
-	}
-
-	target_event->enabled = state;
-}
-
 static int lua_event_enable_trigger(lua_State * L)
 {
 	const char *name = luaL_checkstring(L, 1);
-	event_modify_trigger(name, 1);
+	event_modify_trigger_state(name, 1);
 	return 0;
 }
 
 static int lua_event_disable_trigger(lua_State * L)
 {
 	const char *name = luaL_checkstring(L, 1);
-	event_modify_trigger(name, 0);
+	event_modify_trigger_state(name, 0);
 	return 0;
 }
 
@@ -127,10 +168,13 @@ static int event_change_obstacle_type(const char *obslabel, int type)
 	obstacle *our_obstacle = give_pointer_to_obstacle_with_label(obslabel, &obstacle_level_num);
 	level *obstacle_level = curShip.AllLevels[obstacle_level_num];
 
+	// tell the automap to update
+	update_obstacle_automap(obstacle_level->levelnum, our_obstacle);
+	
 	if (type != -1) {
 		our_obstacle->type = type;
 	} else {
-		action_remove_obstacle(obstacle_level, our_obstacle);
+		del_obstacle(our_obstacle);
 	}
 
 	// Now we make sure the door lists and that are all updated...
@@ -205,6 +249,12 @@ static int lua_event_get_tux_hp(lua_State * L)
 	return 1;
 }
 
+static int lua_event_get_tux_max_hp(lua_State * L)
+{
+	lua_pushinteger(L, (int)Me.maxenergy);
+	return 1;
+}
+
 static int lua_event_heat_tux(lua_State * L)
 {
 	int temp = luaL_checkinteger(L, 1);
@@ -271,9 +321,7 @@ static int lua_event_improve_program(lua_State * L)
 static int lua_event_downgrade_program(lua_State * L)
 {
 	const char *pname = luaL_checkstring(L, 1);
-	Me.skill_level[get_program_index_with_name(pname)]--;
-	if (Me.skill_level[get_program_index_with_name(pname)] < 0)
-		Me.skill_level[get_program_index_with_name(pname)] = 0;
+	downgrade_program(get_program_index_with_name(pname));
 	return 0;
 }
 
@@ -303,12 +351,13 @@ static int lua_event_give_item(lua_State * L)
 	// Either we put the new item directly into inventory or we issue a warning
 	// that there is no room and then drop the item to the floor directly under 
 	// the current Tux position.  That can't fail, right?
-	//
+	char msg[1000];
 	if (!give_item(&NewItem)) {
-		SetNewBigScreenMessage(_("1 Item received (on floor)"));
+		sprintf(msg, _("Received Item: %s (on floor)"), itemname);
 	} else {
-		SetNewBigScreenMessage(_("1 Item received!"));
+		sprintf(msg, _("Received Item: %s"), itemname);
 	}
+	SetNewBigScreenMessage(msg);
 	return 0;
 }
 
@@ -332,6 +381,20 @@ static int lua_event_has_item_backpack(lua_State * L)
 	return 1;
 }
 
+static int lua_event_has_item_equipped(lua_State * L)
+{
+	const char *itemname = luaL_checkstring(L, 1);
+	int item = GetItemIndexByName(itemname);
+	if ((Me.weapon_item.type == item) || (Me.drive_item.type == item) 
+		|| (Me.armour_item.type == item) || (Me.shield_item.type == item) 
+		|| (Me.special_item.type == item)) {
+		lua_pushboolean(L, TRUE);
+	} else {
+		lua_pushboolean(L, FALSE);
+	}
+	return 1;
+}
+
 static int lua_event_equip_item(lua_State * L)
 {
 	const char *item_name = luaL_checkstring(L, 1);
@@ -347,6 +410,16 @@ static int lua_event_equip_item(lua_State * L)
 	return 0;
 }
 
+static int lua_set_death_item(lua_State * L)
+{
+	const char *item_name = luaL_checkstring(L, 1);
+	enemy *en = get_enemy_arg(L, 2);
+	if (!strcmp(item_name, "NONE"))
+		en->on_death_drop_item_code = -1;
+	else
+		en->on_death_drop_item_code = GetItemIndexByName(item_name);
+	return 0;
+}
 
 static int lua_event_add_diary_entry(lua_State * L)
 {
@@ -357,39 +430,14 @@ static int lua_event_add_diary_entry(lua_State * L)
 	return 0;
 }
 
-static int lua_event_plant_cookie(lua_State * L)
+static int lua_event_has_met(lua_State *L)
 {
-	const char *cookie = luaL_checkstring(L, 1);
-	PlantCookie(cookie);
-	return 0;
-}
-
-static int lua_event_cookie_planted(lua_State * L)
-{
-	const char *cookie = luaL_checkstring(L, 1);
-	int i;
-	int cond = 0;
-	for (i = 0; i < MAX_COOKIES; i++) {
-		if (!Me.cookie_list[i])
-			break;
-		
-		if (!strcmp(Me.cookie_list[i], cookie)) {
-			cond = 1;
-			break;
-		}
-	}
-
-	lua_pushboolean(L, cond);
+	const char *npc_name = luaL_checkstring(L, 1);
+	struct npc *npc = npc_get(npc_name);
+	lua_pushboolean(L, npc->chat_character_initialized);
 	return 1;
 }
 
-static int lua_event_remove_cookie(lua_State * L)
-{
-	const char *cookie = luaL_checkstring(L, 1);
-	DeleteCookie(cookie);
-	return 0;
-}
-
 static int lua_event_assign_mission(lua_State * L)
 {
 	const char *misname = luaL_checkstring(L, 1);
@@ -434,9 +482,9 @@ static int lua_event_is_mission_complete(lua_State * L)
 
 static int lua_event_give_xp(lua_State * L)
 {
-	int xp = luaL_checkinteger(L, 1);
+	int xp = luaL_checkinteger(L, 1) * Me.experience_factor;
 	char tmpstr[150];
-	Me.Experience += xp * Me.experience_factor;
+	Me.Experience += xp;
 	sprintf(tmpstr, _("+%d Experience Points"), xp);
 	SetNewBigScreenMessage(tmpstr);
 	return 0;
@@ -445,10 +493,7 @@ static int lua_event_give_xp(lua_State * L)
 static int lua_event_eat_training_points(lua_State * L)
 {
 	int nb = luaL_checkinteger(L, 1);
-	char tmpstr[150];
 	Me.points_to_distribute -= nb;
-	sprintf(tmpstr, _("%d training point(s) spent!"), nb);
-	SetNewBigScreenMessage(tmpstr);
 	return 0;
 }
 
@@ -497,9 +542,9 @@ static int lua_event_change_stat(lua_State * L)
 	} else if (!strcmp(characteristic, "dexterity")) {
 		statptr = &Me.base_dexterity;
 	} else if (!strcmp(characteristic, "CPU")) {
-		statptr = &Me.base_magic;
+		statptr = &Me.base_cooling;
 	} else if (!strcmp(characteristic, "vitality")) {
-		statptr = &Me.base_vitality;
+		statptr = &Me.base_physique;
 	} else {
 		ErrorMessage(__FUNCTION__,
 			     "I was called with characteristic name %s - accepted values are \"strength\", \"dexterity\", \"CPU\", and \"vitality\".",
@@ -586,17 +631,21 @@ static int lua_event_freeze_tux_npc(lua_State * L)
 
 static int lua_chat_player_name(lua_State * L)
 {
-	lua_pushstring(L, Me.character_name);
+	if (Me.character_name)
+		lua_pushstring(L, Me.character_name);
+	else
+		lua_pushstring(L, "");
 	return 1;
 }
 
 static int lua_chat_tux_says(lua_State * L)
 {
 	const char *answer = luaL_checkstring(L, 1);
-	const char *sample = luaL_optstring(L, 2, "Sorry_No_Voice_Sample_Yet_0.wav");
+	const char *waitop = luaL_optstring(L, 2, "WAIT");
+	int no_wait = !strcmp(waitop, "NO_WAIT");
 
 	autostr_append(chat_log.text, "\1- ");
-	GiveSubtitleNSample(L_(answer), sample, chat_control_chat_droid, TRUE);
+	chat_add_response(L_(answer), no_wait, chat_control_chat_droid);
 	autostr_append(chat_log.text, "\n");
 	return 0;
 }
@@ -604,10 +653,11 @@ static int lua_chat_tux_says(lua_State * L)
 static int lua_chat_npc_says(lua_State * L)
 {
 	const char *answer = luaL_checkstring(L, 1);
-	const char *sample = luaL_optstring(L, 2, "Sorry_No_Voice_Sample_Yet_0.wav");
+	const char *waitop = luaL_optstring(L, 2, "WAIT");
+	int no_wait = !strcmp(waitop, "NO_WAIT");
 
 	autostr_append(chat_log.text, "\2");
-	GiveSubtitleNSample(L_(answer), sample, chat_control_chat_droid, TRUE);
+	chat_add_response(L_(answer), no_wait, chat_control_chat_droid);
 	autostr_append(chat_log.text, "\n");
 	return 0;
 }
@@ -615,14 +665,30 @@ static int lua_chat_npc_says(lua_State * L)
 static int lua_chat_cli_says(lua_State * L)
 {
 	const char *answer = luaL_checkstring(L, 1);
-	const char *sample = luaL_optstring(L, 2, "Sorry_No_Voice_Sample_Yet_0.wav");
+	const char *waitop = luaL_optstring(L, 2, "WAIT");
+	int no_wait = !strcmp(waitop, "NO_WAIT");
 
 	autostr_append(chat_log.text, "\3");
-	GiveSubtitleNSample(L_(answer), sample, chat_control_chat_droid, TRUE);
+	chat_add_response(L_(answer), no_wait, chat_control_chat_droid);
 
 	return 0;
 }
 
+static int lua_chat_push_topic(lua_State * L)
+{
+	const char *topic = luaL_checkstring(L, 1);
+	
+	chat_push_topic(topic);
+
+	return 0;
+}
+
+static int lua_chat_pop_topic(lua_State * L)
+{
+	chat_pop_topic();
+
+	return 0;
+}
 
 static int lua_chat_run_subdialog(lua_State * L)
 {
@@ -705,27 +771,19 @@ static int lua_chat_set_bot_state(lua_State * L)
 {
 	const char *cmd = luaL_checkstring(L, 1);
 	enemy *en = get_enemy_arg(L, 2); 
-	if (!strcmp(cmd, "follow_tux")) {
-		en->follow_tux = TRUE;
-		en->CompletelyFixed = FALSE;
-	} else if (!strcmp(cmd, "fixed")) {
-		en->follow_tux = FALSE;
-		en->CompletelyFixed = TRUE;
-	} else if (!strcmp(cmd, "free")) {
-		en->follow_tux = FALSE;
-		en->CompletelyFixed = FALSE;
-	} else if (!strcmp(cmd, "home")) {
-		en->follow_tux = FALSE;
-		en->CompletelyFixed = FALSE;
-		en->combat_state = RETURNING_HOME;
-	} else if (!strcmp(cmd, "patrol")) {
-		en->follow_tux = FALSE;
-		en->CompletelyFixed = FALSE;
-		en->combat_state = SELECT_NEW_WAYPOINT;
-	} else {
-		ErrorMessage(__FUNCTION__,
-			     "I was called with an invalid state named %s. Accepted values are \"follow_tux\", \"fixed\", \"free\", \"home\", and \"patrol\".\n",
-			     PLEASE_INFORM, IS_FATAL, cmd);
+	set_bot_state(en, cmd);
+	return 0;
+}
+
+static int lua_chat_broadcast_bot_state(lua_State * L)
+{
+	const char *cmd = luaL_checkstring(L, 1);	
+	const char *dialogname = chat_control_chat_droid->dialog_section_name;
+	enemy *en;
+	BROWSE_LEVEL_BOTS(en, chat_control_chat_droid->pos.z) {
+		if (!strcmp(en->dialog_section_name, dialogname) && (is_friendly(en->faction, FACTION_SELF))) {
+			set_bot_state(en, cmd);
+		}
 	}
 	return 0;
 }
@@ -752,7 +810,7 @@ static int lua_set_bot_destination(lua_State * L)
 				Map label %s (found on level %d) does not have a waypoint. Cannot send bot %s\n\
 				to this location. Doing nothing.\n\
 				Location coordinates x=%d, y=%d.",
-				PLEASE_INFORM, IS_WARNING_ONLY, label, TempLocation.level, en->pos.z, en->dialog_section_name, TempLocation.x, TempLocation.y);
+				PLEASE_INFORM, IS_WARNING_ONLY, label, TempLocation.level, en->dialog_section_name, TempLocation.x, TempLocation.y);
 		return 0;
 	}
 
@@ -763,19 +821,41 @@ static int lua_set_bot_destination(lua_State * L)
 	return 0;
 }
 
+static int lua_set_rush_tux(lua_State * L)
+{
+	const uint8_t cmd = (uint8_t)luaL_checkinteger(L, 1);
+	enemy *en = get_enemy_arg(L, 2);
+	en->will_rush_tux = cmd;
+	return 0;
+}
+
+static int lua_will_rush_tux(lua_State * L)
+{
+	enemy *en = get_enemy_arg(L, 1);
+	lua_pushboolean(L, en->will_rush_tux);
+	return 1;
+}
+
 static int lua_chat_takeover(lua_State * L)
 {
 	int opponent_capsules = luaL_checkinteger(L, 1);
 	int player_capsules = 2 + Me.skill_level[get_program_index_with_name("Hacking")];
 	int game_length = luaL_optint(L, 2, 100);
 
-	int won = do_takeover(player_capsules, opponent_capsules, game_length);
+	int won = do_takeover(player_capsules, opponent_capsules, game_length, NULL);
 
 	lua_pushboolean(L, won);
 
 	return 1;
 }
 
+static int lua_chat_bot_exists(lua_State *L)
+{
+	int exists = get_enemy_opt(L, 1, TRUE) != NULL;
+	lua_pushboolean(L, exists);
+	return 1;
+}
+
 static int lua_chat_get_bot_type(lua_State * L)
 {
 	enemy *en = get_enemy_arg(L, 1); 
@@ -844,7 +924,7 @@ static int lua_set_faction_state(lua_State *L)
 	else if (!strcmp(state_str, "friendly"))
 		state = FRIENDLY;
 	else {
-		ErrorMessage(__FUNCTION__, "Unknown faction state %s.", PLEASE_INFORM, IS_WARNING_ONLY);
+		ErrorMessage(__FUNCTION__, "Unknown faction state %s.", PLEASE_INFORM, IS_WARNING_ONLY, state_str);
 		return 0;
 	}
 
@@ -944,9 +1024,14 @@ luaL_reg lfuncs[] = {
 	,
 	{"heat_tux", lua_event_heat_tux}
 	,
+	/* get_tux_hp()             - Returns Tux's current health
+	 * get_tux_max_hp()         - Returns Tux's current maximum health
+	 * see also: tux_hp_ratio() - Returns the ratio of the two
+	 */
 	{"get_tux_hp", lua_event_get_tux_hp}
 	,
-
+	{"get_tux_max_hp", lua_event_get_tux_max_hp}
+	,
 	/* improve_skill(string skill_name)
 	 * get_skill()
 	 * improve_skill improves one of the three "melee", "ranged" and "programming" skills
@@ -982,17 +1067,20 @@ luaL_reg lfuncs[] = {
 	,
 	{"has_item_backpack", lua_event_has_item_backpack}
 	,
+	{"has_item_equipped", lua_event_has_item_equipped}
+	,
 	{"equip_item", lua_event_equip_item}
 	,
 	{"sell_item", lua_event_sell_item}
 	,
+	/* set_death_item(string item_name [, string  npc]) 
+	 * changes the item dropped when the droid dies
+	*/
+	{"set_death_item", lua_set_death_item}
+	,
 	{"add_diary_entry", lua_event_add_diary_entry}
 	,
-	{"add_cookie", lua_event_plant_cookie}
-	,
-	{"has_cookie", lua_event_cookie_planted}
-	,
-	{"del_cookie", lua_event_remove_cookie}
+	{"has_met", lua_event_has_met}
 	,
 	{"assign_quest", lua_event_assign_mission}
 	,
@@ -1022,8 +1110,7 @@ luaL_reg lfuncs[] = {
 	,
 	{"craft_addons", lua_event_craft_addons}
 	,
-
-	{"get_player_name", lua_chat_player_name}
+	{"get_player_name", lua_chat_player_name} 
 	,
 	{"tux_says", lua_chat_tux_says}
 	,
@@ -1032,6 +1119,10 @@ luaL_reg lfuncs[] = {
 	,
 	{"cli_says", lua_chat_cli_says}
 	,
+	{"topic", lua_chat_push_topic}
+	,
+	{"pop_topic", lua_chat_pop_topic}
+	,
 	{"run_subdialog", lua_chat_run_subdialog}
 	,
 	{"start_chat", lua_start_chat}
@@ -1049,10 +1140,21 @@ luaL_reg lfuncs[] = {
 
 	{"drop_dead", lua_chat_drop_dead}
 	,
+	{"bot_exists", lua_chat_bot_exists}
+	,
 	{"set_bot_state", lua_chat_set_bot_state}
 	,
 	{"set_bot_destination", lua_set_bot_destination}
 	,
+	{"broadcast_bot_state", lua_chat_broadcast_bot_state}
+	,
+	/* set_rush_tux()   - Sets or unsets if the NPC should rush and talk to Tux
+	 * will_rush_tux() - Checks if the NPC is planning on rushing Tux
+	 */
+	{"set_rush_tux", lua_set_rush_tux}
+	,
+	{"will_rush_tux", lua_will_rush_tux}
+	,
 	{"takeover", lua_chat_takeover}
 	,
 	{"heal_npc", lua_event_heal_npc}
@@ -1084,19 +1186,65 @@ luaL_reg lfuncs[] = {
 	{NULL, NULL}
 };
 
-void run_lua(const char *code)
+void run_lua(enum lua_target target, const char *code)
 {
-	if (luaL_dostring(global_lua_state, code)) {
-		ErrorMessage(__FUNCTION__, "Error running Lua code {%s}: %s.\n", PLEASE_INFORM, IS_FATAL, code,
-			     lua_tostring(global_lua_state, -1));
+	lua_State *L = get_lua_state(target);
+
+	if (luaL_dostring(L, code)) {
+		fflush(stdout);
+
+		const char *error = lua_tostring(L, -1);
+		char *display_code = strdup(code);
+		const char *ptr = error;
+		int err_line = 0;
+		int cur_line = 2;
+		struct auto_string *erronous_code;
+
+		erronous_code = alloc_autostr(16);
+
+		//Find which line the error is on (if there is a line number in the error message)
+		while (*ptr != 0 && *ptr != ':') {
+			ptr++;
+		}
+		if (*ptr != 0) {
+			// Line number found
+			ptr++;
+			err_line = strtol(ptr, NULL, 10);
+		}
+
+		//Break up lua code by newlines then insert line numbers & error notification
+		ptr = strtok(display_code,"\n");
+
+		while (ptr != NULL) {
+			if (err_line != cur_line) {
+				autostr_append(erronous_code, "%d %s\n", cur_line, ptr);
+#ifndef __WIN32__
+			} else if (!strcmp(getenv("TERM"), "xterm")) { //color highlighting for Linux/Unix terminals
+				autostr_append(erronous_code, "\033[41m>%d %s\033[0m\n", cur_line, ptr);
+#endif
+			} else {
+				autostr_append(erronous_code, ">%d %s\n", cur_line, ptr);
+			}
+
+			ptr = strtok(NULL, "\n");
+			cur_line++;
+		}
+
+		ErrorMessage(__FUNCTION__, "Error running Lua code: %s.\nErroneous LuaCode={\n%s}",
+				 PLEASE_INFORM, IS_WARNING_ONLY, error, erronous_code->value);
+
+		free(display_code);
+		free_autostr(erronous_code);
 	}
 }
 
-void run_lua_file(const char *path)
+void run_lua_file(enum lua_target target, const char *path)
 {
-	if (luaL_dofile(global_lua_state, path)) {
+	lua_State *L = get_lua_state(target);
+
+	if (luaL_dofile(L, path)) {
 		ErrorMessage(__FUNCTION__, "Cannot run script file %s: %s.\n",
-		         PLEASE_INFORM, IS_FATAL, path, lua_tostring(global_lua_state, -1));
+		         PLEASE_INFORM, IS_FATAL, path, lua_tostring(L, -1));
 	}
 }
 
@@ -1105,15 +1253,90 @@ void init_lua()
 	char fpath[2048];
 	int i;
 
-	global_lua_state = lua_open();
-	luaL_openlibs(global_lua_state);
+	dialog_lua_state = lua_open();
+	luaL_openlibs(dialog_lua_state);
+	config_lua_state = lua_open();
+	luaL_openlibs(config_lua_state);
 
 	for (i = 0; lfuncs[i].name != NULL; i++) {
-		lua_pushcfunction(global_lua_state, lfuncs[i].func);
-		lua_setglobal(global_lua_state, lfuncs[i].name);
+		lua_pushcfunction(dialog_lua_state, lfuncs[i].func);
+		lua_setglobal(dialog_lua_state, lfuncs[i].name);
 	}
 
 	if (!find_file("script_helpers.lua", MAP_DIR, fpath, 1)) {
-		run_lua_file(fpath);
+		run_lua_file(LUA_DIALOG, fpath);
+		run_lua_file(LUA_CONFIG, fpath);
 	}
 }
+
+/**
+ * Reset Lua state
+ */
+void reset_lua_state(void)
+{
+	int i;
+	char fpath[2048];
+
+	lua_close(dialog_lua_state);
+	dialog_lua_state = lua_open();
+	luaL_openlibs(dialog_lua_state);
+
+	for (i = 0; lfuncs[i].name != NULL; i++) {
+		lua_pushcfunction(dialog_lua_state, lfuncs[i].func);
+		lua_setglobal(dialog_lua_state, lfuncs[i].name);
+	}
+
+	if (!find_file("script_helpers.lua", MAP_DIR, fpath, 1)) {
+		run_lua_file(LUA_DIALOG, fpath);
+	}
+}
+
+/**
+ * Save Lua variables as lua code.
+ * Variables prefixed with '_' are omitted because these are Lua predefined variables.
+ */
+void write_lua_variables(struct auto_string *savestruct_autostr)
+{
+	int boolean;
+	const char *value;
+	lua_State *L = get_lua_state(LUA_DIALOG);
+
+	lua_pushnil(L);
+	while (lua_next(L, LUA_GLOBALSINDEX) != 0) {
+		int value_type = lua_type(L, -1);
+		int key_type = lua_type(L, -2);
+
+		if (key_type != LUA_TSTRING) {
+			lua_pop(L, 1);
+			continue;
+		}
+
+		const char *name = lua_tostring(L, -2);
+		if (name[0] == '_') {
+			lua_pop(L, 1);
+			continue;
+		}
+
+		switch (value_type)
+		{
+			case LUA_TBOOLEAN:
+				boolean = lua_toboolean(L, -1);
+				autostr_append(savestruct_autostr, "_G[\"%s\"] = %s\n", name, boolean ? "true" : "false");
+				break;
+			case LUA_TSTRING:
+				value = lua_tostring(L, -1);
+				autostr_append(savestruct_autostr, "_G[\"%s\"] = \"%s\"\n", name, value);
+				break;
+			case LUA_TNUMBER:
+				value = lua_tostring(L, -1);
+				autostr_append(savestruct_autostr, "_G[\"%s\"] = %s\n", name, value);
+				break;
+			default:
+				break;
+		}
+
+		lua_pop(L, 1);
+	}
+
+	autostr_append(savestruct_autostr, "\n");
+}
