@@ -36,14 +36,12 @@
 #include "global.h"
 #include "proto.h"
 
-#include "SDL_rotozoom.h"
 
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_validator.h"
 
 #include "lvledit/lvledit_actions.h"
 #include "lvledit/lvledit_display.h"
-#include "lvledit/lvledit_grass_actions.h"
 #include "lvledit/lvledit_map.h"
 #include "lvledit/lvledit_menu.h"
 #include "lvledit/lvledit_tools.h"
@@ -115,6 +113,7 @@ int level_editor_item_drop_index(int row_len, int line_len)
  */
 item *ItemDropFromLevelEditor(void)
 {
+#define SKIP_CTD_ITEM 1
 	int SelectionDone = FALSE;
 	int NewItemCode = (-1);
 	int i;
@@ -132,12 +131,12 @@ item *ItemDropFromLevelEditor(void)
 
 	while (!SelectionDone) {
 		SDL_Event event;
-		our_SDL_fill_rect_wrapper(Screen, NULL, 0);
+		clear_screen();
 
 		for (j = 0; j < row_len; j++) {
 			for (i = 0; i < line_len; i++) {
 				init_item(&temp_item);
-				temp_item.type = i + j * line_len + item_group * line_len * row_len;
+				temp_item.type = i + j * line_len + item_group * line_len * row_len + SKIP_CTD_ITEM;
 				if (temp_item.type >= Number_Of_Item_Types)
 					continue;	//temp_item.type = 1 ;
 				ShowRescaledItem(i, 32 + (64 * GameConfig.screen_height / 480 + 2) * j, &(temp_item));
@@ -152,7 +151,7 @@ item *ItemDropFromLevelEditor(void)
 		if (MouseCursorIsOnButton(LEVEL_EDITOR_CANCEL_ITEM_DROP_BUTTON, GetMousePos_x(), GetMousePos_y()))
 			PutStringFont(Screen, FPS_Display_BFont, 20, 440 * GameConfig.screen_height / 480, _("Cancel item drop"));
 		if (level_editor_item_drop_index(row_len, line_len) != (-1)) {
-			previous_mouse_position_index = level_editor_item_drop_index(row_len, line_len) + item_group * line_len * row_len;
+			previous_mouse_position_index = level_editor_item_drop_index(row_len, line_len) + item_group * line_len * row_len + SKIP_CTD_ITEM;
 			if (previous_mouse_position_index >= Number_Of_Item_Types) {
 				previous_mouse_position_index = Number_Of_Item_Types - 1;
 			} else
@@ -160,7 +159,7 @@ item *ItemDropFromLevelEditor(void)
 					      D_(ItemMap[previous_mouse_position_index].item_name));
 		}
 
-		blit_our_own_mouse_cursor();
+		blit_mouse_cursor();
 		our_SDL_flip_wrapper();
 
 		while (SDL_PollEvent(&event)) {
@@ -180,7 +179,7 @@ item *ItemDropFromLevelEditor(void)
 				if (MouseCursorIsOnButton(LEVEL_EDITOR_CANCEL_ITEM_DROP_BUTTON, event.button.x, event.button.y)) {
 					return NULL;
 				} else if (level_editor_item_drop_index(row_len, line_len) != (-1)) {
-					NewItemCode = level_editor_item_drop_index(row_len, line_len) + item_group * line_len * row_len;
+					NewItemCode = level_editor_item_drop_index(row_len, line_len) + item_group * line_len * row_len + SKIP_CTD_ITEM;
 					if (NewItemCode < 0)
 						NewItemCode = 0;	// just if the mouse has moved away in that little time...
 					if (NewItemCode < Number_Of_Item_Types)
@@ -202,17 +201,6 @@ item *ItemDropFromLevelEditor(void)
 	return dropped_item;
 };				// void ItemDropFromLevelEditor( void )
 
-/**
- * There is a 'help' screen for the level editor too.  This help screen
- * is presented as a scrolling text, giving a short introduction and also
- * explaining the keymap to the level editor.  The info for this scrolling
- * text is all in a title file in the maps directory, much like the initial
- * scrolling text at any new game startup.
- */
-void ShowLevelEditorKeymap(void)
-{
-	PlayATitleFile("level_editor_help.title");
-};				// void ShowLevelEditorKeymap ( void )
 
 static void leveleditor_init()
 {
@@ -233,14 +221,13 @@ static void leveleditor_init()
 	strcpy(VanishingMessage, "");
 	VanishingMessageEndDate = 0;
 
+	// Reset tooltips.
+	widget_set_tooltip(NULL, NULL);
+
 	// For drawing new waypoints, we init this.
 	//
 	OriginWaypoint = (-1);
 
-	leveleditor_init_widgets();
-
-	global_ingame_mode = GLOBAL_INGAME_MODE_NORMAL;
-
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
 }
 
@@ -250,9 +237,8 @@ static void leveleditor_cleanup()
 	action_freestack();
 	clear_selection(-1);
 
-	global_ingame_mode = GLOBAL_INGAME_MODE_NORMAL;
-
 	SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL);
+	Me.mouse_move_target = Me.pos;
 }
 
 void TestMap(void)
@@ -260,6 +246,7 @@ void TestMap(void)
 	leveleditor_cleanup();	//free current selection and undo stack as a workaround for existing problems
 	if (game_root_mode == ROOT_IS_GAME)	/*don't allow map testing if root mode is GAME */
 		return;
+	game_status = INSIDE_GAME;
 	SaveGame();
 	Game();
 	LoadGame();
@@ -269,7 +256,7 @@ void TestMap(void)
 
 /**
  * This function provides the leveleditor integrated into
- * freedroid.  Actually this function is a submenu of the big
+ * FreedroidRPG.  Actually this function is a submenu of the big
  * Escape Menu.  In here you can edit the level and, upon pressing
  * escape, you can enter a new submenu where you can save the level,
  * change level name and quit from level editing.
@@ -285,18 +272,23 @@ void LevelEditor()
 	while (!level_editor_done) {
 		game_status = INSIDE_LVLEDITOR;
 
+		StartTakingTimeForFPSCalculation();
+		update_frames_displayed();
+
 		if (GameConfig.limit_framerate)
 			SDL_Delay(10);
 
 		leveleditor_process_input();
+		update_widgets();
 
-		struct leveleditor_widget *w = get_active_widget(GetMousePos_x(), GetMousePos_y());
-		if (w->type == WIDGET_MAP) {
+		struct widget *w = get_active_widget(GetMousePos_x(), GetMousePos_y());
+		if (w && w->type == WIDGET_MAP) {
 			leveleditor_update_tool();
 		}
 
 		leveleditor_display();
 
+		ComputeFPSForThisFrame();
 	}
 
 	leveleditor_cleanup();
