@@ -35,7 +35,6 @@
 #include "global.h"
 #include "proto.h"
 
-#include "SDL_rotozoom.h"
 
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_widgets.h"
@@ -59,7 +58,7 @@ static void clear_action(action * action)
 		free(action->d.change_label_name.new_name);
 
 	list_del(&action->node);	//< removes an action from a list
-	free(action);		//< free's the action
+	free(action);		//< frees the action
 }
 
 /**
@@ -126,6 +125,11 @@ static action *action_create(int type, va_list args)
 			act->d.delete_waypoint.x = va_arg(args, int);
 			act->d.delete_waypoint.y = va_arg(args, int);
 			break;
+		case ACT_MOVE_WAYPOINT:
+			act->d.move_waypoint.w = va_arg(args, waypoint *);
+			act->d.move_waypoint.newx = va_arg(args, int);
+			act->d.move_waypoint.newy = va_arg(args, int);
+			break;
 		case ACT_TOGGLE_WAYPOINT_RSPAWN:
 			act->d.toggle_waypoint_rspawn.x = va_arg(args, int);
 			act->d.toggle_waypoint_rspawn.y = va_arg(args, int);
@@ -137,6 +141,7 @@ static action *action_create(int type, va_list args)
 		case ACT_TILE_FLOOR_SET:
 			act->d.change_floor.x = va_arg(args, int);
 			act->d.change_floor.y = va_arg(args, int);
+			act->d.change_floor.layer = va_arg(args, int);
 			act->d.change_floor.type = va_arg(args, int);
 			break;
 		case ACT_MULTIPLE_ACTIONS:
@@ -157,6 +162,18 @@ static action *action_create(int type, va_list args)
 			act->d.jump_to_level.x = (float)va_arg(args, double);
 			act->d.jump_to_level.y = (float)va_arg(args, double);
 			break;
+		case ACT_CHANGE_FLOOR_LAYER:
+			act->d.target_layer = va_arg(args, int);
+			break;
+		case ACT_CREATE_MAP_LABEL:
+			act->d.create_map_label.x = va_arg(args, int);
+			act->d.create_map_label.y = va_arg(args, int);
+			act->d.create_map_label.label_name = va_arg(args, char *);
+			break;
+		case ACT_REMOVE_MAP_LABEL:
+			act->d.delete_map_label.x = va_arg(args, int);
+			act->d.delete_map_label.y = va_arg(args, int);
+			break;
 		default:
 			ErrorMessage(__FUNCTION__, "Unknown action type %d\n", PLEASE_INFORM, IS_FATAL, type);
 	}
@@ -184,43 +201,16 @@ void action_push(int type, ...)
 	va_end(args);
 }
 
-obstacle *action_create_obstacle(level * EditLevel, double x, double y, int new_obstacle_type)
-{
-	int i;
-
-	for (i = 0; i < MAX_OBSTACLES_ON_MAP; i++) {
-		if (EditLevel->obstacle_list[i].type == (-1)) {
-			EditLevel->obstacle_list[i].type = new_obstacle_type;
-			EditLevel->obstacle_list[i].pos.x = x;
-			EditLevel->obstacle_list[i].pos.y = y;
-			EditLevel->obstacle_list[i].pos.z = EditLevel->levelnum;
-			glue_obstacles_to_floor_tiles_for_level(EditLevel->levelnum);
-			// Now that we have disturbed the order of the obstacles on this level, we need
-			// to re-assemble the lists of pointers to obstacles, like the door list, the
-			// teleporter list and the refreshes list.
-			//
-			dirty_animated_obstacle_lists(EditLevel->levelnum);
-			
-			return (&(EditLevel->obstacle_list[i]));
-		}
-	}
-
-	ErrorMessage(__FUNCTION__, "\
-	    Ran out of obstacle positions (%d) in level %d!", PLEASE_INFORM, IS_FATAL, MAX_OBSTACLES_ON_MAP, EditLevel->levelnum);
-	return (NULL);
-}
-
 void action_move_obstacle(level * EditLevel, obstacle * obs, float newx, float newy)
 {
 	action_push(ACT_MOVE_OBSTACLE, obs, obs->pos.x, obs->pos.y);
-	obs->pos.x = newx;
-	obs->pos.y = newy;
-	glue_obstacles_to_floor_tiles_for_level(EditLevel->levelnum);
+
+	move_obstacle(obs, newx, newy);
 }
 
 obstacle *action_create_obstacle_user(Level EditLevel, double x, double y, int new_obstacle)
 {
-	obstacle *o = action_create_obstacle(EditLevel, x, y, new_obstacle);
+	obstacle *o = add_obstacle(EditLevel, x, y, new_obstacle);
 	if (o) {
 		action_push(ACT_REMOVE_OBSTACLE, o);
 	}
@@ -258,20 +248,15 @@ static int action_change_obstacle_label(level *EditLevel, obstacle *obstacle, ch
 	return undoable;
 }
 
-void action_change_obstacle_label_user(level *EditLevel, obstacle *our_obstacle, char *predefined_name)
+void action_change_obstacle_label_user(level *EditLevel, obstacle *our_obstacle)
 {
 	char *name = NULL;
 
 	if (!our_obstacle)
 		return;
 
-	if (predefined_name == NULL) {
-		char *old_label = get_obstacle_extension(EditLevel, our_obstacle, OBSTACLE_EXTENSION_LABEL);
-		name = GetEditableStringInPopupWindow(1000, _("\nPlease enter obstacle label: \n\n"),
-						   old_label ? old_label : "");
-	} else {
-		name = strdup(predefined_name);
-	}
+	char *old_label = get_obstacle_extension(EditLevel, our_obstacle, OBSTACLE_EXTENSION_LABEL);
+	name = GetEditableStringInPopupWindow(1000, _("\nPlease enter obstacle label: \n\n"), old_label ? old_label : "");
 	
 	if (name) {
 		action_change_obstacle_label(EditLevel, our_obstacle, name, 1);
@@ -279,38 +264,6 @@ void action_change_obstacle_label_user(level *EditLevel, obstacle *our_obstacle,
 	}
 }
 
-/**
- * Remove an obstacle from the map.
- * @param undoable indicates whether the action should be made undoable by pushing 
- * undo actions on the stack.
- */
-void action_remove_obstacle(level *EditLevel, obstacle *our_obstacle)
-{
-	level *lvl = curShip.AllLevels[our_obstacle->pos.z];
-
-	// The likely case that no obstacle was currently marked.
-	//
-	if (our_obstacle == NULL)
-		return;
-
-	our_obstacle->type = (-1);
-
-	// Remove the obstacle label if we had one
-	// with the current design this won't be undoable
-	action_change_obstacle_label(lvl, our_obstacle, NULL, 0);
-
-	// Now doing that must have shifted the glue!  That is a problem.  We need to
-	// reglue everything to the map...
-	//
-	glue_obstacles_to_floor_tiles_for_level(lvl->levelnum);
-
-	// Now that we have disturbed the order of the obstacles on this level, we need
-	// to re-assemble the lists of pointers to obstacles, like the door list, the
-	// teleporter list and the refreshes list.
-	//
-	dirty_animated_obstacle_lists(lvl->levelnum);
-}
-
 void action_remove_obstacle_user(Level EditLevel, obstacle * our_obstacle)
 {
 	/* Save obstacle information for the undo action */
@@ -322,7 +275,7 @@ void action_remove_obstacle_user(Level EditLevel, obstacle * our_obstacle)
 	posy = our_obstacle->pos.y;
 
 	// make an undoable removal
-	action_remove_obstacle(EditLevel, our_obstacle);
+	del_obstacle(our_obstacle);
 	action_push(ACT_CREATE_OBSTACLE, posx, posy, type);
 }
 
@@ -342,10 +295,12 @@ item *action_create_item(level *EditLevel, float x, float y, int type)
 		// Display a popup window with a number selector in order to choose the
 		// multiplicity of the item
 		multiplicity =
-		    do_graphical_number_selection_in_range(1, (!MatchItemWithName(type, "Valuable Circuits")) ? 100 : 1000, 1);
-		
-		// We always want create an item with a multiplicity higher than 0
-		multiplicity = (multiplicity > 0) ? multiplicity : 1;
+		    do_graphical_number_selection_in_range(1, (!MatchItemWithName(type, "Valuable Circuits")) ? 100 : 1000, 1, 0);
+
+		if (multiplicity == 0) {
+			// Do not drop an item on the floor when the user cancelled the action.
+			return NULL;
+		}
 	}
 
 	// Create an item on the map
@@ -454,6 +409,18 @@ void action_remove_waypoint(level *EditLevel, int x, int y)
 	action_push(ACT_CREATE_WAYPOINT, x, y, old_random_spawn);
 }
 
+void action_move_waypoint(level *lvl, waypoint *w, int x, int y)
+{
+	int oldx, oldy;
+
+	oldx = w->x;
+	oldy = w->y;
+
+	move_waypoint(lvl, w, x, y);
+
+	action_push(ACT_MOVE_WAYPOINT, w, oldx, oldy);
+}
+
 /**
  * Set/unset the flag for random bots and add this action in the undo/redo stack
  * \param EditLevel Pointer towards the currently edited level where toggle the waypoint
@@ -508,54 +475,72 @@ int action_toggle_waypoint_connection(level *EditLevel, int id_origin, int id_ta
 
 void level_editor_action_toggle_waypoint_connection_user(level * EditLevel, int xpos, int ypos)
 {
-	waypoint *wpts;
-	int i;
+	int wpnum;
 
-	// Determine which waypoint is currently targeted
-	wpts = EditLevel->waypoints.arr;
-	for (i = 0; i < EditLevel->waypoints.size; i++) {
-		if ((wpts[i].x == xpos) && (wpts[i].y == ypos))
-			break;
+	wpnum = get_waypoint(EditLevel, xpos, ypos);
+	if (wpnum < 0) {
+		// No waypoint is currently targeted.
+		return;
 	}
 
-	if (i == EditLevel->waypoints.size) {
-		sprintf(VanishingMessage, _("Sorry, don't know which waypoint you mean."));
-		VanishingMessageEndDate = SDL_GetTicks() + 7000;
+	if (OriginWaypoint == -1) {
+		// Set the origin waypoint for the next connection.
+		OriginWaypoint = wpnum;
+		return;
+	}
+
+	if (OriginWaypoint != wpnum) {
+		// Toggle a connection between the origin waypoint and the currently targeted.
+		action_toggle_waypoint_connection(EditLevel, OriginWaypoint, wpnum, 1, 1);
+	}
+
+	OriginWaypoint = -1;
+}
+
+void lvledit_action_toggle_waypoint(int randomspawn)
+{
+	int wpnum = get_waypoint(EditLevel(), EditX(), EditY());
+	if (wpnum < 0) {
+		// If the waypoint doesn't exist at the map position, create it
+		action_create_waypoint(EditLevel(), EditX(), EditY(), randomspawn);
 	} else {
-		sprintf(VanishingMessage, _("You specified waypoint nr. %d."), i);
-		VanishingMessageEndDate = SDL_GetTicks() + 7000;
-		if (OriginWaypoint == (-1)) {
-			OriginWaypoint = i;
-			strcat(VanishingMessage, _("\nIt has been marked as the origin of the next connection."));
-			DebugPrintf(1, "\nWaypoint nr. %d. selected as origin\n", i);
+		// An existing waypoint will be removed or have its
+		// randomspawn flag toggled
+		waypoint *wpts = EditLevel()->waypoints.arr;
+		if (randomspawn) {
+			action_toggle_waypoint_randomspawn(EditLevel(), wpts[wpnum].x, wpts[wpnum].y);
 		} else {
-			if (OriginWaypoint == i) {
-				strcat(VanishingMessage, _("\n\nOrigin==Target --> Connection Operation cancelled."));
-				OriginWaypoint = (-1);
-			} else {
-				sprintf(VanishingMessage, _("\n\nOrigin: %d Target: %d. Operation makes sense."), OriginWaypoint, i);
-				if (action_toggle_waypoint_connection(EditLevel, OriginWaypoint, i, 1, 1) < 0) {
-					strcat(VanishingMessage, _("\nOperation done, connection removed."));
-				} else {
-					strcat(VanishingMessage, _("\nOperation done, connection added."));
-				}
-				OriginWaypoint = (-1);
-			}
+			action_remove_waypoint(EditLevel(), wpts[wpnum].x, wpts[wpnum].y);
 		}
 	}
+}
 
-	return;
+void action_set_floor_layer(Level EditLevel, int x, int y, int layer, int type)
+{
+	if (layer >= EditLevel->floor_layers)
+		EditLevel->floor_layers++;
 
+	int old = EditLevel->map[y][x].floor_values[layer];
+	EditLevel->map[y][x].floor_values[layer] = type;
+	action_push(ACT_TILE_FLOOR_SET, x, y, layer, old);
 }
 
 void action_set_floor(Level EditLevel, int x, int y, int type)
 {
-	int old = EditLevel->map[y][x].floor_value;
-	EditLevel->map[y][x].floor_value = type;
-	action_push(ACT_TILE_FLOOR_SET, x, y, old);
+	// Underlay floor tiles are placed in layer #0.
+	// Overlay floor tiles are placed in layer #1.
+	int layer = type < MAX_UNDERLAY_FLOOR_TILES ? 0 : 1;
+	action_set_floor_layer(EditLevel, x, y, layer, type);
 }
 
-static int action_change_map_label(level *EditLevel, int i, char *name, int x, int y, int undoable)
+void action_change_floor_layer(level *lvl, int layer)
+{
+	int old_floor_layer = current_floor_layer;
+	current_floor_layer = layer;
+	action_push(ACT_CHANGE_FLOOR_LAYER, old_floor_layer);
+}
+
+static void action_change_map_label(level *EditLevel, int i, char *name, int x, int y)
 {
 	struct map_label *map_label;
 	char *old_label = NULL;
@@ -572,26 +557,19 @@ static int action_change_map_label(level *EditLevel, int i, char *name, int x, i
 		del_map_label(EditLevel, old_label);
 	}
 
-	// Create the undo action if appropriate
-	if (undoable) {
-		action_push(ACT_SET_MAP_LABEL, i, old_label, x, y);
-	} else {
-		free(old_label);
-	}
+	action_push(ACT_SET_MAP_LABEL, i, old_label, x, y);
 
 	// If the new label is empty, we are done
 	if (!name || !strlen(name))
-		return 0;
+		return;
 
 	name = strdup(name);
 
 	// Create a new map label at the position of cursor
 	add_map_label(EditLevel, x, y, name);
-
-	return undoable;
 }
 
-void level_editor_action_change_map_label_user(level *EditLevel)
+void level_editor_action_change_map_label_user(level *EditLevel, float x, float y)
 {
 	struct map_label *map_label = NULL;
 	char *name;
@@ -605,8 +583,8 @@ void level_editor_action_change_map_label_user(level *EditLevel)
 	for (i = 0; i < EditLevel->map_labels.size; i++) {
 		map_label = &ACCESS_MAP_LABEL(EditLevel->map_labels, i);
 
-		if ((fabsf(map_label->pos.x + 0.5 - Me.pos.x) < 0.5) && 
-			 (fabsf(map_label->pos.y + 0.5 - Me.pos.y) < 0.5)) {
+		if ((fabsf(map_label->pos.x + 0.5 - x) < 0.5) &&
+			 (fabsf(map_label->pos.y + 0.5 - y) < 0.5)) {
 			
 			// Use the old label as a suggestion
 			old_name = map_label->label_name;
@@ -625,7 +603,7 @@ void level_editor_action_change_map_label_user(level *EditLevel)
 			return;
 		}
 
-		map_label = get_map_label(EditLevel, name);
+		map_label = map_label_exists(name);
 		if (!map_label) {
 			// When the new name of the map label does not exist, we are done
 			break;
@@ -634,7 +612,7 @@ void level_editor_action_change_map_label_user(level *EditLevel)
 		// When the new name already exists, we must not create an other map
 		// label with the same name, but we want to display an alert window,
 		// and then go back to input box with the name still present
-		alert_window(_("The new name of the map label already exists on this map, please choose an other name."));
+		alert_window("%s", _("The new name of the map label already exists, please choose an other name."));
 
 		// Copy the name in order to have it in the input box
 		strcpy(suggested_label, name);
@@ -645,7 +623,44 @@ void level_editor_action_change_map_label_user(level *EditLevel)
 	}
 
 	// Change a map label when the name enter by the user is valid
-	action_change_map_label(EditLevel, i, name, rintf(Me.pos.x - 0.5), rintf(Me.pos.y - 0.5), 1);
+	action_change_map_label(EditLevel, i, name, rintf(x - 0.5), rintf(y - 0.5));
+}
+
+/**
+ * @brief Create a map label on the map and push this action on the undo/redo stack.
+ *
+ * @param lvl Pointer towards the level where the map label is created.
+ * @param x The x position of the map label.
+ * @param y The y position of the map label.
+ * @param label_name The name of the map label.
+ */
+void action_create_map_label(level *lvl, int x, int y, char *label_name)
+{
+	add_map_label(lvl, x, y, label_name);
+
+	action_push(ACT_REMOVE_MAP_LABEL, x, y);
+}
+
+/**
+ * @brief Remove a map label on the map and push this action on the undo/redo stack.
+ *
+ * @param lvl Pointer towards the level where the map label is removed.
+ * @param x The x position of the map label.
+ * @param y The y position of the map label.
+ */
+void action_remove_map_label(level *lvl, int x, int y)
+{
+	char *old_label_name;
+	map_label *m;
+
+	m = get_map_label_from_coords(lvl, x, y);
+	if (m) {
+		old_label_name = strdup(m->label_name);
+
+		del_map_label(lvl, m->label_name);
+
+		action_push(ACT_CREATE_MAP_LABEL, x, y, old_label_name);
+	}
 }
 
 /**
@@ -656,11 +671,11 @@ void level_editor_action_change_map_label_user(level *EditLevel)
 void action_jump_to_level(int target_level, double x, double y)
 {
 	// When the user wants to change the current edited level, reset tools
-	leveleditor_reset_tools();
+	lvledit_reset_tools();
 
 	action_push(ACT_JUMP_TO_LEVEL, EditLevel()->levelnum, Me.pos.x, Me.pos.y);	//< sets undo or redo stack, depending on push_mode state
 	reset_visible_levels();
-	Teleport(target_level, (float)x, (float)y, FALSE);
+	Teleport(target_level, (float)x, (float)y, FALSE, TRUE);
 }
 
 /**
@@ -704,6 +719,9 @@ static void action_do(level * level, action * a)
 	case ACT_REMOVE_WAYPOINT:
 		action_remove_waypoint(level, a->d.delete_waypoint.x, a->d.delete_waypoint.y);
 		break;
+	case ACT_MOVE_WAYPOINT:
+		action_move_waypoint(level, a->d.move_waypoint.w, a->d.move_waypoint.newx, a->d.move_waypoint.newy);
+		break;
 	case ACT_TOGGLE_WAYPOINT_RSPAWN:
 		action_toggle_waypoint_randomspawn(level, a->d.toggle_waypoint_rspawn.x, a->d.toggle_waypoint_rspawn.y);
 		break;
@@ -711,7 +729,7 @@ static void action_do(level * level, action * a)
 		action_toggle_waypoint_connection(level, a->d.toggle_waypoint_connection.x, a->d.toggle_waypoint_connection.y, 1, 1);
 		break;
 	case ACT_TILE_FLOOR_SET:
-		action_set_floor(level, a->d.change_floor.x, a->d.change_floor.y, a->d.change_floor.type);
+		action_set_floor_layer(level, a->d.change_floor.x, a->d.change_floor.y, a->d.change_floor.layer, a->d.change_floor.type);
 		break;
 	case ACT_MULTIPLE_ACTIONS:
 		ErrorMessage(__FUNCTION__, "Passed a multiple actions meta-action as parameter. A real action is needed.\n", PLEASE_INFORM, IS_WARNING_ONLY);
@@ -720,12 +738,20 @@ static void action_do(level * level, action * a)
 		action_change_obstacle_label(level, a->d.change_obstacle_name.obstacle, a->d.change_obstacle_name.new_name, 1);
 		break;
 	case ACT_SET_MAP_LABEL:
-		action_change_map_label(level, a->d.change_label_name.id, a->d.change_label_name.new_name, a->d.change_label_name.x, a->d.change_label_name.y, 1);
+		action_change_map_label(level, a->d.change_label_name.id, a->d.change_label_name.new_name, a->d.change_label_name.x, a->d.change_label_name.y);
 		break;
 	case ACT_JUMP_TO_LEVEL:
 		action_jump_to_level(a->d.jump_to_level.target_level, a->d.jump_to_level.x, a->d.jump_to_level.y);
 		break;
-
+	case ACT_CHANGE_FLOOR_LAYER:
+		action_change_floor_layer(level, a->d.target_layer);
+		break;
+	case ACT_CREATE_MAP_LABEL:
+		action_create_map_label(level, a->d.create_map_label.x, a->d.create_map_label.y, a->d.create_map_label.label_name);
+		break;
+	case ACT_REMOVE_MAP_LABEL:
+		action_remove_map_label(level, a->d.delete_map_label.x, a->d.delete_map_label.y);
+		break;
 	}
 }
 
@@ -782,7 +808,7 @@ void level_editor_place_aligned_object(int positionid)
 {
 	float position_offset_x[9] = { 0, 0.5, 1.0, 0, 0.5, 1.0, 0, 0.5, 1.0 };
 	float position_offset_y[9] = { 1.0, 1.0, 1.0, 0.5, 0.5, 0.5, 0, 0, 0 };
-	struct leveleditor_categoryselect *cs = get_current_object_type();
+	struct widget_lvledit_categoryselect *cs = get_current_object_type();
 	int type = cs->indices[cs->selected_tile_nb];
 	moderately_finepoint pos;
 
@@ -792,6 +818,12 @@ void level_editor_place_aligned_object(int positionid)
 	pos.x = (int)Me.pos.x + position_offset_x[positionid];
 	pos.y = (int)Me.pos.y + position_offset_y[positionid];
 
+	if (!pos_inside_level(pos.x, pos.y, EditLevel()))
+	{
+		// Do not place an aligned object outside the current level.
+		return;
+	}
+
 	switch (cs->type) {
 	case OBJECT_OBSTACLE:
 			action_create_obstacle_user(EditLevel(), pos.x, pos.y, type);
@@ -832,6 +864,7 @@ void CreateNewMapLevel(int level_num)
 	NewLevel->levelnum = level_num;
 	NewLevel->xlen = 90;
 	NewLevel->ylen = 90;
+	NewLevel->floor_layers = 1;
 	NewLevel->light_bonus = 19;
 	NewLevel->minimum_light_value = 19;
 	NewLevel->infinite_running_on_this_level = FALSE;
@@ -845,10 +878,10 @@ void CreateNewMapLevel(int level_num)
 	for (i = 0; i < NewLevel->ylen; i++) {
 		NewLevel->map[i] = MyMalloc(NewLevel->xlen * sizeof(map_tile));
 		for (k = 0; k < NewLevel->xlen; k++) {
-			NewLevel->map[i][k].floor_value = ISO_FLOOR_SAND;
-			for (l = 0; l < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; l++) {
-				NewLevel->map[i][k].obstacles_glued_to_here[l] = (-1);
-			}
+			NewLevel->map[i][k].floor_values[0] = ISO_FLOOR_SAND;
+			for (l = 1; l < MAX_FLOOR_LAYERS; l++)
+				NewLevel->map[i][k].floor_values[l] = ISO_FLOOR_EMPTY;
+			dynarray_init(&NewLevel->map[i][k].glued_obstacles, 0, sizeof(int));
 		}
 	}
 	// Now we initialize the level jump interface variables with 'empty' values
@@ -887,16 +920,19 @@ void CreateNewMapLevel(int level_num)
 	dynarray_init(&NewLevel->waypoints, 10, sizeof(struct waypoint));
 	
 	curShip.AllLevels[level_num] = NewLevel;
-
-	glue_obstacles_to_floor_tiles_for_level(level_num);
-
-};				// void CreateNewMapLevel( int )
+}
 
 void delete_map_level(int lnum)
 {
 	int i;
 	level *l = curShip.AllLevels[lnum];
 
+	free(l->Levelname);
+	free(l->Background_Song_Name);
+
+	// Clear out all the existing glue information.
+	free_glued_obstacles(l);
+
 	// Delete floor tiles
 	for (i = 0; i < l->ylen; i++)
 		free(l->map[i]);
@@ -916,6 +952,15 @@ void delete_map_level(int lnum)
 			curShip.AllLevels[i]->jump_target_east = -1;
 	}
 
+	// Free obstacle extensions.
+	dynarray_free(&l->obstacle_extensions);
+
+	// Free map labels.
+	dynarray_free(&l->map_labels);
+
+	// Free waypoints.
+	dynarray_free(&l->waypoints);
+
 	// Free memory
 	free(curShip.AllLevels[lnum]);
 
@@ -991,7 +1036,7 @@ void level_editor_edit_chest(obstacle *o)
 		done = GreatShopInterface(chest_nb_items, chest_items, 1, user_items, &shop_order);
 
 		// BUY removes an item from the chest
-		// SELL spaws the drop item interface
+		// SELL spawns the drop item interface
 		switch (shop_order.shop_command) {
 		case BUY_1_ITEM:
 			DeleteItem(chest_items[shop_order.item_selected]);
@@ -1018,4 +1063,4 @@ void level_editor_edit_chest(obstacle *o)
 	}
 }
 
-#undef _leveledijtor_action_c
+#undef _leveleditor_actions_c
