@@ -23,7 +23,7 @@
  *
  */
 
-#define _leveleditor_map_c
+#define _widget_lvledit_map_c
 
 #include "system.h"
 
@@ -32,8 +32,6 @@
 #include "global.h"
 #include "proto.h"
 
-#include "SDL_rotozoom.h"
-
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_actions.h"
 #include "lvledit/lvledit_map.h"
@@ -120,18 +118,13 @@ static void move_obstacles(level *EditLevel, int x, int y)
 		if (o->type <= (-1))
 			continue;
 
-		// Move the obstacle
-		o->pos.x += x;
-		o->pos.y += y;
-		
-		if (!pos_inside_level(o->pos.x, o->pos.y, EditLevel)) {
+		if (!pos_inside_level(o->pos.x + x, o->pos.y + y, EditLevel)) {
 			// When the obstacle is outside of the map, we must remove it
-			action_remove_obstacle(EditLevel, o);
+			del_obstacle(o);
+		} else {
+			move_obstacle(o, o->pos.x + x, o->pos.y + y);
 		}
 	}
-
-	// And finally, re-glue all obstacles to the map
-	glue_obstacles_to_floor_tiles_for_level(EditLevel->levelnum);
 }
 
 /**
@@ -217,11 +210,12 @@ void insert_line_south(level *EditLevel)
 	
 	// Create the new line, and fill it with default values
 	EditLevel->map[EditLevel->ylen - 1] = MyMalloc((EditLevel->xlen + 1) * sizeof(map_tile));
+	map_tile *tiles = EditLevel->map[EditLevel->ylen - 1];
 	for (i = 0; i < EditLevel->xlen; i++) {
-		EditLevel->map[EditLevel->ylen - 1][i].floor_value = ISO_FLOOR_SAND;
-		for (j = 0; j < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; j++) {
-			EditLevel->map[EditLevel->ylen - 1][i].obstacles_glued_to_here[j] = (-1);
-		}
+		tiles[i].floor_values[0] = ISO_FLOOR_SAND;
+		for (j = 1; j < MAX_FLOOR_LAYERS; j++)
+			EditLevel->map[EditLevel->ylen - 1][i].floor_values[j] = ISO_FLOOR_EMPTY;
+		dynarray_init(&EditLevel->map[EditLevel->ylen - 1][i].glued_obstacles, 0, sizeof(int));
 	}
 }
 
@@ -242,10 +236,10 @@ void insert_column_east(level *EditLevel)
 	// We have to enlarge each map line, and fill those new tiles with default values
 	for (i = 0; i < EditLevel->ylen; i++) {
 		MapPointer = (map_tile*)realloc(EditLevel->map[i], sizeof(map_tile) * (EditLevel->xlen + 1));
-		MapPointer[EditLevel->xlen - 1].floor_value = ISO_FLOOR_SAND;
-		for (j = 0; j < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; j++) {
-			MapPointer[EditLevel->xlen - 1].obstacles_glued_to_here[j] = (-1);
-		}
+		MapPointer[EditLevel->xlen - 1].floor_values[0] = ISO_FLOOR_SAND;
+		for (j = 1; j < MAX_FLOOR_LAYERS; j++)
+			MapPointer[EditLevel->xlen - 1].floor_values[j] = ISO_FLOOR_EMPTY;
+		dynarray_init(&MapPointer[EditLevel->xlen - 1].glued_obstacles, 0, sizeof(int));
 		EditLevel->map[i] = MapPointer;
 	}
 }
@@ -286,6 +280,8 @@ void remove_column_east(level *EditLevel)
 	if (EditLevel->xlen - 1 < MIN_MAP_LINES)
 		return;
 
+	free_glued_obstacles(EditLevel);
+
 	// Remove a column at the east is always easy, we must just modify the
 	// value of size on the horizontal axis, allocation of new memory or other
 	// things are not necessary
@@ -295,6 +291,7 @@ void remove_column_east(level *EditLevel)
 	// (ie. obstacles, map labels, items, waypoints) but remove those which are
 	// outside of the map
 	move_all_objects(EditLevel, 0, 0);
+	teleport_to_level_center(EditLevel->levelnum);
 }
 
 /**
@@ -309,6 +306,8 @@ void remove_column_west(level *EditLevel)
 	if (EditLevel->xlen - 1 < MIN_MAP_LINES)
 		return;
 
+	free_glued_obstacles(EditLevel);
+
 	// Now the new memory and everything is done.  All we
 	// need to do is move the information to the east
 	for (i = 0; i < EditLevel->ylen; i++) {
@@ -320,6 +319,7 @@ void remove_column_west(level *EditLevel)
 
 	// Now we also have to shift the position of all elements
 	move_all_objects(EditLevel, -1, 0);
+	teleport_to_level_center(EditLevel->levelnum);
 }
 
 /**
@@ -333,6 +333,8 @@ void remove_line_north(level *EditLevel)
 	if (EditLevel->ylen - 1 < MIN_MAP_LINES)
 		return;
 
+	free_glued_obstacles(EditLevel);
+
 	// Now we do some shifting of lines
 	free(EditLevel->map[0]);
 	for (i = 0; i < EditLevel->ylen - 1; i++) {
@@ -343,6 +345,7 @@ void remove_line_north(level *EditLevel)
 
 	// Now we also have to shift the position of all elements
 	move_all_objects(EditLevel, 0, -1);
+	teleport_to_level_center(EditLevel->levelnum);
 }
 
 /**
@@ -354,6 +357,8 @@ void remove_line_south(level *EditLevel)
 	if (EditLevel->ylen - 1 < MIN_MAP_LINES)
 		return;
 
+	free_glued_obstacles(EditLevel);
+
 	// Remove a line at the very south is always easy, we must just modify the
 	// value of size on the vertical axis, allocation of new memory or other
 	// things are not necessary
@@ -363,4 +368,31 @@ void remove_line_south(level *EditLevel)
 	// (ie. obstacles, map labels, items, waypoints) but remove those which are
 	// outside of the map
 	move_all_objects(EditLevel, 0, 0);
+	teleport_to_level_center(EditLevel->levelnum);
+}
+
+/**
+ * Save the map - either to the real location, or to the home directory 
+ * as a fallback if the main location is not writable.
+ *
+ */
+void save_map(void)
+{
+	char fname[2048];
+	find_file("levels.dat", MAP_DIR, fname, 0);
+	if (SaveShip(fname, TRUE, 0)) {
+		ErrorMessage(__FUNCTION__, "Saving ship file to %s failed, possibly because of permission issues. Saving your ship to %s instead.\n", NO_NEED_TO_INFORM, IS_WARNING_ONLY, MAP_DIR, our_config_dir);
+		alert_window("Saving levels to %s/levels.dat instead of the default location map/levels.dat.", our_config_dir);
+		sprintf(fname, "%s/levels.dat", our_config_dir);
+		SaveShip(fname, TRUE, 0);
+	}
+
+	find_file("ReturnOfTux.droids", MAP_DIR, fname, 0);
+	if (save_special_forces(fname)) {
+		sprintf(fname, "%s/ReturnOfTux.droids", our_config_dir);
+		save_special_forces(fname);
+	}
+
+	CenteredPutString(Screen, 11 * FontHeight(Menu_BFont), _("Your ship was saved..."));
+	our_SDL_flip_wrapper();
 }
