@@ -31,21 +31,26 @@
 #include "global.h"
 #include "proto.h"
 
-#include "SDL_rotozoom.h"
-
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_actions.h"
 #include "lvledit/lvledit_map.h"
 #include "lvledit/lvledit_validator.h"
 #include "lvledit/lvledit_menu.h"
+#include "lvledit/lvledit_widgets.h"
 
 #include "mapgen/mapgen.h"
 
-int EditLevelNrPopup(void)
+/**
+ * Get a natural value from the user
+ * @param displayed_text Text to be displayed
+ * @param suggested_val Suggested value for the number
+ * @return Input number, -1 if the input is not made of digits only and -2 if the input is NULL
+ */
+int get_number_popup(const char *displayed_text, const char *suggested_val)
 {
 	char *str, *cstr;
 	int tgt = 0;
-	str = GetEditableStringInPopupWindow(1000, _("\n Please enter new level number: \n\n"), "");
+	str = GetEditableStringInPopupWindow(10, displayed_text, suggested_val);
 	if (str) {
 		cstr = str;
 		while (*cstr) {
@@ -57,13 +62,13 @@ int EditLevelNrPopup(void)
 		if (!tgt)
 			tgt = atoi(str);
 	} else
-		tgt = -1;
+		tgt = -2;
 	
 	free(str);
 
 	return (tgt);
 
-}				// int EditLevelNrPopup ( void )
+}
 
 /**
  * This a a menu interface to allow to edit the level dimensions in a
@@ -93,7 +98,7 @@ void EditLevelDimensions(void)
 
 	while (!proceed_now) {
 
-		InitiateMenu(-1);
+		InitiateMenu("--EDITOR_BACKGROUND--");
 
 		i = 0;
 		sprintf(Options[i], _("North"));
@@ -133,7 +138,7 @@ void EditLevelDimensions(void)
 		MenuTexts[i++] = _("Back");
 		MenuTexts[i++] = "";
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, -1, FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -200,7 +205,6 @@ void EditLevelDimensions(void)
 		}
 
 		gps_transform_map_init();
-		dirty_animated_obstacle_lists(EditLevel->levelnum);
 
 	}			// while (!proceed_now)
 
@@ -220,8 +224,6 @@ static void SetLevelInterfaces(void)
 	int i;
 	Level EditLevel;
 
-	EditLevel = curShip.AllLevels[Me.pos.z];
-
 	enum {
 		JUMP_TARGET_NORTH = 1,
 		JUMP_TARGET_EAST,
@@ -232,7 +234,7 @@ static void SetLevelInterfaces(void)
 
 	while (!proceed_now) {
 		EditLevel = curShip.AllLevels[Me.pos.z];
-		InitiateMenu(-1);
+		InitiateMenu("--EDITOR_BACKGROUND--");
 
 		i = 0;
 
@@ -267,7 +269,7 @@ static void SetLevelInterfaces(void)
 		MenuTexts[i++] = _("Back");
 		MenuTexts[i++] = "";
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, -1, FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
 
 		while (EnterPressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -294,11 +296,10 @@ static void SetLevelInterfaces(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = EditLevelNrPopup();
+			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
 
 			if (tgt >= -1 && tgt < curShip.num_levels) {
 				EditLevel->jump_target_north = (tgt);
-				proceed_now = !proceed_now;
 			}
 			gps_transform_map_dirty_flag = TRUE;
 			break;
@@ -310,11 +311,10 @@ static void SetLevelInterfaces(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = EditLevelNrPopup();
+			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
 
 			if (tgt >= -1 && tgt < curShip.num_levels) {
 				EditLevel->jump_target_east = (tgt);
-				proceed_now = !proceed_now;
 			}
 			gps_transform_map_dirty_flag = TRUE;
 			break;
@@ -326,11 +326,10 @@ static void SetLevelInterfaces(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = EditLevelNrPopup();
+			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
 
 			if (tgt >= -1 && tgt < curShip.num_levels) {
 				EditLevel->jump_target_south = (tgt);
-				proceed_now = !proceed_now;
 			}
 			gps_transform_map_dirty_flag = TRUE;
 			break;
@@ -342,11 +341,10 @@ static void SetLevelInterfaces(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = EditLevelNrPopup();
+			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
 
 			if (tgt >= -1 && tgt < curShip.num_levels) {
 				EditLevel->jump_target_west = (tgt);
-				proceed_now = !proceed_now;
 			}
 			gps_transform_map_dirty_flag = TRUE;
 			break;
@@ -438,7 +436,7 @@ static void AddRemLevel(void)
 	game_status = INSIDE_MENU;
 
 	while (!proceed_now) {
-		InitiateMenu(-1);
+		InitiateMenu("--EDITOR_BACKGROUND--");
 		i = 0;
 		MenuTexts[i++] = _("Add New Level");
 		MenuTexts[i++] = _("Remove Current Level");
@@ -448,7 +446,7 @@ static void AddRemLevel(void)
 		while (EscapePressed())
 			SDL_Delay(1);
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, -1, FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -479,13 +477,12 @@ static void AddRemLevel(void)
 
 				gps_transform_map_dirty_flag = TRUE;
 				gps_transform_map_init();
-				dirty_animated_obstacle_lists(new_level_num);
 				// Teleporting Tux will re-render the menu background
 				reset_visible_levels();
 				action_jump_to_level_center(new_level_num);
 			}
 			else      //curShip.num_levels >= MAX_LEVELS
-				alert_window(_("Reached the maximum number of levels."));
+				alert_window("%s", _("Reached the maximum number of levels."));
 			break;
 		case REMOVE_CURRENT_LEVEL:
 			if (game_root_mode == ROOT_IS_GAME)
@@ -494,21 +491,22 @@ static void AddRemLevel(void)
 				SDL_Delay(1);
 
 			if (EditLevel()->levelnum == 0) {
-				alert_window("Cannot remove level number 0.\n");
+				alert_window("%s", _("Cannot remove level number 0.\n"));
 				break;
 			}
 
 			{
 				int tmp = EditLevel()->levelnum; // needed due to Teleport changing EditLevel() result
 
-				dirty_animated_obstacle_lists(tmp);
 				delete_map_level(tmp);
 
+				lvledit_reset_tools();
+
 				gps_transform_map_dirty_flag = TRUE;
 				gps_transform_map_init();
 				// Teleporting Tux will re-render the menu background
 				reset_visible_levels();
-				Teleport(0, 3, 3, FALSE);
+				Teleport(0, 3, 3, FALSE, TRUE);
 			}
 			break;
 		case LEAVE_OPTIONS_MENU:
@@ -534,18 +532,20 @@ static void LevelOptions(void)
 	int proceed_now = FALSE;
 	int MenuPosition = 1;
 	int tgt = -1;
-	int i;
+	int i, j;
 	int l = 0;
+	int *droid_types;
 
 	enum {
 		CHANGE_LEVEL_POSITION = 1,
 		SET_LEVEL_NAME,
 		EDIT_LEVEL_DIMENSIONS,
+		EDIT_LEVEL_FLOOR_LAYERS,
 		SET_LEVEL_INTERFACE_POSITION,
 		SET_RANDOM_LEVEL,
+		SET_RANDOM_DROIDS,
 		SET_TELEPORT_PAIR,
 		CHANGE_LIGHT,
-		CHANGE_INDOOR_OUTDOOR,
 		SET_BACKGROUND_SONG_NAME,
 		CHANGE_INFINITE_RUNNING,
 		ADD_REM_LEVEL,
@@ -554,7 +554,7 @@ static void LevelOptions(void)
 
 	while (!proceed_now) {
 
-		InitiateMenu(-1);
+		InitiateMenu("--EDITOR_BACKGROUND--");
 
 		i = 0;
 		sprintf(Options[i], _("Level"));
@@ -572,6 +572,11 @@ static void LevelOptions(void)
 		strcat(Options[i], Options[i + 1]);
 		MenuTexts[i] = Options[i];
 		i++;
+		sprintf(Options[i], _("Floor layers"));
+		sprintf(Options[i + 1], ":  %d", EditLevel()->floor_layers);
+		strcat(Options[i], Options[i + 1]);
+		MenuTexts[i] = Options[i];
+		i++;
 		sprintf(Options[i], _("Edge"));
 		strcat(Options[i], " ");
 		sprintf(Options[i + 1], _("Interface"));
@@ -600,6 +605,16 @@ static void LevelOptions(void)
 		MenuTexts[i] = Options[i];
 		i++;
 
+		droid_types = EditLevel()->random_droids.types;
+		sprintf(Options[i], "Random droids: %d Types: ", EditLevel()->random_droids.nr);
+		for (j = 0; j < EditLevel()->random_droids.types_size; j++) {
+				if (j)
+					strcat(Options[i], ", ");
+				strcat(Options[i], Druidmap[droid_types[j]].druidname);
+		}
+		MenuTexts[i] = Options[i];
+		i++;
+
 		sprintf(Options[i], "%s: %s", _("Teleport pair"),
 				mapgen_teleport_pair_str(EditLevel()->teleport_pair));
 		MenuTexts[i] = Options[i];
@@ -624,20 +639,12 @@ static void LevelOptions(void)
 		MenuTexts[i] = Options[i];
 		i++;
 
-		sprintf(Options[i], _("Daylight cycle"));
-		strcat(Options[i], ": ");
-		if (EditLevel()->use_underground_lighting)
-			strcat(Options[i], _("Indoors"));
-		else
-			(strcat(Options[i], _("Outdoors")));	//Or was it the opposite?
-		MenuTexts[i] = Options[i];
-		i++;
 		sprintf(Options[i], _("Background Music"));
 		sprintf(Options[i + 1], ": %s", EditLevel()->Background_Song_Name);
 		strcat(Options[i], Options[i + 1]);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Infinite Running"));
+		sprintf(Options[i], _("Infinite Running Stamina"));
 		strcat(Options[i], ": ");
 		if (EditLevel()->infinite_running_on_this_level)
 			strcat(Options[i], _("YES"));
@@ -652,7 +659,7 @@ static void LevelOptions(void)
 		while (EscapePressed())
 			SDL_Delay(1);
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, -1, FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -669,7 +676,7 @@ static void LevelOptions(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = EditLevelNrPopup();
+			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
 
 			if (tgt >= 0 && tgt < curShip.num_levels) {
 				if (level_exists(tgt)) {
@@ -683,6 +690,9 @@ static void LevelOptions(void)
 			EditLevel()->random_dungeon++;
 			EditLevel()->random_dungeon %= 3;
 			break;
+		case SET_RANDOM_DROIDS:
+			get_random_droids_from_user();
+			break;
 		case SET_TELEPORT_PAIR:
 			EditLevel()->teleport_pair = mapgen_cycle_teleport_pair(EditLevel()->teleport_pair);
 			break;
@@ -733,10 +743,23 @@ static void LevelOptions(void)
 				SDL_Delay(1);
 			EditLevelDimensions();
 			break;
-		case CHANGE_INDOOR_OUTDOOR:
+		case EDIT_LEVEL_FLOOR_LAYERS:
+			if (LeftPressed() || RightPressed())
+				break;
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
-			EditLevel()->use_underground_lighting = !EditLevel()->use_underground_lighting;
+			char popup_message[1000];
+			sprintf(popup_message, _("\n Please enter new number of floor layers (valid values are between 1 and %d): \n\n"), MAX_FLOOR_LAYERS);
+			char *new_floor_layers = GetEditableStringInPopupWindow(1000, popup_message, "");
+			if (new_floor_layers) {
+				char *endptr;
+				long layers = strtol(new_floor_layers, &endptr, 10);
+				if (layers > MAX_FLOOR_LAYERS) layers = MAX_FLOOR_LAYERS;
+				else if (layers < 1) layers = 1;
+				if (*endptr != *new_floor_layers)
+					EditLevel()->floor_layers = layers;
+				free(new_floor_layers);
+			}
 			break;
 		case CHANGE_INFINITE_RUNNING:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
@@ -791,6 +814,31 @@ static void LevelOptions(void)
 				}
 				break;
 
+			case EDIT_LEVEL_FLOOR_LAYERS:
+				if (LeftPressed()) {
+					int new_layers = EditLevel()->floor_layers - 1;
+					if (new_layers > 0) {
+						int x, y;
+						EditLevel()->floor_layers = new_layers;
+						// Clear the removed floor layer
+						for (y = 0; y < EditLevel()->ylen; y++) {
+							for (x = 0; x < EditLevel()->xlen; x++) {
+								EditLevel()->map[y][x].floor_values[new_layers] = ISO_FLOOR_EMPTY;
+							}
+						}
+					}
+					if (current_floor_layer >= EditLevel()->floor_layers)
+						current_floor_layer = EditLevel()->floor_layers - 1;
+					while (LeftPressed()) ;
+				}
+				if (RightPressed()) {
+					int new_layers = EditLevel()->floor_layers + 1;
+					if (new_layers <= MAX_FLOOR_LAYERS)
+						EditLevel()->floor_layers = new_layers;
+					while (RightPressed()) ;
+				}
+				break;
+
 			case CHANGE_LIGHT:
 				if (l == 0) {
 					if (RightPressed()) {
@@ -806,7 +854,7 @@ static void LevelOptions(void)
 						while (LeftPressed()) ;
 					}
 					reset_visible_levels();
-					Teleport(EditLevel()->levelnum, Me.pos.x, Me.pos.y, FALSE);
+					Teleport(EditLevel()->levelnum, Me.pos.x, Me.pos.y, FALSE, TRUE);
 					break;
 				} else {
 					if (RightPressed()) {
@@ -820,7 +868,7 @@ static void LevelOptions(void)
 						while (LeftPressed()) ;
 					}
 					reset_visible_levels();
-					Teleport(EditLevel()->levelnum, Me.pos.x, Me.pos.y, FALSE);
+					Teleport(EditLevel()->levelnum, Me.pos.x, Me.pos.y, FALSE, TRUE);
 					break;
 				}
 
@@ -847,7 +895,7 @@ static void AdvancedOptions(void)
 
 	while (!proceed_now) {
 
-		InitiateMenu(-1);
+		InitiateMenu("--EDITOR_BACKGROUND--");
 
 		i = 0;
 		MenuTexts[i++] = _("Run Map Level Validator");
@@ -858,7 +906,7 @@ static void AdvancedOptions(void)
 		while (EscapePressed())
 			SDL_Delay(1);
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, -1, FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -871,11 +919,7 @@ static void AdvancedOptions(void)
 		case RUN_MAP_VALIDATION:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
-			LevelValidation();
-			while (!SpacePressed() && !EnterPressed() && !MouseLeftPressed() && !EscapePressed())
-				SDL_Delay(0);
-			//Hack: eat all pending events.
-			input_handle();
+			level_validation();
 			break;
 		case LEAVE_OPTIONS_MENU:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
@@ -919,13 +963,14 @@ int DoLevelEditorMainMenu()
 		SAVE_LEVEL_POSITION,
 //              MANAGE_LEVEL_POSITION,
 		ESCAPE_FROM_MENU_POSITION,
+                SHOW_HELP,
 		QUIT_TO_MAIN_POSITION,
 		QUIT_POSITION,
 	};
 
 	while (!proceed_now) {
 
-		InitiateMenu(-1);
+		InitiateMenu("--EDITOR_BACKGROUND--");
 
 		i = 0;
 		sprintf(Options[i], _("Level"));
@@ -945,6 +990,7 @@ int DoLevelEditorMainMenu()
 			//              MenuTexts[i++] = " ";
 		}
 		MenuTexts[i++] = _("Continue Editing");
+                MenuTexts[i++] = _("Show Help");
 		MenuTexts[i++] = _("Quit to Main Menu");
 		MenuTexts[i++] = _("Exit FreedroidRPG");
 		MenuTexts[i++] = "";
@@ -952,7 +998,7 @@ int DoLevelEditorMainMenu()
 		while (EscapePressed())
 			SDL_Delay(1);
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, -1, FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -973,7 +1019,7 @@ int DoLevelEditorMainMenu()
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = EditLevelNrPopup();
+			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
 
 			if (tgt >= 0 && tgt < curShip.num_levels) {
 				if (level_exists(tgt)) {
@@ -1004,14 +1050,15 @@ int DoLevelEditorMainMenu()
 				break;
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
-			char fp[2048];
-			find_file("freedroid.levels", MAP_DIR, fp, 0);
-			SaveShip(fp, TRUE, 0);
-			CenteredPutString(Screen, 11 * FontHeight(Menu_BFont), _("Your ship was saved..."));
-			our_SDL_flip_wrapper();
+			save_map();
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 			break;
+                case SHOW_HELP:
+			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
+				SDL_Delay(1);
+			PlayATitleFile("level_editor_help.title");
+			break;
 		case QUIT_TO_MAIN_POSITION:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
@@ -1027,7 +1074,7 @@ int DoLevelEditorMainMenu()
 			break;
 		case QUIT_POSITION:
 			DebugPrintf(2, "\nvoid EscapeMenu( void ): Quit Requested by user.  Terminating...");
-			Terminate(0);
+			Terminate(EXIT_SUCCESS, TRUE);
 			break;
 		default:
 			break;
