@@ -32,8 +32,6 @@
 #include "global.h"
 #include "proto.h"
 
-#include "SDL_rotozoom.h"
-
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_actions.h"
 #include "lvledit/lvledit_display.h"
@@ -68,7 +66,6 @@ static struct leveleditor_place {
 	int r_tile_used;
 
 	/* Waypoint mode */
-	int origwp;
 	int startwp;
 	int nbactions;
 } state;
@@ -104,9 +101,9 @@ static int do_waypoint_route(int rspawn)
 
 	if (state.startwp != -1 && wpnum != state.startwp) {
 		// Connect to the previous waypoint
-		if (action_toggle_waypoint_connection(EditLevel(), state.startwp, wpnum, 0, 0) == 1)
+		if (action_toggle_waypoint_connection(EditLevel(), state.startwp, wpnum, 0, 1) == 1)
 			state.nbactions++;
-		if (action_toggle_waypoint_connection(EditLevel(), wpnum, state.startwp, 0, 0) == 1)
+		if (action_toggle_waypoint_connection(EditLevel(), wpnum, state.startwp, 0, 1) == 1)
 			state.nbactions++;
 	}
 
@@ -119,14 +116,8 @@ static int do_waypoint_route(int rspawn)
 		return 1;
 	}
 
-	if (!isnew && wpnum == state.origwp) {
-		end_waypoint_route();
-		return 1;
-	}
-
 	if (our_mode != CONNECT_WAYPOINT) {
 		our_mode = CONNECT_WAYPOINT;
-		state.origwp = wpnum;
 	}
 
 	state.startwp = wpnum;
@@ -237,98 +228,74 @@ static void end_rectangle_floor(int commit)
 		level_editor_action_undo();
 }
 
+/**
+ * Place an enemy on the map
+ * @param droid_type Type of droid to pe placed
+ */
+void place_enemy(int droid_type)
+{
+	gps pos;
+
+	pos.x = mouse_mapcoord.x;
+	pos.y = mouse_mapcoord.y;
+	pos.z = EditLevel()->levelnum;
+
+	place_special_force(pos, droid_type);
+}
+
+/**
+ * Place a map label on the map.
+ */
+static void place_map_label()
+{
+	level_editor_action_change_map_label_user(EditLevel(), mouse_mapcoord.x, mouse_mapcoord.y);
+}
+
 /*
  * Pick type of walls (vertical and horizontal).
- * This function is temporary, because it's terrible hack ...
  */
 static void pick_walls(int windex)
 {
+	int i;
+	struct obstacle_spec *spec = get_obstacle_spec(windex);
+	struct obstacle_group *group = find_obstacle_group(windex);
+
 	state.l_type.nw = state.l_type.ne = -1;
 	state.l_type.sw = state.l_type.se = -1;
+	state.l_type.h = state.l_type.v = -1;
 
-	switch (windex) {
-	case ISO_BRICK_WALL_CABLES_V:
-	case ISO_BRICK_WALL_CABLES_H:
-		state.l_type.v = ISO_BRICK_WALL_CABLES_V;
-		state.l_type.h = ISO_BRICK_WALL_CABLES_H;
-
-		// Pick the corners
-		state.l_type.nw = ISO_BRICK_WALL_CABLES_CORNER_3;
-		state.l_type.ne = ISO_BRICK_WALL_CABLES_CORNER_1;
-		state.l_type.sw = ISO_BRICK_WALL_CABLES_CORNER_2;
-		state.l_type.se = ISO_BRICK_WALL_CABLES_CORNER_4;
-		break;
-	case ISO_THICK_WALL_V:
-	case ISO_THICK_WALL_H:
-		state.l_type.v = ISO_THICK_WALL_V;
-		state.l_type.h = ISO_THICK_WALL_H;
-
-		// Pick the corners
-		state.l_type.nw = ISO_THICK_WALL_CORNER_NW;
-		state.l_type.ne = ISO_THICK_WALL_CORNER_NE;
-		state.l_type.sw = ISO_THICK_WALL_CORNER_SW;
-		state.l_type.se = ISO_THICK_WALL_CORNER_SE;
-		break;
-	case ISO_V_DOOR_LOCKED:
-	case ISO_H_DOOR_LOCKED:
-		state.l_type.v = ISO_V_DOOR_LOCKED;
-		state.l_type.h = ISO_H_DOOR_LOCKED;
-		break;
-	case ISO_V_DOOR_000_OPEN:
-	case ISO_H_DOOR_000_OPEN:
-		state.l_type.v = ISO_V_DOOR_000_OPEN;
-		state.l_type.h = ISO_H_DOOR_000_OPEN;
-		break;
-	case ISO_OUTER_WALL_W1:
-	case ISO_OUTER_WALL_N1:
-		state.l_type.v = ISO_OUTER_WALL_W1;
-		state.l_type.h = ISO_OUTER_WALL_N1;
-		break;
-	case ISO_OUTER_WALL_W2:
-	case ISO_OUTER_WALL_N2:
-		state.l_type.v = ISO_OUTER_WALL_W2;
-		state.l_type.h = ISO_OUTER_WALL_N2;
-		break;
-	case ISO_OUTER_WALL_W3:
-	case ISO_OUTER_WALL_N3:
-		state.l_type.v = ISO_OUTER_WALL_W3;
-		state.l_type.h = ISO_OUTER_WALL_N3;
-		break;
-	case ISO_OUTER_WALL_E1:
-	case ISO_OUTER_WALL_S1:
-		state.l_type.v = ISO_OUTER_WALL_E1;
-		state.l_type.h = ISO_OUTER_WALL_S1;
-		break;
-	case ISO_OUTER_WALL_E2:
-	case ISO_OUTER_WALL_S2:
-		state.l_type.v = ISO_OUTER_WALL_E2;
-		state.l_type.h = ISO_OUTER_WALL_S2;
-		break;
-	case ISO_OUTER_WALL_E3:
-	case ISO_OUTER_WALL_S3:
-		state.l_type.v = ISO_OUTER_WALL_E3;
-		state.l_type.h = ISO_OUTER_WALL_S3;
-		break;
-	case ISO_CAVE_WALL_V:
-	case ISO_CAVE_WALL_H:
-		state.l_type.v = ISO_CAVE_WALL_V;
-		state.l_type.h = ISO_CAVE_WALL_H;
-		
-		// Pick the corners
-		state.l_type.nw = ISO_CAVE_CORNER_NW;
-		state.l_type.ne = ISO_CAVE_CORNER_NE;
-		state.l_type.sw = ISO_CAVE_CORNER_SW;
-		state.l_type.se = ISO_CAVE_CORNER_SE;
-		break;
-	default:
-		if (obstacle_map[windex].flags & IS_HORIZONTAL) {
+	// If obstacle doesn't belong to any group it's assumed that
+	// it has only vertical and horizontal wall types. In this case,
+	// horizontal and vertical obstacle types should be subsequent
+	// obstacle types.
+	if (!group) {
+		if (spec->flags & IS_HORIZONTAL) {
 			state.l_type.h = windex;
 			state.l_type.v = windex - 1;
-		}
-		else {
+		} else {
 			state.l_type.v = windex;
-			state.l_type.h = windex + 1;		
+			state.l_type.h = windex + 1;
 		}
+		return;
+	}
+
+	// Find corners, horizontal and vertical wall types in the obstacle
+	// group.
+	for (i = 0; i < group->members.size; i++) {
+		int member = ((int *)(group->members.arr))[i];
+		unsigned int member_flags = get_obstacle_spec(member)->flags;
+		if (member_flags & CORNER_NW)
+			state.l_type.nw = member;
+		else if (member_flags & CORNER_NE)
+			state.l_type.ne = member;
+		else if (member_flags & CORNER_SE)
+			state.l_type.se = member;
+		else if (member_flags & CORNER_SW)
+			state.l_type.sw = member;
+		else if (member_flags & IS_HORIZONTAL)
+			state.l_type.h = member;
+		else if (member_flags & IS_VERTICAL)
+			state.l_type.v = member;
 	}
 }
 
@@ -338,7 +305,7 @@ static void pick_walls(int windex)
  */
 static int horizontal_wall(int type)
 {
-	return (obstacle_map[type].flags & IS_HORIZONTAL);
+	return (get_obstacle_spec(type)->flags & IS_HORIZONTAL);
 }
 
 /**
@@ -347,7 +314,7 @@ static int horizontal_wall(int type)
  */
 static int vertical_wall(int type)
 {
-	return (obstacle_map[type].flags & IS_VERTICAL);
+	return (get_obstacle_spec(type)->flags & IS_VERTICAL);
 }
 
 /**
@@ -485,7 +452,7 @@ static void line_moving_forwards(moderately_finepoint offset)
 	line_element *last_wall;	// The last wall of the line
 	int new_direction;			// New direction of the wall line
 	float distance;				// Difference of distance since last time
-	int type;						// The type of the new wall
+	int type = 0;						// The type of the new wall
 
 	// Get the last wall in the line of walls
 	last_wall = list_entry(state.l_elements_head.prev, line_element, list);
@@ -657,7 +624,7 @@ static void end_wall_line(int commit)
 		if (!commit) {
 			// When the user wants cancel the line of walls, we must remove all
 			// the walls
-			action_remove_obstacle(EditLevel(), e->address);
+			del_obstacle(e->address);
 		}
 		nb_actions++;
 	}
@@ -668,7 +635,7 @@ static void end_wall_line(int commit)
 
 int leveleditor_place_input(SDL_Event *event)
 {
-	struct leveleditor_categoryselect *cs = get_current_object_type();
+	struct widget_lvledit_categoryselect *cs = get_current_object_type();
 	int type = cs->indices[cs->selected_tile_nb];
 
 	if (!mouse_in_level) {
@@ -706,8 +673,16 @@ int leveleditor_place_input(SDL_Event *event)
 				place_item(type);
 				return 1;
 				break;
+			case OBJECT_ENEMY:
+				place_enemy(type);
+				return 1;
+				break;
+			case OBJECT_MAP_LABEL:
+				place_map_label();
+				return 1;
+				break;
 			default:
-				alert_window("Place tool does not support this type of object.");
+				alert_window("%s", _("Place tool does not support this type of object."));
 			}
 		}
 	} else if (our_mode == RECTANGLE_FLOOR) {
@@ -739,7 +714,7 @@ int leveleditor_place_display()
 		waypoint *wpts = EditLevel()->waypoints.arr;
 
 		draw_connection_between_tiles(wpts[state.startwp].x + 0.5, wpts[state.startwp].y + 0.5, mouse_mapcoord.x, mouse_mapcoord.y,
-					      GameConfig.zoom_is_on ? ZOOM_OUT : 0);
+					      GameConfig.zoom_is_on ? ZOOM_OUT : 0, wpts[state.startwp].suppress_random_spawn);
 	}
 
 	return 0;
@@ -760,4 +735,5 @@ void leveleditor_place_reset()
 	default:
 			break;
 	}
+	OriginWaypoint = -1;
 }
