@@ -1,7 +1,7 @@
 
 /* 
  *
- *   Copyright (c) 2009 Arthur Huillet
+ *   Copyright (c) 2009-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -23,5 +23,7 @@
  *
  */
 
-int leveleditor_place_input(SDL_Event * event);
-int leveleditor_place_display();
+int leveleditor_place_input(SDL_Event *);
+int leveleditor_place_display(void);
+
+void leveleditor_place_reset(void);
