@@ -32,8 +32,6 @@
 #include "global.h"
 #include "proto.h"
 
-#include "SDL_rotozoom.h"
-
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_actions.h"
 #include "lvledit/lvledit_display.h"
@@ -55,17 +53,20 @@ static struct leveleditor_select {
 enum { DISABLED, FD_RECT, FD_RECTDONE, DRAGDROP } mode;
 
 struct selected_element {
-	enum leveleditor_object_type type;
+	enum lvledit_object_type type;
 	struct list_head node;
 	void *data;
 };
 
+#define ALL_FLOOR_LAYERS (-1)
+
 /* Selected floor tiles are wrapped into a struct lvledit_map_tile, that holds 
  * coordinate information. struct map_tile does not have coordinate information,
  * and the game doesn't need it, only the leveleditor. */
 struct lvledit_map_tile {
 	map_tile *tile;
 	point coord;
+	int layer;
 };
 
 static LIST_HEAD(selected_elements);
@@ -115,6 +116,18 @@ int element_in_selection(void *data)
 			if (((struct lvledit_map_tile *) (e->data))->tile == data)
 				return 1;
 			break;
+		case OBJECT_WAYPOINT:
+			// We must compare the coordinates of the two waypoints
+			if (((waypoint *)e->data)->x == ((waypoint *)data)->x &&
+				((waypoint *)e->data)->y == ((waypoint *)data)->y)
+				return 1;
+			break;
+		case OBJECT_MAP_LABEL:
+			// We must compare the coordinates of the two map labels
+			if (((map_label *)e->data)->pos.x == ((map_label *)data)->pos.x &&
+				((map_label *)e->data)->pos.y == ((map_label *)data)->pos.y)
+				return 1;
+			break;
 		default:
 			if (e->data == data)
 				return 1;
@@ -143,6 +156,8 @@ static void calc_min_max_selection(struct list_head *list, moderately_finepoint
 	waypoint *w;
 	obstacle *o;
 	item *it;
+	map_label *m;
+	enemy *en;
 	
 	cmin->x = 9999;
 	cmin->y = 9999;
@@ -171,12 +186,37 @@ static void calc_min_max_selection(struct list_head *list, moderately_finepoint
 
 			__calc_min_max(w->x, w->y, cmin, cmax);
 			break;
+		case OBJECT_MAP_LABEL:
+			m = e->data;
+
+			__calc_min_max(m->pos.x, m->pos.y, cmin, cmax);
+			break;
+		case OBJECT_ENEMY:
+			en = e->data;
+
+			__calc_min_max(en->pos.x, en->pos.y, cmin, cmax);
+			break;
 		default:
 			;
 		}
 	}
 }
 
+static int set_floor_layers(level *lvl, struct lvledit_map_tile *tile)
+{
+	int i;
+	struct map_tile *t = tile->tile;
+
+	if (tile->layer == ALL_FLOOR_LAYERS) {
+		for (i = 0; i < lvl->floor_layers; i++)
+			action_set_floor_layer(EditLevel(), tile->coord.x, tile->coord.y, i, t->floor_values[i]);
+		return i;
+	} else {
+		action_set_floor_layer(lvl, tile->coord.x, tile->coord.y, current_floor_layer, t->floor_values[tile->layer]);
+		return 1;
+	}
+}
+
 point selection_start() {
 	return state.rect_start;
 }
@@ -187,7 +227,7 @@ point selection_len() {
 
 int selection_type()
 {
-	struct leveleditor_categoryselect *cs = get_current_object_type();
+	struct widget_lvledit_categoryselect *cs = get_current_object_type();
 
 	if (!cs) {
 		return OBJECT_NONE;
@@ -196,7 +236,7 @@ int selection_type()
 	return cs->type;
 }
 
-static void add_object_to_list(struct list_head *list, void *data, enum leveleditor_object_type type)
+static void add_object_to_list(struct list_head *list, void *data, enum lvledit_object_type type)
 {
 	struct selected_element *e = MyMalloc(sizeof(struct selected_element));
 	e->type = type;
@@ -211,9 +251,8 @@ static void __clear_selected_list(struct list_head *lst, int nbelem)
 	list_for_each_entry_safe(e, ne, lst, node) {
 		if (nbelem-- == 0)
 			return;
-
-		if (e->type == OBJECT_FLOOR) {
-			/* Unselecting a floor tile requires freeing its wrapper structure. */				
+		if (e->type == OBJECT_FLOOR || e->type == OBJECT_WAYPOINT || e->type == OBJECT_MAP_LABEL) {
+			/* Unselecting floor tiles, waypoints or map labels requires freeing the duplicated data */
 			free(e->data);
 		}
 		list_del(&e->node);
@@ -237,8 +276,8 @@ static void select_floor_on_tile(int x, int y)
 static void select_obstacles_on_tile(int x, int y)
 {
 	int idx, a;
-	for (a = 0; a < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE && EditLevel()->map[y][x].obstacles_glued_to_here[a] != -1; a++) {
-		idx = EditLevel()->map[y][x].obstacles_glued_to_here[a];
+	for (a = 0; a < EditLevel()->map[y][x].glued_obstacles.size; a++) {
+		idx = ((int *)(EditLevel()->map[y][x].glued_obstacles.arr))[a];
 		if (!element_in_selection(&EditLevel()->obstacle_list[idx])) {
 		add_object_to_list(&selected_elements, &EditLevel()->obstacle_list[idx], OBJECT_OBSTACLE);
 			state.rect_nbelem_selected++;
@@ -254,7 +293,10 @@ static void select_waypoint_on_tile(int x, int y)
 	for (i = 0; i < EditLevel()->waypoints.size; i++) {
 		if (wpts[i].x == x && wpts[i].y == y) {
 			if (!element_in_selection(&wpts[i])) {
-				add_object_to_list(&selected_elements, &wpts[i], OBJECT_WAYPOINT);
+				waypoint *w = MyMalloc(sizeof(waypoint));
+				memcpy(w, &wpts[i], sizeof(waypoint));
+
+				add_object_to_list(&selected_elements, w, OBJECT_WAYPOINT);
 				state.rect_nbelem_selected++;
 			}
 		}
@@ -282,6 +324,41 @@ static void select_item_on_tile(int x, int y)
 	}
 }
 
+static void select_map_label_on_tile(int x, int y)
+{
+	map_label *labels = EditLevel()->map_labels.arr;
+	int i;
+
+	for (i = 0; i < EditLevel()->map_labels.size; i++) {
+		if (labels[i].pos.x == x && labels[i].pos.y == y) {
+			if (!element_in_selection(&labels[i])) {
+				map_label *m = MyMalloc(sizeof(map_label));
+				memcpy(m, &labels[i], sizeof(map_label));
+
+				add_object_to_list(&selected_elements, m, OBJECT_MAP_LABEL);
+				state.rect_nbelem_selected++;
+			}
+		}
+	}
+}
+
+static void select_special_forces_on_tile(int x, int y)
+{
+	enemy *en;
+
+	BROWSE_LEVEL_BOTS(en, EditLevel()->levelnum) {
+		if (!en->SpecialForce)
+			continue;
+
+		if ((int)en->pos.x == x && (int)en->pos.y == y) {
+			if (!element_in_selection(en)) {
+				add_object_to_list(&selected_elements, en, OBJECT_ENEMY);
+				state.rect_nbelem_selected++;
+			}
+		}
+	}
+}
+
 static void select_object_on_tile(int x, int y)
 {
 	switch (selection_type()) {
@@ -297,6 +374,12 @@ static void select_object_on_tile(int x, int y)
 	case OBJECT_ITEM:
 		select_item_on_tile(x, y);
 		break;
+	case OBJECT_MAP_LABEL:
+		select_map_label_on_tile(x, y);
+		break;
+	case OBJECT_ENEMY:
+		select_special_forces_on_tile(x, y);
+		break;
 	default:
 		ErrorMessage(__FUNCTION__,
 				"Abstract object type %d for leveleditor not supported.\n", PLEASE_INFORM, IS_WARNING_ONLY, selection_type());
@@ -426,58 +509,30 @@ static void start_drag_drop()
 	state.cur_drag_pos.y = mouse_mapcoord.y;
 }
 
-static void do_drag_drop_obstacle()
+static void do_drag_drop_obstacle(moderately_finepoint diff)
 {
 	struct selected_element *e;
-	moderately_finepoint cmin, cmax, diff;
-	
-	// Calculate the coordinates min/max of the selection
-	calc_min_max_selection(&selected_elements, &cmin, &cmax);
-	
-	// Calculate the new displacement of the selection
-	diff.x = mouse_mapcoord.x - state.cur_drag_pos.x;
-	diff.y = mouse_mapcoord.y - state.cur_drag_pos.y;
 
 	list_for_each_entry(e, &selected_elements, node) {
 		if (e->type != OBJECT_OBSTACLE)
 			return;
 
-		if ((cmax.y + diff.y) >= EditLevel()->ylen || (cmax.x + diff.x) >= EditLevel()->xlen
-			|| (cmin.x + diff.x) < 0 || (cmin.y + diff.y) < 0) {
-			// Do not place obstacles outside of the level
-			return;
-		}
-		
-		// Calculate the new coordinates of the obstacle
-		((obstacle *) (e->data))->pos.x += diff.x;
-		((obstacle *) (e->data))->pos.y += diff.y;
+		obstacle *o = ((obstacle *)(e->data));
+
+		move_obstacle(o, o->pos.x + diff.x, o->pos.y + diff.y);
 	}	
 	state.cur_drag_pos.x = mouse_mapcoord.x;
 	state.cur_drag_pos.y = mouse_mapcoord.y;
 }
 
-static void do_drag_drop_floor()
+static void do_drag_drop_floor(moderately_finepoint diff)
 {
 	struct selected_element *e;
 	struct lvledit_map_tile *t;
-	moderately_finepoint cmin, cmax, diff;
 	int changed_tiles = 0;
 
-	// Calculate the coordinates min/max of the selection
-	calc_min_max_selection(&clipboard_elements, &cmin, &cmax);
-	
-	// Calculate the new displacement of the selection
-	diff.x = mouse_mapcoord.x - state.cur_drag_pos.x;
-	diff.y = mouse_mapcoord.y - state.cur_drag_pos.y;
-
 	// Move the selection if the displacement exceeds half a tile
-	if (abs(diff.x) >= 0.5 || abs(diff.y) >= 0.5 ) {
-		
-		if ((cmax.y + diff.y) >= EditLevel()->ylen || (cmax.x + diff.x) >= EditLevel()->xlen
-			|| (cmin.x + diff.x) <= 0 || (cmin.y + diff.y) <= 0) {
-			// Do not place obstacles outside of the level
-			return;
-		}
+	if (fabsf(diff.x) >= 0.5 || fabsf(diff.y) >= 0.5 ) {
 
 		if ((state.drag_start.x != state.cur_drag_pos.x) 
 			|| (state.drag_start.y != state.cur_drag_pos.y)) {
@@ -502,49 +557,34 @@ static void do_drag_drop_floor()
 		list_for_each_entry(e, &clipboard_elements, node) {
 			if (e->type != OBJECT_FLOOR)
 				return;
-				
+
 			t = e->data;
 
 			// Calculate the new coordinates of the tile
-			t->coord.x += (int)diff.x;
-			t->coord.y += (int)diff.y;
+			t->coord.x += (int)rintf(diff.x);
+			t->coord.y += (int)rintf(diff.y);
 
 			// Set the floor for the new current tile
-			action_set_floor(EditLevel(), t->coord.x, t->coord.y, t->tile->floor_value);
-			changed_tiles++;
+			changed_tiles += set_floor_layers(EditLevel(), t);
 
 			// Select the new tile
 			select_floor_on_tile(t->coord.x, t->coord.y);
-		}		
-		state.cur_drag_pos.x += (int)diff.x;
-		state.cur_drag_pos.y += (int)diff.y;
+		}
+		state.cur_drag_pos.x += (int)rintf(diff.x);
+		state.cur_drag_pos.y += (int)rintf(diff.y);
 		
-		action_push(ACT_MULTIPLE_ACTIONS, changed_tiles);		
+		action_push(ACT_MULTIPLE_ACTIONS, changed_tiles);
 	}
 }
 
-static void do_drag_drop_item()
+static void do_drag_drop_item(moderately_finepoint diff)
 {
 	struct selected_element *e;
-	moderately_finepoint cmin, cmax, diff;
-	
-	// Calculate the coordinates min/max of the selection
-	calc_min_max_selection(&selected_elements, &cmin, &cmax);
-	
-	// Calculate the new displacement of the selection
-	diff.x = mouse_mapcoord.x - state.cur_drag_pos.x;
-	diff.y = mouse_mapcoord.y - state.cur_drag_pos.y;
 
 	list_for_each_entry(e, &selected_elements, node) {
 		if (e->type != OBJECT_ITEM)
 			return;
 
-		if ((cmax.y + diff.y) >= EditLevel()->ylen || (cmax.x + diff.x) >= EditLevel()->xlen
-			|| (cmin.x + diff.x) < 0 || (cmin.y + diff.y) < 0) {
-			// Do not place items outside of the level
-			return;
-		}
-		
 		// Calculate the new coordinates of the item
 		((item *) (e->data))->pos.x += diff.x;
 		((item *) (e->data))->pos.y += diff.y;
@@ -553,20 +593,71 @@ static void do_drag_drop_item()
 	state.cur_drag_pos.y = mouse_mapcoord.y;
 }
 
+static void do_drag_drop_waypoint(moderately_finepoint diff)
+{
+	struct selected_element *e;
+	waypoint *w, *w2;
+	int wp_pos;
+
+	// Move the selection if the displacement exceeds half a tile
+	if (fabsf(diff.x) >= 0.5 || fabsf(diff.y) >= 0.5 ) {
+		list_for_each_entry(e, &selected_elements, node) {
+			if (e->type != OBJECT_WAYPOINT)
+				return;
+
+			w = e->data;
+
+			// In order to retrieve the actual waypoint we must obtain its index in the level's waypoint array
+			wp_pos = get_waypoint(EditLevel(), w->x, w->y);
+
+			// Modify the selection coordinates
+			w->x += (int)rintf(diff.x);
+			w->y += (int)rintf(diff.y);
+
+			// Retrieve the original waypoint
+			w2 = &(((waypoint *)(EditLevel()->waypoints.arr))[wp_pos]);
+
+			// Modify the actual waypoint
+			move_waypoint(EditLevel(), w2, w->x, w->y);
+		}
+
+		state.cur_drag_pos.x += (int)rintf(diff.x);
+		state.cur_drag_pos.y += (int)rintf(diff.y);
+	}
+}
+
 static void do_drag_drop()
 {
+	moderately_finepoint cmin, cmax, diff;
+
+	// Calculate the coordinates min/max of the selection.
+	calc_min_max_selection(&selected_elements, &cmin, &cmax);
+
+	// Calculate the new displacement of the selection.
+	diff.x = mouse_mapcoord.x - state.cur_drag_pos.x;
+	diff.y = mouse_mapcoord.y - state.cur_drag_pos.y;
+
+	if ((cmax.y + diff.y) >= EditLevel()->ylen || (cmax.x + diff.x) >= EditLevel()->xlen
+		|| (cmin.x + diff.x) < 0 || (cmin.y + diff.y) < 0) {
+		// Do not place objects outside the level.
+		return;
+	}
+
 	switch (selection_type()) {
 	case OBJECT_OBSTACLE:
-		do_drag_drop_obstacle();
+		do_drag_drop_obstacle(diff);
 		break;
 	case OBJECT_FLOOR:
-		do_drag_drop_floor();
+		do_drag_drop_floor(diff);
 		break;
 	case OBJECT_ITEM:
-		do_drag_drop_item();
+		do_drag_drop_item(diff);
+		break;
+	case OBJECT_WAYPOINT:
+		do_drag_drop_waypoint(diff);
 		break;
 	default:
-		break;	
+		break;
 	}
 }
 
@@ -575,6 +666,8 @@ static void end_drag_drop()
 	struct selected_element *e;
 	int enb = 0;
 	double x, y;
+	int wp_pos;
+	waypoint *w, *w2;
 
 	// Set up undo actions
 	list_for_each_entry(e, &selected_elements, node) {
@@ -591,13 +684,26 @@ static void end_drag_drop()
 			action_push(ACT_MOVE_ITEM, (item *) e->data, x, y);
 			enb++;
 			break;
+		case OBJECT_WAYPOINT:
+			// In order to store the undo action for each waypoint we need to
+			// compute the initial position of the waypoint
+			w = e->data;
+			x = (w->x - (int)(state.cur_drag_pos.x - state.drag_start.x));
+			y = (w->y - (int)(state.cur_drag_pos.y - state.drag_start.y));
+
+			// In order to retrieve the actual waypoint we must obtain its index in the level's waypoints array
+			wp_pos = get_waypoint(EditLevel(), w->x, w->y);
+
+			// Retrieve the original waypoint and push the undo action
+			w2 = &(((waypoint *)(EditLevel()->waypoints.arr))[wp_pos]);
+			action_push(ACT_MOVE_WAYPOINT, w2, (int)x, (int)y);
+			enb++;
+			break;
 		default:
 			;
 		}
 	}
 
-	glue_obstacles_to_floor_tiles_for_level(EditLevel()->levelnum);
-
 	if (enb)
 		action_push(ACT_MULTIPLE_ACTIONS, enb);
 
@@ -612,7 +718,26 @@ static void end_drag_drop()
 	}
 }
 
-int level_editor_can_cycle_obs()
+static int level_editor_number_of_marked_items()
+{
+	item *it;
+	int i, num = 0;
+
+	for (i = 0; i < MAX_ITEMS_PER_LEVEL; i++) {
+		it = &EditLevel()->ItemList[i];
+
+		if (it->type == -1)
+			continue;
+
+		if ((fabsf(state.rect_start.x - it->pos.x) <= 0.5) && (fabsf(state.rect_start.y - it->pos.y) <= 0.5)) {
+			num++;
+		}
+	}
+
+	return num;
+}
+
+int level_editor_can_cycle_marked_object()
 {
 	if (mode != FD_RECTDONE) {
 		// We get called from the outside so check mode coherency first
@@ -621,40 +746,97 @@ int level_editor_can_cycle_obs()
 	// N key does nothing on a selection larger than one tile
 	if (abs(state.rect_len.x) != 1 || abs(state.rect_len.y) != 1)
 		return 0;
-
-	if ((EditLevel()->map[state.rect_start.y][state.rect_start.x].obstacles_glued_to_here[0] == -1) ||
-	    (EditLevel()->map[state.rect_start.y][state.rect_start.x].obstacles_glued_to_here[1] == -1))
+	
+	if (!pos_inside_level(state.rect_start.x, state.rect_start.y, EditLevel()))
 		return 0;
 
+	switch (selection_type()) {	
+		case OBJECT_OBSTACLE: 		
+			if (EditLevel()->map[state.rect_start.y][state.rect_start.x].glued_obstacles.size <= 1)
+				return 0;
+			break;		
+		case OBJECT_ITEM:
+			if (level_editor_number_of_marked_items() <= 1)
+				return 0;
+			break;
+		default:
+			return 0;
+			break;				
+	}
+
 	return 1;
 }
 
-void level_editor_cycle_marked_obstacle()
+static void level_editor_cycle_marked_obstacle()
 {
 	//This function will select only one of the obstacles attached to a given tile,
 	//and browse through them for each subsequent call.
 
-	if (!level_editor_can_cycle_obs())
-		return;
-
-	// clear the selection and take the next obstacle in the list
-	clear_selection(state.rect_nbelem_selected);
-	state.rect_nbelem_selected = 0;
-
-	if (EditLevel()->map[state.rect_start.y][state.rect_start.x].obstacles_glued_to_here[state.single_tile_mark_index] == -1) {
+	if (state.single_tile_mark_index >= EditLevel()->map[state.rect_start.y][state.rect_start.x].glued_obstacles.size) {
 		state.single_tile_mark_index = 0;
 	}
 
-	int idx = EditLevel()->map[state.rect_start.y][state.rect_start.x].obstacles_glued_to_here[state.single_tile_mark_index];
-add_object_to_list(&selected_elements, &EditLevel()->obstacle_list[idx], OBJECT_OBSTACLE);
+	int idx = ((int *)(EditLevel()->map[state.rect_start.y][state.rect_start.x].glued_obstacles.arr))[state.single_tile_mark_index];
+	add_object_to_list(&selected_elements, &EditLevel()->obstacle_list[idx], OBJECT_OBSTACLE);
+}
+
+static void level_editor_cycle_marked_item()
+{
+	item *it;
+	int i, j;	
+
+	if (state.single_tile_mark_index >= level_editor_number_of_marked_items()) {
+		state.single_tile_mark_index = 0;
+	}
+
+	for (i = 0,j = 0; i < MAX_ITEMS_PER_LEVEL; i++) {
+		it = &EditLevel()->ItemList[i];
+
+		if (it->type == -1)
+			continue;
+
+		if ((fabsf(state.rect_start.x - it->pos.x) <= 0.5) && (fabsf(state.rect_start.y- it->pos.y) <= 0.5)) {
+			if (state.single_tile_mark_index == j) {
+				break;
+			}
+
+			j++;
+		}
+ 
+	}
+
+	add_object_to_list(&selected_elements, it, OBJECT_ITEM);
+}
+
+void level_editor_cycle_marked_object()
+{
+	if (!level_editor_can_cycle_marked_object())
+		return;
+
+	clear_selection(state.rect_nbelem_selected);
+	state.rect_nbelem_selected = 0;	
+	
+	switch (selection_type()) {
+		case OBJECT_OBSTACLE:
+			level_editor_cycle_marked_obstacle();
+			break;
+		case OBJECT_ITEM:
+			level_editor_cycle_marked_item();
+			break;
+		default:
+			ErrorMessage(__FUNCTION__, "Unhandled selection type. \n", IS_WARNING_ONLY, NO_NEED_TO_INFORM);
+			break;
+	}
+	
 	state.single_tile_mark_index++;
-	state.rect_nbelem_selected++;
+	state.rect_nbelem_selected = 1;
 }
 
 void level_editor_copy_selection()
 {
 	struct selected_element *e;
 	struct lvledit_map_tile *t;	
+	map_label *m;
 	waypoint *w;
 	obstacle *o;
 	item *it;
@@ -687,6 +869,7 @@ void level_editor_copy_selection()
 			memcpy(t->tile, ((struct lvledit_map_tile *) (e->data))->tile, sizeof(map_tile));
 			t->coord.x = ((struct lvledit_map_tile *) (e->data))->coord.x;
 			t->coord.y = ((struct lvledit_map_tile *) (e->data))->coord.y;		
+			t->layer = GameConfig.show_all_floor_layers ? ALL_FLOOR_LAYERS : current_floor_layer;
 			
 			add_object_to_list(&clipboard_elements, t, OBJECT_FLOOR);
 			break;
@@ -702,12 +885,32 @@ void level_editor_copy_selection()
 
 			add_object_to_list(&clipboard_elements, w, OBJECT_WAYPOINT);
 			break;
+		case OBJECT_MAP_LABEL:
+			m = MyMalloc(sizeof(map_label));
+			memcpy(m, e->data, sizeof(map_label));
+
+			add_object_to_list(&clipboard_elements, m, OBJECT_MAP_LABEL);
+			break;
 		default:
 			;
 		}
 	}
 }
 
+static int clear_current_floor_layers(level *lvl, int coord_x, int coord_y)
+{
+	int i;
+
+	if (GameConfig.show_all_floor_layers) {
+		for (i = 0; i < lvl->floor_layers; i++)
+			action_set_floor_layer(lvl, coord_x, coord_y, i, ISO_FLOOR_EMPTY);
+		return i;
+	} else {
+		action_set_floor_layer(lvl, coord_x, coord_y, current_floor_layer, ISO_FLOOR_EMPTY);
+		return 1;
+	}
+}
+
 void level_editor_cut_selection()
 {
 	struct selected_element *e;
@@ -727,11 +930,9 @@ void level_editor_cut_selection()
 			break;
 		case OBJECT_FLOOR:
 			// We replace the tile we cut by a black tile
-			action_set_floor(EditLevel(), 
+			nbelem += clear_current_floor_layers(EditLevel(),
 				((struct lvledit_map_tile *) (e->data))->coord.x,
-				((struct lvledit_map_tile *) (e->data))->coord.y,
-				ISO_COMPLETELY_DARK);
-			nbelem++;
+				((struct lvledit_map_tile *) (e->data))->coord.y);
 			break;
 		case OBJECT_ITEM:
 			action_remove_item(EditLevel(), e->data);
@@ -741,6 +942,10 @@ void level_editor_cut_selection()
 			action_remove_waypoint(EditLevel(), ((waypoint *)(e->data))->x, ((waypoint *)(e->data))->y);
 			nbelem++;
 			break;
+		case OBJECT_MAP_LABEL:
+			action_remove_map_label(EditLevel(), ((map_label *)(e->data))->pos.x, ((map_label *)(e->data))->pos.y);
+			nbelem++;
+			break;
 		default:
 			break;
 		}
@@ -755,13 +960,14 @@ void level_editor_paste_selection()
 {		
 	struct selected_element *e;
 	struct lvledit_map_tile *t;	
+	map_label *m;
 	waypoint *w;
 	obstacle *o;
 	item *it;
 	int nbact = 0;
 	moderately_finepoint cmin, cmax, center;
 
-	if (mode != FD_RECTDONE) {
+	if (mode == FD_RECT || mode == DRAGDROP) {
 		// We get called from the outside so check mode coherency first
 		return;
 	}
@@ -819,10 +1025,8 @@ void level_editor_paste_selection()
 			}
 
 			// Set and select	current tile
-			action_set_floor(EditLevel(), t->coord.x, t->coord.y, t->tile->floor_value);
+			nbact += set_floor_layers(EditLevel(), t);
 			select_floor_on_tile(t->coord.x, t->coord.y);
-			
-			nbact++;
 			break;
 		case OBJECT_ITEM:
 			it = e->data;
@@ -860,7 +1064,29 @@ void level_editor_paste_selection()
 			}
 
 			// Add and select
-			add_object_to_list(&selected_elements, action_create_waypoint(EditLevel(), w->x, w->y, w->suppress_random_spawn), OBJECT_WAYPOINT);
+			action_create_waypoint(EditLevel(), w->x, w->y, w->suppress_random_spawn);
+			select_waypoint_on_tile(w->x, w->y);
+
+			nbact++;
+			break;
+		case OBJECT_MAP_LABEL:
+			m = e->data;
+
+			m->pos.x -= ceil(center.x);
+			m->pos.y -= ceil(center.y);
+
+			m->pos.x += (int)Me.pos.x;
+			m->pos.y += (int)Me.pos.y;
+
+			if (m->pos.x >= EditLevel()->xlen || m->pos.y >= EditLevel()->ylen
+				|| m->pos.x < 0 || m->pos.y < 0) {
+				// We must not paste map labels outside the boundaries of level
+				break;
+			}
+
+			// Add and select
+			action_create_map_label(EditLevel(), m->pos.x, m->pos.y, m->label_name);
+			select_map_label_on_tile(m->pos.x, m->pos.y);
 
 			nbact++;
 			break;
@@ -933,27 +1159,10 @@ int leveleditor_select_display()
 		translate_map_point_to_screen_pixel(state.rect_start.x + state.rect_len.x, state.rect_start.y + state.rect_len.y, &r3, &c3);
 		translate_map_point_to_screen_pixel(state.rect_start.x + state.rect_len.x, state.rect_start.y, &r4, &c4);
 
-		if (!use_open_gl) {
-			DrawHatchedQuad(Screen, r1, c1, r2, c2, r3, c3, r4, c4, 0x1f, 0x7f, 0x8f);
-		}
-#ifdef HAVE_LIBGL
-		else {
-			glDisable(GL_TEXTURE_2D);
-			glEnable(GL_BLEND);
+		short x[4] = { r1, r2, r3, r4 };
+		short y[4] = { c1, c2, c3, c4 };
 
-			glColor4ub(0x1f, 0x7f, 0x8f, 0x8f);
-
-			glBegin(GL_QUADS);
-			glVertex2i(r1, c1);
-			glVertex2i(r2, c2);
-			glVertex2i(r3, c3);
-			glVertex2i(r4, c4);
-			glEnd();
-
-			glEnable(GL_TEXTURE_2D);
-			glDisable(GL_BLEND);
-		}
-#endif
+		draw_quad(x, y, 31, 127, 143, 143);
 		break;
 	default:
 		break;
@@ -974,3 +1183,13 @@ void leveleditor_select_reset()
 			break;
 	}
 }
+
+void level_editor_switch_selection_type(int direction)
+{
+	int type = selection_type() + direction;
+	if (type < 0)
+		type = OBJECT_MAP_LABEL;
+	else if (type > OBJECT_MAP_LABEL)
+		type = OBJECT_OBSTACLE;
+	lvledit_select_type(type);
+}
