@@ -36,10 +36,12 @@ point selection_start(void);
 point selection_len(void);
 int selection_type(void);
 
-int level_editor_can_cycle_obs(void);
-void level_editor_cycle_marked_obstacle(void);
+int level_editor_can_cycle_marked_object(void);
+void level_editor_cycle_marked_object(void);
 void level_editor_cut_selection(void);
 void level_editor_copy_selection(void);
 void level_editor_paste_selection(void);
 
 void clear_selection(int);
+
+void level_editor_switch_selection_type(int direction);
