@@ -43,11 +43,27 @@
 #define EXTERN extern
 #endif
 
+enum validator_return_code {
+	VALIDATION_PASS    = 0,
+	VALIDATION_WARNING = 1,
+	VALIDATION_ERROR   = 2
+};
+
 #ifdef _leveleditor_validator_c
 
 struct lvlval_ctx {
 	SDL_Rect *report_rect;
 	Level this_level;
+	int in_report_section;
+	enum validator_return_code return_code;
+};
+
+struct lvlval_error {
+	char *title;
+	char *comment;
+	char *format;
+	int caught;
+	enum validator_return_code code;
 };
 
 struct lvlval_excpt_item {
@@ -60,13 +76,14 @@ struct lvlval_excpt_item {
 struct level_validator {
 	char initial;
 	struct list_head excpt_list;
-	int (*execute) (struct level_validator * this, struct lvlval_ctx * validator_ctx);
+	void (*execute) (struct level_validator * this, struct lvlval_ctx * validator_ctx);
 	void *(*parse_excpt) (char *string);
 	int (*cmp) (void *opaque_data1, void *opaque_data2);
 };
 
 #endif
 
-EXTERN void LevelValidation(void);
+EXTERN int level_validation(void);
+EXTERN int level_validation_on_console_only(void);
 
 #endif				// _leveleditor_validator_h_
