@@ -22,12 +22,10 @@
  *
  */
 
-void leveleditor_minimap_mouseenter(SDL_Event *, struct leveleditor_widget *);
-void leveleditor_minimap_mouseleave(SDL_Event *, struct leveleditor_widget *);
-void leveleditor_minimap_mouserelease(SDL_Event *, struct leveleditor_widget *);
-void leveleditor_minimap_mousepress(SDL_Event *, struct leveleditor_widget *);
-void leveleditor_minimap_mouserightrelease(SDL_Event *, struct leveleditor_widget *);
-void leveleditor_minimap_mouserightpress(SDL_Event *, struct leveleditor_widget *);
-void leveleditor_minimap_mousewheelup(SDL_Event *, struct leveleditor_widget *);
-void leveleditor_minimap_mousewheeldown(SDL_Event *, struct leveleditor_widget *);
-void leveleditor_minimap_display(struct leveleditor_widget *);
+#define WIDGET_MINIMAP_WIDTH 210
+#define WIDGET_MINIMAP_HEIGHT 140
+
+struct widget_lvledit_minimap {
+};
+
+struct widget *widget_lvledit_minimap_create();
