@@ -36,6 +36,7 @@
 #include "global.h"
 #include "proto.h"
 #include "vars.h"
+#include "widgets/widgets.h"
 
 #ifdef __OpenBSD__
 #include "ieeefp.h"
@@ -54,8 +55,6 @@ void UpdateCountersForThisFrame();
  */
 void DoAllMovementAndAnimations(void)
 {
-	CheckForTriggeredEvents();
-
 	animate_obstacles();	// this is a pure client issue.  Not dependent upon the players.
 
 	animate_blasts();	// move blasts to the right current "phase" of the blast
@@ -77,6 +76,9 @@ void Game()
 	reset_visible_levels();
 	get_visible_levels();
 	animation_timeline_reset();
+
+	// Reset tooltips.
+	widget_set_tooltip(NULL, NULL);
 	
 	while ((!GameOver && !QuitProgram)) {
 		game_status = INSIDE_GAME;
@@ -85,36 +87,33 @@ void Game()
 
 		save_mouse_state();
 		input_handle();
-
-		UpdateCountersForThisFrame();
+		update_widgets();
+		if (!world_frozen()) {
+			HandleInventoryScreen();
+			HandleCharacterScreen();
+		}
 
 		CollectAutomapData();
-
-		DoAllMovementAndAnimations();
-
-		AssembleCombatPicture(SHOW_ITEMS | USE_OWN_MOUSE_CURSOR);
-		our_SDL_flip_wrapper();
-
-		move_tux();
-		get_visible_levels();
-
-		HandleInventoryScreen();
-		HandleCharacterScreen();
-
 		UpdateAllCharacterStats();
 
-		move_enemies();
+		AssembleCombatPicture(SHOW_ITEMS);
+		our_SDL_flip_wrapper();
 
-		check_tux_enemy_collision();
+		if (!world_frozen()) {
+			UpdateCountersForThisFrame();
 
-		correct_tux_position_according_to_jump();
+			DoAllMovementAndAnimations();
+			move_tux();
+			get_visible_levels();
+
+			move_enemies();
+			check_tux_enemy_collision();
+			correct_tux_position_according_to_jump();
+
+		}
 
 		CheckIfMissionIsComplete();
-
-		limit_fps();
-
 		ComputeFPSForThisFrame();
-
 	}			// while !GameOver 
 }
 
@@ -131,29 +130,8 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "Couldn't change working directory to %s.\n", dirname(argv[0]));
 #endif
 
-	// First we issue some message, that should appear in the debug log for
-	// windows users.
-	//
-#ifdef __WIN32__
-	fprintf(stderr, "\n\
-Hello!  This window contains the DEBUG OUTPUT of FreedroidRPG.\n\
-\n\
-Normally you would not see this message or this window, but apparently\n\
-FreedroidRPG has terminated because of an error of some sort.\n\
-\n\
-You might wish to inspect the debug output below.  Maybe sending the\n\
-debug output (or at least the lower end of the debug output) to the\n\
-FreedroidRPG developers could help them to track down the problem.\n\
-\n\
-Well, it's no guarantee that we can solve any bug, but it's certainly\n\
-better than nothing.  Thanks anyway for your interest in FreedroidRPG.\n\
-\n\n\n\
---start of real debug log--\n\n");
-#endif
-
 	QuitProgram = FALSE;
 	draw_collision_rectangles = FALSE;
-	draw_grid = TRUE;
 	gps_transform_map_dirty_flag = TRUE;
 
 	/*
@@ -164,26 +142,40 @@ better than nothing.  Thanks anyway for your interest in FreedroidRPG.\n\
 	sound_on = TRUE;	// default value, can be overridden by command-line 
 	use_open_gl = TRUE;	// default value, can be overridden by command-line 
 	start_editor = FALSE;	// default value, can be overridden by command-line 
+	load_saved = FALSE;	// default value, can be overridden by command-line
+	saved_game_name = NULL;
 	debug_level = -1;	// -1: shut up all debug ... 0=no debug 1=first debug level (at the moment=all) 
-	GameConfig.fullscreen_on = TRUE;	// use X11-window or full screen 
 
 	InitFreedroid(argc, argv);	// Initialisation of global variables and arrays
 
 	if (do_benchmark) {
 		/* Benchmark mode? Do not run the regular game from here. */
-		benchmark();
-		Terminate(0);
+		int failed = benchmark();
+		Terminate(failed ? EXIT_FAILURE : EXIT_SUCCESS, FALSE);
+	}
+
+	int skip_menu = FALSE;
+	if (start_editor) {
+		// When the user wants to start editor from the command line, skip the
+		// startup menu and load the level editor directly
+		skip_menu = TRUE;
+		prepare_level_editor();
+		start_editor = FALSE;
+	} else if (load_saved && saved_game_name != NULL) {
+		// Skip the startup menu and load saved game directly
+		if (load_named_game(saved_game_name) == OK) {
+			skip_menu = TRUE;
+			game_root_mode = ROOT_IS_GAME;
+		}
+		load_saved = FALSE;
 	}
 
 	while (!QuitProgram) {
-		if (start_editor) {
-			// When the user wants to start editor from the command line, skip the
-			// startup menu and load the level editor directly
-			prepare_level_editor();
-			start_editor = FALSE;
-		} else {
+		if (!skip_menu) {
 			StartupMenu();
 			input_handle();
+		} else {
+			skip_menu = FALSE;
 		}
 		switch (game_root_mode) {
 		case ROOT_IS_GAME:
@@ -201,13 +193,13 @@ better than nothing.  Thanks anyway for your interest in FreedroidRPG.\n\
 
 	LightRadiusClean();
 
-	Terminate(0);
+	Terminate(EXIT_SUCCESS, TRUE);
 	return (0);
 };				// int main ( void )
 
 /**
  * Some bots might be frozen and some might be poisoned, some might still 
- * have a 'firewait' or a normal wait or a paralysation.  Other bots have
+ * have a 'firewait' or a normal wait or a paralysis.  Other bots have
  * a text, that is being displayed and that will timeout at some point.
  *
  * In any case those counteres must be updated, which is what this 
@@ -216,7 +208,7 @@ better than nothing.  Thanks anyway for your interest in FreedroidRPG.\n\
  * NOTE:  This whole updating business is a bit in-efficient.  It might 
  *        be better to use some sort of 'game_time' for this and then
  *        not use 'duration left' but rather 'end time' for all these 
- *        poison, paralysation, etc. effects.  That way, we would be able
+ *        poison, paralysis, etc. effects.  That way, we would be able
  *        be skip this whole counter advancement here...
  *
  *        Maybe later it will finally be implemented this way...
@@ -275,24 +267,17 @@ void update_timeouts_for_bots_on_level(int level_num, float latest_frame_time)
  * ----------------------------------------------------------------- */
 void UpdateCountersForThisFrame()
 {
-	static long Overall_Frames_Displayed = 0;
 	int i;
 	float my_speed;
 	float latest_frame_time = Frame_Time();
 	int level_num;
 	float lose_life;
-	// The next couter counts the frames displayed by freedroid during this
-	// whole run!!  DO NOT RESET THIS COUNTER WHEN THE GAME RESTARTS!!
-	Overall_Frames_Displayed++;
-	Overall_Average = (Overall_Average * (Overall_Frames_Displayed - 1)
-			   + latest_frame_time) / Overall_Frames_Displayed;
+
+	update_frames_displayed();
 
 	LastGotIntoBlastSound += latest_frame_time;
 	LastRefreshSound += latest_frame_time;
 
-	if (SkipAFewFrames)
-		SkipAFewFrames--;
-
 	// This is the timeout, that the tux should not start a movement
 	// some fraction of a second after an item drop.
 	//
@@ -327,9 +312,13 @@ void UpdateCountersForThisFrame()
 		}
 	}
 	
+	// We stop here to prevent animation glitches when Tux is dead
+	if (GameOver)
+		return;	
+
 	// Some bots might be frozen and some might be poisoned, some
-	// might still have a 'firewait' or a normal wait or a paralysation.
-	// In any case those counteres must be updated, but we'll only to 
+	// might still have a 'firewait' or a normal wait or a paralysis.
+	// In any case those counters must be updated, but we'll only to 
 	// that for the Tux current level (at present).
 	//
 	for (level_num = 0; level_num < curShip.num_levels; level_num++) {
@@ -338,7 +327,7 @@ void UpdateCountersForThisFrame()
 	}
 
 	// Now we do all the things, that need to be updated for each connected
-	// player separatedly.
+	// player separately.
 	//
 	Me.current_game_date += latest_frame_time;
 
@@ -411,12 +400,16 @@ void UpdateCountersForThisFrame()
 		Me.slowdown_duration -= latest_frame_time;
 	if (Me.invisible_duration > 0)
 		Me.invisible_duration -= latest_frame_time;
+	if (Me.nmap_duration > 0)
+		Me.nmap_duration -= latest_frame_time;
 	if (Me.paralyze_duration < 0)
 		Me.paralyze_duration = 0;
 	if (Me.slowdown_duration < 0)
 		Me.slowdown_duration = 0;
 	if (Me.invisible_duration < 0)
 		Me.invisible_duration = 0;
+	if (Me.nmap_duration < 0)
+		Me.nmap_duration = 0;
 
 	// In order to know when a level can finally be respawned with
 	// enemies, we keep track to the time spent actually in the game, i.e.
