@@ -24,19 +24,9 @@
  *
  */
 
-/*
- * This file has been checked for remaining german comments. However, if you still
- * find some, please either translate them to english or tell me where you found them.
- * Thanks a lot, Johannes Prix.
- */
-
-// This stuff is nescessary, so that global.h and proto.h know that they have been
-// included this time from the file map.h
 #ifndef _map_h
 #define _map_h
 
-/* some defines */
-
 #define CONNECTION_STRING	"c: "
 #define WHITE_SPACE 		" \t"
 #define MAP_BEGIN_STRING	"beginning_of_map"
@@ -56,8 +46,8 @@
 #define ITEM_POS_X_STRING " X="
 #define ITEM_POS_Y_STRING " Y="
 #define ITEM_ARMOR_CLASS_BASE_STRING " AC="
-#define ITEM_MAX_DURATION_STRING " MDur="
-#define ITEM_CUR_DURATION_STRING " CDur="
+#define ITEM_MAX_DURABILITY_STRING " MDur="
+#define ITEM_CUR_DURABILITY_STRING " CDur="
 #define ITEM_AMMO_CLIP_STRING " AClip="
 #define ITEM_MULTIPLICITY_STRING " Multi="
 #define ITEM_SOCKETS_SIZE_STRING " Sockets="
@@ -91,8 +81,6 @@
 #define POSITION_Y_OF_BIG_MAP_INSERT_STRING "BigGraphicsInsertPosY="
 #define BIG_MAP_INSERT_TYPE_STRING "BigGraphicsInsertType="
 
-#define SPECIAL_FORCE_INDICATION_STRING "SForce: T="
-
 #define OBSTACLE_DATA_BEGIN_STRING "obsdata"
 #define OBSTACLE_DATA_END_STRING "/obsdata"
 #define OBSTACLE_TYPE_STRING "t"
