@@ -4,7 +4,6 @@
 #include "struct.h"
 #include "../src/global.h"
 #include "proto.h"
-#include "savestruct.h"
 
 #include "mapgen/mapgen.h"
 #include "mapgen/themes.h"
@@ -17,7 +16,6 @@
 
 static struct mapgen_gamelevel map;
 static level *target_level;
-static int curobstacle;
 
 struct roominfo *rooms;
 int total_rooms = 0;
@@ -74,26 +72,16 @@ static void free_level()
 void set_dungeon_output(level * output)
 {
 	target_level = output;
-	curobstacle = 0;
 }
 
 void mapgen_add_obstacle(double x, double y, int type)
 {
-	if (curobstacle >= MAX_OBSTACLES_ON_MAP) {
-		ErrorMessage(__FUNCTION__, "Too many obstacles on random dungeon level %d\n", PLEASE_INFORM, IS_FATAL,
-			     target_level->levelnum);
-	}
-
-	target_level->obstacle_list[curobstacle].type = type;
-	target_level->obstacle_list[curobstacle].pos.x = x;
-	target_level->obstacle_list[curobstacle].pos.y = y;
-	target_level->obstacle_list[curobstacle].pos.z = target_level->levelnum;
-	curobstacle++;
+	add_obstacle(target_level, x, y, type);
 }
 
 void mapgen_set_floor(int x, int y, int type)
 {
-	target_level->map[y][x].floor_value = type;
+	target_level->map[y][x].floor_values[0] = type;
 }
 
 static void split_wall(int w, int h, unsigned char *tiles) 
@@ -521,7 +509,7 @@ void mapgen_convert(struct dungeon_info *di, int w, int h, unsigned char *tiles)
 static void add_teleport(int telnum, int x, int y, int tpair)
 {
 	const int helpers[2][4] = {
-		{ ISO_REFRESH_3,  ISO_REFRESH_3,  ISO_REFRESH_3,  ISO_REFRESH_3 },
+		{ ISO_REFRESH_1,  ISO_REFRESH_1,  ISO_REFRESH_1,  ISO_REFRESH_1 },
 		{ ISO_ENHANCER_RU, ISO_ENHANCER_LU, ISO_ENHANCER_RD, ISO_ENHANCER_LD }
 	};
 
@@ -535,19 +523,24 @@ static void add_teleport(int telnum, int x, int y, int tpair)
 	sprintf(tmp, "%dfromX%d", target_level->levelnum, telnum);
 	fromwarp = strdup(tmp);
 
+	add_map_label(target_level, x, y, warp);
+	add_map_label(target_level, x + 1, y, fromwarp);
+
+	// A label placed at (x, y) is actually positioned at (x+0.5, y+0.5).
+	// That 0.5 translation is added to the other obstacles around the labels
+	// to have a coherent position.
+
 	obstacle = telnum ? teleport_pairs[tpair].exit : teleport_pairs[tpair].enter;
+	mapgen_add_obstacle(x + 0.5, y + 0.5, obstacle);
+
 	// Decorate room with teleport if the obstacle is the cloud
 	if (obstacle == ISO_TELEPORTER_1) {
 		helper = MyRandom(1);
-		mapgen_add_obstacle(x + 1, y - 1, helpers[helper][0]);
-		mapgen_add_obstacle(x - 1, y - 1, helpers[helper][1]);
-		mapgen_add_obstacle(x + 1, y + 1, helpers[helper][2]);
-		mapgen_add_obstacle(x - 1, y + 1, helpers[helper][3]);
+		mapgen_add_obstacle(x + 1 + 0.5, y - 1 + 0.5, helpers[helper][0]);
+		mapgen_add_obstacle(x - 1 + 0.5, y - 1 + 0.5, helpers[helper][1]);
+		mapgen_add_obstacle(x + 1 + 0.5, y + 1 + 0.5, helpers[helper][2]);
+		mapgen_add_obstacle(x - 1 + 0.5, y + 1 + 0.5, helpers[helper][3]);
 	}
-
-	mapgen_add_obstacle(x, y, obstacle);
-	add_map_label(target_level, x, y, warp);
-	add_map_label(target_level, x + 1, y, fromwarp);
 }
 
 void mapgen_entry_at(struct roominfo *r, int tpair)
@@ -584,9 +577,10 @@ void mapgen_gift(struct roominfo *r)
 		r->x + r->w / 2, r->y + r->h - 1}
 	};
 
-	if (mapgen_get_tile(positions[pos].x + dx[pos], positions[pos].y + dy[pos]) == TILE_WALL) { 
-		positions[pos].x += obstacle_map[gifts[pos]].right_border;
-		positions[pos].y += obstacle_map[gifts[pos]].lower_border;
+	if (mapgen_get_tile(positions[pos].x + dx[pos], positions[pos].y + dy[pos]) == TILE_WALL) {
+		obstacle_spec *spec = get_obstacle_spec(gifts[pos]);
+		positions[pos].x += spec->right_border;
+		positions[pos].y += spec->lower_border;
 		mapgen_add_obstacle(positions[pos].x, positions[pos].y, gifts[pos]);
 	}
 }
@@ -1012,7 +1006,7 @@ static int get_middle_room(int entrance, int *distance)
 	}
 	// Fill array of distances for a room with the entrance
 	for (i = 0; i < total_rooms; i++)
-		distance[entrance] = dist[entrance][i];
+		distance[i] = dist[entrance][i];
 
 	return m;
 }
@@ -1057,8 +1051,6 @@ int generate_dungeon(int w, int h, int nbconnec, int tpair)
 	di.distance = dist;
 	mapgen_convert(&di, w, h, map.m);
 
-	glue_obstacles_to_floor_tiles_for_level(target_level->levelnum);
-
 	// Place random waypoints
 	place_waypoints();
 
