@@ -26,14 +26,13 @@
 #include "struct.h"
 #include "../src/global.h"
 #include "proto.h"
-#include "savestruct.h"
 
 #include "mapgen/mapgen.h"
 #include "mapgen/themes.h"
 
 #define RAND_THEME(t)	t[MyRandom(sizeof(t) / sizeof(t[0]) - 1)]
-#define OBSTACLE_DIM_X(x)	ceil(obstacle_map[x].right_border - obstacle_map[x].left_border)
-#define OBSTACLE_DIM_Y(x)	ceil(obstacle_map[x].lower_border - obstacle_map[x].upper_border)
+#define OBSTACLE_DIM_X(x)	ceil(get_obstacle_spec(x)->right_border - get_obstacle_spec(x)->left_border)
+#define OBSTACLE_DIM_Y(x)	ceil(get_obstacle_spec(x)->lower_border - get_obstacle_spec(x)->upper_border)
 
 static void apply_default_theme(int, int, int);
 static void apply_metal_theme(int, int, int);
@@ -336,7 +335,7 @@ static void place_garden(int room)
 
 	for (i = y1; i <= y2; i++)
 		for (j = x1; j <= x2; j++)
-			mapgen_set_floor(j, i, ISO_FLOOR_SAND_WITH_GRASS_1 + MyRandom(4));
+			mapgen_set_floor(j, i, ISO_OVERLAY_FLOOR_SAND_WITH_GRASS_1 + MyRandom(4));
 
 	// Cycle through doors of the given room
 	for (i = 0; i < ri->num_doors; i++)
@@ -417,8 +416,8 @@ static int place_work_office(int room)
 					mapgen_add_obstacle(x0 + j * 2 + x, y0 + i * d + y + 1, obj);
 				} else {
 					obj = ISO_TABLE_GLASS_2;
-					//x = -OBSTACLE_DIM_X(obj) / 2;
-					y = OBSTACLE_DIM_Y(obj) / 2 + k;
+					x = OBSTACLE_DIM_X(obj) / 3;
+					y = 3 * OBSTACLE_DIM_Y(obj) / 4 + k;
 					mapgen_add_obstacle(x0 + j * 2 + x, y0 + i * d + y, obj);
 					mapgen_add_obstacle(x0 + j * 2 + x, y0 + i * d + y - OBSTACLE_DIM_Y(obj) / 3, ISO_RED_CHAIR_S);
 					if (MyRandom(1))
@@ -596,7 +595,7 @@ void mapgen_place_obstacles(struct dungeon_info *di, int w, int h, unsigned char
 					themes[rooms[room].theme](x, y, wall);
 					break;	
 				case TILE_WALL:
-					mapgen_set_floor(x, y, ISO_COMPLETELY_DARK);
+					mapgen_set_floor(x, y, ISO_FLOOR_EMPTY);
 					break;
 				default:
 					mapgen_set_floor(x, y, tiles[y * w + x]);
@@ -608,11 +607,13 @@ void mapgen_place_obstacles(struct dungeon_info *di, int w, int h, unsigned char
 	for (i = 0; i < total_rooms; i++) {
 		// vis[i] equal to 1 means that the room is free to decorate
 		if (rooms[i].w != 2 && rooms[i].h != 2 && vis[i] == 1) {
-			if (vis[i] && MyRandom(1)) {
+			if (MyRandom(1)) {
 				place_work_office(i);
 				vis[i] = OFFICE_ROOM;
-			}
-			else if (di->distance[i] < num + 3 && !MyRandom(3)) {
+			} else if (MyRandom(1)) {
+				place_library(i);
+				vis[i] = LIBRARY_ROOM;
+			} else if (di->distance[i] < num + 5 && !MyRandom(2)) {
 				place_garden(i);
 				vis[i] = GARDEN_ROOM;
 			}
