@@ -26,6 +26,7 @@
 #define _proto_h
 
 #include "struct.h"
+#include "lua.h"
 
 // main.c 
 void Game(void);
@@ -34,28 +35,37 @@ void Game(void);
 void display_automap(void);
 void toggle_automap(void);
 void CollectAutomapData(void);
+void update_obstacle_automap(int z, obstacle *our_obstacle);
 
 // init.c
 void ResetGameConfigToDefaultValues(void);
+void clear_out_arrays_for_fresh_game(void);
 void next_startup_percentage(int Percentage);
 void ParseCommandLine(int argc, char *const argv[]);
 void ClearAutomapData(void);
 void InitFreedroid(int, char **);
 void PrepareStartOfNewCharacter(char *startpos);
-void prepare_level_editor();
+void prepare_level_editor(void);
 void ThouArtDefeated(void);
 void ThouHastWon(void);
 void PlayATitleFile(char *Filename);
 
 // event.c
 void GetEventTriggers(const char *EventsAndEventTriggersFilename);
-void CheckForTriggeredEvents(void);
-int teleporter_square_below_mouse_cursor(char *ItemDescText);
+void trigger_position_events(void);
+void event_level_changed(int past_lvl, int cur_lvl);
+void event_enemy_died(enemy *dead);
+const char *teleporter_square_below_mouse_cursor(void);
+void event_modify_trigger_state(const char *name, int state);
+struct event_trigger * visible_event_at_location(int x, int y, int z);
 
 // lua.c
 void init_lua(void);
-void run_lua(const char *code);
-void run_lua_file(const char *);
+lua_State *get_lua_state(enum lua_target target);
+void run_lua(enum lua_target, const char *code);
+void run_lua_file(enum lua_target, const char *);
+void reset_lua_state(void);
+void write_lua_variables(struct auto_string *savestruct_autostr);
 
 // luaconfig.c
 void init_luaconfig(void);
@@ -79,6 +89,9 @@ void hit_tux(float, int);
 void animate_tux(void);
 void check_tux_enemy_collision(void);
 void start_tux_death_explosions(void);
+void init_tux(void);
+void set_movement_with_keys(int move_x, int move_y);
+void do_death_menu(void);
 
 // action.c
 void chest_open_action(level *chest_lvl, int chest_index);
@@ -87,6 +100,7 @@ void terminal_connect_action(level *, int);
 void sign_read_action(level *sign_lvl, int sign_index);
 int clickable_obstacle_below_mouse_cursor(level **obst_lvl);
 int check_for_items_to_pickup(level *item_lvl, int item_index);
+action_fptr get_action_by_name(const char *action_name);
 
 // pathfinder.c
 int set_up_intermediate_course_between_positions(gps * curpos, moderately_finepoint * move_target, moderately_finepoint * waypoints,
@@ -98,9 +112,10 @@ void RotateVectorByAngle(moderately_finepoint * vector, float rot_angle);
 void MoveBullets(void);
 void DoMeleeDamage(void);
 void DeleteBullet(int num, int StartBlast);
-void StartBlast(float x, float y, int level, int type, int dmg);
+void StartBlast(float x, float y, int level, int type, int dmg, int faction);
 void animate_blasts(void);
 void DeleteBlast(int num);
+int get_blast_type_by_name(const char *name);
 void MoveActiveSpells(void);
 void DeleteSpell(int num);
 void clear_active_spells(void);
@@ -110,16 +125,16 @@ void CheckBlastCollisions(int num);
 int find_free_bullet_index(void);
 int find_free_melee_shot_index(void);
 void delete_melee_shot(melee_shot *);
+int GetBulletByName(const char *bullet_name);
 
 // view.c 
-void DisplayItemImageAtMouseCursor(int);
+void get_floor_boundaries(int, int *, int *, int *, int *);
 void gps_transform_map_init(void);
 void update_virtual_position(gps * target_pos, gps * source_pos, int level_num);
 int resolve_virtual_position(gps * actual_pos, gps * virtual_pos);
 int pos_inside_level(float x, float y, level * lvl);
 int pos_near_level(float x, float y, level * lvl, float dist);
 void ShowPosition(void);
-void ShowCombatScreenTexts(int mask);
 void set_up_ordered_blitting_list(int mask);
 void blit_preput_objects_according_to_blitting_list(int mask);
 void blit_nonpreput_objects_according_to_blitting_list(int mask);
@@ -128,18 +143,15 @@ void blit_leveleditor_point(int x, int y);
 void update_item_text_slot_positions(void);
 void AssembleCombatPicture(int);
 void blit_tux(int x, int y);
-void (*iso_put_tux_get_function(char *part_name))(int, int, int, int, int);
-void tux_rendering_init();
-void tux_rendering_validate();
+struct tux_part_render_data *tux_get_part_render_data(char *part_name);
+void tux_rendering_load_specs(const char *config_filename);
 void PutBullet(int Bullet_number, int mask);
 void PutItem(item *CurItem, int ItemNumber, int mask, int put_thrown_items_flag, int highlight_item);
 void PutBlast(int);
 void PutEnemy(enemy * e, int x, int y, int mask, int highlight);
 void PutMouseMoveCursor(void);
-void clear_all_loaded_tux_images(int with_free);
 int set_rotation_index_for_this_robot(enemy * ThisRobot);
 int set_rotation_model_for_this_robot(enemy * ThisRobot);
-void grab_enemy_images_from_archive(int enemy_model_nr);
 int level_is_visible(int level_num);
 void get_visible_levels(void);
 void reset_visible_levels(void);
@@ -147,6 +159,8 @@ void PutIndividuallyShapedDroidBody(enemy *, SDL_Rect, int, int);
 void object_vtx_color(void *, float *, float *, float *);
 int get_motion_class_id_by_name(char *);
 char *get_motion_class_name_by_id(int);
+int get_motion_class_id(void);
+void show_inventory_screen(void);
 
 #define next_valid_visible_level(pos, start) ({ \
 	pos = start; \
@@ -194,99 +208,80 @@ void LightRadiusInit(void);
 void LightRadiusClean(void);
 int get_light_strength_screen(int x, int y);
 int get_light_strength_cell(uint32_t x, uint32_t y);
-int get_light_strength(moderately_finepoint target_pos);
 void update_light_list(void);
 void blit_light_radius(void);
 
 // open_gl.c 
-
-enum {
-	Z_DIR,
-	X_DIR,
-	Y_DIR
-};
-
 int our_SDL_flip_wrapper(void);
 int our_SDL_blit_surface_wrapper(SDL_Surface * src, SDL_Rect * srcrect, SDL_Surface * dst, SDL_Rect * dstrect);
 void our_SDL_update_rect_wrapper(SDL_Surface * screen, Sint32 x, Sint32 y, Sint32 w, Sint32 h);
-int our_SDL_fill_rect_wrapper(SDL_Surface * dst, SDL_Rect * dstrect, Uint32 color);
 int blit_quad(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, Uint32 color);
-void drawIsoEnergyBar(int dir, int x, int y, int z, int w, int d, int length, float fill, myColor * c1, myColor * c2);
+void drawIsoEnergyBar(int x, int y, int z, int w, int d, int length, float fill, myColor * c1, myColor * c2);
 
-SDL_Surface *our_SDL_display_format_wrapperAlpha(SDL_Surface * surface);
 SDL_Surface *our_IMG_load_wrapper(const char *file);
 void flip_image_vertically(SDL_Surface * tmp1);
-void make_texture_out_of_surface(iso_image * our_image);
-void make_texture_out_of_prepadded_image(iso_image * our_image);
-SDL_Surface *pad_image_for_texture(SDL_Surface * our_surface);
+void make_texture_out_of_surface(struct image *our_image);
+void make_texture_out_of_prepadded_image(struct image *our_image);
 void blit_open_gl_stretched_texture_light_radius(int decay_x, int decay_y);
-void PutPixel_open_gl(int x, int y, Uint32 pixel);
 void gl_draw_rectangle(SDL_Rect *, int, int, int, int);
 void show_character_screen_background(void);
 int safely_initialize_our_default_open_gl_parameters(void);
-void draw_gl_textured_quad_at_map_position(iso_image * our_floor_iso_image, float our_col, float our_line, float r, float g, float b,
-					   int highlight_texture, int blend, float zoom_factor);
-void draw_gl_textured_quad_at_screen_position(iso_image * our_floor_iso_image, int x, int y);
-void draw_gl_scaled_textured_quad_at_screen_position(iso_image * our_floor_iso_image, int x, int y, float scale_factor);
-void blit_special_background(int background_code);
+void blit_background(const char *background);
 void open_gl_check_error_status(const char *name_of_calling_function);
-void draw_gl_bg_textured_quad_at_screen_position(iso_image * our_floor_iso_image, int x, int y);
+void set_gl_clip_rect(const SDL_Rect *clip);
+void unset_gl_clip_rect(void);
 
 // blocks.c 
 int wall_orientation(int wall);
-void try_to_load_ingame_item_surface(int item_type);
 void iso_load_bullet_surfaces(void);
-int iso_image_loaded(iso_image *);
-void get_iso_image_from_file_and_path(char *fpath, iso_image * our_iso_image, int use_offset_file);
-void load_iso_image(iso_image *, const char *, int);
-void make_sure_zoomed_surface_is_there(iso_image * our_iso_image);
-void load_item_surfaces_for_item_type(int item_type);
 void Load_Mouse_Move_Cursor_Surfaces(void);
-void Load_Skill_Level_Button_Surfaces(void);
-void LoadOneSkillSurfaceIfNotYetLoaded(int SkillSpellNr);
 void LoadAndPrepareEnemyRotationModelNr(int RotationModel);
-void LoadAndPrepareRedEnemyRotationModelNr(int RotationModel);
-void LoadAndPrepareGreenEnemyRotationModelNr(int RotationModel);
-void LoadAndPrepareBlueEnemyRotationModelNr(int RotationModel);
+void free_enemy_graphics(void);
 void Load_Enemy_Surfaces(void);
 void Load_Tux_Surfaces(void);
 void Load_Bullet_Surfaces(void);
 void Load_Blast_Surfaces(void);
 void load_floor_tiles(void);
-iso_image *get_obstacle_image(int);
-void load_obstacle(int);
-void load_all_obstacles(void);
-void blit_iso_image_to_map_position(iso_image * our_iso_image, float pos_x, float pos_y);
-void blit_iso_image_to_screen_position(iso_image * our_iso_image, float pos_x, float pos_y);
-void sdl_highlight_iso_image(iso_image * our_iso_image, float pos_x, float pos_y);
-void blit_zoomed_iso_image_to_map_position(iso_image * our_iso_image, float pos_x, float pos_y);
-
-// block_areas.c
-void init_obstacle_data(void);
+void free_floor_tiles(void);
+struct image *get_obstacle_image(int, int);
+struct image *get_obstacle_shadow_image(int, int);
+struct image *get_map_label_image(void);
+struct image *get_droid_portrait_image(int);
+void load_all_obstacles(int with_startup_bar);
+void free_obstacle_graphics(void);
+struct image *get_item_shop_image(int type);
+struct image *get_item_ingame_image(int type);
+struct image *get_item_inventory_image(int type);
+void load_all_items(void);
+void free_item_graphics(void);
+void load_tux_graphics(int motion_class, int tux_part_group, const char *part_string);
+void reload_tux_graphics(void);
+void get_offset_for_iso_image_from_file_and_path(char *fpath, struct image * our_iso_image);
 
 // graphics.c 
-void set_mouse_cursor_to_shape(int given_shape);
-void make_sure_system_mouse_cursor_is_turned_off(void);
-void make_sure_system_mouse_cursor_is_turned_on(void);
-void blit_our_own_mouse_cursor(void);
-SDL_Surface *rip_rectangle_from_alpha_image(SDL_Surface * our_surface, SDL_Rect our_rect);
-SDL_Surface *CreateColorFilteredSurface(SDL_Surface * FirstSurface, int FilterType);
-void fade_out_using_gamma_ramp(void);
-void fade_in_using_gamma_ramp(void);
+void blit_mouse_cursor(void);
+void fade_out_screen(void);
+void fade_in_screen(void);
 void InitPictures(void);
-void InitTimer(void);
+void init_timer(void);
 void InitVideo(void);
 void InitOurBFonts(void);
-void FreeOurBFonts(void);
-void ClearGraphMem(void);
 void HighlightRectangle(SDL_Surface * Surface, SDL_Rect Area);
 void ShadowingRectangle(SDL_Surface * Surface, SDL_Rect Area);
-int do_graphical_number_selection_in_range(int lower_range, int upper_range, int default_value);
-Uint8 GetAlphaComponent(SDL_Surface * surface, int x, int y);
+int do_graphical_number_selection_in_range(int lower_range, int upper_range, int default_value, int unit_price);
 void draw_line(float x1, float y1, float x2, float y2, Uint32 color, int glwidth);
-void DrawHatchedQuad(SDL_Surface * Surface, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int r, int g, int b);
+void draw_line_on_map(float x1, float y1, float x2, float y2, Uint32 color, int width);
+void sdl_draw_rectangle(SDL_Rect *rect, int r, int g, int b, int a);
+SDL_Surface *sdl_create_colored_surface(SDL_Surface *surf, float r, float g, float b, float a, int highlight);
+void draw_rectangle(SDL_Rect *rect, int r, int g, int b, int a);
+void draw_quad(const int16_t vx[4], const int16_t vy[4], int r, int g, int b, int a);
+void sdl_put_pixel(SDL_Surface *surf, int x, int y, uint8_t red, uint8_t green, uint8_t blue, uint8_t alpha);
+void save_screenshot(const char *filename, int width);
+void reload_graphics(void);
+void clear_screen(void);
 
 // saveloadgame.c 
+int find_saved_games(struct dirent ***);
 void LoadAndShowThumbnail(char *CoreFilename);
 int SaveGame(void);
 int LoadBackupGame(void);
@@ -294,68 +289,6 @@ int LoadGame(void);
 int DeleteGame(void);
 void LoadAndShowStats(char *CoreFilename);
 
-/* Primitive types */
-
-/* Saving is done via macros */
-#define save_pritype(Z,X,Y) autostr_append(savestruct_autostr, Z, X, *(Y))
-#define save_char(X,Y) save_pritype("%s: %hhd\n", X, Y)
-#define save_uchar(X,Y) save_pritype("%s: %hhu\n", X, Y)
-#define save_uint16_t(X,Y) save_pritype("%s: %hu\n", X, Y)
-#define save_int16_t(X,Y) save_pritype("%s: %hd\n", X, Y)
-#define save_int32_t(X,Y) save_pritype("%s: %d\n", X, Y)
-#define save_uint32_t(X,Y) save_pritype("%s: %u\n", X, Y)
-#define save_float(X,Y) save_pritype("%s: %f\n", X, Y)
-#define save_double(X,Y) save_pritype("%s: %lf\n", X, Y)
-#define save_string(X,Y) if (*Y) save_pritype("%s: %s\n", X, Y)
-
-/* Reading is slightly more difficult so we do it with functions */
-void read_int32_t(const char *, const char *, int32_t *);
-void read_int16_t(const char *, const char *, int16_t *);
-void read_char(const char *, const char *, char *);
-void read_uint32_t(const char *, const char *, uint32_t *);
-void read_uint16_t(const char *, const char *, uint16_t *);
-void read_uchar(const char *, const char *, unsigned char *);
-void read_double(const char *, const char *, double *);
-void read_float(const char *, const char *, float *);
-void read_string(const char *, const char *, string *);
-
-/* Array writing/reading */
-void save_moderately_finepoint_array(const char *, moderately_finepoint *, int);
-void read_moderately_finepoint_array(char *, const char *, moderately_finepoint *, int);
-void save_mission_array(const char *, mission *, int);
-void read_mission_array(char *, const char *, mission *, int);
-void save_int32_t_array(const char *, int *, int);
-void read_int32_t_array(char *, const char *, int *, int);
-void save_item_array(const char *, item *, int);
-void read_item_array(char *, const char *, item *, int);
-void save_uchar_array(const char *, unsigned char *, int);
-void read_uchar_array(char *, const char *, unsigned char *, int);
-void save_char_array(const char *, char *, int);
-void read_char_array(char *, const char *, char *, int);
-void save_uint16_t_array(const char *, uint16_t *, int);
-void read_uint16_t_array(char *, const char *, uint16_t *, int);
-void save_gps_array(const char *, gps *, int);
-void read_gps_array(char *, const char *, gps *, int);
-void save_float_array(const char *, float *, int);
-void read_float_array(char *, const char *, float *, int);
-void save_string_array(const char *, string *, int);
-void read_string_array(char *, const char *, string *, int);
-void save_upgrade_socket_dynarray(const char *, struct upgrade_socket_dynarray *);
-void read_upgrade_socket_dynarray(char *, const char *, struct upgrade_socket_dynarray *);
-
-/* Hacks */
-void save_keybind_t_array(const char *, keybind_t *, int);
-void read_keybind_t_array(const char *, const char *, keybind_t *, int);
-#define save_automap_data_t_array save_automap_data
-void save_automap_data(const char *, automap_data_t *, int);
-void read_automap_data_t_array(char *, char *, automap_data_t *, int);
-void save_sdl_rect(const char *, SDL_Rect *);
-int read_sdl_rect(const char *, const char *, SDL_Rect *);
-#define save_list_head_t(X,Y)
-#define read_list_head_t(X,Y,Z)
-void save_luacode(const char *, luacode *);
-void read_luacode(const char *, const char *, luacode *);
-
 // mission.c 
 void CompleteMission(const char *);
 void AssignMission(const char *);
@@ -365,18 +298,19 @@ void CheckIfMissionIsComplete(void);
 void mission_diary_add(const char *, const char *);
 int GetMissionIndexByName(const char *);
 
-// quest_browser.c
-void quest_browser_interface(void);
+// quest_browser_ui.c
+void toggle_quest_browser(void);
+struct widget_group *create_quest_browser(void);
 
 // map.c 
 void respawn_level(int level_num);
-void glue_obstacles_to_floor_tiles_for_level(int level_num);
 void ResolveMapLabelOnShip(const char *MapLabel, location * PositionPointer);
 int smash_obstacle(float x, float y, int level);
-Uint16 GetMapBrick(level *, float, float);
+Uint16 get_map_brick(level *, float, float, int);
 void CountNumberOfDroidsOnShip(void);
 int LoadShip(char *filename, int);
 int SaveShip(const char *filename, int reset_random_levels, int);
+int save_special_forces(const char *filename);
 void get_animated_obstacle_lists(struct visible_level *vis_lvl);
 void dirty_animated_obstacle_lists(int lvl_num);
 void clear_animated_obstacle_lists(struct visible_level *vis_lvl);
@@ -394,6 +328,11 @@ void translate_map_point_to_screen_pixel_func(float x_map_pos, float y_map_pos,
 float translate_pixel_to_map_location(float axis_x, float axis_y, int give_x);
 float translate_pixel_to_zoomed_map_location(float axis_x, float axis_y, int give_x);
 
+//floor_tiles.c
+int next_glue_timestamp(void);
+void free_glued_obstacles(level *lvl);
+struct image *get_floor_tile_image(int floor_value);
+
 //colldet.c
 int WalkablePassFilterCallback(colldet_filter * this, obstacle * obs, int obs_idx);
 int FlyablePassFilterCallback(colldet_filter * this, obstacle * obs, int obs_idx);
@@ -406,18 +345,21 @@ colldet_filter VisiblePassFilter;
 colldet_filter ObstacleByIdPassFilter;
 colldet_filter WalkableExceptIdPassFilter;
 colldet_filter FlyableExceptIdPassFilter;
-int CheckIfWayIsFreeOfDroids(float x1, float y1, float x2, float y2, int OurLevel, freeway_context * ctx);
+int location_free_of_droids(float x, float y, int levelnum, freeway_context *ctx);
+int way_free_of_droids(float x1, float y1, float x2, float y2, int OurLevel, freeway_context * ctx);
 int EscapeFromObstacle(float *posX, float *posY, int posZ, colldet_filter * filter);
 int SinglePointColldet(float x, float y, int z, colldet_filter * filter);
 int DirectLineColldet(float x1, float y1, float x2, float y2, int z, colldet_filter * filter);
 int normalize_vect(float, float, float *, float *);
 
-// sound.c  OR nosound.c 
-void PlayOnceNeededSoundSample(const char *SoundSampleFileName, const int With_Waiting, const int no_double_catching);
+// sound.c
+void play_sound(const char *);
 void InitAudio(void);
 void SetBGMusicVolume(float);
 void SetSoundFXVolume(float);
 void SwitchBackgroundMusicTo(char *filename_raw);
+
+// sound_effects.c
 void tux_scream_sound(void);
 void No_Ammo_Sound(void);
 void Not_Enough_Power_Sound(void);
@@ -427,31 +369,33 @@ void MenuItemSelectedSound(void);
 void MenuItemDeselectedSound(void);
 void MoveMenuPositionSound(void);
 void teleport_arrival_sound(void);
-void Fire_Bullet_Sound(int);
+void fire_bullet_sound(int BulletType, struct gps *shooter_pos);
+void play_blast_sound(int blast_type, struct gps *blast_pos);
 void Mission_Status_Change_Sound(void);
-void DruidBlastSound(void);
-void ExterminatorBlastSound(void);
+void droid_blast_sound(struct gps *blast_pos);
+void exterminator_blast_sound(struct gps *blast_pos);
 void ThouArtDefeatedSound(void);
 void Takeover_Set_Capsule_Sound(void);
 void Takeover_Game_Won_Sound(void);
 void Takeover_Game_Deadlock_Sound(void);
 void Takeover_Game_Lost_Sound(void);
-void PlayGreetingSound(int SoundCode);
-void play_enter_attack_run_state_sound(int SoundCode);
+void play_greeting_sound(enemy *ThisRobot);
+void play_enter_attack_run_state_sound(enemy *ThisRobot);
 void play_death_sound_for_bot(enemy * ThisRobot);
-void play_item_sound(int item_type);
+void play_item_sound(int item_type, struct gps *item_pos);
 void PlayEnemyGotHitSound(int enemytype);
 void BulletReflectedSound(void);
 void Play_Spell_ForceToEnergy_Sound(void);
 void play_melee_weapon_hit_something_sound(void);
-void play_melee_weapon_missed_sound(void);
+void play_melee_weapon_missed_sound(struct gps *attacker_pos);
 void play_open_chest_sound(void);
-void play_sample_using_WAV_cache(char *SoundSampleFileName, int With_Waiting, int no_double_catching);
-void play_sample_using_WAV_cache_v(char *SoundSampleFileName, int With_Waiting, int no_double_catching, double volume);
+void play_sound_cached(const char *SoundSampleFileName);
+void play_sound_cached_v(const char *SoundSampleFileName, double volume);
+void play_sound_at_position(const char *filename, struct gps *listener, struct gps *emitter);
 
 // items.c
 void init_item(item *);
-item create_item_with_name(const char *item_name, int full_duration, int multiplicity);
+item create_item_with_name(const char *item_name, int full_durability, int multiplicity);
 void equip_item(item *new_item);
 item *get_equipped_item_in_slot_for(int item_type);
 void MoveItem(item * SourceItem, item * DestItem);
@@ -466,9 +410,9 @@ int ItemCanBeDroppedInInv(int ItemType, int InvPos_x, int InvPos_y);
 unsigned long calculate_item_buy_price(item * BuyItem);
 unsigned long calculate_item_repair_price(item * repair_item);
 unsigned long calculate_item_sell_price(item * BuyItem);
-void FillInItemProperties(item * ThisItem, int FullDuration, int multiplicity);
+void FillInItemProperties(item *it, int full_durability, int multiplicity);
 void DamageProtectiveEquipment(void);
-void write_full_item_name_into_string(item * ShowItem, char *full_item_name);
+void append_item_name(item *ShowItem, struct auto_string *str);
 item *DropItemAt(int, int, float, float, int);
 void Quick_ApplyItem(int ItemKey);
 int MatchItemWithName(int type, const char *name);
@@ -487,11 +431,10 @@ int GetInventorySquare_x(int x);
 int GetInventorySquare_y(int x);
 void DropHeldItemToInventory(void);
 item *drop_item(item *, float, float, int);
-void show_quick_inventory(void);
 void HandleInventoryScreen(void);
 int try_give_item(item *it);
 int give_item(item *);
-void CopyItem(item * SourceItem, item * DestItem, int MakeSound);
+void CopyItem(item * SourceItem, item * DestItem);
 void DeleteItem(item * Item);
 void DropRandomItem(int level_num, float x, float y, int class, int ForceMagical);
 int get_floor_item_index_under_mouse_cursor(level **item_lvl);
@@ -505,14 +448,17 @@ void copy_upgrade_sockets(item *, item *);
 int item_can_be_customized(item *);
 int item_can_be_installed_to_socket(item *, item *, int);
 struct addon_spec *get_addon_spec(int);
-struct dynarray *get_addon_specs();
+struct dynarray *get_addon_specs(void);
 void add_addon_spec(struct addon_spec *);
 void get_item_bonus_string(item *, const char *, struct auto_string *);
 void print_addon_description(struct addon_spec *, struct auto_string *);
 void calculate_item_bonuses(item *);
+int count_used_sockets(item *);
+int item_upgrade_ui_visible(void);
 
 // character.c
-void DisplayButtons(void);
+void display_buttons(void);
+int get_experience_required(int);
 void UpdateAllCharacterStats(void);
 void ShowCharacterScreen(void);
 void HandleCharacterScreen(void);
@@ -532,13 +478,16 @@ void RadialFireWave(gps ExpCenter, int SpellCostsMana);
 void ShowSkillsScreen(void);
 void HandleCurrentlyActivatedSkill(void);
 int DoSkill(int skill_index, int SpellCost);
+void do_radial_skill(int skill_index, int pos_x, int pos_y, int from_tux);
 void activate_nth_skill(int skill_num);
 void set_nth_quick_skill(int quick_skill);
 void ImproveSkill(int *skill);
 int improve_program(int);
+void downgrade_program(int program_id);
 int get_program_index_with_name(const char *);
 int calculate_program_heat_cost(int program_id);
 int CursorIsOnWhichSkillButton(int x, int y);
+void load_skill_icon_if_needed(spell_skill_spec *spec);
 
 // input.c 
 Uint8 *key_state_array;
@@ -580,64 +529,65 @@ int input_key_press(SDL_Event *);
 void keychart(void);
 void input_set_default(void);
 void input_keyboard_init(void);
+void input_hold_keyboard(void);
+void input_release_keyboard(void);
 int getchar_raw(int *);
-void input_get_keybind(char *cmdname, SDLKey * key, SDLMod * mod);
+void input_get_keybind(const char *cmdname, SDLKey * key, SDLMod * mod);
+void input_get_keybind_string(const char *cmd, char *out);
 void input_set_keybind(char *keybind, SDLKey key, SDLMod mod);
 
 // menu.c 
 void clear_player_inventory_and_stats(void);
 void StoreMenuBackground(int backup_slot);
 void RestoreMenuBackground(int backup_slot);
-int DoMenuSelection(char *InitialText, char *MenuTexts[], int FirstItem, int background_code, void *MenuFont);
-int ChatDoMenuSelectionFlagged(char *InitialText, char **MenuTexts,
-			       int FirstItem, int background_code, void *MenuFont, enemy * ChatDroid);
-int ChatDoMenuSelection(char **MenuTexts, int FirstItem, void *MenuFont, enemy * ChatDroid);
+int DoMenuSelection(char *InitialText, char *MenuTexts[], int FirstItem, const char *background_name, void *MenuFont);
+int chat_do_menu_selection_filtered(char **MenuTexts, enemy *chat_droid, const char *topic);
+int chat_do_menu_selection(char **MenuTexts, enemy *chat_droid);
 void StartupMenu(void);
-void InitiateMenu(int background_code);
+void InitiateMenu(const char *background_name);
 void Cheatmenu(void);
 void EscapeMenu(void);
+int load_named_game(const char *name);
 
 // misc.c 
 #define CURLEVEL() (curShip.AllLevels[Me.pos.z])
 void print_trace(int signum);
-void implant_backtrace_into_signal_handlers(void);
 void adapt_button_positions_to_screen_resolution(void);
-void ErrorMessage(const char *FunctionName, const char *ProblemDescription, int InformDevelopers, int IsFatal, ...);
+#ifdef __GNUC__
+#define PRINTF_FMT_ATTRIBUTE(fmt,firstarg) __attribute__ ((format(printf,fmt,firstarg)));
+#else
+#define PRINTF_FMT_ATTRIBUTE(fmt,firstarg)
+#endif
+void ErrorMessage(const char *FunctionName, const char *ProblemDescription, int InformDevelopers, int IsFatal, ...) PRINTF_FMT_ATTRIBUTE(2,5);
 void ShowGenericButtonFromList(int ButtonIndex);
-int mouse_cursor_is_on_that_iso_image(float pos_x, float pos_y, iso_image *our_iso_image);
+int mouse_cursor_is_on_that_image(float pos_x, float pos_y, struct image *our_iso_image);
 int MouseCursorIsInRect(const SDL_Rect *, int, int);
 int MouseCursorIsOnButton(int ButtonIndex, int x, int y);
 void *MyMemmem(char *haystack, size_t haystacklen, char *needle, size_t needlelen);
-char *ReadAndMallocStringFromData(char *SearchString, const char *StartIndicationString, const char *EndIndicationString);
-char *ReadAndMallocStringFromDataOptional(char *SearchString, const char *StartIndicationString, const char *EndIndicationString);
-int CountStringOccurences(char *SearchString, const char *TargetString);
-void ReadValueFromStringWithDefault(char *SearchBeginPointer, const char *ValuePreceedText, const char *FormatString,
-				    const char *DefaultValueString, void *TargetValue, char *EndOfSearchSectionPointer);
-void ReadValueFromString(char *SearchBeginPointer, const char *ValuePreceedText, const char *FormatString, void *TargetValue,
-			 char *EndOfSearchSectionPointer);
-char *ReadAndMallocAndTerminateFile(const char *filename, const char *File_End_String);
-char *LocateStringInData(char *SearchBeginPointer, const char *SearchTextPointer);
 int find_file(const char *fname, const char *datadir, char *File_Path, int silent);
 void Pause(void);
 void ComputeFPSForThisFrame(void);
 void StartTakingTimeForFPSCalculation(void);
 int Get_Average_FPS(void);
 float Frame_Time(void);
-void limit_fps();
 void Activate_Conservative_Frame_Computation(void);
+void update_frames_displayed(void);
 int MyRandom(int);
-void Teleport(int LNum, float X, float Y, int WithSound);
+void Teleport(int LNum, float X, float Y, int WithSound, int with_animation_reset);
 void teleport_to_level_center(int);
 int SaveGameConfig(void);
 int LoadGameConfig(void);
 void InsertNewMessage(void);
-void Terminate(int);
+void Terminate(int, int);
 void ShowDebugInfos(void);
 Sint16 ReadSint16(void *memory);
 void endian_swap(char *pdata, size_t dsize, size_t nelements);
 uint32_t pot_gte(uint32_t v);
 obstacle *give_pointer_to_obstacle_with_label(const char *, int *);
 int level_exists(int);
+void freeze_world(void);
+void unfreeze_world(void);
+int world_frozen(void);
 
 // enemy.c 
 void SetRestOfGroupToState(Enemy ThisRobot, short NewState);
@@ -646,7 +596,7 @@ int CheckEnemyEnemyCollision(enemy *);
 void move_enemies(void);
 void clear_enemies(void);
 void enemy_reset(enemy *this_bot);
-void enemy_reset_fabric();
+void enemy_reset_fabric(void);
 enemy *enemy_new(int type);
 void enemy_insert_into_lists(enemy *this_enemy, int is_living);
 void animate_enemy(enemy * our_bot);
@@ -668,65 +618,76 @@ enemy *get_enemy_with_dialog(const char *dialog);
 #define BROWSE_LEVEL_BOTS(T,L) list_for_each_entry(T, &level_bots_head[(L)], level_list)
 
 // text.c
-int get_lines_needed(const char *text, SDL_Rect t_rect, float text_stretch);
+int get_lines_needed(const char *text, SDL_Rect t_rect, float line_height_factor);
 void show_backgrounded_label_at_map_position(char *LabelText, float fill_status, float pos_x, float pos_y, int zoom_is_on);
 char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, const char *DefaultString);
 int show_backgrounded_text_rectangle(const char *, struct BFont_Info *, int, int, int, int);
-void alert_window(const char *text);
+void alert_window(const char *text, ...) PRINTF_FMT_ATTRIBUTE(1,2);
 int CutDownStringToMaximalSize(char *StringToCut, int LengthInPixels);
 void SetNewBigScreenMessage(const char *ScreenMessageText);
 void DisplayBigScreenMessage(void);
 void ChatWithFriendlyDroid(Enemy ChatDroid);
-void EnemyHitByBulletText(enemy *);
 void EnemyInfluCollisionText(enemy *);
 
-int DisplayText(const char *text, int startx, int starty, const SDL_Rect * clip, float text_stretch);
+int display_text_using_line_height(const char *, int, int, const SDL_Rect*, float);
+int display_text(const char *, int, int, const SDL_Rect*);
 void show_chat_log(enemy *);
 
-int ScrollText(char *text, int background_code);
+int ScrollText(char *text, const char *background_name);
 
-int ImprovedCheckLineBreak(char *text, const SDL_Rect * clip, float text_stretch);
+int ImprovedCheckLineBreak(char *, const SDL_Rect*, float);
 char *PreviousLine(char *textstart, char *text);
 char *NextLine(char *text);
-char *GetString(int max_len, int background_code, const char *text_for_overhead_promt);
-void printf_SDL(SDL_Surface * screen, int x, int y, const char *fmt, ...);
+char *get_string(int max_len, const char *background_name, const char *text_for_overhead_promt);
+void printf_SDL(SDL_Surface * screen, int x, int y, const char *fmt, ...) PRINTF_FMT_ATTRIBUTE(4,5);
 int longest_line_width(char *text);
 
 // text_public.c 
-
-void DebugPrintf(int db_level, const char *fmt, ...);
+char *ReadAndMallocStringFromData(char *SearchString, const char *StartIndicationString, const char *EndIndicationString);
+char *ReadAndMallocStringFromDataOptional(char *SearchString, const char *StartIndicationString, const char *EndIndicationString);
+int CountStringOccurences(char *SearchString, const char *TargetString);
+void ReadValueFromStringWithDefault(char *SearchBeginPointer, const char *ValuePreceedText, const char *FormatString,
+				    const char *DefaultValueString, void *TargetValue, char *EndOfSearchSectionPointer);
+void ReadValueFromString(char *SearchBeginPointer, const char *ValuePreceedText, const char *FormatString, void *TargetValue,
+			 char *EndOfSearchSectionPointer);
+int ReadRangeFromString(char *SearchString, const char *StartIndicationString, const char *EndIndicationString, int *min, int *max, int default_val);
+char *ReadAndMallocAndTerminateFile(const char *filename, const char *File_End_String);
+char *LocateStringInData(char *SearchBeginPointer, const char *SearchTextPointer);
+void DebugPrintf(int db_level, const char *fmt, ...) PRINTF_FMT_ATTRIBUTE(2,3);
 void *MyMalloc(long);
 int FS_filelength(FILE * f);
 int inflate_stream(FILE *, unsigned char **, int *);
 int deflate_to_stream(unsigned char *, int, FILE *);
 
-// text_widget.c
-void init_text_widget(text_widget *, const char *);
-int widget_handle_mouse(text_widget *);
-void show_text_widget(text_widget *);
-
 // hud.c 
-void give_item_description(char *, item *);
+void append_item_description(struct auto_string *str, item *);
 void ShowCurrentSkill(void);
 void ShowCurrentWeapon(void);
 void ShowCurrentHealthAndForceLevel(void);
-void DisplayBanner(void);
+void show_texts_and_banner(void);
 int get_days_of_game_duration(float current_game_date);
 int get_hours_of_game_duration(float current_game_date);
 int get_minutes_of_game_duration(float current_game_date);
-void append_new_game_message(const char *fmt, ...);
+void append_new_game_message(const char *fmt, ...) PRINTF_FMT_ATTRIBUTE(1,2);
 void init_message_log(void);
 void toggle_game_config_screen_visibility(int screen_visible);
+int get_current_fps(void);
+void display_tooltip(const char *, int, SDL_Rect);
+void blit_vertical_status_bar(float, float, Uint32, Uint32, int, int, int, int);
+
+// game_ui.c
+struct widget_group *get_game_ui(void);
 
 // item_upgrades_ui.c
-int get_item_upgrade_ui_tooltip(const point *, char *);
-void show_item_upgrade_ui();
-void item_upgrade_ui();
+int append_item_upgrade_ui_tooltip(const point *, struct auto_string *str);
+void show_item_upgrade_ui(void);
+void item_upgrade_ui(void);
 
 // addon_crafting_ui.c
 int cursor_is_on_addon_crafting_ui(const point *);
-void show_addon_crafting_ui();
-void addon_crafting_ui();
+void show_addon_crafting_ui(void);
+void addon_crafting_ui(void);
+int addon_crafting_ui_visible(void);
 
 // shop.c 
 void ShowItemPicture(int, int, int);
@@ -739,9 +700,8 @@ int GreatShopInterface(int, item * ShowPointerList[MAX_ITEMS_IN_INVENTORY], int,
 // takeover.c 
 
 int droid_takeover(enemy *);
-int do_takeover(int, int, int);
+int do_takeover(int, int, int, enemy *);
 
-int GetTakeoverGraphics(void);
 void InventPlayground(void);
 
 void ProcessPlayground(void);
@@ -753,9 +713,7 @@ void ClearPlayground(void);
 int IsActive(int color, int row);
 
 // BFont.c
-void PutPixel32(SDL_Surface * surface, int x, int y, Uint32 pixel);
 Uint32 FdGetPixel(SDL_Surface * Surface, Sint32 X, Sint32 Y);
-void PutPixel(SDL_Surface * surface, int x, int y, Uint32 pixel);
 
 void list_add(list_head_t * new, list_head_t * head);
 void list_add_tail(list_head_t * new, list_head_t * head);
@@ -767,14 +725,14 @@ int list_empty(const list_head_t * head);
 void list_splice(list_head_t * list, list_head_t * head);
 void list_splice_init(list_head_t * list, list_head_t * head);
 
-int load_texture_atlas(const char *, const char *, char *filenames[], iso_image *, int);
+int load_texture_atlas(const char *, const char *, struct image *(*get_storage_for_key)(const char *key));
 
 // chat.c
-void PlantCookie(const char *);
-void DeleteCookie(const char *);
+void chat_push_topic(const char *topic);
+void chat_pop_topic(void);
 void run_subdialog(const char *);
-void GiveSubtitleNSample(const char *, const char *, enemy *, int);
-void validate_dialogs(void);
+void chat_add_response(const char *, int, enemy *);
+int validate_dialogs(void);
 
 // leveleditor_input.c
 void leveleditor_process_input(void);
@@ -791,9 +749,9 @@ void set_dungeon_output(level *);
 // string.c
 struct auto_string *alloc_autostr(int);
 void free_autostr(struct auto_string *);
-int autostr_printf(struct auto_string *, const char *, ...);
+int autostr_printf(struct auto_string *, const char *, ...) PRINTF_FMT_ATTRIBUTE(2,3);
 int autostr_vappend(struct auto_string *str, const char *fmt, va_list args);
-int autostr_append(struct auto_string *, const char *, ...);
+int autostr_append(struct auto_string *, const char *, ...) PRINTF_FMT_ATTRIBUTE(2,3);
 
 // dynarray.c
 struct dynarray *dynarray_alloc(int, size_t);
@@ -802,17 +760,18 @@ void dynarray_resize(struct dynarray *, int, size_t);
 void dynarray_free(struct dynarray *);
 void dynarray_add(struct dynarray *, void *, size_t);
 void dynarray_del(struct dynarray *, int, size_t);
+void *dynarray_member(struct dynarray *, int, size_t);
 
 // animate.c
 void animation_timeline_reset(void);
 void animation_timeline_advance(void);
 int animate_door(level* obstacle_lvl, int obstacle_idx);
-int animate_teleporter(level* obstacle_lvl, int obstacle_idx);
-int animate_refresh(level* obstacle_lvl, int obstacle_idx);
 int animate_autogun(level* obstacle_lvl, int obstacle_idx);
+int animate_obstacle(level *obstacle_lvl, int obstacle_idx);
+animation_fptr get_animation_by_name(const char *animation_name);
 
 // benchmark.c
-void benchmark(void);
+int benchmark(void);
 
 // npc.c
 struct npc *npc_get(const char *);
@@ -821,35 +780,67 @@ void npc_add(const char *);
 int npc_add_shoplist(const char *, const char *, int);
 void init_npcs(void);
 void clear_npcs(void);
-item *npc_get_inventory(struct npc *);
+item_dynarray *npc_get_inventory(struct npc *);
 void npc_inventory_delete_item(struct npc *, int);
 
 // faction.c
 enum faction_id get_faction_id(const char *);
+const char *get_faction_from_id(int);
 void set_faction_state(enum faction_id, enum faction_id, enum faction_state);
 int is_friendly(enum faction_id, enum faction_id);
 void init_factions(void);
-void save_factions(struct auto_string *);
-void load_factions(char *);
 
 // obstacle_extension.c
 void *get_obstacle_extension(level *, obstacle *, enum obstacle_extension_type);
 int get_obstacle_index(level *, obstacle *);
 void add_obstacle_extension(level *, obstacle *, enum obstacle_extension_type, void *);
 void del_obstacle_extension(level *, obstacle *, enum obstacle_extension_type);
+void del_obstacle_extensions(level *, obstacle *);
 void defrag_obstacle_array(level *);
+void free_obstacle_extensions(level *lvl);
 
 // map_label.c
 void add_map_label(level *, int, int, char *);
 void del_map_label(level *, const char *);
+void free_map_labels(level *lvl);
 struct map_label *get_map_label(level *, const char *);
+struct map_label *get_map_label_from_coords(level *, float, float);
+struct map_label *map_label_exists(const char *);
 
 // lvledit_display.c
-float lvledit_zoomfact_inv();
+float lvledit_zoomfact_inv(void);
+
+// lvledit_widgets.c
+struct widget_group *get_lvledit_ui(void);
 
 // waypoint.c
 int add_waypoint(level *, int, int, int);
 void del_waypoint(level *, int, int);
 int get_waypoint(level *, int, int);
+void move_waypoint(level *, waypoint *, int, int);
+
+// image.c
+void start_image_batch(void);
+void end_image_batch(void);
+void display_image_on_screen(struct image *img, int x, int y, struct image_transformation t);
+void display_image_on_map(struct image *img, float X, float Y, struct image_transformation t);
+void create_subimage(struct image *source, struct image *new_img, SDL_Rect *rect);
+void load_image(struct image *, const char *, int);
+void load_image_surface(struct image *img, const char *filename, int use_offset_file);
+void free_image_surface(struct image *img);
+void delete_image(struct image *img);
+int image_loaded(struct image *);
+struct image_transformation set_image_transformation(float scale_x, float scale_y, float r, float g, float b, float a, int highlight);
+
+// obstacle.c
+obstacle *add_obstacle(level *lvl, float x, float y, int type);
+void del_obstacle(obstacle *o);
+obstacle_spec *get_obstacle_spec(int index);
+void glue_obstacle(level *lvl, obstacle *o);
+void unglue_obstacle(level *lvl, obstacle *o);
+void move_obstacle(obstacle *o, float x, float y);
+struct obstacle_group *get_obstacle_group_by_name(const char *group_name);
+void add_obstacle_to_group(const char *group_name, int type);
+struct obstacle_group *find_obstacle_group(int type);
 
 #endif
