@@ -35,7 +35,8 @@
 #include "struct.h"
 #include "global.h"
 #include "proto.h"
-#include "SDL_rotozoom.h"
+
+#include "widgets/widgets.h"
 
 #define SHOP_ROW_LENGTH 8
 
@@ -116,7 +117,7 @@ int AssemblePointerListForItemShow(item ** ItemPointerListPointer, int IncludeWo
  * course we must find out and return the index of the item clicked on.
  * If no item was clicked on, then a -1 will be returned as index.
  */
-int ClickWasOntoItemRowPosition(int x, int y, int TuxItemRow)
+static int ClickWasOntoItemRowPosition(int x, int y, int TuxItemRow)
 {
 	if (TuxItemRow) {
 		if (y < TuxItemRowRect.y)
@@ -163,10 +164,10 @@ void ShowRescaledItem(int position, int TuxItemRow, item * ShowItem)
 {
 	SDL_Rect TargetRectangle =
 	    { 0, 0, INITIAL_BLOCK_WIDTH * GameConfig.screen_width / 640, INITIAL_BLOCK_HEIGHT * GameConfig.screen_height / 480 };
-	static iso_image equipped_icon;
+	static struct image equipped_icon;
 
-	if (!iso_image_loaded(&equipped_icon)) {
-		load_iso_image(&equipped_icon, "mouse_cursor_0003.png", FALSE);
+	if (!image_loaded(&equipped_icon)) {
+		load_image(&equipped_icon, "cursors/mouse_cursor_0003.png", FALSE);
 	}
 
 	TuxItemRowRect.x = 55 * GameConfig.screen_width / 640;
@@ -193,14 +194,14 @@ void ShowRescaledItem(int position, int TuxItemRow, item * ShowItem)
 		TargetRectangle.x = ShopItemRowRect.x + position * INITIAL_BLOCK_WIDTH * GameConfig.screen_width / 640;
 		TargetRectangle.y = TuxItemRow;
 	}
-	our_SDL_blit_surface_wrapper(ItemMap[ShowItem->type].inv_image.shop_iso_image.surface, NULL, Screen, &TargetRectangle);
+	
+	struct image *img = get_item_shop_image(ShowItem->type);
+	if (img) {
+		display_image_on_screen(img, TargetRectangle.x, TargetRectangle.y, IMAGE_NO_TRANSFO);
+	}
+
 	if (item_is_currently_equipped(ShowItem)) {
-		if (use_open_gl) {
-			draw_gl_textured_quad_at_screen_position(&equipped_icon, TargetRectangle.x + TargetRectangle.w - 24,
-								 TargetRectangle.y);
-		} else {
-			blit_iso_image_to_screen_position(&equipped_icon, TargetRectangle.x + TargetRectangle.w - 24, TargetRectangle.y);
-		}
+		display_image_on_screen(&equipped_icon, TargetRectangle.x + TargetRectangle.w - 24, TargetRectangle.y, IMAGE_NO_TRANSFO);
 	}
 };				// void ShowRescaledItem ( int position , item* ShowItem )
 
@@ -209,208 +210,118 @@ void ShowRescaledItem(int position, int TuxItemRow, item * ShowItem)
  */
 void ShowItemPicture(int PosX, int PosY, int Number)
 {
-	SDL_Surface *tmp;
-	SDL_Rect target;
-	char ConstructedFileName[2048];
-	char fpath[2048];
 	static char LastImageSeriesPrefix[1000] = "NONE_AT_ALL";
-	static int NumberOfImagesInPreviousRotation = 0;
-	static int NumberOfImagesInThisRotation = 0;
+	static int NumberOfImagesInThisRotation = 1;
 #define MAX_NUMBER_OF_IMAGES_IN_ITEM_ROTATION 64
-	static SDL_Surface *ItemRotationSurfaces[MAX_NUMBER_OF_IMAGES_IN_ITEM_ROTATION] = { NULL };
-	SDL_Surface *Whole_Image;
+	static struct image item_rotation_img[MAX_NUMBER_OF_IMAGES_IN_ITEM_ROTATION] = { EMPTY_IMAGE };
 	int i;
 	int RotationIndex;
 
-	// DebugPrintf (2, "\nvoid ShowItemPicture(...): Function call confirmed.");
-
-	// if ( !strcmp ( ItemMap[ Number ] . item_rotation_series_prefix , "NONE_AVAILABLE_YET" ) )
-	// return; // later this should be a default-correction instead
-
-	// Maybe we have to reload the whole image series
-	//
-	if (strcmp(LastImageSeriesPrefix, ItemMap[Number].item_rotation_series_prefix)) {
-		// Maybe we have to free the series from an old item display first
-		//
-		if (ItemRotationSurfaces[0] != NULL) {
-			for (i = 0; i < NumberOfImagesInPreviousRotation; i++) {
-				SDL_FreeSurface(ItemRotationSurfaces[i]);
-			}
+	if (!image_loaded(&item_rotation_img[0])) {
+		// Initialize image structures
+		struct image empty = EMPTY_IMAGE;
+		for (i = 0; i < sizeof(item_rotation_img)/sizeof(item_rotation_img[0]); i++) {
+			memcpy(&item_rotation_img[i], &empty, sizeof(struct image));
 		}
-		// Now we can start to load the whole series into memory
-		//
+	}
+
+	if (strcmp(LastImageSeriesPrefix, ItemMap[Number].item_rotation_series_prefix)) {
+
+		// Free previous images
 		for (i = 0; i < MAX_NUMBER_OF_IMAGES_IN_ITEM_ROTATION; i++) {
-			// At first we will try to find some item rotation models in the
-			// new directory structure.
-			//
+			delete_image(&item_rotation_img[i]);
+		}
+
+		// Load new images
+		for (i = 0; i < MAX_NUMBER_OF_IMAGES_IN_ITEM_ROTATION; i++) {
+			char ConstructedFileName[2048];
+			char fpath[2048];
 			sprintf(ConstructedFileName, "items/%s/portrait_%04d.jpg", ItemMap[Number].item_rotation_series_prefix, i + 1);
-			if (find_file(ConstructedFileName, GRAPHICS_DIR, fpath, 1))
-				Whole_Image = NULL;
-			else
-				Whole_Image = our_IMG_load_wrapper(fpath);	// This is a surface with alpha channel, since the picture is one of this type
 
-			// If that didn't work, then it's time to try the same directory with 'png' ending...
-			// Maybe there's still some (old) rotation image of this kind.
-			//
-			if (Whole_Image == NULL) {
-				DebugPrintf(1, "\nNo luck trying to load .jpg item image series from the 'bastian' dir... trying png...");
-				sprintf(ConstructedFileName, "items/%s/portrait_%04d.png", ItemMap[Number].item_rotation_series_prefix,
-					i + 1);
-				if (find_file(ConstructedFileName, GRAPHICS_DIR, fpath, 1))
-					Whole_Image = NULL;
-				else
-					Whole_Image = our_IMG_load_wrapper(fpath);	// This is a surface with alpha channel, since the picture is one of this type
-			}
-			// But at this point, we should have found the image!!
-			// or if not, this maybe indicates that we have reached the
-			// last image in the image series...
-			//
-			if (Whole_Image == NULL) {
+			// Look for the next file
+			if (!find_file(ConstructedFileName, GRAPHICS_DIR, fpath, 1)) {
+				load_image(&item_rotation_img[i], ConstructedFileName, FALSE);
+			} else {
 				NumberOfImagesInThisRotation = i;
-				NumberOfImagesInPreviousRotation = NumberOfImagesInThisRotation;
-				DebugPrintf(1, "\nDONE LOADING ITEM IMAGE SERIES.  Loaded %d images into memory.",
-					    NumberOfImagesInThisRotation);
 
-				// Maybe we've received the nothing loaded case even on the first attempt
-				// to load something.  This of course would mean a severe error in Freedroid!
-				//
-				if (NumberOfImagesInThisRotation <= 0) {
-					fprintf(stderr, "\n\nfpath: %s. \n", fpath);
-					ErrorMessage(__FUNCTION__, "\
-Freedroid was unable to load even one image of a rotated item image series into memory.\n\
-This error indicates some installation problem with freedroid.", PLEASE_INFORM, IS_FATAL);
-				}
+				if (!NumberOfImagesInThisRotation)
+					ErrorMessage(__FUNCTION__, "Unable to load any item rotation image for item \"%s\". File \"%s\" was not found.", PLEASE_INFORM, IS_WARNING_ONLY, ItemMap[Number].item_name, ConstructedFileName);
 
 				break;
 			}
-			// Also we must check for our upper bound of the list of 
-			// item images.  This will most likely never be exceeded, but it
-			// can hurt to just be on the safe side.
-			//
-			if (i >= MAX_NUMBER_OF_IMAGES_IN_ITEM_ROTATION - 2) {
-				fprintf(stderr, "\n\nfpath: %s. \n", fpath);
-				ErrorMessage(__FUNCTION__, "\
-Freedroid was encountered more item images in an item rotation image series\n\
-than it is able to handle.  This is a very strange error.  Someone has been\n\
-trying to make the ultra-fine item rotation series.  Strange.", PLEASE_INFORM, IS_FATAL);
-			}
-
-			SDL_SetAlpha(Whole_Image, 0, SDL_ALPHA_OPAQUE);
-			ItemRotationSurfaces[i] = SDL_CreateRGBSurface(0, Whole_Image->w, Whole_Image->h, 32, rmask, gmask, bmask, amask);
-			SDL_BlitSurface(Whole_Image, NULL, ItemRotationSurfaces[i], NULL);
-			SDL_FreeSurface(Whole_Image);
-
-			// We must remember, that his is already loaded of course
-			strcpy(LastImageSeriesPrefix, ItemMap[Number].item_rotation_series_prefix);
 
 		}
 
+		// Remember what series we have just loaded
+		strcpy(LastImageSeriesPrefix, ItemMap[Number].item_rotation_series_prefix);
 	}
 
 	RotationIndex = (SDL_GetTicks() / 70);
 
 	RotationIndex = RotationIndex - (RotationIndex / NumberOfImagesInThisRotation) * NumberOfImagesInThisRotation;
 
-	tmp = ItemRotationSurfaces[RotationIndex];
-
-	SDL_SetClipRect(Screen, NULL);
-	Set_Rect(target, PosX, PosY, GameConfig.screen_width, GameConfig.screen_height);
-	our_SDL_blit_surface_wrapper(tmp, NULL, Screen, &target);
+	if (image_loaded(&item_rotation_img[RotationIndex]))
+			display_image_on_screen(&item_rotation_img[RotationIndex], PosX, PosY, IMAGE_NO_TRANSFO);
 }
 
 /**
  * Assemble item description.
  */
-static void fill_item_description(text_widget *desc, item *show_item)
+static void fill_item_description(struct widget_text *desc, item *show_item, int buy)
 {
-	char TextChunk[2000];
-	char *class_string;
 	long int repair_price = 0;
 	itemspec *info;
+	int price = calculate_item_buy_price(show_item);
+	const char *action = _("Buy");
+	const char *buyone_highlight = font_switchto_neon;
+	const char *buyall_highlight = font_switchto_neon;
 
 	if (show_item == NULL)
 		return;
 
-	if (MatchItemWithName(show_item->type, "Valuable Circuits"))
-		ErrorMessage(__FUNCTION__, "Valuable Circuits in shop.", PLEASE_INFORM, IS_WARNING_ONLY);
-
 	info = &ItemMap[show_item->type];
 
-	init_text_widget(desc, "");
+	widget_text_init(desc, "");
 
-	if (info->item_can_be_installed_in_weapon_slot)
-		class_string = _("Weapon");
-	else if (info->item_can_be_installed_in_drive_slot)
-		class_string = _("Drive");
-	else if (info->item_can_be_installed_in_armour_slot)
-		class_string = _("Armor");
-	else if (info->item_can_be_installed_in_shield_slot)
-		class_string = _("Shield");
-	else if (info->item_can_be_installed_in_special_slot)
-		class_string = _("Helm");
-	else
-		class_string = _("Miscellaneous");
-
-	write_full_item_name_into_string(show_item, TextChunk);
-	autostr_printf(desc->text, _("Item: %s \nClass: %s\n"), TextChunk, class_string);
-
-	// Append item bonuses.
-	struct auto_string *bonuses = alloc_autostr(128);
-	get_item_bonus_string(show_item, ", ", bonuses);
-	if (bonuses->length)
-		autostr_append(desc->text, "%s%s%s%s\n", _("Specials: "), font_switchto_red,
-					   bonuses->value, font_switchto_neon);
-	free_autostr(bonuses);
-
-	if (info->item_group_together_in_inventory)
-		autostr_append(desc->text, _("Multiplicity: %d\n"), (int)show_item->multiplicity);
-
-	autostr_append(desc->text, _("Durability: "));
-	if (show_item->max_duration >= 0)
-		autostr_append(desc->text, "%d / %d\n", (int)show_item->current_duration, show_item->max_duration);
-	else
-		autostr_append(desc->text, _("Indestructible\n"));
-
-	if (!info->item_can_be_applied_in_combat) {
-		autostr_append(desc->text, _("Attributes required: "));
-
-		if ((info->item_require_strength <= 0) &&
-			(info->item_require_dexterity <= 0) && (info->item_require_magic <= 0)) {
-			autostr_append(desc->text, _("NONE\n"));
-		} else {
-			if (info->item_require_strength > 0)
-				autostr_append(desc->text, _("Str: %d "), info->item_require_strength);
-			if (info->item_require_dexterity > 0)
-				autostr_append(desc->text, _("Dex: %d "), info->item_require_dexterity);
-			if (info->item_require_magic > 0)
-				autostr_append(desc->text, _("Mag: %d "), info->item_require_magic);
-			autostr_append(desc->text, "\n");
-		}
-	}
+	append_item_description(desc->text, show_item);
+	
 	// Now we give some pricing information, the base list price for the item,
 	// the repair price and the sell value
-	if (calculate_item_buy_price(show_item)) {
-		autostr_append(desc->text, _("Base list price: %ld\n"), calculate_item_buy_price(show_item));
-		autostr_append(desc->text, _("Sell value: %ld\n"), calculate_item_sell_price(show_item));
-		if (show_item->current_duration == show_item->max_duration || show_item->max_duration == (-1))
+	if (price) {
+		if (!buy) {
+ 			price = calculate_item_sell_price(show_item);
+			action = _("Sell");
+		} else if (price > Me.Gold) {
+			buyone_highlight = font_switchto_red;
+			buyall_highlight = font_switchto_red;
+		} else if ((price * show_item->multiplicity) > Me.Gold) {
+			buyall_highlight = font_switchto_red;
+		}
+
+		if (ItemMap[show_item->type].item_group_together_in_inventory) {
+			autostr_append(desc->text, _("Price per unit: %s%d%s\n"), buyone_highlight, price, font_switchto_neon);
+			autostr_append(desc->text, _("%s all: %s%d%s\n"), action, buyall_highlight, price * show_item->multiplicity, font_switchto_neon);
+		} else {
+			autostr_append(desc->text, _("%s Price: %s%d%s\n"), action, buyone_highlight, price, font_switchto_neon);
+		}
+
+
+		if (show_item->current_durability == show_item->max_durability || show_item->max_durability == (-1))
 			repair_price = 0;
 		else
 			repair_price = calculate_item_repair_price(show_item);
-		if (show_item->max_duration == (-1))
-			autostr_append(desc->text, _("Indestructible\n"));
-		else
-			autostr_append(desc->text, _("Repair cost: %ld\n"), repair_price);
+
+		if (ItemMap[show_item->type].base_item_durability != (-1)) {
+			if (show_item->max_durability == (-1))
+				autostr_append(desc->text, _("Indestructible\n"));
+			else if (!buy)
+				autostr_append(desc->text, _("Repair cost: %ld\n"), repair_price);
+		}
 	} else {
 		autostr_append(desc->text, _("Unsellable\n"));
 	}
 
 	/* If the item is a weapon, then we print out some weapon stats. */
-	if (info->base_item_gun_damage + info->item_gun_damage_modifier > 0) {
-		autostr_append(desc->text, _("Damage: %d - %d\n"),
-					   info->base_item_gun_damage,
-					   info->base_item_gun_damage + info->item_gun_damage_modifier);
-	}
-
 	if (info->item_gun_recharging_time > 0)
 		autostr_append(desc->text, _("Recharge time: %3.2f\n"),
 					   info->item_gun_recharging_time);
@@ -419,13 +330,10 @@ static void fill_item_description(text_widget *desc, item *show_item)
 		autostr_append(desc->text, _("Time to reload ammo clip: %3.2f\n"),
 					   info->item_gun_reloading_time);
 
-	if (show_item->armor_class + show_item->bonus_to_armor_class > 0)
-		autostr_append(desc->text, _("Armor class: %d\n"), show_item->armor_class + show_item->bonus_to_armor_class);
-
 	autostr_append(desc->text, _("Notes: %s"), D_(info->item_description));
 
 	if (info->item_gun_use_ammunition)
-		autostr_append(desc->text, _(" This weapon requires %s."),
+		autostr_append(desc->text, _("\nThis weapon requires %s."),
 					   ammo_desc_for_weapon(show_item->type));
 }
 
@@ -453,15 +361,11 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 	int SellButtonActive = FALSE;
 	int ret = 0;
 	int old_game_status = game_status;
-	static text_widget item_description;
+	static struct widget_text item_description;
 	const int scroll_to_top = -1000;
 
 	game_status = INSIDE_MENU;
 
-	// For the shop, we'll also try to use our own mouse cursor
-	//
-	make_sure_system_mouse_cursor_is_turned_off();
-
 	// We add some security against indexing beyond the
 	// range of items given in the list.
 	if (NumberOfItems <= 0) {
@@ -497,25 +401,21 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 	}
 
 	/* Initialize the text widget. */
-	init_text_widget(&item_description, "");
-	item_description.rect.x = UNIVERSAL_COORD_W(258);
-	item_description.rect.y = UNIVERSAL_COORD_H(108);
-	item_description.rect.w = UNIVERSAL_COORD_W(346);
-	item_description.rect.h = UNIVERSAL_COORD_H(255);
+	widget_text_init(&item_description, "");
+	widget_set_rect(WIDGET(&item_description), UNIVERSAL_COORD_W(258), UNIVERSAL_COORD_H(108), UNIVERSAL_COORD_W(346), UNIVERSAL_COORD_H(255));
 	item_description.font = FPS_Display_BFont;
-	item_description.text_stretch = TEXT_STRETCH;
+	item_description.line_height_factor = LINE_HEIGHT_FACTOR;
 	item_description.scroll_offset = scroll_to_top;
 
 	if (ItemIndex >= 0) {
-		fill_item_description(&item_description, ShowPointerList[ItemIndex]);
+		fill_item_description(&item_description, ShowPointerList[ItemIndex], 1);
 		item_description.scroll_offset = scroll_to_top;
 	} else if (TuxItemIndex >= 0) {
-		fill_item_description(&item_description, TuxItemsList[TuxItemIndex]);
+		fill_item_description(&item_description, TuxItemsList[TuxItemIndex], 0);
 		item_description.scroll_offset = scroll_to_top;
 	}
 
 	while (1) {
-		StartTakingTimeForFPSCalculation();
 		save_mouse_state();
 		input_handle();
 
@@ -524,9 +424,9 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 
 		// We show all the info and the buttons that should be in this
 		// interface...
-		AssembleCombatPicture(USE_OWN_MOUSE_CURSOR | ONLY_SHOW_MAP);
+		AssembleCombatPicture(ONLY_SHOW_MAP);
 		SDL_SetClipRect(Screen, NULL);
-		blit_special_background(ITEM_BROWSER_SHOP_BACKGROUND_CODE);
+		blit_background("item_browser_shop.png");
 
 		/* This is a magic formula to place the item picture. */
 		int x = 40 * GameConfig.screen_width / 1024 + ((250 * GameConfig.screen_width / 1024) - 132) / 2;
@@ -538,7 +438,7 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 			ShowItemPicture(x, y, TuxItemsList[TuxItemIndex]->type);
 		}
 
-		show_text_widget(&item_description);
+		widget_text_display(WIDGET(&item_description));
 
 		for (i = 0; i < RowLength; i++) {
 			ShowRescaledItem(i, FALSE, ShowPointerList[i + RowStart]);
@@ -578,8 +478,8 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 			}
 			BuyButtonActive = FALSE;
 
-			if ((ItemMap[TuxItemsList[TuxItemIndex]->type].base_item_duration >= 0) &&
-			    (TuxItemsList[TuxItemIndex]->max_duration > TuxItemsList[TuxItemIndex]->current_duration))
+			if ((ItemMap[TuxItemsList[TuxItemIndex]->type].base_item_durability >= 0) &&
+			    (TuxItemsList[TuxItemIndex]->max_durability > TuxItemsList[TuxItemIndex]->current_durability))
 				ShowGenericButtonFromList(REPAIR_BUTTON);
 		} else {
 			BuyButtonActive = FALSE;
@@ -591,7 +491,7 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 		PutStringFont(Screen, FPS_Display_BFont, 40 * GameConfig.screen_width / 640 - 15,
 			      370 * GameConfig.screen_height / 480, GoldString);
 
-		blit_our_own_mouse_cursor();
+		blit_mouse_cursor();
 		our_SDL_flip_wrapper();
 
 
@@ -613,7 +513,7 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 					if (ItemIndex != -1) {
 						if (ItemIndex >= RowStart + RowLength)
 							ItemIndex--;
-						fill_item_description(&item_description, ShowPointerList[ItemIndex]);
+						fill_item_description(&item_description, ShowPointerList[ItemIndex], 1);
 						item_description.scroll_offset = scroll_to_top;
 					}
 				}
@@ -624,7 +524,7 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 					if (ItemIndex != -1) {
 						if (ItemIndex < RowStart)
 							ItemIndex++;
-						fill_item_description(&item_description, ShowPointerList[ItemIndex]);
+						fill_item_description(&item_description, ShowPointerList[ItemIndex], 1);
 						item_description.scroll_offset = scroll_to_top;
 					}
 				}
@@ -635,7 +535,7 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 					if (TuxItemIndex != -1) {
 						if (TuxItemIndex >= TuxRowStart + TuxRowLength)
 							TuxItemIndex--;
-						fill_item_description(&item_description, TuxItemsList[TuxItemIndex]);
+						fill_item_description(&item_description, TuxItemsList[TuxItemIndex], 0);
 						item_description.scroll_offset = scroll_to_top;
 					}
 				}
@@ -646,36 +546,36 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 					if (TuxItemIndex != -1) {
 						if (TuxItemIndex < TuxRowStart)
 							TuxItemIndex++;
-						fill_item_description(&item_description, TuxItemsList[TuxItemIndex]);
+						fill_item_description(&item_description, TuxItemsList[TuxItemIndex], 0);
 						item_description.scroll_offset = scroll_to_top;
 					}
 				}
 				MoveMenuPositionSound();
 			} else if (((ClickTarget = ClickWasOntoItemRowPosition(GetMousePos_x(), GetMousePos_y(), FALSE)) >= 0)) {
-				if (RowStart + ClickTarget < NumberOfItems) {
+				if (ClickTarget < RowLength) {
 					ItemIndex = RowStart + ClickTarget;
 					TuxItemIndex = (-1);
-					fill_item_description(&item_description, ShowPointerList[ItemIndex]);
+					fill_item_description(&item_description, ShowPointerList[ItemIndex], 1);
 					item_description.scroll_offset = scroll_to_top;
 				}
 			} else if (((ClickTarget = ClickWasOntoItemRowPosition(GetMousePos_x(), GetMousePos_y(), TRUE)) >= 0)) {
-				if (TuxRowStart + ClickTarget < NumberOfItemsInTuxRow) {
+				if (ClickTarget < TuxRowLength) {
 					TuxItemIndex = TuxRowStart + ClickTarget;
 					ItemIndex = (-1);
-					fill_item_description(&item_description, TuxItemsList[TuxItemIndex]);
+					fill_item_description(&item_description, TuxItemsList[TuxItemIndex], 0);
 					item_description.scroll_offset = scroll_to_top;
 				}
 			} else if (MouseCursorIsOnButton(BUY_BUTTON, GetMousePos_x(), GetMousePos_y())) {
 				if (BuyButtonActive && ItemIndex != -1) {
 					ShopOrder->item_selected = ItemIndex;
 					ShopOrder->shop_command = BUY_1_ITEM;
-					if ((ItemMap[ShowPointerList[ItemIndex]->type].item_group_together_in_inventory) &&
-					    (Me.Gold / ItemMap[ShowPointerList[ItemIndex]->type].base_list_price >= 1)) {
+					int afford = Me.Gold / ItemMap[ShowPointerList[ItemIndex]->type].base_list_price;
+					if ((ItemMap[ShowPointerList[ItemIndex]->type].item_group_together_in_inventory) && (afford >= 1)) {
 						ShopOrder->number_selected =
-						    do_graphical_number_selection_in_range(0,
-											   Me.Gold /
-											   ItemMap[ShowPointerList[ItemIndex]->type].
-											   base_list_price, 1);
+							do_graphical_number_selection_in_range(0,
+									(ShowPointerList[ItemIndex]->multiplicity <= afford) ?
+										ShowPointerList[ItemIndex]->multiplicity : afford,
+									1, calculate_item_buy_price(ShowPointerList[ItemIndex]));
 					} else {
 						ShopOrder->number_selected = 1;
 					}
@@ -691,7 +591,8 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 					    (TuxItemsList[TuxItemIndex]->multiplicity > 1)) {
 						ShopOrder->number_selected =
 						    do_graphical_number_selection_in_range(0, TuxItemsList[TuxItemIndex]->multiplicity,
-											   TuxItemsList[TuxItemIndex]->multiplicity);
+											   TuxItemsList[TuxItemIndex]->multiplicity,
+											calculate_item_sell_price(TuxItemsList[TuxItemIndex]));
 					} else
 						ShopOrder->number_selected = 1;
 					ret = 0;
@@ -706,8 +607,8 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 					// really something to repair (and therefore the button is shown at
 					// all further above.
 					//
-					if ((ItemMap[TuxItemsList[TuxItemIndex]->type].base_item_duration >= 0) &&
-					    (TuxItemsList[TuxItemIndex]->max_duration > TuxItemsList[TuxItemIndex]->current_duration)) {
+					if ((ItemMap[TuxItemsList[TuxItemIndex]->type].base_item_durability >= 0) &&
+					    (TuxItemsList[TuxItemIndex]->max_durability > TuxItemsList[TuxItemIndex]->current_durability)) {
 						ShopOrder->item_selected = TuxItemIndex;
 						ShopOrder->shop_command = REPAIR_ITEM;
 						ShopOrder->number_selected = 1;
@@ -729,10 +630,6 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 			item_description.scroll_offset++;
 		}
 
-		// Limit framerate if configured to do so.
-		limit_fps();
-		ComputeFPSForThisFrame();
-
 		if (EscapePressed()) {
 			while (EscapePressed()) ;
 			ret = -1;
@@ -746,77 +643,28 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 	return ret;
 }
 
-/**
- * This function tells us which item in the menu has been clicked upon.
- * It does not check for lower than 4 items in the menu available.
- */
-int ClickedMenuItemPosition(void)
-{
-	int CursorY;
-	int i;
-
-	CursorY = GetMousePos_y();	// this is already the position corrected for 16 pixels!!
-
-#define ITEM_MENU_DISTANCE 80
-#define ITEM_FIRST_POS_Y 130
-#define NUMBER_OF_ITEMS_ON_ONE_SCREEN 4
-
-	// When a character is blitted to the screen at x y, then the x and y
-	// refer to the top left corner of the coming blit.  Using this information
-	// we will define the areas where a click 'on the blitted text' has occurred
-	// or not.
-	//
-	if (CursorY < ITEM_FIRST_POS_Y)
-		return (-1);
-	if (CursorY > ITEM_FIRST_POS_Y + NUMBER_OF_ITEMS_ON_ONE_SCREEN * ITEM_MENU_DISTANCE)
-		return (-1);
-
-	for (i = 0; i < NUMBER_OF_ITEMS_ON_ONE_SCREEN; i++) {
-		if (CursorY < ITEM_FIRST_POS_Y + (i + 1) * ITEM_MENU_DISTANCE)
-			return i;
-	}
-
-	// At this point we've already determined and returned to right click-area.
-	// if this point is ever reached, a severe error has occurred, and Freedroid
-	// should therefore also say so.
-	//
-	ErrorMessage(__FUNCTION__, "\
-The MENU CODE was unable to properly resolve a mouse button press.", PLEASE_INFORM, IS_FATAL);
-
-	return (3);		// to make compilers happy :)
-
-};				// int ClickedMenuItemPosition( void )
-
 /**
  * This function repairs the item given as parameter.
  */
-void TryToRepairItem(item * RepairItem)
+static void repair_item(item * RepairItem)
 {
-	char *MenuTexts[10];
-	MenuTexts[0] = _("Yes");
-	MenuTexts[1] = _("No");
-	MenuTexts[2] = "";
-
 	while (SpacePressed() || EnterPressed() || MouseLeftPressed())
 		SDL_Delay(1);
 
 	if (calculate_item_repair_price(RepairItem) > Me.Gold) {
-		MenuTexts[0] = _(" BACK ");
-		MenuTexts[1] = "";
-		SetCurrentFont(Menu_BFont);
-		DoMenuSelection(_("\n\nYou can't afford to have this item repaired! "), MenuTexts, 1, -1, NULL);
+		alert_window("%s\n\n%s", ItemMap[RepairItem->type].item_name, _("You can not afford to have this item repaired."));
 		return;
 	}
 
 	Me.Gold -= calculate_item_repair_price(RepairItem);
-	RepairItem->current_duration = RepairItem->max_duration;
-	PlayOnceNeededSoundSample("effects/Shop_ItemRepairedSound_0.ogg", FALSE, FALSE);
-};				// void TryToRepairItem( item* RepairItem )
+	RepairItem->current_durability = RepairItem->max_durability;
+	play_sound("effects/Shop_ItemRepairedSound_0.ogg");
+}
 
 /**
  * This function tries to sell the item given as parameter.
  */
-void TryToSellItem(item * SellItem, int AmountToSellAtMost)
+static void TryToSellItem(item * SellItem, int AmountToSellAtMost)
 {
 	// We catch the case, that not even one item was selected
 	// for buying in the number selector...
@@ -833,7 +681,7 @@ void TryToSellItem(item * SellItem, int AmountToSellAtMost)
 	if (SellItem->multiplicity < 1) {
 		ErrorMessage(__FUNCTION__, "\
 		An item sold seemed to have multiplicity < 1.  This might be due to some\n\
-		fatal errors in the engine OR it might be due to some items droped on the\n\
+		fatal errors in the engine OR it might be due to some items dropped on the\n\
 		maps somewhere long ago still had multiplicity=0 setting, which should not\n\
 		normally occur with 'freshly' generated items.  Well, that's some dust from\n\
 		the past, but now it should be fixed and not occur in future releases (0.9.10\n\
@@ -848,14 +696,14 @@ void TryToSellItem(item * SellItem, int AmountToSellAtMost)
 
 	// Ok.  Here we silently sell the item.
 	//
-	Me.Gold += calculate_item_sell_price(SellItem) * ((float)AmountToSellAtMost) / ((float)SellItem->multiplicity);
+	Me.Gold += calculate_item_sell_price(SellItem) * AmountToSellAtMost;
 	if (AmountToSellAtMost < SellItem->multiplicity)
 		SellItem->multiplicity -= AmountToSellAtMost;
 	else
 		DeleteItem(SellItem);
 
-	PlayOnceNeededSoundSample("effects/Shop_ItemSoldSound_0.ogg", FALSE, TRUE);
-};				// void TryToSellItem( item* SellItem )
+	play_sound("effects/Shop_ItemSoldSound_0.ogg");
+}
 
 /**
  * This function tries to buy the item given as parameter.
@@ -870,27 +718,24 @@ static int buy_item(item *BuyItem, int amount)
 		return -1;
 	}
 
-	CopyItem(BuyItem, &new_item, 0);
+	CopyItem(BuyItem, &new_item);
 	
 	if (BuyItem->multiplicity < amount)
 		amount = BuyItem->multiplicity;
 		
 	new_item.multiplicity = amount;
-	item_price = calculate_item_buy_price(&new_item);
+	item_price = calculate_item_buy_price(&new_item) * new_item.multiplicity;
 
 	// If the item is too expensive, bail out
 	if (item_price > Me.Gold) {
-		char linebuf[1000];
-		sprintf(linebuf, _("%s\n\nYou can't afford to purchase this item!"),
-		        ItemMap[BuyItem->type].item_name);
-		alert_window(linebuf);
+		alert_window("%s\n\n%s", ItemMap[BuyItem->type].item_name, _("You can not afford this item."));
 		return -1;
 	}
 
 	// Subtract money, give item, play sound.
 	Me.Gold -= item_price;
 	give_item(&new_item);
-	PlayOnceNeededSoundSample("effects/Shop_ItemBoughtSound_0.ogg", FALSE, FALSE);	
+	play_sound("effects/Shop_ItemBoughtSound_0.ogg");
 	
 	// Did player want all of the items?
 	if(BuyItem->multiplicity == amount) {
@@ -914,23 +759,20 @@ void InitTradeWithCharacter(struct npc *npc)
 {
 #define NUMBER_OF_ITEMS_IN_SHOP 17
 
-	item *SalesList;
 	item *BuyPointerList[MAX_ITEMS_IN_INVENTORY];
 	item *TuxItemsList[MAX_ITEMS_IN_INVENTORY];
 	int i;
 	int ItemSelected = 0;
 	shop_decision ShopOrder;
 	int NumberOfItemsInTuxRow = 0;
-	int NumberOfItemsInShop = 0;
+	struct dynarray *sold_items;
 
-	SalesList = npc_get_inventory(npc);
+	sold_items = npc_get_inventory(npc);
 
-	for (i = 0; i < MAX_ITEMS_IN_INVENTORY; i++) {
-		if (SalesList[i].type == (-1))
-			BuyPointerList[i] = NULL;
-		else
-			BuyPointerList[i] = &(SalesList[i]);
+	for (i = 0; i < sold_items->size && i < sizeof(BuyPointerList)/sizeof(BuyPointerList[0]); i++) {
+			BuyPointerList[i] = &((item *)(sold_items->arr))[i];
 	}
+
 	// Now here comes the new thing:  This will be a loop from now
 	// on.  The buy and buy and buy until at one point we say 'BACK'
 	//
@@ -938,14 +780,7 @@ void InitTradeWithCharacter(struct npc *npc)
 
 		NumberOfItemsInTuxRow = AssemblePointerListForItemShow(&(TuxItemsList[0]), TRUE);
 
-		for (i = 0; i < MAX_ITEMS_IN_INVENTORY; i++) {
-			if (BuyPointerList[i] == NULL) {
-				NumberOfItemsInShop = i;
-				break;
-			}
-		}
-
-		ItemSelected = GreatShopInterface(NumberOfItemsInShop, BuyPointerList, NumberOfItemsInTuxRow, TuxItemsList, &(ShopOrder));
+		ItemSelected = GreatShopInterface(sold_items->size, BuyPointerList, NumberOfItemsInTuxRow, TuxItemsList, &(ShopOrder));
 		switch (ShopOrder.shop_command) {
 		case BUY_1_ITEM:
 			if (buy_item(BuyPointerList[ShopOrder.item_selected], ShopOrder.number_selected) == 1) {
@@ -957,18 +792,15 @@ void InitTradeWithCharacter(struct npc *npc)
 			TryToSellItem(TuxItemsList[ShopOrder.item_selected], ShopOrder.number_selected);
 			break;
 		case REPAIR_ITEM:
-			TryToRepairItem(TuxItemsList[ShopOrder.item_selected]);
+			repair_item(TuxItemsList[ShopOrder.item_selected]);
 			break;
 		default:
 
 			break;
 		};
 
-		for (i = 0; i < MAX_ITEMS_IN_INVENTORY; i++) {
-			if (SalesList[i].type == (-1))
-				BuyPointerList[i] = NULL;
-			else
-				BuyPointerList[i] = &(SalesList[i]);
+		for (i = 0; i < sold_items->size && i < sizeof(BuyPointerList)/sizeof(BuyPointerList[0]); i++) {
+			BuyPointerList[i] = &((item *)(sold_items->arr))[i];
 		}
 	}
 }
