@@ -68,8 +68,12 @@
 #define SPELL_LEVEL_BUTTONS_X 60
 #define SPELL_LEVEL_BUTTONS_Y 423
 #define SPELL_LEVEL_BUTTON_WIDTH 30
+#define SPELL_LEVEL_BUTTON_HEIGHT SPELL_LEVEL_BUTTON_WIDTH
 
-int Override_Power_Limit = 0;
+#define NUMBER_OF_SKILL_PAGES 8
+
+static int Override_Power_Limit = 0;
+static struct image skill_level_images[NUMBER_OF_SKILL_PAGES];
 
 /**
  * This function improves a generic skill (hack melee ranged magic) by one
@@ -99,6 +103,30 @@ int improve_program(int prog_id)
 	return 0;
 }
 
+void downgrade_program(int prog_id)
+{
+	if (prog_id < 0)
+		return;
+
+	Me.skill_level[prog_id]--;
+	if (Me.skill_level[prog_id] < 0)
+		Me.skill_level[prog_id] = 0;
+
+	// If the current skill level was downgraded to 0, the current
+	// skill needs to be unselected.
+	if (!Me.skill_level[prog_id] && Me.readied_skill == prog_id) {
+		int i;
+		for (i = 0; i < MAX_NUMBER_OF_PROGRAMS; i++) {
+			if (Me.skill_level[i] > 0) {
+				activate_nth_skill(i);
+				return;
+			}
+		}
+
+		ErrorMessage(__FUNCTION__, "No skills are available for selection.", PLEASE_INFORM, IS_WARNING_ONLY);
+	}
+}
+
 /* ------------------
  * This function calculates the heat cost of running a given program (source or blob), based on current program level and casting ability
  * -----------------*/
@@ -203,7 +231,7 @@ static void TeleportHome(void)
 		// Teleport
 		teleport_arrival_sound();
 		reset_visible_levels();
-		Teleport(Me.teleport_anchor.z, Me.teleport_anchor.x, Me.teleport_anchor.y, TRUE);
+		Teleport(Me.teleport_anchor.z, Me.teleport_anchor.x, Me.teleport_anchor.y, TRUE, TRUE);
 		clear_active_bullets();
 
 		// Reset teleport anchor
@@ -222,7 +250,7 @@ static void TeleportHome(void)
 
 	teleport_arrival_sound();
 	reset_visible_levels();
-	Teleport(HomeSpot.level, HomeSpot.x, HomeSpot.y, TRUE);
+	Teleport(HomeSpot.level, HomeSpot.x, HomeSpot.y, TRUE, TRUE);
 	clear_active_bullets();
 
 }				// void TeleportHome ( void )
@@ -237,10 +265,8 @@ void HandleCurrentlyActivatedSkill()
 	if (!MouseRightClicked() || Me.busy_time > 0)
 		return;
 
-	int Grabbed_InvPos = GetInventoryItemAt(GetInventorySquare_x(GetMousePos_x()), GetInventorySquare_y(GetMousePos_y()));
-	if (Grabbed_InvPos != -1)
-		if (ItemMap[Me.Inventory[Grabbed_InvPos].type].item_can_be_applied_in_combat)
-			return;	//if the cursor is over an item that can be applied, then the player wants to apply it not trigger a spell
+	if (MouseCursorIsInInventoryGrid(GetMousePos_x(), GetMousePos_y()))
+		return;
 
 	if (Me.skill_level[Me.readied_skill] <= 0)
 		return;
@@ -326,6 +352,7 @@ int DoSkill(int skill_index, int SpellCost)
 		Me.slowdown_duration += strcmp(SpellSkillMap[skill_index].effect, "slowdown") ? 0 : effdur;
 		Me.paralyze_duration += strcmp(SpellSkillMap[skill_index].effect, "paralyze") ? 0 : effdur;
 		Me.invisible_duration += strcmp(SpellSkillMap[skill_index].effect, "invisibility") ? 0 : effdur;
+		Me.nmap_duration += strcmp(SpellSkillMap[skill_index].effect, "nmap") ? 0 : effdur;
 		Me.light_bonus_end_date = Me.current_game_date + (strcmp(SpellSkillMap[skill_index].effect, "light") ? 0 : effdur);
 		break;
 
@@ -341,7 +368,7 @@ int DoSkill(int skill_index, int SpellCost)
 		if (SpellSkillMap[skill_index].graphics_code != -1)
 			FillInDefaultBulletStruct(&bul_parms, SpellSkillMap[skill_index].graphics_code, GetItemIndexByName("Laser pistol"));
 		else
-			FillInDefaultBulletStruct(&bul_parms, MAGENTA_BULLET, GetItemIndexByName("Laser pistol"));
+			FillInDefaultBulletStruct(&bul_parms, GetBulletByName("half_magenta"), GetItemIndexByName("Laser pistol"));
 
 		bul_parms.freezing_level = strcmp(SpellSkillMap[skill_index].effect, "slowdown") ? 0 : effdur;
 		bul_parms.poison_duration = strcmp(SpellSkillMap[skill_index].effect, "poison") ? 0 : effdur;
@@ -361,38 +388,8 @@ int DoSkill(int skill_index, int SpellCost)
 
 	case PROGRAM_FORM_RADIAL:
 		Me.temperature += SpellCost;
-
-		int i, j;
-		for (i = 0; i < MAX_ACTIVE_SPELLS; i++) {
-			if (AllActiveSpells[i].img_type == (-1))
-				break;
-		}
-		if (i >= MAX_ACTIVE_SPELLS)
-			i = 0;
-
-		AllActiveSpells[i].img_type =
-		    (SpellSkillMap[skill_index].graphics_code == -1 ? 2 : SpellSkillMap[skill_index].graphics_code);
-		AllActiveSpells[i].spell_center.x = Me.pos.x;
-		AllActiveSpells[i].spell_center.y = Me.pos.y;
-		AllActiveSpells[i].spell_radius = 0.3;
-		AllActiveSpells[i].spell_age = 0;
-		AllActiveSpells[i].mine = 1;
-		if (SpellSkillMap[skill_index].hurt_humans && SpellSkillMap[skill_index].hurt_bots)
-			AllActiveSpells[i].hit_type = ATTACK_HIT_ALL;
-		else if (SpellSkillMap[skill_index].hurt_humans)
-			AllActiveSpells[i].hit_type = ATTACK_HIT_HUMANS;
-		else
-			AllActiveSpells[i].hit_type = ATTACK_HIT_BOTS;
-
-		for (j = 0; j < RADIAL_SPELL_DIRECTIONS; j++) {
-			AllActiveSpells[i].active_directions[j] = TRUE;
-		}
-
-		AllActiveSpells[i].freeze_duration = strcmp(SpellSkillMap[skill_index].effect, "slowdown") ? 0 : effdur;
-		AllActiveSpells[i].poison_duration = strcmp(SpellSkillMap[skill_index].effect, "poison") ? 0 : effdur;
-		AllActiveSpells[i].poison_dmg = strcmp(SpellSkillMap[skill_index].effect, "poison") ? 0 : hitdmg;
-		AllActiveSpells[i].paralyze_duration = strcmp(SpellSkillMap[skill_index].effect, "paralyze") ? 0 : effdur;
-		AllActiveSpells[i].damage = hitdmg;
+		
+		do_radial_skill(skill_index, Me.pos.x, Me.pos.y, 1);
 
 		return 1;
 
@@ -448,7 +445,7 @@ int DoSkill(int skill_index, int SpellCost)
 			// Do nothing here.  The right mouse click while in inventory screen
 			// will be handled in the inventory screen management function.
 			//
-			PlayOnceNeededSoundSample("effects/tux_ingame_comments/CantRepairThat.ogg", FALSE, FALSE);
+			play_sound("effects/tux_ingame_comments/CantRepairThat.ogg");
 		}
 		goto out;
 	}
@@ -463,6 +460,53 @@ int DoSkill(int skill_index, int SpellCost)
 	return 1;
 };				// void HandleCurrentlyActivatedSkill( void )
 
+
+/**
+ * This function starts a new radial skill (grenade blast, etc) from 
+ * the given x and y coordinates.
+ */
+void do_radial_skill(int skill_index, int pos_x, int pos_y, int from_tux)
+{
+	float hitdmg = calculate_program_hit_damage(skill_index);
+	float effdur = calculate_program_effect_duration(skill_index);
+
+	int i, j;
+	for (i = 0; i < MAX_ACTIVE_SPELLS; i++) {
+		if (AllActiveSpells[i].img_type == (-1))
+			break;
+	}
+	if (i >= MAX_ACTIVE_SPELLS)
+		i = 0;
+
+	AllActiveSpells[i].img_type =
+	    (SpellSkillMap[skill_index].graphics_code == -1 ? 2 : SpellSkillMap[skill_index].graphics_code);
+	AllActiveSpells[i].spell_center.x = pos_x;
+	AllActiveSpells[i].spell_center.y = pos_y;
+	AllActiveSpells[i].spell_radius = 0.3;
+	if (!strcmp(SpellSkillMap[skill_index].effect, "short"))
+		AllActiveSpells[i].spell_age = 0.5;
+	else
+		AllActiveSpells[i].spell_age = 0;
+
+	AllActiveSpells[i].mine = from_tux;
+	if (SpellSkillMap[skill_index].hurt_humans && SpellSkillMap[skill_index].hurt_bots)
+		AllActiveSpells[i].hit_type = ATTACK_HIT_ALL;
+	else if (SpellSkillMap[skill_index].hurt_humans)
+		AllActiveSpells[i].hit_type = ATTACK_HIT_HUMANS;
+	else
+		AllActiveSpells[i].hit_type = ATTACK_HIT_BOTS;
+
+	for (j = 0; j < RADIAL_SPELL_DIRECTIONS; j++) {
+		AllActiveSpells[i].active_directions[j] = TRUE;
+	}
+
+	AllActiveSpells[i].freeze_duration = strcmp(SpellSkillMap[skill_index].effect, "slowdown") ? 0 : effdur;
+	AllActiveSpells[i].poison_duration = strcmp(SpellSkillMap[skill_index].effect, "poison") ? 0 : effdur;
+	AllActiveSpells[i].poison_dmg = strcmp(SpellSkillMap[skill_index].effect, "poison") ? 0 : hitdmg;
+	AllActiveSpells[i].paralyze_duration = strcmp(SpellSkillMap[skill_index].effect, "paralyze") ? 0 : effdur;
+	AllActiveSpells[i].damage = hitdmg;
+}
+
 /**
  * This function checks if a given screen position lies within the 
  * one of the skill icons and returns the number of that skill icon.
@@ -521,11 +565,9 @@ static int CursorIsOnWhichSpellPageButton(int x, int y)
 	// First we check if the cursor is in at least horizontally
 	// and vertically in the line with the spell level buttons.
 	//
-	if (x > SkillScreenRect.x + SPELL_LEVEL_BUTTONS_X + 300)
-		return (-1);
 	if (x < SkillScreenRect.x + SPELL_LEVEL_BUTTONS_X)
 		return (-1);
-	if (y > SkillScreenRect.y + SPELL_LEVEL_BUTTONS_Y + 16)
+	if (y > SkillScreenRect.y + SPELL_LEVEL_BUTTONS_Y + SPELL_LEVEL_BUTTON_HEIGHT)
 		return (-1);
 	if (y < SkillScreenRect.y + SPELL_LEVEL_BUTTONS_Y)
 		return (-1);
@@ -535,12 +577,12 @@ static int CursorIsOnWhichSpellPageButton(int x, int y)
 	// at least, the row of skill icons.
 	//
 	for (i = 0; i < NUMBER_OF_SKILL_PAGES; i++) {
-		if (x < SkillScreenRect.x + SPELL_LEVEL_BUTTONS_X + (i + 1) * SPELL_LEVEL_BUTTON_WIDTH)
+		if (x < SkillScreenRect.x + (2.0 * ((float)GameConfig.screen_width / 640.0)) + SPELL_LEVEL_BUTTONS_X + (i + 1) * SPELL_LEVEL_BUTTON_WIDTH)
 			return i;
 	}
 
-	return (-1);
-};				// int CursorIsOnWhichSpellLevelButton( int x , int y )
+	return -1;
+}
 
 /** 
  *
@@ -552,34 +594,29 @@ static void ShowSkillsExplanationScreen(void)
 	int ICON_OFFSET_Y = 20;
 	int TEXT_OFFSET_X = 15;
 	SDL_Rect TargetSkillRect;
+	spell_skill_spec *spec = &SpellSkillMap[Me.readied_skill];;
 
 	// This should draw the background...
 	//
-	blit_special_background(SKILL_SCREEN_EXPLANATION_BACKGROUND_CODE);
+	blit_background("SkillExplanationScreen.png");
 
 	// Draws the skill icon at the correct position
 	//
 	TargetSkillRect.x = ICON_OFFSET_X;
 	TargetSkillRect.y = ICON_OFFSET_Y;
-	LoadOneSkillSurfaceIfNotYetLoaded(Me.readied_skill);
-	if (use_open_gl) {
-		draw_gl_textured_quad_at_screen_position(&SpellSkillMap[Me.readied_skill].icon_surface,
-							 TargetSkillRect.x, TargetSkillRect.y);
-	} else {
-		our_SDL_blit_surface_wrapper(SpellSkillMap[Me.readied_skill].icon_surface.surface, NULL, Screen, &TargetSkillRect);
-	}
+
+	load_skill_icon_if_needed(spec);
+	display_image_on_screen(&spec->icon_surface,
+						 TargetSkillRect.x, TargetSkillRect.y, IMAGE_NO_TRANSFO);
 
 	// Draws the explanation text
-	// (We will use the FPS display font, cause the small one isn't 
-	// very well readable on the silver background.)
 	//
 	TargetSkillRect.x = TEXT_OFFSET_X;
 	TargetSkillRect.w = 320 - 2 * TEXT_OFFSET_X;
 	TargetSkillRect.h = 480 - 15;
 	SetCurrentFont(FPS_Display_BFont);
-	DisplayText(D_(SpellSkillMap[Me.readied_skill].description), 16, 16 + 64 + 16, &TargetSkillRect, TEXT_STRETCH);
-
-};				// void ShowSkillsExplanationScreen( void )
+	display_text(D_(spec->description), 16, 16 + 64 + 16, &TargetSkillRect);
+}
 
 /** 
  * We will draw only those skills to the skills inventory, that are
@@ -623,7 +660,7 @@ void activate_nth_skill(int skill_num)
 		Me.readied_skill = skill_num;
 	} else {
 		ErrorMessage(__FUNCTION__,
-			"Tried to activate skill number %d which was not aquired yet.\n",
+			"Tried to activate skill number %d which was not acquired yet.\n",
 				PLEASE_INFORM, IS_WARNING_ONLY, skill_num);
 	}
 
@@ -648,7 +685,7 @@ void set_nth_quick_skill(int quick_skill)
 		if (Me.skill_level[ski] <=  0) {
 			// Invalid skill was selected
 			ErrorMessage(__FUNCTION__,
-				"Tried to set skill number %d in quick skills. Skill was not aquired yet.\n",
+				"Tried to set skill number %d in quick skills. Skill was not acquired yet.\n",
 					PLEASE_INFORM, IS_WARNING_ONLY, ski);
 			return;
 		}
@@ -662,6 +699,47 @@ void set_nth_quick_skill(int quick_skill)
 		Me.program_shortcuts[quick_skill] = ski;
 }; // void set_nth_quick_skill(int quick_skill)
 
+
+void load_skill_icon_if_needed(spell_skill_spec *spec)
+{
+	if (!image_loaded(&spec->icon_surface)) {
+		char filename[1000];
+		sprintf(filename, "skill_icons/%s", spec->icon_name);
+		load_image(&spec->icon_surface, filename, FALSE); 
+	}
+}
+
+/**
+ * This function loads the image containing the different buttons for the
+ * different skills in the skill book of the Tux.
+ */
+static void load_skill_level_images_if_needed(void)
+{
+#define SKILL_LEVEL_BUTTON_FILE "mouse_buttons/skill_buttons.png"
+	int i = 0;
+	struct image img = EMPTY_IMAGE;
+	SDL_Rect src;
+
+	if (image_loaded(&skill_level_images[0]))
+		return;
+
+	// Load the image
+	load_image(&img, SKILL_LEVEL_BUTTON_FILE, FALSE);
+
+	// Create the subimages
+	for (i = 0; i < NUMBER_OF_SKILL_PAGES; i++) {
+		src.x = i * (SKILL_LEVEL_BUTTON_WIDTH);
+		src.y = 0;
+		src.w = SKILL_LEVEL_BUTTON_WIDTH;
+		src.h = SKILL_LEVEL_BUTTON_HEIGHT;
+		create_subimage(&img, &skill_level_images[i], &src);
+	}
+	
+	// Delete the big image
+	free_image_surface(&img);
+}
+
+
 /**
  * This function displays the SKILLS SCREEN.  This is NOT the same as the
  * CHARACTER SCREEN.  In the skills screen you can see what skills/spells
@@ -706,11 +784,7 @@ void ShowSkillsScreen(void)
 	//
 	SetCurrentFont(FPS_Display_BFont);
 
-	// Maybe the skill circle images for clicking between different spell circles
-	// have not been loaded yet.  Then it is time to do so.  If this was already
-	// done before, then the function will know it and don't do anything anyway.
-	//
-	Load_Skill_Level_Button_Surfaces();
+	load_skill_level_images_if_needed();
 
 	// We will need the current mouse position on several spots...
 	//
@@ -727,10 +801,7 @@ void ShowSkillsScreen(void)
 	// At this point we know, that the skill screen is desired and must be
 	// displayed in-game:
 	//
-	// SDL_SetClipRect( Screen, NULL );
-	// our_SDL_blit_surface_wrapper ( SkillScreenImage , NULL , Screen , &SkillScreenRect );
-	//
-	blit_special_background(SKILL_SCREEN_BACKGROUND_CODE);
+	blit_background("SkillScreen.png");
 
 	if (GameConfig.skill_explanation_screen_visible)
 		ShowSkillsExplanationScreen();
@@ -740,7 +811,7 @@ void ShowSkillsScreen(void)
 	//
 	SpellLevelRect.x = SkillScreenRect.x + SPELL_LEVEL_BUTTONS_X + SPELL_LEVEL_BUTTON_WIDTH * GameConfig.spell_level_visible;
 	SpellLevelRect.y = SkillScreenRect.y + SPELL_LEVEL_BUTTONS_Y;
-	our_SDL_blit_surface_wrapper(SpellLevelButtonImageList[GameConfig.spell_level_visible].surface, NULL, Screen, &SpellLevelRect);
+	display_image_on_screen(&skill_level_images[GameConfig.spell_level_visible], SpellLevelRect.x, SpellLevelRect.y, IMAGE_NO_TRANSFO);
 
 	// Now we fill in the skills available to this bot.  ( For now, these skills 
 	// are not class-specific, like in diablo or something, but this is our first
@@ -758,13 +829,9 @@ void ShowSkillsScreen(void)
 		if (SkillOfThisSlot < 0)
 			continue;
 
-		LoadOneSkillSurfaceIfNotYetLoaded(SkillOfThisSlot);
+		load_skill_icon_if_needed(&SpellSkillMap[SkillOfThisSlot]);
 
-		if (use_open_gl) {
-			draw_gl_textured_quad_at_screen_position(&SpellSkillMap[SkillOfThisSlot].icon_surface, ButtonRect.x, ButtonRect.y);
-		} else {
-			our_SDL_blit_surface_wrapper(SpellSkillMap[SkillOfThisSlot].icon_surface.surface, NULL, Screen, &ButtonRect);
-		}
+		display_image_on_screen(&SpellSkillMap[SkillOfThisSlot].icon_surface, ButtonRect.x, ButtonRect.y, IMAGE_NO_TRANSFO);
 
 		SetCurrentFont(FPS_Display_BFont);
 
@@ -779,13 +846,13 @@ void ShowSkillsScreen(void)
 			// print the quick key number
 			char str[10];
 			sprintf(str, "F%d\n", 5 + sci);
-			DisplayText(str, ButtonRect.x + ButtonRect.w - 2 - TextWidth(str), ButtonRect.y, &SkillScreenRect, TEXT_STRETCH);
+			display_text(str, ButtonRect.x + ButtonRect.w - 2 - TextWidth(str), ButtonRect.y, &SkillScreenRect);
 		}
 		// Name of the skill
 
-		DisplayText(D_(SpellSkillMap[SkillOfThisSlot].name),
+		display_text(D_(SpellSkillMap[SkillOfThisSlot].name),
 			    16 + 64 + 16 + SkillScreenRect.x,
-			    FIRST_SKILLRECT_Y - 6 + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y, &SkillScreenRect, TEXT_STRETCH);
+			    FIRST_SKILLRECT_Y - 6 + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y, &SkillScreenRect);
 
 		SetCurrentFont(Messagestat_BFont);
 		int tmp, tmp2;
@@ -795,7 +862,7 @@ void ShowSkillsScreen(void)
 		// Program revision
 		sprintf(CharText, _("Program revision: %c%d%c "), font_switchto_msgvar[0], Me.skill_level[SkillOfThisSlot],
 			font_switchto_msgstat[0]);
-		DisplayText(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, TEXT_STRETCH);
+		display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect);
 		nextypos += FontHeight(GetCurrentFont());
 
 		// Heat cost/cooling
@@ -805,7 +872,7 @@ void ShowSkillsScreen(void)
 				sprintf(CharText, _("Heat produced: %c%d%c "), font_switchto_msgvar[0], tmp, font_switchto_msgstat[0]);
 			else
 				sprintf(CharText, _("Cooling: %c%d%c "), font_switchto_msgvar[0], -tmp, font_switchto_msgstat[0]);
-			DisplayText(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, TEXT_STRETCH);
+			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect);
 			nextypos += FontHeight(GetCurrentFont());
 		}
 		// Damage/healing
@@ -825,7 +892,7 @@ void ShowSkillsScreen(void)
 					sprintf(CharText, _("Healing: %c%d-%d%c "), font_switchto_msgvar[0], -tmp, -tmp2,
 						font_switchto_msgstat[0]);
 			}
-			DisplayText(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, TEXT_STRETCH);
+			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect);
 			nextypos += FontHeight(GetCurrentFont());
 		}
 		// Special effect and duration
@@ -850,7 +917,7 @@ void ShowSkillsScreen(void)
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "weapon"))
 				sprintf(CharText, " ");
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "repair"))
-				sprintf(CharText, " ");
+				sprintf(CharText, "Repair items, degrading them a bit");
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "satellite"))
 				sprintf(CharText, " ");
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "light"))
@@ -863,43 +930,36 @@ void ShowSkillsScreen(void)
 				sprintf(CharText + strlen(CharText), _(" for %c%.1f%c seconds"), font_switchto_msgvar[0], dur,
 					font_switchto_msgstat[0]);
 
-			DisplayText(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, TEXT_STRETCH);
+			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect);
 			nextypos += FontHeight(GetCurrentFont());
 		}
-
 	}
 
-	// Now we see if perhaps the player has just clicked on one of the skills
-	// available to this class.  In this case of course we must set a different
-	// skill/spell as the currently activated skill/spell.
-	//
-	if ((CursorIsOnWhichSkillButton(CurPos.x, CurPos.y) != (-1)) && MouseLeftClicked()) {
-		if (CursorIsOnWhichSkillButton(CurPos.x, CurPos.y) +
-		    NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible < number_of_skills)
-			if (SkillSubsetMap[CursorIsOnWhichSkillButton(CurPos.x, CurPos.y) +
-					   NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible] >= 0)
-				Me.readied_skill = SkillSubsetMap[CursorIsOnWhichSkillButton(CurPos.x, CurPos.y) +
-								  NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible];
-	}
+	if (!world_frozen()) {
+		// Now we see if perhaps the player has just clicked on one of the skills
+		// available to this class.  In this case of course we must set a different
+		// skill/spell as the currently activated skill/spell.
+		//
+		if ((CursorIsOnWhichSkillButton(CurPos.x, CurPos.y) != (-1)) && MouseLeftClicked()) {
+			if (CursorIsOnWhichSkillButton(CurPos.x, CurPos.y) +
+				NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible < number_of_skills)
+				if (SkillSubsetMap[CursorIsOnWhichSkillButton(CurPos.x, CurPos.y) +
+						   NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible] >= 0)
+					Me.readied_skill = SkillSubsetMap[CursorIsOnWhichSkillButton(CurPos.x, CurPos.y) +
+									  NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible];
+		}
 
-	if (MouseCursorIsOnButton(OPEN_CLOSE_SKILL_EXPLANATION_BUTTON, CurPos.x, CurPos.y) && MouseLeftClicked()) {
-		toggle_game_config_screen_visibility(GAME_CONFIG_SCREEN_VISIBLE_SKILL_EXPLANATION);
-		while (MouseLeftPressed())
-			SDL_Delay(1);
-	}
-	// Now we see if perhaps the player has just clicked on another skill level
-	// button.  In this case of course we must set a different skill/spell level
-	// as the currently visible spell level.
-	//
-	if ((CursorIsOnWhichSpellPageButton(CurPos.x, CurPos.y) != (-1)) && MouseLeftClicked()) {
-		GameConfig.spell_level_visible = CursorIsOnWhichSpellPageButton(CurPos.x, CurPos.y);
-	}
-	// We want to know, if the button was pressed the previous frame when we
-	// are in the next frame and back in this function.  Therefore we store
-	// the current button situation, so that we can conclude on button just
-	// pressed later.
-	//
+		if (MouseCursorIsOnButton(OPEN_CLOSE_SKILL_EXPLANATION_BUTTON, CurPos.x, CurPos.y) && MouseLeftClicked()) {
+			toggle_game_config_screen_visibility(GAME_CONFIG_SCREEN_VISIBLE_SKILL_EXPLANATION);
+			while (MouseLeftPressed())
+				SDL_Delay(1);
+		}
 
-};				// ShowSkillsScreen ( void )
+		// Handle clicks on page numbers
+		if ((CursorIsOnWhichSpellPageButton(CurPos.x, CurPos.y) != (-1)) && MouseLeftClicked()) {
+			GameConfig.spell_level_visible = CursorIsOnWhichSpellPageButton(CurPos.x, CurPos.y);
+		}
+	}
+}
 
 #undef _skills_c
