@@ -30,16 +30,12 @@
 #include "system.h"
 #include "defs.h"
 
-typedef struct tColorRGBA {
+typedef struct {
 	Uint8 r;
 	Uint8 g;
 	Uint8 b;
 	Uint8 a;
-} tColorRGBA, myColor;
-
-typedef struct tColorY {
-	Uint8 y;
-} tColorY;
+} myColor;
 
 /**
  * Simple doubly linked list implementation.
@@ -58,6 +54,8 @@ struct dynarray {
 	int capacity;
 };
 
+typedef struct dynarray item_dynarray;
+
 typedef struct upgrade_socket_dynarray {
 	struct upgrade_socket *arr;
 	int size;
@@ -67,63 +65,60 @@ typedef struct upgrade_socket_dynarray {
 /* prototype for BFont_Info from BFont.h */
 struct BFont_Info;
 
-/* ----------------------------------------------------------------------
- * Here comes the struct for an iso image.  It also contains some 
- * placeholder for a possible 'zoomed-out' version of the iso image and
- * also some placeholder for a possible OpenGL texture as well.
- *
- * In order to have a convenient means to initialize variables of this
- * type in several places, even in case this struct changes shape in the
- * future, the 'UNLOADED_ISO_IMAGE' definition below is made.  This
- * definition should be used always when initializing a variable of this
- * so that later changes to the struct can be made with minimal effort 
- * and mistakes.
- * ---------------------------------------------------------------------- */
-typedef struct iso_image {
+struct image_transformation {
+	SDL_Surface *surface;		
+	float scale_x;
+	float scale_y;
+	float c[4]; /**< color transformation, r g b a in that order */
+	int highlight;
+};
+
+/**
+ * This structure defines an image in FreedroidRPG. It contains information
+ * that enables rendering using SDL or OpenGL.
+ */
+struct image {
 	SDL_Surface *surface;
 	short offset_x;
 	short offset_y;
-	SDL_Surface *zoomed_out_surface;
-	short texture_width;
-	short texture_height;
-	short original_image_width;
-	short original_image_height;
+	short w;
+	short h;
 	int texture_has_been_created;
 #ifdef HAVE_LIBGL
-	GLuint texture;		// this is to store an open_gl texture...
+	GLuint texture;
 #else
-	int placeholder_for_texture_value;	// this is to store an open_gl texture...
+	int texture;
 #endif
-	float tx0;
-	float tx1;
-	float ty0;
-	float ty1;
-} iso_image, *Iso_image;
-#define UNLOADED_ISO_IMAGE { NULL , 0 , 0 , NULL , 0 , 0 , 0, 0, 0, 0, 0.0, 0.0, 0.0, 0.0 }
+	float tex_x0;
+	float tex_x1;
+	float tex_y0;
+	float tex_y1;
+	short tex_w;
+	short tex_h;
+
+	struct image_transformation cached_transformation;
+};
+#define EMPTY_IMAGE { .surface = NULL , .offset_x = 0 , .offset_y = 0 , .texture_has_been_created = 0 , .cached_transformation = { NULL, 0.0, 0.0, { 0.0, 0.0, 0.0, 0.0}, 0 } }
 
 typedef struct mouse_press_button {
-	iso_image button_image;
+	struct image button_image;
 	char *button_image_file_name;
 	SDL_Rect button_rect;
 	char scale_this_button;
 } mouse_press_button, *Mouse_press_button;
 
-typedef struct keybind {
-	char *name;
-		/**< keybinding name, taken from keybindNames */
-	int key;
-	     /**< key/axis/button event number */
-	int mod;
-	     /**< Key modifiers */
-} keybind_t;
-
 typedef char *string;
 
+typedef struct keybind {
+	string name; /**< keybinding name, taken from keybindNames */
+	int key;      /**< key/axis/button event number */
+	int mod;      /**< Key modifiers */
+} keybind_t;
+
 typedef struct configuration_for_freedroid {
 	float WantedTextVisibleTime;
 	int Draw_Framerate;
 	int Draw_Position;
-	int Enemy_Hit_Text;
 	int All_Texts_Switch;
 	float Current_BG_Music_Volume;
 	float Current_Sound_FX_Volume;
@@ -142,12 +137,14 @@ typedef struct configuration_for_freedroid {
 	int skill_explanation_screen_visible;
 	int enemy_energy_bars_visible;
 	int limit_framerate;
-	int highlighting_mode_full;
 	int omit_obstacles_in_level_editor;
 	int omit_enemies_in_level_editor;
 	int zoom_is_on;
 	int show_blood;		// this can be used to make the game more friendly for children...
-	int show_tooltips;
+	int show_lvledit_tooltips;
+	int show_grid;
+	int show_wp_connections;
+	int grid_mode;
 	int number_of_big_screen_messages;
 	float delay_for_big_screen_messages;
 	int enable_cheatkeys;
@@ -156,16 +153,17 @@ typedef struct configuration_for_freedroid {
 	int screen_height;
 	int next_time_width_of_screen;
 	int next_time_height_of_screen;
-	float automap_display_scale;
 	int skip_shadow_blitting;
 	int do_fadings;		// do the screen fadings
 	int fullscreen_on;
 	int talk_to_bots_after_takeover;
 	int xray_vision_for_tux;
 	int cheat_running_stamina;
+	int cheat_double_speed;
 	int lazyload;
 	int show_item_labels;
 	int last_edited_level;
+	int show_all_floor_layers;
 
 	int difficulty_level;
 
@@ -232,31 +230,6 @@ typedef struct mission {
 	int expanded_display_for_this_mission;
 } mission;
 
-//--------------------
-// This structure can contain conditions that must be fulfilled, so that a special
-// event is triggered.  Such conditions may be specified in the mission file as well
-//
-typedef struct event_trigger {
-	// Maybe the event is triggerd by the influencer stepping somewhere
-	int Influ_Must_Be_At_Level;
-	point Influ_Must_Be_At_Point;
-
-	int DeleteTriggerAfterExecution;
-
-	luacode lua_code;
-
-	char *name;
-	int enabled;		//is the trigger enabled?
-	int silent;		//do we have to advertise this trigger to the user? (teleporters..)
-} event_trigger;
-
-typedef struct item_image_spec {
-	point inv_size;
-	SDL_Surface *Surface;
-	iso_image ingame_iso_image;
-	iso_image shop_iso_image;
-} item_image_spec;
-
 struct addon_bonus {
 	char *name;
 	int value;
@@ -283,12 +256,13 @@ typedef struct itemspec {
 	char *item_drop_sound_file_name;
 	char *item_inv_file_name;
 
-	char item_can_be_applied_in_combat;
+	char *item_combat_use_description;
 	char item_can_be_installed_in_weapon_slot;
 	char item_can_be_installed_in_drive_slot;
 	char item_can_be_installed_in_armour_slot;
 	char item_can_be_installed_in_shield_slot;
 	char item_can_be_installed_in_special_slot;
+	char *tux_part_instance;
 
 	char item_group_together_in_inventory;
 
@@ -300,10 +274,7 @@ typedef struct itemspec {
 	short base_item_gun_damage;	//   damage done by this bullettype 
 	short item_gun_damage_modifier;	// modifier to the damage done by this bullettype 
 	float item_gun_bullet_lifetime;	// how long does a 'bullet' from this gun type live?
-	char item_gun_bullet_reflect_other_bullets;	// can this 'bullet' reflect other bullets
-	char item_gun_bullet_pass_through_explosions;	// can this 'bullet' reflect other bullets
 	char item_gun_bullet_pass_through_hit_bodies;	// does this bullet go through hit bodies (e.g. like a laser sword)
-	char item_gun_bullet_ignore_wall_collisions;	// can this bullet pass through walls and map barriers?
 	short item_gun_ammo_clip_size;	//max. number of bullets in the charger
 
 	// the following values have only relevance in case of a melee weapon
@@ -317,22 +288,26 @@ typedef struct itemspec {
 	short base_armor_class;
 	short armor_class_modifier;
 
-	// which requirement for strength, dexterity and magic (force) does the item have?
+	// which requirement for strength, dexterity and cooling does the item have?
 	short int item_require_strength;
 	short int item_require_dexterity;
-	short int item_require_magic;
+	short int item_require_cooling;
 
-	// what duration does the item have?
-	short int base_item_duration;
-	short int item_duration_modifier;
+	// what durability does the item have?
+	short int base_item_durability;
+	short int item_durability_modifier;
+
+	point inv_size;
+	struct image inventory_image;
+	struct image ingame_image;
+	struct image shop_image;
 
-	// Which picture to use for this item, when it's lying on the floor?
-	// int picture_number;
-	item_image_spec inv_image;
 	short int base_list_price;	// the base price of this item at the shop
 
 	short int min_drop_class;
 	short int max_drop_class;
+	int drop_amount;		// minimum number of items to drop at once
+	int drop_amount_max;	// maximum of items to drop at once
 
 } itemspec;
 
@@ -347,8 +322,8 @@ typedef struct item {
 	gps virt_pos;
 	SDL_Rect text_slot_rectangle;
 	int type;
-	int max_duration;	// the maximum item durability reachable for this item
-	float current_duration;	// the currently remaining durability for this item
+	int max_durability;	// the maximum item durability reachable for this item
+	float current_durability;	// the currently remaining durability for this item
 	float throw_time;	// has this item just jumped out from a chest maybe or is it jumping right now?
 
 	int bonus_to_dex;
@@ -362,8 +337,6 @@ typedef struct item {
 	int bonus_to_all_attributes;
 	int bonus_to_armor_class;
 	int bonus_to_damage;
-	int bonus_to_resist_fire;	// this is a percentage
-	int bonus_to_resist_electricity;	// this is a percentage
 	int bonus_to_paralyze_enemy;
 	int bonus_to_slow_enemy;
 	int bonus_to_light_radius;
@@ -377,6 +350,7 @@ typedef struct item {
 	point inventory_position;
 
 	struct upgrade_socket_dynarray upgrade_sockets;
+	int quality;
 } item;
 
 typedef struct druidspec {
@@ -398,10 +372,10 @@ typedef struct druidspec {
 	float aggression_distance;
 	float time_spent_eyeing_tux;
 
-	short int monster_level;
-	short int forced_magic_items;
+	short int drop_class;
 
 	item weapon_item;
+	int gun_muzzle_height;
 
 	short int amount_of_plasma_transistors;
 	short int amount_of_superconductors;
@@ -418,6 +392,8 @@ typedef struct druidspec {
 	short individual_shape_nr;
 } druidspec;
 
+typedef char s_char; // Used for pointer to static string which are not to be saved
+
 typedef struct enemy {
 	// There are three sets of attributes, which are initialized and
 	// possibly re-initialized by 3 different codes:
@@ -445,7 +421,7 @@ typedef struct enemy {
 	//
 	short int id;                       // unique id of the droid. start at 1.
 	short int type;                     // the number of the droid specifications in Druidmap
-	char SpecialForce;                  // this flag will exclude the droid from initial shuffling of droids
+	uint8_t SpecialForce;               // this flag will exclude the droid from initial shuffling of droids
 	int marker;                         // this provides a marker for special mission targets
 	short int max_distance_to_home;     // how far this robot will go before returning to it's home waypoint
 	string dialog_section_name;
@@ -456,17 +432,17 @@ typedef struct enemy {
 	// 2nd set ('global state')
 	//
 	int faction;
-	char will_rush_tux;           // will this robot approach the Tux on sight and open communication?
-	int combat_state;             // current state of the bot
-	float state_timeout;          // time spent in this state (used by "timeout" states such as STOP_AND_EYE_TARGET only)
-	short int CompletelyFixed;    // set this flag to make the robot entirely immobile
-	char has_been_taken_over;     // has the Tux made this a friendly bot via takeover subgame?
-	char follow_tux;              // does this robot try to follow tux via it's random movements?
-	char has_greeted_influencer;  // has this robot issued his first-time-see-the-Tux message?
-	short int nextwaypoint;       // the next waypoint target
-	short int lastwaypoint;       // the waypoint from whence this robot just came from
-	short int homewaypoint;       // the waypoint this robot started at
-	gps pos;                      // coordinates of the current position in the level
+	uint8_t will_rush_tux;          // will this robot approach the Tux on sight and open communication?
+	int combat_state;               // current state of the bot
+	float state_timeout;            // time spent in this state (used by "timeout" states such as STOP_AND_EYE_TARGET only)
+	short int CompletelyFixed;      // set this flag to make the robot entirely immobile
+	uint8_t has_been_taken_over;    // has the Tux made this a friendly bot via takeover subgame?
+	uint8_t follow_tux;             // does this robot try to follow tux via it's random movements?
+	uint8_t has_greeted_influencer; // has this robot issued his first-time-see-the-Tux message?
+	short int nextwaypoint;         // the next waypoint target
+	short int lastwaypoint;         // the waypoint from whence this robot just came from
+	short int homewaypoint;         // the waypoint this robot started at
+	gps pos;                        // coordinates of the current position in the level
 
 	//--------------------
 	// 3rd set ('transient state')
@@ -482,7 +458,7 @@ typedef struct enemy {
 	float pure_wait;                   // time till the droid will start to move again
 	float firewait;                    // time this robot still takes until its weapon will be fully reloaded
 	short int ammo_left;               // ammunition left in the charger
-	char attack_target_type;           // attack NOTHING, PLAYER, or BOT
+	uint8_t attack_target_type;        // attack NOTHING, PLAYER, or BOT
 	short int bot_target_n;
 	struct enemy *bot_target_addr;
 	float previous_angle;              // which angle has this robot been facing the frame before?
@@ -491,9 +467,9 @@ typedef struct enemy {
 	float last_phase_change;           // when did the robot last change his (8-way-)direction of facing
 	float last_combat_step;            // when did this robot last make a step to move in closer or farther away from Tux in combat?
 	float TextVisibleTime;
-	char *TextToBeDisplayed;           // WARNING!!! Only use static texts
+	s_char *TextToBeDisplayed;         // WARNING!!! Only use static texts
 	moderately_finepoint PrivatePathway[5];
-	char bot_stuck_in_wall_at_previous_check;
+	uint8_t bot_stuck_in_wall_at_previous_check;
 	float time_since_previous_stuck_in_wall_check;
 
 	//--------------------
@@ -506,12 +482,12 @@ typedef struct enemy {
 
 typedef struct npc {
 	string dialog_basename;
-	unsigned char chat_character_initialized;
-	unsigned char chat_flags[MAX_ANSWERS_PER_PERSON];
+	uint8_t chat_character_initialized;
+	uint8_t chat_flags[MAX_ANSWERS_PER_PERSON];
 
 	string shoplist[MAX_ITEMS_IN_INVENTORY]; //list of items that can be put in the inventory of the NPC
 	int shoplistweight[MAX_ITEMS_IN_INVENTORY]; //weight of each item: relative probability of appearance in inventory
-	item npc_inventory[MAX_ITEMS_IN_NPC_INVENTORY]; // current NPC stock (what can be readily bought)
+	item_dynarray npc_inventory;
 
 	float last_trading_date; // when did we trade with this NPC the last time?
 
@@ -534,7 +510,7 @@ typedef struct tux {
 	gps mouse_move_target;	// where the tux is going automatically by virtue of mouse move
 
 	short int current_enemy_target_n;	//which enemy has been targeted 
-	char god_mode;
+	uint8_t god_mode;
 	enemy *current_enemy_target_addr;	// which enemy has been targeted, address
 
 	int mouse_move_target_combo_action_type;	// what extra action has to be done upon arrival?
@@ -565,41 +541,36 @@ typedef struct tux {
 	float MissionTimeElapsed;
 	float got_hit_time;	// how long stunned now since the last time tux got hit 
 
-	char savegame_version_string[1000];	// a string to identify games from older freedroid versions
+	string savegame_version_string;	// a string to identify games from older FreedroidRPG versions
 
-	int Strength;		// character Strength value = 'power supply max. capacity'
-	int Magic;		// character Magic value = 
-	int Dexterity;		// character Dexterity value = 'power redistribution speed'
-	int base_vitality;	// character Vitality value = 'cloaking field maximum strength'
-	int base_strength;	// character Strength value = 'power supply max. capacity'
-	int base_magic;		// character Magic value = 
-	int base_dexterity;	// character Dexterity value = 'power redistribution speed'
-	int Vitality;		// character Vitality value = 'cloaking field maximum strength'
+	int base_cooling;
+	int base_dexterity;
+	int base_physique;
+	int base_strength;
+	int cooling;
+	int dexterity;
+	int physique;
+	int strength;
 	int points_to_distribute;	// these are the points that are available to distribute upon the character stats
 	float base_damage;	// the current damage the influencer does
 	float damage_modifier;	// the modifier to the damage the influencer currently does
 	float armor_class;
 	float to_hit;		// percentage chance, that Tux will hit a random lv 1 bot
-	int resist_fire;	// percentage to reduce from fire damage
-	int resist_electricity;	// percentage to reduce from electricity damage
 
 	int slowing_melee_targets;	// duration for how long hit enemies are slowed down
 	int paralyzing_melee_targets;	// duration for how long hit enemies are paralyzed
-	int double_ranged_damage;	// does this Tux do double ranged weapon damage?
 	float experience_factor; // multiplier for the experience gained from bots
 
-	unsigned int Experience;	// character Experience = 'spare droid elements found'
-	int exp_level;		// which 'experience level' is the influencer currenly at?
-	unsigned int ExpRequired;	// how much experience required for the next level?
-	unsigned int ExpRequired_previously;	// how was required for the previous level?
+	unsigned int Experience;	
+	int exp_level;			
 
 	unsigned int Gold;
-	char character_name[MAX_CHARACTER_NAME_LENGTH];
+	string character_name;
 	mission AllMissions[MAX_MISSIONS_IN_GAME];	// What must be done to fullfill this mission?
 	int marker;		// In case you've taken over a marked droid, this will contain the marker
 	float LastCrysoundTime;
 	float TextVisibleTime;
-	char *TextToBeDisplayed;                    // WARNING!!! Only use static texts
+	s_char *TextToBeDisplayed;                    // WARNING!!! Only use static texts
 
 	//--------------------
 	// Here we note all the 'skill levels' of the Tux and also which skill is
@@ -610,10 +581,9 @@ typedef struct tux {
 	int melee_weapon_skill;
 	int ranged_weapon_skill;
 	int spellcasting_skill;
-
 	//--------------------
 	// The inventory slots.  Some items are residing in general inventory,
-	// other items might be equiped in some of the corresponding slots of
+	// other items might be equipped in some of the corresponding slots of
 	// the inventory screen.
 	//
 	item Inventory[MAX_ITEMS_IN_INVENTORY];
@@ -626,11 +596,9 @@ typedef struct tux {
 	//--------------------
 	// A record of when and if the tux has been on some maps...
 	//
-	unsigned char HaveBeenToLevel[MAX_LEVELS];	// record of the levels the player has visited yet.
+	uint8_t HaveBeenToLevel[MAX_LEVELS];	            // record of the levels the player has visited yet.
 	float time_since_last_visit_or_respawn[MAX_LEVELS];	// record of the levels the player has visited yet.
 
-	string cookie_list[MAX_COOKIES];
-
 	//--------------------
 	// THE FOLLOWING ARE INFORMATION, THAT ARE HUGE AND THAT ALSO DO NOT NEED
 	// TO BE COMMUNICATED FROM THE CLIENT TO THE SERVER OR VICE VERSA
@@ -647,6 +615,7 @@ typedef struct tux {
 	float slowdown_duration;
 	float paralyze_duration;
 	float invisible_duration;
+	float nmap_duration;
 
 	int quest_browser_changed;
 
@@ -663,63 +632,66 @@ typedef struct tux {
 } tux_t;
 
 typedef struct bulletspec {
+	char *name;		// what is the name of this bullet type
+	char *sound;		// what sound to play
 	int phases;		// how many phases in motion to show 
 	double phase_changes_per_second;	// how many different phases to display every second
-	iso_image image[BULLET_DIRECTIONS][MAX_PHASES_IN_A_BULLET];
+	int blast_type;
+	struct image image[BULLET_DIRECTIONS][MAX_PHASES_IN_A_BULLET];
 } bulletspec;
 
 typedef struct bullet {
 	short int type;
-	unsigned char phase;
-	signed char mine;
+	int phase;
+	uint8_t mine;
 	gps pos;
+	int height;
 	moderately_finepoint speed;
-	short int time_in_frames;	// how long does the bullet exist, measured in number of frames
 	short int damage;	// damage done by this particular bullet
 	float time_in_seconds;	// how long does the bullet exist in seconds
 	float bullet_lifetime;	// how long can this bullet exist at most
 	float time_to_hide_still;	// countdown to when the bullet will actually appear
-	char reflect_other_bullets;
 	short int owner;
 	float angle;
 
-	char was_reflected;
-	char ignore_wall_collisions;
-	char pass_through_hit_bodies;	// does this bullet go through hit bodies (e.g. like a laser sword stike)
-	char pass_through_explosions;	// does this bullet go through explosions (e.g. laser sword stike though dead droid)
+	uint8_t pass_through_hit_bodies; // does this bullet go through hit bodies (e.g. like a laser sword strike)
 	short int freezing_level;	// does this bullet freeze the target?
 	float poison_duration;
 	float poison_damage_per_sec;
 	float paralysation_duration;
 
 	int faction;
-	char hit_type;		//hit bots, humans, both?
+	uint8_t hit_type;		//hit bots, humans, both?
 } bullet, *Bullet;
 
 typedef struct melee_shot	// this is a melee shot
 {
-	char attack_target_type;	//type of attack
-	char mine;		//is it mine?
+	uint8_t attack_target_type;	//type of attack
+	uint8_t mine;		//is it mine?
 	short int bot_target_n;	//which enemy has been targeted 
 	enemy *bot_target_addr;	// which enemy has been targeted, address
 	short int to_hit;	//chance to hit, percent
 	short int damage;
 	short int owner;
+	float time_to_hit;	//Time until the 'shot' makes contact
 } melee_shot;
 
 typedef struct blastspec {
 	int phases;
 	float total_animation_time;
-	iso_image image[PHASES_OF_EACH_BLAST];
+	int do_damage;
+	struct image *images;
+	char *name;
+	string sound_file;
 } blastspec;
 
 typedef struct blast {
 	gps pos;
 	int type;
 	float phase;
-	int MessageWasDone;
 	float damage_per_second;
-} blast, *Blast;
+	int faction;
+} blast;
 
 typedef struct spell_active {
 	int img_type;		// what type of spell is active?
@@ -731,15 +703,15 @@ typedef struct spell_active {
 	moderately_finepoint spell_center;
 	float spell_radius;
 	float spell_age;
-	char active_directions[RADIAL_SPELL_DIRECTIONS];
+	uint8_t active_directions[RADIAL_SPELL_DIRECTIONS];
 	int mine;
-	char hit_type;
+	uint8_t hit_type;
 } spell_active;
 
 typedef struct spell_skill_spec {
 	char *name;
 	char *icon_name;
-	iso_image icon_surface;
+	struct image icon_surface;
 	short heat_cost;
 	short heat_cost_per_level;
 	short damage_base;
@@ -766,11 +738,13 @@ typedef struct waypoint {
 typedef struct obstacle {
 	int type;
 	gps pos;
+	int timestamp;
+	int frame_index;
 } obstacle;
 
 typedef struct map_tile {
-	Uint16 floor_value;
-	int obstacles_glued_to_here[MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE];
+	Uint16 floor_values[MAX_FLOOR_LAYERS];
+	struct dynarray glued_obstacles;
 } map_tile;
 
 struct obstacle_extension {
@@ -791,12 +765,12 @@ typedef struct level {
 	char *Levelname;
 	char *Background_Song_Name;
 
+	int floor_layers;
 	map_tile *map[MAX_MAP_LINES];	// this is a vector of pointers
 	int jump_target_north;
 	int jump_target_south;
 	int jump_target_east;
 	int jump_target_west;
-	int use_underground_lighting;
 
 	obstacle obstacle_list[MAX_OBSTACLES_ON_MAP];
 	item ItemList[MAX_ITEMS_PER_LEVEL];
@@ -804,16 +778,26 @@ typedef struct level {
 	struct dynarray obstacle_extensions;
 	struct dynarray map_labels;
 	struct dynarray waypoints;
+	struct {
+		int nr;
+		int types[10];
+		int types_size;
+	} random_droids;
 
 	int teleport_pair;
 } level, *Level;
 
-typedef struct obstacle_spec {
-	iso_image image;
-	iso_image shadow_image;
+typedef void (*action_fptr) (level *obst_lvl, int obstacle_idx);
+typedef int (*animation_fptr) (level *obst_lvl, int obstacle_idx);
 
-	char image_loaded;
-    char *label; 
+struct obstacle_graphics {
+	int count;
+	struct image *images;
+	struct image *shadows;
+};
+
+typedef struct obstacle_spec {
+	char *label; 
 
 	//--------------------
 	// Some obstacles will block the Tux from walking through them.
@@ -836,21 +820,27 @@ typedef struct obstacle_spec {
 	unsigned int flags;
 
 	//--------------------
-	// Some obstacles will emitt light.  Specify light strength here.
+	// Some obstacles will emit light.  Specify light strength here.
 	// A value of 0 light will be sufficient in most cases...
 	//
-	short emitted_light_strength;
+	struct dynarray emitted_light_strength;
 	char transparent;
-	
-	char *filename;
-    void (*action) (level *obst_lvl, int obstacle_id);
-	
+
+	struct dynarray filenames;
+	action_fptr action_fn;
+
 	//--------------------
 	// Some obstacles have an associated animation.
 	// This property defines the function to call to animate them
-	int (*animate_fn) (level *obstacle_lvl, int obstacle_idx);
+	animation_fptr animation_fn;
+	float animation_fps;
 } obstacle_spec;
 
+struct obstacle_group {
+	const char *name;
+	struct dynarray members;
+};
+
 struct visible_level {
 	int valid;
 	level *lvl_pointer;
@@ -866,10 +856,10 @@ typedef struct ship {
 } ship;
 
 typedef struct dialogue_option {
+	char *topic;
 	char *option_text;
-	char *option_sample_file_name;
+	int no_text;
 	luacode lua_code;
-
 	int exists;
 } dialogue_option;
 
@@ -897,7 +887,7 @@ typedef struct screen_resolution {
 } screen_resolution;
 
 /*
- * CheckIfWayIsFreeOfDroids's execution context
+ * [way|location]_free_of_droids's execution context
  * 
  * Note : '2' excepted bots are needed when the pathfinder is called
  *         to let a bot reach an other one (used during attack, for example).
@@ -912,7 +902,7 @@ typedef struct freeway_context {
  */
 typedef struct pathfinder_context {
 	colldet_filter *dlc_filter;	// DLC filter to use
-	freeway_context *frw_ctx;	// CheckIfWayIsFreeOfDroids's execution context to use
+	freeway_context *frw_ctx;	// [way|location]_free_of_droids's execution context to use
 } pathfinder_context;
 
 typedef struct {
@@ -927,20 +917,6 @@ struct auto_string {
 	unsigned long capacity;
 };
 
-/* text_widget.c */
-typedef struct text_widget {
-	SDL_Rect rect;             /* The area in which the text should be displayed */
-	struct BFont_Info *font;
-	struct auto_string *text;
-	float text_stretch;
-
-	int scroll_offset;         /* 0 means bottom, negative means above bottem. */
-	int mouse_already_handled;
-
-	void (*content_below_func)(void);
-	void (*content_above_func)(void);
-} text_widget;
-
 /*
  * Specification of Animations
  */
@@ -959,18 +935,60 @@ struct distancebased_animation {
 	int nb_keyframes;		// Set to (last - first). Avoid to compute it each time this information is needed
 };
 
+/**
+ * Specification of Tux's parts rendering data
+ * Pointers to data needed to render one given Tux' part
+ */
+struct tux_part_render_data {
+	char *name;                      // Name of the part (used as a key of the struct)
+	char **default_part_instance;    // Pointer to one of the tuxrendering.default_instances
+	item *wearable_item;             // Pointer to one of the Me.XXXX_item
+	int part_group;                  // One of the PART_GROUP_XXXX values
+};
+
 /**
  * Specification of Tux's parts rendering order
  * Defines how to order Tux's parts for a 'set' of animation phases.
  * A linked list of sets is associated to a (motion_class, rotation) pair
  * (see the definition of struct tux_rendering_s in global.h).
  */
-
 struct tux_part_render_set {
 	int phase_start;	// First animation phase of the set
 	int phase_end;		// Last animation phase of the set
-	void (*render_funcs[ALL_PART_GROUPS])(int, int, int, int, int);	// Ordered array of rendering functions to call
+	struct tux_part_render_data *part_render_data[ALL_PART_GROUPS];	// Ordered array of the data needed to render Tux parts
 	struct tux_part_render_set *next;
 };
+typedef struct tux_part_render_set *tux_part_render_motionclass[MAX_TUX_DIRECTIONS];
+
+/**
+ * Contains the prefix of the animation archive files needed to render
+ * each Tux's part.
+ */
+struct tux_part_instances {
+	char *head;
+	char *torso;
+	char *weaponarm;
+	char *weapon;
+	char *shieldarm;
+	char *feet;
+};
+
+/**
+ * Contains all the informations needed to render Tux
+ */
+struct tux_rendering {
+	struct dynarray motion_class_names;             // All motion classes
+	struct tux_part_instances default_instances;    // Default part instances
+	tux_part_render_motionclass *render_order;      // The render_sets of each motion class
+	int gun_muzzle_height;							// Vertical offset to apply to bullets
+} tux_rendering;
+
+/**
+ * Contains a set of Tux's parts images for a motion class.
+ */
+struct tux_motion_class_images {
+	struct image part_images[ALL_PART_GROUPS][TUX_TOTAL_PHASES][MAX_TUX_DIRECTIONS];
+	char part_names[ALL_PART_GROUPS][64];
+};
 
 #endif
