@@ -28,11 +28,6 @@
  *           and take into account the AC-defined conditionals
  */
 
-/* 
- * well, for the time being we actually don't care too much about the 
- * AC-defined conditionals and pretend it's standard linux
- */
-
 #ifndef _SYSTEM_H
 #define _SYSTEM_H
 
@@ -112,6 +107,12 @@
 #include "SDL.h"
 #include "SDL_image.h"
 
+#ifdef HAVE_LIBSDL_GFX
+#include "SDL_rotozoom.h"
+#include "SDL_framerate.h"
+#include "SDL_gfxPrimitives.h"
+#endif
+
 #ifdef HAVE_LIBSDL_MIXER
 #include "SDL_mixer.h"
 #endif
