@@ -38,26 +38,17 @@
 #include "proto.h"
 #include "takeover.h"
 #include "map.h"
+#include "widgets/widgets.h"
 
-enemy *cDroid;
 Uint32 cur_time;		// current time in ms 
-SDL_Surface *to_blocks;		// the global surface containing all game-blocks 
 
-//--------------------
-// The rectangles containing the blocks for the takeover game
-//
-iso_image FillBlocks[NUM_FILL_BLOCKS];
-SDL_Rect FillRects[NUM_FILL_BLOCKS];
-iso_image CapsuleBlocks[NUM_CAPS_BLOCKS];
-SDL_Rect CapsuleRects[NUM_CAPS_BLOCKS];
-iso_image ToGameBlocks[NUM_TO_BLOCKS];
-SDL_Rect ToGameRects[NUM_TO_BLOCKS];
-iso_image ToGroundBlocks[NUM_GROUND_BLOCKS];
-SDL_Rect ToGroundRects[NUM_GROUND_BLOCKS];
-iso_image ToColumnBlock;
-SDL_Rect ToColumnRect;
-iso_image ToLeaderBlock;
-SDL_Rect ToLeaderRect;
+// Takeover images
+static struct image FillBlocks[NUM_FILL_BLOCKS];
+static struct image CapsuleBlocks[NUM_CAPS_BLOCKS];
+static struct image ToGameBlocks[NUM_TO_BLOCKS];
+static struct image ToGroundBlocks[NUM_GROUND_BLOCKS];
+static struct image ToColumnBlock;
+static struct image ToLeaderBlock;
 
 //--------------------
 // Class seperation of the blocks 
@@ -139,7 +130,7 @@ float EvaluatePosition(const int color, const int row, const int layer, const in
 float EvaluateCenterPosition(const int color, const int row, const int layer, const int endgame);
 void AdvancedEnemyTakeoverMovements(const int countdown);
 
-static void ShowPlayground(void);
+static void ShowPlayground(enemy *target);
 
 static void display_takeover_help()
 {
@@ -149,65 +140,30 @@ static void display_takeover_help()
 /** 
  * Display the picture of a droid
  */
-static void ShowDroidPicture(int PosX, int PosY, int Number)
+static void show_droid_picture(int PosX, int PosY, int type)
 {
-	SDL_Surface *tmp;
-	SDL_Rect target;
-	char ConstructedFileName[5000];
-	char fpath[2048];
+	char filename[5000];
 	static char LastImageSeriesPrefix[1000] = "NONE_AT_ALL";
 #define NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION 32
-	static SDL_Surface *DroidRotationSurfaces[NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION] = { NULL };
-	SDL_Surface *Whole_Image;
-	int i;
+	static struct image droid_images[NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION];
 	int RotationIndex;
 
-	DebugPrintf(2, "\nvoid ShowDroidPicture(...): Function call confirmed.");
-
-	if (!strcmp(Druidmap[Number].droid_portrait_rotation_series_prefix, "NONE_AVAILABLE_YET"))
-		return;		// later this should be a default-correction instead
+	if (!strcmp(Druidmap[type].droid_portrait_rotation_series_prefix, "NONE_AVAILABLE_YET"))
+		return;
 
 	// Maybe we have to reload the whole image series
 	//
-	if (strcmp(LastImageSeriesPrefix, Druidmap[Number].droid_portrait_rotation_series_prefix)) {
-		// Maybe we have to free the series from an old item display first
-		//
-		if (DroidRotationSurfaces[0] != NULL) {
-			for (i = 1; i < NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION; i++) {
-				SDL_FreeSurface(DroidRotationSurfaces[i]);
-			}
-		}
-		// Now we can start to load the whole series into memory
-		//
+	if (strcmp(LastImageSeriesPrefix, Druidmap[type].droid_portrait_rotation_series_prefix)) {
+		int i;
 		for (i = 0; i < NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION; i++) {
-			if (!strcmp(Druidmap[Number].droid_portrait_rotation_series_prefix, "NONE_AVAILABLE_YET")) {
-				Terminate(ERR);
-			} else {
-				sprintf(ConstructedFileName, "droids/%s/portrait_%04d.jpg",
-					Druidmap[Number].droid_portrait_rotation_series_prefix, i + 1);
-				DebugPrintf(1, "\nConstructedFileName = %s ", ConstructedFileName);
-			}
+			delete_image(&droid_images[i]);
 
-			// We must remember, that his is already loaded of course
-			strcpy(LastImageSeriesPrefix, Druidmap[Number].droid_portrait_rotation_series_prefix);
+			sprintf(filename, "droids/%s/portrait_%04d.jpg", Druidmap[type].droid_portrait_rotation_series_prefix, i + 1);
 
-			find_file(ConstructedFileName, GRAPHICS_DIR, fpath, 0);
-
-			Whole_Image = our_IMG_load_wrapper(fpath);	// This is a surface with alpha channel, since the picture is one of this type
-			if (Whole_Image == NULL) {
-				fprintf(stderr, "\n\nfpath: %s. \n", fpath);
-				ErrorMessage(__FUNCTION__, "\
-Freedroid was unable to load an image of a rotated droid into memory.\n\
-This error indicates some installation problem with freedroid.", PLEASE_INFORM, IS_FATAL);
-			}
-
-			SDL_SetAlpha(Whole_Image, 0, SDL_ALPHA_OPAQUE);
-
-			DroidRotationSurfaces[i] = our_SDL_display_format_wrapperAlpha(Whole_Image);	// now we have an alpha-surf of right size
-			SDL_SetColorKey(DroidRotationSurfaces[i], 0, 0);	// this should clear any color key in the dest surface
-
-			SDL_FreeSurface(Whole_Image);
+			load_image(&droid_images[i], filename, FALSE);
 		}
+			
+		strcpy(LastImageSeriesPrefix, Druidmap[type].droid_portrait_rotation_series_prefix);
 	}
 
 	RotationIndex = (SDL_GetTicks() / 50);
@@ -215,82 +171,77 @@ This error indicates some installation problem with freedroid.", PLEASE_INFORM,
 	RotationIndex =
 	    RotationIndex - (RotationIndex / NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION) * NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION;
 
-	tmp = DroidRotationSurfaces[RotationIndex];
+	// Compute the maximum uniform scale to apply to the bot image so that it fills
+	// the droid portrait image, and center the image.
+	struct image *img = &droid_images[RotationIndex];
+	float scale = min((float)Droid_Image_Window.w / (float)img->w, (float)Droid_Image_Window.h / (float)img->h);
+	moderately_finepoint pos;
+	pos.x = (float)PosX + ((float)Droid_Image_Window.w - (float)img->w * scale) / 2.0;
+	pos.y = (float)PosY + ((float)Droid_Image_Window.h - (float)img->h * scale) / 2.0;
+	display_image_on_screen(img, pos.x, pos.y, IMAGE_SCALE_TRANSFO(scale));
+}
+
+/**
+ * Display infopage page of droidtype.
+ */
+static void show_droid_info(int droidtype)
+{
+	SDL_Rect clip;
 
 	SDL_SetClipRect(Screen, NULL);
-	Set_Rect(target, PosX, PosY, GameConfig.screen_width, GameConfig.screen_height);
-	our_SDL_blit_surface_wrapper(tmp, NULL, Screen, &target);
 
-	DebugPrintf(2, "\nvoid ShowDroidPicture(...): Usual end of function reached.");
+	// Show background
+	blit_background("title.jpg");
+	blit_background("takeover_browser.png");
 
-};				// void ShowDroidPicture ( ... )
+	// Show droid portrait
+	show_droid_picture(UNIVERSAL_COORD_W(45), UNIVERSAL_COORD_H(213), droidtype);
 
-/* ------------------------------------------------------------
- * display infopage page of droidtype
- * does update the screen, no our_SDL_flip_wrapper() necesary !
- * ------------------------------------------------------------ */
-static void ShowDroidInfo(int droidtype, int Displacement, char ShowArrows)
+	// Show the droid name
+	SetCurrentFont(Menu_BFont);
+	clip.x = UNIVERSAL_COORD_W(330);
+	clip.y = UNIVERSAL_COORD_H(57);
+	clip.w = UNIVERSAL_COORD_W(200);
+	clip.h = UNIVERSAL_COORD_H(30);
+	display_text_using_line_height(Druidmap[droidtype].default_short_description, clip.x, clip.y, &clip, 1.0);
+}
+
+/**
+ * Initialize text widget with a description of the specified droidtype.
+ */
+static void init_droid_description(struct widget_text *w, int droidtype)
 {
 	char *item_name;
-	int type;
-	char InfoText[10000];
-	char TextChunk[2000];
+	int weapon_type;
 
-	// We initialize the text rectangle
-	//
-	Cons_Text_Rect.x = 258 * GameConfig.screen_width / 640;
-	Cons_Text_Rect.y = 89 * GameConfig.screen_height / 480;
-	Cons_Text_Rect.w = 346 * GameConfig.screen_width / 640;
-	Cons_Text_Rect.h = 282 * GameConfig.screen_height / 480;
+	autostr_append(w->text, _("Unit Type %s\n"), Druidmap[droidtype].druidname);
+	autostr_append(w->text, _("Entry : %d\n"), droidtype + 1);
 
-	SDL_SetClipRect(Screen, NULL);
-
-	blit_special_background(ITEM_BROWSER_BG_PIC_BACKGROUND_CODE);
-
-	ShowDroidPicture(45 * GameConfig.screen_width / 640, 190 * GameConfig.screen_height / 480, droidtype);
-
-	// We fill out the header area of the items browser.
-	//
-	SetCurrentFont(Menu_BFont);
-	strcpy(TextChunk, Druidmap[droidtype].druidname);
-	CutDownStringToMaximalSize(TextChunk, 225);
-	PutString(Screen, 330 * GameConfig.screen_width / 640, 38 * GameConfig.screen_height / 480, TextChunk);
-
-	sprintf(InfoText, _("\
-Unit type %s\n\
-Entry : %d\n"), Druidmap[droidtype].druidname, droidtype + 1);
-
-	if ((type = Druidmap[droidtype].weapon_item.type) >= 0)	// make sure item=-1 
-		item_name = D_(ItemMap[type].item_name);	// does not segfault 
+	if ((weapon_type = Druidmap[droidtype].weapon_item.type) >= 0)	// make sure item != -1 
+		item_name = D_(ItemMap[weapon_type].item_name);	// does not segfault
 	else
 		item_name = _("none");
 
-	sprintf(TextChunk, _("\nArmament : %s\n"), item_name);
-	strcat(InfoText, TextChunk);
+	autostr_append(w->text, _("\nArmament : %s\n"), item_name);
+	autostr_append(w->text, _("\nCores : %d\n"), 2 + Druidmap[droidtype].class);
+
+
 	if (Me.TakeoverSuccesses[droidtype]+Me.TakeoverFailures[droidtype]) {
-		sprintf(TextChunk, _("\nTakeover Success : %2d%%\n"),
-        		((100*Me.TakeoverSuccesses[droidtype])/ (Me.TakeoverSuccesses[droidtype]+Me.TakeoverFailures[droidtype])) );
-		strcat(InfoText, TextChunk);
+		int success_ratio =
+			((100*Me.TakeoverSuccesses[droidtype])/
+			 (Me.TakeoverSuccesses[droidtype]+Me.TakeoverFailures[droidtype]));
+
+		autostr_append(w->text, _("\nTakeover Success : %2d%%\n"), success_ratio);
 	}
 
-	sprintf(TextChunk, _("\nNotes: %s\n"), D_(Druidmap[droidtype].notes));
-	strcat(InfoText, TextChunk);
-
-	SetCurrentFont(FPS_Display_BFont);
-	DisplayText(InfoText, Cons_Text_Rect.x, Cons_Text_Rect.y + Displacement, &Cons_Text_Rect, TEXT_STRETCH);
-
-	if (ShowArrows) {
-		ShowGenericButtonFromList(UP_BUTTON);
-		ShowGenericButtonFromList(DOWN_BUTTON);
-	}
-
-};				// void ShowDroidInfo ( ... )
+	autostr_append(w->text, _("\nNotes: %s\n"), D_(Druidmap[droidtype].notes));
+}
 
 /**
  * This function does the countdown where you still can changes your
  * color.
  */
-static void ChooseColor(void)
+static void ChooseColor(enemy *target)
 {
 	int countdown = 100;	// duration in 1/10 seconds given for color choosing 
 	int ColorChosen = FALSE;
@@ -313,7 +264,7 @@ static void ChooseColor(void)
 		while (SDL_PollEvent(&event)) {
 
 			if (event.type == SDL_QUIT) {
-				Terminate(0);
+				Terminate(EXIT_SUCCESS, TRUE);
 			}
 
 			if (event.type == SDL_MOUSEBUTTONDOWN) {
@@ -351,9 +302,6 @@ static void ChooseColor(void)
 				case SDLK_SPACE:
 					ColorChosen = TRUE;
 					break;
-				case SDLK_F1:
-					display_takeover_help();
-					break;
 				default:
 					break;
 				}
@@ -364,7 +312,7 @@ static void ChooseColor(void)
 		countdown--;	// Count down 
 		sprintf(count_text, _("Color-%d.%d"), countdown/10, countdown%10);
 
-		ShowPlayground();
+		ShowPlayground(target);
 		to_show_banner(count_text, NULL);
 		our_SDL_flip_wrapper();
 
@@ -377,7 +325,7 @@ static void ChooseColor(void)
 		SDL_Delay(1);
 };				// void ChooseColor ( void ) 
 
-static void PlayGame(int countdown)
+static void PlayGame(int countdown, enemy *target)
 {
 	char count_text[80];
 	int FinishTakeover = FALSE;
@@ -427,16 +375,14 @@ static void PlayGame(int countdown)
 					break;
 				}
 			} else if (event.type == SDL_KEYDOWN) {
-				/* allow for a WIN-key that give immedate victory */
+				/* allow for a WIN-key that gives immediate victory */
 				event.key.keysym.mod &= ~(KMOD_CAPS | KMOD_NUM | KMOD_MODE);	/* We want to ignore "global" modifiers. */
 				if (event.key.keysym.sym == SDLK_w && (event.key.keysym.mod == (KMOD_LCTRL | KMOD_LALT))) {
 					LeaderColor = YourColor;	/* simple as that */
 					return;	/* leave now, to avoid changing of LeaderColor! */
 				}
-				if (event.key.keysym.sym == SDLK_F1)
-					display_takeover_help();
 			} else if (event.type == SDL_QUIT) {
-				Terminate(0);
+				Terminate(EXIT_SUCCESS, TRUE);
 			}
 		}
 
@@ -525,13 +471,13 @@ static void PlayGame(int countdown)
 
 		}
 		/* if (motion_tick has occurred) */
-		ShowPlayground();
+		ShowPlayground(target);
 		to_show_banner(count_text, NULL);
 		our_SDL_flip_wrapper();
 		SDL_Delay(10);
 	}			/* while !FinishTakeover */
 
-	/* Final contdown */
+	/* Final countdown */
 	countdown = CAPSULE_COUNTDOWN + 10;
 
 	while (countdown--) {
@@ -546,7 +492,7 @@ static void PlayGame(int countdown)
 		ProcessPlayground();
 		ProcessPlayground();	/* this has to be done several times to be sure */
 		ProcessDisplayColumn();
-		ShowPlayground();
+		ShowPlayground(target);
 		our_SDL_flip_wrapper();
 	}			/* while (countdown) */
 
@@ -554,113 +500,42 @@ static void PlayGame(int countdown)
 
 }
 
-int do_takeover(int player_capsules, int opponent_capsules, int game_length)
-{
-	char *message;
-	int player_won = 0;
-	int FinishTakeover = FALSE;
-	int row;
-	int old_status;
-
-	old_status = game_status;
-
-	Activate_Conservative_Frame_Computation();
-
-	// Maybe takeover graphics haven't been loaded yet.  Then we do this
-	// here now and for once.  Later calls will be ignored inside the function.
-	//
-	GetTakeoverGraphics();
-
-	// eat pending events
-	input_handle();
-
-	while (!FinishTakeover) {
-		// Init Color-column and Capsule-Number for each opponenet and your color 
-		//
-		for (row = 0; row < NUM_LINES; row++) {
-			DisplayColumn[row] = (row % 2);
-			CapsuleCountdown[YELLOW][0][row] = -1;
-			CapsuleCountdown[PURPLE][0][row] = -1;
-		}		// for row 
-
-		YourColor = YELLOW;
-		OpponentColor = PURPLE;
-
-		CapsuleCurRow[YELLOW] = 0;
-		CapsuleCurRow[PURPLE] = 0;
-
-		NumCapsules[YOU] = player_capsules;
-		NumCapsules[ENEMY] = opponent_capsules;
-		InventPlayground();
-
-		EvaluatePlayground();
-
-		ShowPlayground();
-		our_SDL_flip_wrapper();
-
-		ChooseColor();
-
-		// This following function plays the takeover game, until one
-		// of THREE states is reached, i.e. until YOU WON, YOU LOST
-		// or until DEADLOCK is reached.  Well, so maybe after that
-		// the takeover game is finished, but if it's a deadlock, then
-		// the game must be played again in the next loop...
-		//
-		PlayGame(game_length);
-
-		// We we evaluate the final score of the game.  Maybe we're done
-		// already, maybe not...
-		//
-		if (LeaderColor == YourColor) {
-			Takeover_Game_Won_Sound();
-			message = _("Complete");
-			FinishTakeover = TRUE;
-			player_won = 1;
-		} else if (LeaderColor == OpponentColor) {
-			Takeover_Game_Lost_Sound();
-			message = _("Rejected");
-			FinishTakeover = TRUE;
-			player_won = 0;
-		} else {
-			Takeover_Game_Deadlock_Sound();
-			message = _("Deadlock");
-		}
-
-		ShowPlayground();
-		to_show_banner(message, NULL);
-		our_SDL_flip_wrapper();
-		SDL_Delay(100);
-
-	}
-
-	game_status = old_status;
-	return player_won;
+static void show_info_up_button() {
+	ShowGenericButtonFromList(UP_BUTTON);
 }
 
-/*-----------------------------------------------------------------
- *
+static void show_info_down_button() {
+	ShowGenericButtonFromList(DOWN_BUTTON);
+}
+
+/**
  * This function manages the whole takeover game of Tux against 
  * some bot.
  *
  * The return value is TRUE/FALSE depending on whether the game was
  * finally won/lost.
- *
- *-----------------------------------------------------------------*/
-int droid_takeover(enemy * target)
+ */
+int droid_takeover(enemy *target)
 {
-	int Finished = FALSE;
-	int Displacement = 0;
+	int menu_finished = FALSE;
 	int reward = 0;
 	SDL_Event event;
+	static struct widget_text droid_info;
+
+	// Set up the droid description widget
+	widget_text_init(&droid_info, "");
+	init_droid_description(&droid_info, target->type);
+	widget_set_rect(WIDGET(&droid_info), UNIVERSAL_COORD_W(258), UNIVERSAL_COORD_H(107), UNIVERSAL_COORD_W(346), UNIVERSAL_COORD_H(282));
+	droid_info.font = FPS_Display_BFont;
+	droid_info.content_above_func = show_info_up_button;
+	droid_info.content_below_func = show_info_down_button;
 
 	// Prevent distortion of framerate by the delay coming from 
 	// the time spent in the menu.
-	//
 	Activate_Conservative_Frame_Computation();
 
 	// We set the UserRect to full again, no matter what other windows might
 	// be open right now...
-	//
 	User_Rect.x = 0;
 	User_Rect.y = 0;
 	User_Rect.w = GameConfig.screen_width;
@@ -670,35 +545,34 @@ int droid_takeover(enemy * target)
 
 	SwitchBackgroundMusicTo(TAKEOVER_BACKGROUND_MUSIC_SOUND);
 
-	DisplayBanner();
-
-	while (!Finished) {
-		ShowDroidInfo(target->type, Displacement, TRUE);
+	while (!menu_finished) {
+		show_droid_info(target->type);
+		widget_text_display(WIDGET(&droid_info));
 		ShowGenericButtonFromList(TAKEOVER_HELP_BUTTON);
-		blit_our_own_mouse_cursor();
+		SetCurrentFont(Para_BFont);
+		CenteredPutString(Screen, GameConfig.screen_height - 30, "For more information, click the help button.");
+		blit_mouse_cursor();
 		our_SDL_flip_wrapper();
 
 		while (SDL_PollEvent(&event)) {
 
 			if (event.type == SDL_QUIT) {
-				Terminate(0);
+				Terminate(EXIT_SUCCESS, TRUE);
 			}
 
 			if (event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_LEFT) {
 				if (MouseCursorIsOnButton(UP_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-					MoveMenuPositionSound();
-					Displacement += FontHeight(GetCurrentFont());
+					droid_info.scroll_offset--;
 				} else if (MouseCursorIsOnButton(DOWN_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-					MoveMenuPositionSound();
-					Displacement -= FontHeight(GetCurrentFont());
+					droid_info.scroll_offset++;
 				} else if (MouseCursorIsOnButton(DROID_SHOW_EXIT_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-					Finished = TRUE;
+					menu_finished = TRUE;
 				} else if (MouseCursorIsOnButton(TAKEOVER_HELP_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-					PlayATitleFile("TakeoverInstructions.title");
+					display_takeover_help();
 				}
 			} else if (event.type == SDL_KEYDOWN
 				   && ((event.key.keysym.sym == SDLK_SPACE) || (event.key.keysym.sym == SDLK_ESCAPE))) {
-				Finished = TRUE;
+				menu_finished = TRUE;
 			}
 		}
 		SDL_Delay(1);
@@ -706,11 +580,10 @@ int droid_takeover(enemy * target)
 
 	while (!(!SpacePressed() && !EscapePressed() && !MouseLeftPressed())) ;
 
-	cDroid = target;
 	int player_capsules = 2 + Me.skill_level[get_program_index_with_name("Hacking")];
 	max_opponent_capsules = 2 + Druidmap[target->type].class;
 
-	if (do_takeover(player_capsules, max_opponent_capsules, 100)) {
+	if (do_takeover(player_capsules, max_opponent_capsules, 100, target)) {
 		/* Won takeover */
 		Me.marker = target->marker;
 
@@ -749,9 +622,7 @@ int droid_takeover(enemy * target)
                 Me.TakeoverFailures[target->type]++;
 	}
 
-	cDroid = NULL;
-
-	ClearGraphMem();
+	clear_screen();
 
 	SwitchBackgroundMusicTo(CURLEVEL()->Background_Song_Name);
 
@@ -877,65 +748,40 @@ void AdvancedEnemyTakeoverMovements(const int countdown)
 	return;
 };				// AdvancedEnemyTakeoverMovements 
 
-/**
- * This function reads in the takeover game elements for later blitting. 
- * It frees previous SDL-surfaces if they were allocated.  T
- * This allows to use this fct also for theme-switching.
- *-----------------------------------------------------------------*/
-int GetTakeoverGraphics(void)
+static void GetTakeoverGraphics(void)
 {
 	static int TakeoverGraphicsAreAlreadyLoaded = FALSE;
+	struct image img = EMPTY_IMAGE;
 	int i, j;
 	int curx = 0, cury = 0;
 	SDL_Rect tmp;
-	SDL_Surface *TempLoadSurface;
 
-	// Maybe this function has been called before and everything
-	// has been loaded already.  Then of course we don't need to
-	// do anything any more and can just return.
-	//
 	if (TakeoverGraphicsAreAlreadyLoaded)
-		return (OK);
+		return;
 
-	// Now we start loading all the takeover graphics.
-	//
-	Set_Rect(tmp, User_Rect.x, User_Rect.y, 0, 0);
-	char fp[2048];
-	find_file(TO_BLOCK_FILE, GRAPHICS_DIR, fp, 0);
-	TempLoadSurface = our_IMG_load_wrapper(fp);
-	to_blocks = our_SDL_display_format_wrapperAlpha(TempLoadSurface);	// the surface is converted
-	SDL_FreeSurface(TempLoadSurface);
-	if (use_open_gl)
-		flip_image_vertically(to_blocks);
+	load_image(&img, TO_BLOCK_FILE, FALSE);
 
 	// Get the fill-blocks 
 	for (i = 0; i < NUM_FILL_BLOCKS; i++, curx += FILL_BLOCK_LEN + 2) {
-		Set_Rect(FillRects[i], curx, cury, FILL_BLOCK_LEN, FILL_BLOCK_HEIGHT);
-		FillBlocks[i].surface = rip_rectangle_from_alpha_image(to_blocks, FillRects[i]);
-		if (use_open_gl)
-			make_texture_out_of_surface(&(FillBlocks[i]));
+		Set_Rect(tmp, curx, cury, FILL_BLOCK_LEN, FILL_BLOCK_HEIGHT);
+		create_subimage(&img, &FillBlocks[i], &tmp);
 	}
 
 	// Get the capsule blocks 
 	for (i = 0; i < NUM_CAPS_BLOCKS; i++, curx += CAPSULE_LEN + 2) {
-		Set_Rect(CapsuleRects[i], curx, cury, CAPSULE_LEN, CAPSULE_HEIGHT);
-		CapsuleBlocks[i].surface = rip_rectangle_from_alpha_image(to_blocks, CapsuleRects[i]);
-		if (use_open_gl)
-			make_texture_out_of_surface(&(CapsuleBlocks[i]));
+		Set_Rect(tmp, curx, cury, CAPSULE_LEN, CAPSULE_HEIGHT);
+		create_subimage(&img, &CapsuleBlocks[i], &tmp);
 	}
 
 	// Get the default background color, to be used when no background picture found! 
-	curx += CAPSULE_LEN + 2;
 	curx = 0;
 	cury += FILL_BLOCK_HEIGHT + 2;
 
 	// get the game-blocks 
 	for (j = 0; j < 2 * NUM_PHASES; j++) {
 		for (i = 0; i < TO_BLOCKS; i++) {
-			Set_Rect(ToGameRects[j * TO_BLOCKS + i], curx, cury, TO_BLOCKLEN, TO_BLOCKHEIGHT);
-			ToGameBlocks[j * TO_BLOCKS + i].surface = rip_rectangle_from_alpha_image(to_blocks, ToGameRects[j * TO_BLOCKS + i]);
-			if (use_open_gl)
-				make_texture_out_of_surface(&(ToGameBlocks[j * TO_BLOCKS + i]));
+			Set_Rect(tmp, curx, cury, TO_BLOCKLEN, TO_BLOCKHEIGHT);
+			create_subimage(&img, &ToGameBlocks[j * TO_BLOCKS + i], &tmp);
 			curx += TO_BLOCKLEN + 2;
 		}
 		curx = 0;
@@ -944,10 +790,8 @@ int GetTakeoverGraphics(void)
 
 	// Get the ground, column and leader blocks 
 	for (i = 0; i < NUM_GROUND_BLOCKS; i++) {
-		Set_Rect(ToGroundRects[i], curx, cury, GROUNDBLOCKLEN, GROUNDBLOCKHEIGHT);
-		ToGroundBlocks[i].surface = rip_rectangle_from_alpha_image(to_blocks, ToGroundRects[i]);
-		if (use_open_gl)
-			make_texture_out_of_surface(&(ToGroundBlocks[i]));
+		Set_Rect(tmp, curx, cury, GROUNDBLOCKLEN, GROUNDBLOCKHEIGHT);
+		create_subimage(&img, &ToGroundBlocks[i], &tmp);
 		curx += GROUNDBLOCKLEN + 2;
 	}
 	cury += GROUNDBLOCKHEIGHT + 2;
@@ -956,132 +800,194 @@ int GetTakeoverGraphics(void)
 	// Now the rectangle for the column blocks will be set and after
 	// that we can create the new surface for blitting.
 	//
-	Set_Rect(ToColumnRect, curx, cury, COLUMNBLOCKLEN, COLUMNBLOCKHEIGHT);
-	ToColumnBlock.surface = rip_rectangle_from_alpha_image(to_blocks, ToColumnRect);
-	if (use_open_gl)
-		make_texture_out_of_surface(&(ToColumnBlock));
+	Set_Rect(tmp, curx, cury, COLUMNBLOCKLEN, COLUMNBLOCKHEIGHT);
+	create_subimage(&img, &ToColumnBlock, &tmp);
 
-	// 
 	curx += COLUMNBLOCKLEN + 2;
 
 	// Now the rectangle for the leader block will be set and after
 	// that we can create the new surface for blitting.
 	//
-	Set_Rect(ToLeaderRect, curx, cury, LEADERBLOCKLEN, LEADERBLOCKHEIGHT);
-	ToLeaderBlock.surface = rip_rectangle_from_alpha_image(to_blocks, ToLeaderRect);
-	if (use_open_gl)
-		make_texture_out_of_surface(&(ToLeaderBlock));
+	Set_Rect(tmp, curx, cury, LEADERBLOCKLEN, LEADERBLOCKHEIGHT);
+	create_subimage(&img, &ToLeaderBlock, &tmp);
 
-	// Now that everything was loaded, we should remember this, so we
-	// don't load anything again next time...
-	//
+	free_image_surface(&img);
 	TakeoverGraphicsAreAlreadyLoaded = TRUE;
+}
 
-	return OK;
-};				// int GetTakeoverGraphics ( void )
+/**
+ * Initiate the takeover game
+ * @param game_length game time in tenths of a second
+ * @param target a pointer to the enemy structure of the target. NULL means none
+ * @return 1 on player success, 0 on player failure
+ */
+int do_takeover(int player_capsules, int opponent_capsules, int game_length, enemy* target)
+{
+	char *message;
+	int player_won = 0;
+	int FinishTakeover = FALSE;
+	int row;
+	int old_status;
 
-static void ShowPlayground(void)
+	old_status = game_status;
+
+	Activate_Conservative_Frame_Computation();
+
+	// Maybe takeover graphics haven't been loaded yet.  Then we do this
+	// here now and for once.  Later calls will be ignored inside the function.
+	GetTakeoverGraphics();
+
+	// eat pending events
+	input_handle();
+
+	while (!FinishTakeover) {
+		// Init Color-column and Capsule-Number for each opponent and your color 
+		//
+		for (row = 0; row < NUM_LINES; row++) {
+			DisplayColumn[row] = (row % 2);
+			CapsuleCountdown[YELLOW][0][row] = -1;
+			CapsuleCountdown[PURPLE][0][row] = -1;
+		}		// for row 
+
+		YourColor = YELLOW;
+		OpponentColor = PURPLE;
+
+		CapsuleCurRow[YELLOW] = 0;
+		CapsuleCurRow[PURPLE] = 0;
+
+		NumCapsules[YOU] = player_capsules;
+		NumCapsules[ENEMY] = opponent_capsules;
+		InventPlayground();
+
+		EvaluatePlayground();
+
+		ShowPlayground(target);
+		our_SDL_flip_wrapper();
+
+		ChooseColor(target);
+
+		// This following function plays the takeover game, until one
+		// of THREE states is reached, i.e. until YOU WON, YOU LOST
+		// or until DEADLOCK is reached.  Well, so maybe after that
+		// the takeover game is finished, but if it's a deadlock, then
+		// the game must be played again in the next loop...
+		//
+		PlayGame(game_length, target);
+
+		// We we evaluate the final score of the game.  Maybe we're done
+		// already, maybe not...
+		//
+		if (LeaderColor == YourColor) {
+			Takeover_Game_Won_Sound();
+			message = _("Complete");
+			FinishTakeover = TRUE;
+			player_won = 1;
+		} else if (LeaderColor == OpponentColor) {
+			Takeover_Game_Lost_Sound();
+			message = _("Rejected");
+			FinishTakeover = TRUE;
+			player_won = 0;
+		} else {
+			Takeover_Game_Deadlock_Sound();
+			message = _("Deadlock");
+		}
+
+		ShowPlayground(target);
+		to_show_banner(message, NULL);
+		our_SDL_flip_wrapper();
+		SDL_Delay(100);
+
+	}
+
+	game_status = old_status;
+	return player_won;
+}
+
+/**
+ * Draws the playground for the takeover game
+ * @param target a pointer to the target's enemy structure. Use NULL for none
+ */
+static void ShowPlayground(enemy * target)
 {
 	int i, j;
 	int color, player;
 	int block;
 	int xoffs, yoffs;
 	SDL_Rect Target_Rect;
-
+	int phase, dheight;
 	xoffs = User_Rect.x + (User_Rect.w - 2 * 290) / 2;
-	yoffs = User_Rect.y + (User_Rect.h - 2 * 140) / 2;
+	yoffs = User_Rect.y + (User_Rect.h - 2 * 140) / 2 + 30;
 
+	phase = (int) Me.phase;
+
+	blit_background("transfer.jpg");
+
+	if (target) {
+		// Find the difference of the droid's height and Tux's height.
+		// Tux's height is measured from the top of his head to the bottom of his feet
+		// This also accounts for the droid's blitting offset
+		struct tux_motion_class_images *current_motion_class_images = &tux_images[get_motion_class_id()];
+		dheight = current_motion_class_images->part_images[PART_GROUP_FEET][0][phase].offset_y +
+			current_motion_class_images->part_images[PART_GROUP_FEET][0][phase].h -
+			current_motion_class_images->part_images[PART_GROUP_HEAD][0][phase].offset_y -
+			enemy_images[target->type][0][0].h -
+			enemy_images[target->type][0][0].offset_y;
+
+		// Offset the droid's drawing rectangle by dheight minus 30 units.
+		Set_Rect(Target_Rect, xoffs + DruidStart[!YourColor].x, yoffs + dheight - 30, User_Rect.w, User_Rect.h);
+		PutIndividuallyShapedDroidBody(target, Target_Rect, FALSE, FALSE);
+	}
 	//  SDL_SetColorKey (Screen, 0, 0);
 	SDL_SetClipRect(Screen, &User_Rect);
 
-	blit_special_background(TAKEOVER_BACKGROUND_CODE);
-
-	blit_tux(xoffs + DruidStart[YourColor].x, yoffs + DruidStart[YourColor].y + 30);
-
-	if (cDroid) {
-		Set_Rect(Target_Rect, xoffs + DruidStart[!YourColor].x, yoffs + DruidStart[!YourColor].y, User_Rect.w, User_Rect.h);
-		PutIndividuallyShapedDroidBody(cDroid, Target_Rect, FALSE, FALSE);
-	}
+	blit_tux(xoffs + DruidStart[YourColor].x, yoffs);
 
 	Set_Rect(Target_Rect, xoffs + LEFT_OFFS_X, yoffs + LEFT_OFFS_Y, User_Rect.w, User_Rect.h);
 
-	if (use_open_gl)
-		draw_gl_textured_quad_at_screen_position(&ToGroundBlocks[YELLOW_HIGH], Target_Rect.x, Target_Rect.y);
-	else
-		our_SDL_blit_surface_wrapper(ToGroundBlocks[YELLOW_HIGH].surface, NULL, Screen, &Target_Rect);
+	display_image_on_screen (&ToGroundBlocks[YELLOW_HIGH], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 
 	Target_Rect.y += GROUNDBLOCKHEIGHT;
 
 	for (i = 0; i < 12; i++) {
-		if (use_open_gl)
-			draw_gl_textured_quad_at_screen_position(&ToGroundBlocks[YELLOW_MIDDLE], Target_Rect.x, Target_Rect.y);
-		else
-			our_SDL_blit_surface_wrapper(ToGroundBlocks[YELLOW_MIDDLE].surface, NULL, Screen, &Target_Rect);
+		display_image_on_screen (&ToGroundBlocks[YELLOW_MIDDLE], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 		Target_Rect.y += GROUNDBLOCKHEIGHT;
 	}
 
-	if (use_open_gl)
-		draw_gl_textured_quad_at_screen_position(&ToGroundBlocks[YELLOW_LOW], Target_Rect.x, Target_Rect.y);
-	else
-		our_SDL_blit_surface_wrapper(ToGroundBlocks[YELLOW_LOW].surface, NULL, Screen, &Target_Rect);
+	display_image_on_screen (&ToGroundBlocks[YELLOW_LOW], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 
 	// the middle column
 	Set_Rect(Target_Rect, xoffs + MID_OFFS_X, yoffs + MID_OFFS_Y, 0, 0);
 
-	if (use_open_gl)
-		draw_gl_textured_quad_at_screen_position(&ToLeaderBlock, Target_Rect.x, Target_Rect.y);
-	else
-		our_SDL_blit_surface_wrapper(ToLeaderBlock.surface, NULL, Screen, &Target_Rect);
+	display_image_on_screen (&ToLeaderBlock, Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 
 	Target_Rect.y += LEADERBLOCKHEIGHT;
 	for (i = 0; i < 12; i++, Target_Rect.y += COLUMNBLOCKHEIGHT) {
-		if (use_open_gl)
-			draw_gl_textured_quad_at_screen_position(&ToColumnBlock, Target_Rect.x, Target_Rect.y);
-		else
-			our_SDL_blit_surface_wrapper(ToColumnBlock.surface, NULL, Screen, &Target_Rect);
+		display_image_on_screen (&ToColumnBlock, Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 	}
 
 	// the right column
 	Set_Rect(Target_Rect, xoffs + RIGHT_OFFS_X, yoffs + RIGHT_OFFS_Y, 0, 0);
-	if (use_open_gl)
-		draw_gl_textured_quad_at_screen_position(&ToGroundBlocks[PURPLE_HIGH], Target_Rect.x, Target_Rect.y);
-	else
-		our_SDL_blit_surface_wrapper(ToGroundBlocks[PURPLE_HIGH].surface, NULL, Screen, &Target_Rect);
+	display_image_on_screen (&ToGroundBlocks[PURPLE_HIGH], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 
 	Target_Rect.y += GROUNDBLOCKHEIGHT;
 
 	for (i = 0; i < 12; i++, Target_Rect.y += GROUNDBLOCKHEIGHT) {
-		if (use_open_gl)
-			draw_gl_textured_quad_at_screen_position(&ToGroundBlocks[PURPLE_MIDDLE], Target_Rect.x, Target_Rect.y);
-		else
-			our_SDL_blit_surface_wrapper(ToGroundBlocks[PURPLE_MIDDLE].surface, NULL, Screen, &Target_Rect);
+		display_image_on_screen (&ToGroundBlocks[PURPLE_MIDDLE], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 	}
 
-	if (use_open_gl)
-		draw_gl_textured_quad_at_screen_position(&ToGroundBlocks[PURPLE_LOW], Target_Rect.x, Target_Rect.y);
-	else
-		our_SDL_blit_surface_wrapper(ToGroundBlocks[PURPLE_LOW].surface, NULL, Screen, &Target_Rect);
+	display_image_on_screen (&ToGroundBlocks[PURPLE_LOW], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 
 	// Fill the leader-LED with its color 
 	Set_Rect(Target_Rect, xoffs + LEADERLED_X, yoffs + LEADERLED_Y, 0, 0);
-	if (use_open_gl)
-		draw_gl_textured_quad_at_screen_position(&FillBlocks[LeaderColor], Target_Rect.x, Target_Rect.y);
-	else
-		our_SDL_blit_surface_wrapper(FillBlocks[LeaderColor].surface, NULL, Screen, &Target_Rect);
+	display_image_on_screen (&FillBlocks[LeaderColor], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 
 	Target_Rect.y += FILL_BLOCK_HEIGHT;
-	if (use_open_gl)
-		draw_gl_textured_quad_at_screen_position(&FillBlocks[LeaderColor], Target_Rect.x, Target_Rect.y);
-	else
-		our_SDL_blit_surface_wrapper(FillBlocks[LeaderColor].surface, NULL, Screen, &Target_Rect);
+	display_image_on_screen (&FillBlocks[LeaderColor], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 
 	// Fill the display column with its colors 
 	for (i = 0; i < NUM_LINES; i++) {
 		Set_Rect(Target_Rect, xoffs + LEDCOLUMN_X, yoffs + LEDCOLUMN_Y + i * (FILL_BLOCK_HEIGHT + 2), 0, 0);
-		if (use_open_gl)
-			draw_gl_textured_quad_at_screen_position(&FillBlocks[DisplayColumn[i]], Target_Rect.x, Target_Rect.y);
-		else
-			our_SDL_blit_surface_wrapper(FillBlocks[DisplayColumn[i]].surface, NULL, Screen, &Target_Rect);
+		display_image_on_screen (&FillBlocks[DisplayColumn[i]], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 	}
 
 	// Show the yellow playground 
@@ -1090,10 +996,7 @@ static void ShowPlayground(void)
 			Set_Rect(Target_Rect, xoffs + PlaygroundStart[YELLOW].x + i * TO_BLOCKLEN,
 				 yoffs + PlaygroundStart[YELLOW].y + j * TO_BLOCKHEIGHT, 0, 0);
 			block = ToPlayground[YELLOW][i][j] + ActivationMap[YELLOW][i][j] * TO_BLOCKS;
-			if (use_open_gl)
-				draw_gl_textured_quad_at_screen_position(&ToGameBlocks[block], Target_Rect.x, Target_Rect.y);
-			else
-				our_SDL_blit_surface_wrapper(ToGameBlocks[block].surface, NULL, Screen, &Target_Rect);
+			display_image_on_screen (&ToGameBlocks[block], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 		}
 
 	// Show the purple playground 
@@ -1103,10 +1006,7 @@ static void ShowPlayground(void)
 				 xoffs + PlaygroundStart[PURPLE].x + (NUM_LAYERS - i - 2) * TO_BLOCKLEN,
 				 yoffs + PlaygroundStart[PURPLE].y + j * TO_BLOCKHEIGHT, 0, 0);
 			block = ToPlayground[PURPLE][i][j] + (NUM_PHASES + ActivationMap[PURPLE][i][j]) * TO_BLOCKS;
-			if (use_open_gl)
-				draw_gl_textured_quad_at_screen_position(&ToGameBlocks[block], Target_Rect.x, Target_Rect.y);
-			else
-				our_SDL_blit_surface_wrapper(ToGameBlocks[block].surface, NULL, Screen, &Target_Rect);
+			display_image_on_screen (&ToGameBlocks[block], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 		}
 
 	// Show the capsules left for each player 
@@ -1119,24 +1019,16 @@ static void ShowPlayground(void)
 		Set_Rect(Target_Rect, xoffs + CurCapsuleStart[color].x,
 			 yoffs + CurCapsuleStart[color].y + CapsuleCurRow[color] * (CAPSULE_HEIGHT + 2), 0, 0);
 		if (NumCapsules[player]) {
-			if (use_open_gl)
-				draw_gl_textured_quad_at_screen_position(&CapsuleBlocks[color], Target_Rect.x, Target_Rect.y);
-			else
-				our_SDL_blit_surface_wrapper(CapsuleBlocks[color].surface, NULL, Screen, &Target_Rect);
+			display_image_on_screen (&CapsuleBlocks[color], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 		}
 
 		for (i = 0; i < NumCapsules[player] - 1; i++) {
 			Set_Rect(Target_Rect, xoffs + LeftCapsulesStart[color].x,
 				 yoffs + LeftCapsulesStart[color].y + i * CAPSULE_HEIGHT, 0, 0);
-			if (use_open_gl)
-				draw_gl_textured_quad_at_screen_position(&CapsuleBlocks[color], Target_Rect.x, Target_Rect.y);
-			else
-				our_SDL_blit_surface_wrapper(CapsuleBlocks[color].surface, NULL, Screen, &Target_Rect);
+			display_image_on_screen (&CapsuleBlocks[color], Target_Rect.x, Target_Rect.y, IMAGE_NO_TRANSFO);
 		}		// for capsules 
 	}			// for player 
 
-	SetCurrentFont(Para_BFont);
-	CenteredPutString(Screen, GameConfig.screen_height - 25, "Press F1 for help.");
 	return;
 
 };				// ShowPlayground 
@@ -1241,7 +1133,7 @@ void InventPlayground(void)
 						break;
 					}
 
-					/* dont destroy verzweigungen in prev. layer */
+					/* don't destroy branch in prev. layer */
 					anElement = ToPlayground[color][layer - 1][row];
 					if (anElement == SEPARATOR_H || anElement == SEPARATOR_L) {
 						row--;
@@ -1253,7 +1145,7 @@ void InventPlayground(void)
 						break;
 					}
 
-					/* cut off kabels in last layer, if any */
+					/* cut off cables in last layer, if any */
 					anElement = ToPlayground[color][layer - 1][row];
 					if (BlockClass[anElement] == CONNECTOR)
 						ToPlayground[color][layer - 1][row] = CABLE_END;
@@ -1262,7 +1154,7 @@ void InventPlayground(void)
 					if (BlockClass[anElement] == CONNECTOR)
 						ToPlayground[color][layer - 1][row + 2] = CABLE_END;
 
-					/* set the verzweigung itself */
+					/* set the branch itself */
 					ToPlayground[color][layer][row] = SEPARATOR_H;
 					ToPlayground[color][layer][row + 1] = SEPARATOR_M;
 					ToPlayground[color][layer][row + 2] = SEPARATOR_L;
@@ -1290,7 +1182,7 @@ void InventPlayground(void)
 						break;
 					}
 
-					/* cut off kabels in last layer, if any */
+					/* cut off cables in last layer, if any */
 					anElement = ToPlayground[color][layer - 1][row + 1];
 					if (BlockClass[anElement] == CONNECTOR)
 						ToPlayground[color][layer - 1][row + 1] = CABLE_END;
@@ -1640,7 +1532,6 @@ void ProcessPlayground(void)
 				if (TurnActive) {
 					if (ActivationMap[color][layer][row] == INACTIVE)
 						ActivationMap[color][layer][row] = ACTIVE1;
-					TurnActive = FALSE;
 				} else
 					ActivationMap[color][layer][row] = INACTIVE;
 
@@ -1656,7 +1547,7 @@ void ProcessPlayground(void)
 /** 
  * This function sets the correct values for the status column in the
  * middle of the takeover game field.
- * Binking leds are realized here as well.
+ * Blinking LEDs are realized here as well.
  */
 void ProcessDisplayColumn(void)
 {
@@ -1751,8 +1642,8 @@ void ProcessCapsules(void)
 };				// void ProcessCapsules ( void )
 
 /**
- * This function tells, wether a Column-connection is active or not.
- * It returns TRUE or FALSE accordinly.
+ * This function tells, whether a Column-connection is active or not.
+ * It returns TRUE or FALSE accordingly.
  */
 int IsActive(int color, int row)
 {
@@ -1794,10 +1685,10 @@ void to_show_banner(const char *left, const char *right)
 {
 	char left_box[LEFT_TEXT_LEN + 10];
 	char right_box[RIGHT_TEXT_LEN + 10];
-	int left_len, right_len;	// the actualy string lengths
+	int left_len, right_len;	// the actual string lengths
 
 	// At first the text is prepared.  This can't hurt.
-	// we will decide whether to dispaly it or not later...
+	// we will decide whether to display it or not later...
 	//
 
 	if (left == NULL)
@@ -1810,12 +1701,12 @@ void to_show_banner(const char *left, const char *right)
 	left_len = strlen(left);
 	if (left_len > LEFT_TEXT_LEN) {
 		printf("\nWarning: String %s too long for Left Infoline!!", left);
-		Terminate(ERR);
+		Terminate(EXIT_FAILURE, TRUE);
 	}
 	right_len = strlen(right);
 	if (right_len > RIGHT_TEXT_LEN) {
 		printf("\nWarning: String %s too long for Right Infoline!!", right);
-		Terminate(ERR);
+		Terminate(EXIT_FAILURE, TRUE);
 	}
 	// Now prepare the left/right text-boxes 
 	memset(left_box, ' ', LEFT_TEXT_LEN);	// pad with spaces 
@@ -1831,8 +1722,8 @@ void to_show_banner(const char *left, const char *right)
 	// time to display it...
 	DebugPrintf(2, "Takeover said: %s -- %s\n", left_box, right_box);
 	SetCurrentFont(Para_BFont);
-	DisplayText(left_box, LEFT_INFO_X, LEFT_INFO_Y, NULL, TEXT_STRETCH);
-	DisplayText(right_box, RIGHT_INFO_X, RIGHT_INFO_Y, NULL, TEXT_STRETCH);
+	display_text(left_box, LEFT_INFO_X, LEFT_INFO_Y, NULL);
+	display_text(right_box, RIGHT_INFO_X, RIGHT_INFO_Y, NULL);
 
 };				// void to_show_banner (const char* left, const char* right)
 
