@@ -39,47 +39,29 @@
 #include "struct.h"
 #include "proto.h"
 #include "global.h"
-#include "SDL_rotozoom.h"
 
 #include "lvledit/lvledit_display.h"
 
-// curent text insertion position
+// current text insertion position
 static int MyCursorX;
 static int MyCursorY;
 
-int display_char_disabled_local;
+static int display_char_disabled;
 
 /**
  * This function determines how many lines of text it takes to write a string
  * with the current font into the given rectangle, provided one would start at
  * the left side.
  */
-int get_lines_needed(const char *text, SDL_Rect t_rect, float text_stretch)
+int get_lines_needed(const char *text, SDL_Rect t_rect, float line_height_factor)
 {
 	t_rect.h = 32767; // arbitrary large number
 	display_char_disabled = TRUE;
-	int lines = DisplayText(text, t_rect.x, t_rect.y, &t_rect, text_stretch);
+	int lines = display_text_using_line_height(text, t_rect.x, t_rect.y, &t_rect, line_height_factor);
 	display_char_disabled = FALSE;
 	return lines;
 }
 
-/**
- * Display one character, updating MyCursorX to new position.
- */
-static void display_char(unsigned char c)
-{
-	if (handle_switch_font_char(c))
-		return;
-
-	if (c < ' ' || c > GetCurrentFont()->number_of_chars - 1)
-		c = '.';
-
-	if ((!display_char_disabled) && (!display_char_disabled_local))
-		PutCharFont(Screen, GetCurrentFont(), MyCursorX, MyCursorY, c);
-
-	MyCursorX += CharWidth(GetCurrentFont(), c) + get_letter_spacing(GetCurrentFont());
-}
-
 /**
  *
  *
@@ -93,8 +75,9 @@ void show_backgrounded_label_at_pixel_position(char *LabelText, int pos_x, int p
 	background_rect.w = TextWidth(LabelText) + 2;
 	background_rect.h = 20;
 
-	our_SDL_fill_rect_wrapper(Screen, &(background_rect), 0);
+	draw_rectangle(&background_rect, 0, 0, 0, 255);
 
+	SetCurrentFont(FPS_Display_BFont);
 	PutString(Screen, pos_x, pos_y, LabelText);
 
 };				// void show_backgrounded_label_at_pixel_position ( char* LabelText , float fill_status , int pos_x , int pos_y )
@@ -141,14 +124,14 @@ int show_backgrounded_text_rectangle(const char *text, struct BFont_Info *font,
 	// Set up and fill the rectangle.
 	t_rect.w = w;
 	t_rect.h = r_height;
-	our_SDL_fill_rect_wrapper(Screen, &t_rect, SDL_MapRGB(Screen->format, 0, 0, 0));
+	draw_rectangle(&t_rect, 0, 0, 0, 255);
 	
 	// Show the text inside our newly drawn rectangle.
 	t_rect.w -= IN_WINDOW_TEXT_OFFSET * 2;
 	t_rect.h -= IN_WINDOW_TEXT_OFFSET;
 	t_rect.x += IN_WINDOW_TEXT_OFFSET;
 	t_rect.y += IN_WINDOW_TEXT_OFFSET;
-	DisplayText(text, t_rect.x, t_rect.y, &t_rect, 1.0);
+	display_text_using_line_height(text, t_rect.x, t_rect.y, &t_rect, 1.0);
 
 	SetCurrentFont(old_font);
 	return r_height;
@@ -159,15 +142,22 @@ int show_backgrounded_text_rectangle(const char *text, struct BFont_Info *font,
  * a big important style.  Then a popup window is suitable, with a mouse
  * click to confirm and make it go away again.
  */
-void alert_window(const char *text)
+void alert_window(const char *text, ...)
 {
+	if (!SDL_WasInit(SDL_INIT_VIDEO))
+		return;
+
+	va_list args;
+	struct auto_string *buffer = alloc_autostr(256);
 	int w = 440;   // arbitrary
 	int h = 60;    // arbitrary
 	int x = (GameConfig.screen_width  - w) / 2;	// center of screen
 	int y = (GameConfig.screen_height - h) / 5 * 2; // 2/5 of screen from top
+	va_start(args, text);
+	autostr_vappend(buffer, text, args);
+	va_end(args);
 
 	Activate_Conservative_Frame_Computation();
-	make_sure_system_mouse_cursor_is_turned_off();
 	StoreMenuBackground(1);
 
 	SDL_Event e;
@@ -175,16 +165,19 @@ void alert_window(const char *text)
 		SDL_Delay(1);
 		RestoreMenuBackground(1);
 
-		int r_height = show_backgrounded_text_rectangle(text, FPS_Display_BFont, x, y, w, h);
+		int r_height = show_backgrounded_text_rectangle(buffer->value, FPS_Display_BFont, x, y, w, h);
 		show_backgrounded_text_rectangle(_("Click to continue..."), Red_BFont, x, y + r_height, w, 10);
 
-		blit_our_own_mouse_cursor();
+		blit_mouse_cursor();
 		our_SDL_flip_wrapper();
 		save_mouse_state();
 		
 		SDL_WaitEvent(&e);
 		
 		switch (e.type) {
+		case SDL_QUIT:
+			Terminate(EXIT_SUCCESS, TRUE);
+			break;
 		case SDL_KEYDOWN:
 			if (e.key.keysym.sym == SDLK_SPACE || e.key.keysym.sym == SDLK_RETURN || e.key.keysym.sym == SDLK_ESCAPE)
 				goto wait_click_and_out;
@@ -198,8 +191,8 @@ void alert_window(const char *text)
 
 wait_click_and_out:
 	while (SpacePressed() || EnterPressed() || EscapePressed() || MouseLeftPressed());
-
-};				// void alert_window( char* text )
+	free_autostr(buffer);
+}
 
 /**
  * 
@@ -227,44 +220,13 @@ int CutDownStringToMaximalSize(char *StringToCut, int LengthInPixels)
 	return TRUE;
 };				// void CutDownStringToMaximalSize ( char* StringToCut , int LengthInPixels )
 
-/**
- * This function assigns a text comment to say for an enemy right after
- * is has been hit.  This can be turned off via a switch in GameConfig.
- */
-void EnemyHitByBulletText(enemy * ThisRobot)
-{
-	if (!GameConfig.Enemy_Hit_Text)
-		return;
-
-	ThisRobot->TextVisibleTime = 0;
-	if (ThisRobot->faction == FACTION_BOTS)
-		switch (MyRandom(4)) {
-		case 0:
-			ThisRobot->TextToBeDisplayed = _("Unhandled exception fault.  Press ok to reboot.");
-			break;
-		case 1:
-			ThisRobot->TextToBeDisplayed = _("System fault. Please buy a newer version.");
-			break;
-		case 2:
-			ThisRobot->TextToBeDisplayed = _("System error. Might be a virus.");
-			break;
-		case 3:
-			ThisRobot->TextToBeDisplayed = _("System error. Please buy an upgrade from MS.");
-			break;
-		case 4:
-			ThisRobot->TextToBeDisplayed = _("System error. Press any key to reboot.");
-			break;
-	} else
-		ThisRobot->TextToBeDisplayed = _("Aargh, I got hit.  Ugh, I got a bad feeling...");
-};				// void EnemyHitByBullet( int Enum );
-
 /* -----------------------------------------------------------------
  *
  * This function scrolls a given text down inside the User-window, 
  * defined by the global SDL_Rect User_Rect
  *
  * ----------------------------------------------------------------- */
-int ScrollText(char *Text, int background_code)
+int ScrollText(char *Text, const char *background_name)
 {
 	int Number_Of_Line_Feeds = 0;	// number of lines used for the text
 	int StartInsertLine, InsertLine;
@@ -282,20 +244,16 @@ int ScrollText(char *Text, int background_code)
 
 	while (!done) {
 		
-		/*MouseLeftPressed()
-	       || (MouseCursorIsOnButton(SCROLL_TEXT_UP_BUTTON, GetMousePos_x(), GetMousePos_y()))
-	       || (MouseCursorIsOnButton(SCROLL_TEXT_DOWN_BUTTON, GetMousePos_x(), GetMousePos_y()))) {*/
-
-		if (background_code != (-1))
-			blit_special_background(background_code);
-		Number_Of_Line_Feeds = DisplayText(Text, ScrollRect.x, InsertLine, &ScrollRect, TEXT_STRETCH);
+		if (background_name)
+			blit_background(background_name);
+		Number_Of_Line_Feeds = display_text(Text, ScrollRect.x, InsertLine, &ScrollRect);
 		// We might add some buttons to be displayed here, so that, if you don't have
 		// a mouse wheel and don't know about cursor keys, you can still click on these
 		// buttons to control the scrolling speed of the text.
 		//
 		ShowGenericButtonFromList(SCROLL_TEXT_UP_BUTTON);
 		ShowGenericButtonFromList(SCROLL_TEXT_DOWN_BUTTON);
-		blit_our_own_mouse_cursor();
+		blit_mouse_cursor();
 		our_SDL_flip_wrapper();
 
 		while (SDL_PollEvent(&event)) {
@@ -347,27 +305,25 @@ int ScrollText(char *Text, int background_code)
 
 		InsertLine -= speed;
 
-		// impose some limit on the amount to scroll away downwards and topwards
+		// impose some limit on the amount to scroll away downwards and upwards
 		//
 		if (InsertLine > StartInsertLine && (speed < 0)) {
 			InsertLine = StartInsertLine;
 			speed = 0;
 		}
-		if (InsertLine + (Number_Of_Line_Feeds + 1) * (int)(FontHeight(GetCurrentFont()) * TEXT_STRETCH) < ScrollRect.y
+		if (InsertLine + (Number_Of_Line_Feeds + 1) * (int)(FontHeight(GetCurrentFont()) * LINE_HEIGHT_FACTOR) < ScrollRect.y
 		    && (speed > 0)) {
-			InsertLine = ScrollRect.y - (Number_Of_Line_Feeds + 1) * (int)(FontHeight(GetCurrentFont()) * TEXT_STRETCH);
+			InsertLine = ScrollRect.y - (Number_Of_Line_Feeds + 1) * (int)(FontHeight(GetCurrentFont()) * LINE_HEIGHT_FACTOR);
 			speed = 0;
 		}
 
-		SDL_Delay(30);
-
-	}			// while !Space_Pressed 
+		SDL_Delay(20);
+	}
 
 	while (MouseLeftPressed()) ;	// so that we don't touch again immediately.
 
 	return OK;
-
-};				// int ScrollText ( ... )
+}
 
 /**
  * This function sets a new text, that will be displayed in huge font 
@@ -404,60 +360,55 @@ void SetNewBigScreenMessage(const char *ScreenMessageText)
 void DisplayBigScreenMessage(void)
 {
 	int i;
-	int next_screen_message_position = 30;
+	int y_pos = 30;
 
 	for (i = 0; i < GameConfig.number_of_big_screen_messages; i++) {
+		if (!Me.BigScreenMessage[i])
+			continue;
+
+		char *text = Me.BigScreenMessage[i];
+
 		if (Me.BigScreenMessageDuration[i] < GameConfig.delay_for_big_screen_messages) {
 			SDL_SetClipRect(Screen, NULL);
-			CenteredPutStringFont(Screen, Menu_BFont, next_screen_message_position, Me.BigScreenMessage[i]);
+
+			SetCurrentFont(Menu_BFont);
+			int x_pos = GameConfig.screen_width/2 - TextWidthFont(GetCurrentFont(), text)/2;
+			display_text_using_line_height(text, x_pos, y_pos, NULL /* clip */, 1.0);
+
 			if (!GameConfig.Inventory_Visible && !GameConfig.SkillScreen_Visible && !GameConfig.CharacterScreen_Visible)
 				Me.BigScreenMessageDuration[i] += Frame_Time();
 
-			next_screen_message_position += FontHeight(Menu_BFont);
+			y_pos += FontHeight(Menu_BFont);
 		}
 	}
 
 };				// void DisplayBigScreenMessage( void )
 
-/*-----------------------------------------------------------------
- * This function prints *Text beginning at positions startx/starty,
- * respecting the text-borders set by clip_rect.  This includes 
- * clipping but also automatic line-breaks when end-of-line is 
- * reached.  If clip_rect==NULL, no clipping is performed.
- *      
- *      NOTE: the previous clip-rectange is restored before
- *            the function returns!
- *
- *     NOTE2: this function _does not_ update the screen
- *
- * @Ret: number of lines written (from the first text line up to the last
- *       displayed line)
- *-----------------------------------------------------------------*/
-int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip, float text_stretch)
+static int display_text_using_line_height_with_cursor(const char *text, int startx, int starty, const SDL_Rect *clip, float line_height_factor, int curpos)
 {
 	char *tmp;		// mobile pointer to the current position in the string to be printed
 	SDL_Rect Temp_Clipping_Rect;	// adding this to prevent segfault in case of NULL as parameter
 	SDL_Rect store_clip;
 	short int nblines = 1;
 	int empty_lines_started = 0;
+	int current_curpos = 0;
 
 	int letter_spacing = get_letter_spacing(GetCurrentFont());
 	int tab_width = TABWIDTH * (CharWidth(GetCurrentFont(), TABCHAR) + letter_spacing);
 
-	if (!*Text) {
+	if (!*text)
 		nblines = 0;
-		return nblines;
-	}
 
 	// We position the internal text cursor on the right spot for
 	// the first character to be printed.
 	//
-	if (startx != -1)
-		MyCursorX = startx;
-	if (starty != -1)
-		MyCursorY = starty;
+	MyCursorX = startx;
+	MyCursorY = starty;
 
-	// We make a backup of the current clipping rect, so we can respore
+	int cursor_x = MyCursorX;
+	int cursor_y = MyCursorY;
+
+	// We make a backup of the current clipping rect, so we can restore
 	// it later.
 	//
 	SDL_GetClipRect(Screen, &store_clip);
@@ -476,31 +427,59 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
 		Temp_Clipping_Rect.h = GameConfig.screen_height;
 	}
 
+	set_gl_clip_rect(clip);
+
+	start_image_batch();
+
 	// Now we can start to print the actual text to the screen.
 	//
 	// The running text pointer must be initialized.
 	//
-	tmp = (char *)Text;	// this is no longer a 'const' char*, but only a char*
+	tmp = (char *)text;	// this is no longer a 'const' char*, but only a char*
 	while (*tmp && (MyCursorY < clip->y + clip->h)) {
-		if (((*tmp == ' ') || (*tmp == '\t'))
-		    && (ImprovedCheckLineBreak(tmp, clip, text_stretch) == 1))	// dont write over right border 
-		{		/*THE CALL ABOVE HAS DONE THE CARRIAGE RETURN FOR US !!! */
-			empty_lines_started++;
-			++tmp;
+		if (current_curpos == curpos) {
+			cursor_x = MyCursorX;
+			cursor_y = MyCursorY;
+		}
+
+		if (handle_switch_font_char(*tmp)) {
+			tmp++;
+			current_curpos++;
 			continue;
 		}
 
-		if (*tmp == '\n') {
-			MyCursorX = clip->x;
-			MyCursorY += (int)(FontHeight(GetCurrentFont()) * text_stretch);
+		if (((*tmp == ' ') || (*tmp == '\t'))
+		    && (ImprovedCheckLineBreak(tmp, clip, line_height_factor) == 1))	// don't write over right border 
+		{		/*THE CALL ABOVE HAS DONE THE CARRIAGE RETURN FOR US !!! */
 			empty_lines_started++;
-		} else if (*tmp == '\t') {
+			tmp++;
+			current_curpos++;
+			continue;
+		}
+
+		// carriage return in the middle of a word if it is too big to fit on one line
+		if (isgraph(*tmp) && (MyCursorX + CharWidth(GetCurrentFont(), *tmp) + letter_spacing) > (clip->x + clip->w)) {
+			MyCursorX = clip->x;
+			MyCursorY += (int)(FontHeight(GetCurrentFont()) * line_height_factor);
+			empty_lines_started++;
+		}
+
+		switch (*tmp) {
+		case '\n':
+			MyCursorX = clip->x;
+			MyCursorY += (int)(FontHeight(GetCurrentFont()) * line_height_factor);
+			empty_lines_started++;
+			break;
+		case '\t':
 			MyCursorX = (int)ceilf((float)MyCursorX / (float)(tab_width)) * (tab_width);
-		} else {
-			if (MyCursorY <= clip->y - (int)(FontHeight(GetCurrentFont()) * text_stretch))
-				display_char_disabled_local = TRUE;
-			display_char(*tmp);
-			display_char_disabled_local = FALSE;
+			break;
+		default:
+			if (MyCursorY > clip->y - (int)(FontHeight(GetCurrentFont()) * line_height_factor)
+				&& !display_char_disabled)
+				PutCharFont(Screen, GetCurrentFont(), MyCursorX, MyCursorY, *tmp);
+
+			MyCursorX += CharWidth(GetCurrentFont(), *tmp)
+				+ get_letter_spacing(GetCurrentFont());
 
 			// At least one visible character must follow a line break or else
 			// the line is a trailing empty line not visible to the user. Such
@@ -510,20 +489,58 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
 				empty_lines_started = 0;
 			}
 		}
-
 		tmp++;
-
+		current_curpos++;
 	}
 
+	end_image_batch();
+
 	SDL_SetClipRect(Screen, &store_clip);	// restore previous clip-rect 
 
+	if (curpos != -1) {
+		if (curpos != 0 && current_curpos == curpos) {
+			cursor_x = MyCursorX;
+			cursor_y = MyCursorY;
+		}
+
+		SDL_Rect cursor_rect;
+		cursor_rect.x = cursor_x;
+		cursor_rect.y = cursor_y;
+		cursor_rect.h = FontHeight(GetCurrentFont());
+		cursor_rect.w = 8;
+		HighlightRectangle(Screen, cursor_rect);
+	}
+	
+	if (use_open_gl)
+		unset_gl_clip_rect();
+
 	return nblines;
 }
 
+/**
+ * Prints text at given positions, automatically word-wrapping at the
+ * edges of clip_rect.  If clip_rect is NULL, no clipping is performed.
+ *      
+ * @return number of lines written (from the first text line up to the
+ *         last displayed line)
+ */
+int display_text_using_line_height(const char *text, int startx, int starty, const SDL_Rect *clip, float line_height_factor)
+{
+	return display_text_using_line_height_with_cursor(text, startx, starty, clip, line_height_factor, -1);
+}
+
+/**
+ * Display text using the default line height.
+ */
+int display_text(const char *text, int x, int y, const SDL_Rect* clip_rect)
+{
+	return display_text_using_line_height(text, x, y, clip_rect, LINE_HEIGHT_FACTOR);
+}
+
 /**
  * This function checks if the next word still fits in this line
  * of text and initiates a carriage return/line feed if not.
- * Very handy and convenient, for that means it is no longer nescessary
+ * Very handy and convenient, for that means it is no longer necessary
  * to enter \n in the text every time its time for a newline. cool.
  *  
  * rp: added argument clip, which contains the text-window we're writing in
@@ -531,7 +548,7 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
  *
  * ah: added return value : 1 if carriage return was done, FALSE otherwise
  */
-int ImprovedCheckLineBreak(char *Resttext, const SDL_Rect * clip, float text_stretch)
+int ImprovedCheckLineBreak(char *Resttext, const SDL_Rect * clip, float line_height_factor)
 {
 	int NeededSpace = 0;
 
@@ -550,7 +567,7 @@ int ImprovedCheckLineBreak(char *Resttext, const SDL_Rect * clip, float text_str
 
 	if ((MyCursorX + NeededSpace) > (clip->x + clip->w)) {
 		MyCursorX = clip->x;
-		MyCursorY += (int)(FontHeight(GetCurrentFont()) * text_stretch);
+		MyCursorY += (int)(FontHeight(GetCurrentFont()) * line_height_factor);
 		return 1;
 	}
 
@@ -558,46 +575,17 @@ int ImprovedCheckLineBreak(char *Resttext, const SDL_Rect * clip, float text_str
 };				// int ImprovedCheckLineBreak()
 
 /**
- * This function reads a string of "MaxLen" from User-input, and
- * echoes it either using graphics-text
- *
- * The function does have some extra complicating elelments coming
- * from the potential slowness of machines with pure SDL and no
- * OpenGL.  These machines might not re-blit the whole background
- * fast enough for swiftly typed keystrokes, resulting in some
- * keypresses not being detected by the game.  To avoid that when
- * using SDL, not the whole screen will be re-blitted but only some
- * relevant parts, so that the CPU can save some time.  With OpenGL,
- * a complete re-blit is done in every cycle.
- *
- * NOTE: MaxLen is the maximal _strlen_ of the string (excl. \0 !)
- *
- * @Ret: char *: String is allocated _here_!!!
- *
- * ----------------------------------------------------------------- */
-char *GetString(int MaxLen, int background_code, const char *text_for_overhead_promt)
+ * Prompt the user for a string no longer than MaxLen (excluding terminating \0).
+ */
+char *get_string(int MaxLen, const char *background_name, const char *text_for_overhead_promt)
 {
 	char *input;		// pointer to the string entered by the user
 	int key;		// last 'character' entered 
 	int curpos;		// counts the characters entered so far
 	int finished;
-	int x0, y0, height;
-	SDL_Rect store_rect, tmp_rect;
-	SDL_Surface *store = NULL;
+	int x0, y0;
 
-	height = FontHeight(GetCurrentFont());
-
-	DisplayText(text_for_overhead_promt, 50, 50, NULL, TEXT_STRETCH);
-	x0 = MyCursorX;
-	y0 = MyCursorY;
-
-	if (use_open_gl)
-		StoreMenuBackground(0);
-	else {
-		store = SDL_CreateRGBSurface(0, GameConfig.screen_width, height, vid_bpp, 0, 0, 0, 0);
-		Set_Rect(store_rect, x0, y0, GameConfig.screen_width, height);
-		our_SDL_blit_surface_wrapper(Screen, &store_rect, store, NULL);
-	}
+	display_text(text_for_overhead_promt, 50, 50, NULL);
 
 	// allocate memory for the users input
 	input = MyMalloc(MaxLen + 5);
@@ -609,13 +597,8 @@ char *GetString(int MaxLen, int background_code, const char *text_for_overhead_p
 	curpos = 0;
 
 	while (!finished) {
-		if (use_open_gl) {
-			blit_special_background(background_code);
-			DisplayText(text_for_overhead_promt, 50, 50, NULL, TEXT_STRETCH);
-		} else {
-			Copy_Rect(store_rect, tmp_rect);
-			our_SDL_blit_surface_wrapper(store, NULL, Screen, &tmp_rect);
-		}
+		blit_background(background_name);
+		display_text(text_for_overhead_promt, 50, 50, NULL);
 
 		x0 = MyCursorX;
 		y0 = MyCursorY;
@@ -629,8 +612,8 @@ char *GetString(int MaxLen, int background_code, const char *text_for_overhead_p
 			// Display the image again so both buffers are in sync
 			// useful for GL drivers that do true pageflipping (win32, nvidia 173.x, ...)
 			if (use_open_gl) {
-				blit_special_background(background_code);
-				DisplayText(text_for_overhead_promt, 50, 50, NULL, TEXT_STRETCH);
+				blit_background(background_name);
+				display_text(text_for_overhead_promt, 50, 50, NULL);
 				x0 = MyCursorX;
 				y0 = MyCursorY;
 				PutString(Screen, x0, y0, input);
@@ -639,7 +622,6 @@ char *GetString(int MaxLen, int background_code, const char *text_for_overhead_p
 			input[curpos] = 0;
 			finished = TRUE;
 		} else if ((key < SDLK_DELETE) && isprint(key) && (curpos < MaxLen)) {
-			DebugPrintf(3, "isprint() true for keycode: %d ('%c')\n", key, (char)key);
 			/* printable characters are entered in string */
 			input[curpos] = (char)key;
 			curpos++;
@@ -654,19 +636,14 @@ char *GetString(int MaxLen, int background_code, const char *text_for_overhead_p
 				curpos--;
 			input[curpos] = '.';
 		} else if (key == SDLK_ESCAPE) {
+			free(input);
 			while (EscapePressed()) ;
 			return (NULL);
 		}
-
-	}			// while(!finished) 
-
-	DebugPrintf(2, "\n\nchar *GetString(..):  The final string is:\n");
-	DebugPrintf(2, input);
-	DebugPrintf(2, "\n\n");
+	}
 
 	return (input);
-
-};
+}
 
 /* -----------------------------------------------------------------
  * This function reads a string of "MaxLen" from User-input.
@@ -682,8 +659,6 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
 	int finished;
 	int x0, y0;
 	SDL_Rect TargetRect;
-	SDL_Rect CursorRect;
-	char tmp_char;
 	int i;
 
 #define EDIT_WINDOW_TEXT_OFFSET 15
@@ -707,7 +682,7 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
 	// Now we find the right position for the new string to start by writing
 	// out the title text once, just to get the cursor positioned right...
 	//
-	DisplayText(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect, TEXT_STRETCH);
+	display_text(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect);
 	x0 = MyCursorX;
 	y0 = MyCursorY;
 
@@ -721,7 +696,7 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
 		TargetRect.h = 340;
 		TargetRect.x = (640 - TargetRect.w) / 2;
 		TargetRect.y = (480 - TargetRect.h) / 2;
-		our_SDL_fill_rect_wrapper(Screen, &TargetRect, SDL_MapRGB(Screen->format, 0, 0, 0));
+		draw_rectangle(&TargetRect, 0, 0, 0, 255);
 
 		TargetRect.w -= EDIT_WINDOW_TEXT_OFFSET;
 		TargetRect.h -= EDIT_WINDOW_TEXT_OFFSET;
@@ -730,25 +705,14 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
 
 		SetCurrentFont(FPS_Display_BFont);
 
-		DisplayText(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect, TEXT_STRETCH);
+		display_text(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect);
 
 		if (PopupWindowTitle[strlen(PopupWindowTitle) - 1] != '\n')
-			DisplayText("\n\n", x0, y0, &TargetRect, TEXT_STRETCH);
+			display_text("\n\n", x0, y0, &TargetRect);
 
-		TargetRect.y = MyCursorX;
+		TargetRect.x = MyCursorX;
 		TargetRect.y = MyCursorY;
-		DisplayText(input, TargetRect.x, TargetRect.y, &TargetRect, TEXT_STRETCH);
-
-		// We position the cursor right on its real location
-		//
-		tmp_char = input[curpos];
-		input[curpos] = 0;
-		CursorRect.x = MyCursorX;
-		input[curpos] = tmp_char;
-		CursorRect.y = TargetRect.y;
-		CursorRect.h = FontHeight(GetCurrentFont());
-		CursorRect.w = 8;
-		HighlightRectangle(Screen, CursorRect);
+		display_text_using_line_height_with_cursor(input, TargetRect.x, TargetRect.y, &TargetRect, LINE_HEIGHT_FACTOR, curpos);
 
 		our_SDL_flip_wrapper();
 
@@ -760,6 +724,7 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
 		} else if (key == SDLK_ESCAPE) {
 			while (EscapePressed())
 				SDL_Delay(1);
+			free(input);
 			return NULL;
 		} else if (isprint(key) && (curpos < MaxLen)) {
 			// If a printable character has been entered, it is either appended to
@@ -817,9 +782,8 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
 
 	SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL);
 
-	return (input);
-
-};				// char* GetEditableStringInPopupWindow ( int MaxLen , char* PopupWindowTitle )
+	return input;
+}
 
 /* -----------------------------------------------------------------
  * behaves similarly as gl_printf() of svgalib, using the BFont
@@ -831,7 +795,7 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
  *
  * Added functionality to PrintString() is: 
  *  o) passing -1 as coord uses previous x and next-line y for printing
- *  o) Screen is updated immediatly after print, using SDL_flip()                       
+ *  o) Screen is updated immediately after print, using SDL_flip()                       
  *
  * ----------------------------------------------------------------- */
 void printf_SDL(SDL_Surface * screen, int x, int y, const char *fmt, ...)
