@@ -39,14 +39,12 @@
 
 #include <zlib.h>
 
-void load_item_surfaces_for_item_type(int item_type);
-
 extern int Number_Of_Item_Types;
 extern int debug_level;
 
 /** 
  * This function works a malloc, except that it also checks for
- * success and terminates in case of "out of memory", so we dont
+ * success and terminates in case of "out of memory", so we don't
  * need to do this always in the code.
  */
 void *MyMalloc(long Mamount)
@@ -60,7 +58,7 @@ void *MyMalloc(long Mamount)
 	if ((Mptr = calloc(1, (size_t) Mamount)) == NULL) {
 		fprintf(stderr, " MyMalloc(%ld) did not succeed!\n", Mamount);
 		fflush(stderr);
-		Terminate(ERR);
+		Terminate(EXIT_FAILURE, TRUE);
 	}
 
 	return Mptr;
@@ -86,50 +84,45 @@ void DebugPrintf(int db_level, const char *fmt, ...)
 
 /**
  * This function should help to simplify and standardize the many error
- * messages possible in Freedroid RPG.
+ * messages possible in FreedroidRPG.
  */
 void ErrorMessage(const char *FunctionName, const char *fmt, int InformDevelopers, int IsFatal, ...)
 {
 	va_list args;
 	va_start(args, IsFatal);
 
-	fprintf(stderr, "\n----------------------------------------------------------------------\n\
-Freedroid has encountered a problem:\n");
-	fprintf(stderr, "In function: %s.\n", FunctionName);
-	fprintf(stderr, "FreedroidRPG package and version number: %s %s.\n", PACKAGE, VERSION);
+	fprintf(stderr, "\n---------------------------------------------------------------------------------\n\
+FreedroidRPG %s encountered a problem ", VERSION);
+	fprintf(stderr, "in function:  %s \n", FunctionName);
 	vfprintf(stderr, fmt, args);
 
 	if (InformDevelopers) {
 		fprintf(stderr, "\
 \n\
 \n\
-If you encounter this message, please inform the Freedroid developers\n\
-about the problem, by either\n\
+If you encounter this message, please inform the FreedroidRPG developers about it\n\
+You can \n\
 \n\
-  sending e-mail to:\n\
-    freedroid-discussion AT lists.sourceforge.net\n\
-  mention it to someone of the developers on our IRC channel:\n\
-    channel: #freedroid on irc.freenode.net\n\
-  post on our forum at:\n\
-    https://sourceforge.net/apps/phpbb/freedroid\n\
-  report a bug on our tracker at:\n\
-    https://sourceforge.net/tracker/?group_id=54521&atid=474016\n\
+  send an e-mail to                    freedroid-discussion AT lists.sourceforge.net\n\
+  mention it on our IRC channel        #freedroid on irc.freenode.net\n\
+  post on our forum at                 https://sourceforge.net/apps/phpbb/freedroid\n\
+  or report the bug on our tracker at  http://bugs.freedroid.org/\n\
 \n\
-Thanks a lot!\n\n");
+Thank you!\n\n");
 	}
 
 	if (IsFatal) {
-		fprintf(stderr, "Freedroid will terminate now to draw attention to the problems it could\n\
-not resolve.  Sorry if that interrupts a major game of yours...\n");
+		fprintf(stderr, "FreedroidRPG will terminate now to draw attention to the problems it could\n\
+not resolve. We are sorry if that interrupts a major game of yours.\n");
 	} else {
 		fprintf(stderr, "The problem mentioned above is not fatal, we continue the execution.\n");
-		fprintf(stderr, "----------------------------------------------------------------------\n");
+		fprintf(stderr, "---------------------------------------------------------------------------------\n");
 	}
 
 	va_end(args);
 
 	if (IsFatal)
-		Terminate(ERR);
+		Terminate(EXIT_FAILURE, TRUE);
 
 };				// void ErrorMessage ( ... )
 
@@ -151,8 +144,6 @@ void *MyMemmem(char *haystack, size_t haystacklen, char *needle, size_t needlele
 	void *MatchPointer;
 	size_t SearchPos = 0;
 
-	DebugPrintf(3, "%s(): haystack = %d, len = %d, needle=%s\n", __FUNCTION__, haystack, haystacklen, needle);
-
 	while (haystacklen - SearchPos > 0) {
 		// We search for the first match OF THE FIRST CHARACTER of needle
 		//
@@ -166,12 +157,7 @@ void *MyMemmem(char *haystack, size_t haystacklen, char *needle, size_t needlele
 		// Otherwise we see, if also the rest of the strings match this time ASSUMING THEY ARE STRINGS!
 		// In case of a match, we can return immediately
 		//
-		DebugPrintf(3, "calling strstr()..");
 		MatchPointer = strstr(NextFoundPointer, needle);
-		if (MatchPointer)
-			DebugPrintf(3, "..survived. MatchPointer = %d\n", MatchPointer);
-		else
-			DebugPrintf(3, "survived, but nothing found\n");
 		if (MatchPointer != NULL)
 			return (MatchPointer);
 
@@ -238,7 +224,7 @@ char *ReadAndMallocStringFromData(char *SearchString, const char *StartIndicatio
 		fprintf(stderr, "\n\nStartIndicationString: '%s'\nEndIndicationString: '%s'\n", StartIndicationString, EndIndicationString);
 		ErrorMessage(__FUNCTION__, "\
 The string that is supposed to prefix an entry in a text data file\n\
-of Freedroid was not found within this text data file.\n\
+of FreedroidRPG was not found within this text data file.\n\
 This indicates some corruption in the data file in question.", PLEASE_INFORM, IS_FATAL);
 	}
 	return result;
@@ -283,28 +269,34 @@ char *ReadAndMallocAndTerminateFile(const char *filename, const char *File_End_S
 		fprintf(stderr, "\n\nfilename: '%s'\n", filename);
 
 		ErrorMessage(__FUNCTION__, "\
-Freedroid was unable to open a given text file,\n\
+FreedroidRPG was unable to open a given text file,\n\
 that should be there and should be accessible.\n\
-This indicates a serious bug in this installation of Freedroid.", PLEASE_INFORM, IS_FATAL);
+This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM, IS_FATAL);
 	} else {
 		DebugPrintf(1, "\nchar* ReadAndMallocAndTerminateFile ( char* filename ) : Opening file succeeded...");
 	}
 
-	MemoryAmount = FS_filelength(DataFile) + 100;
+	int filelen = FS_filelength(DataFile);
+	MemoryAmount = filelen + 100;
 	Data = (char *)MyMalloc(MemoryAmount);
 
-	fread(Data, MemoryAmount, 1, DataFile);
-
-	DebugPrintf(1, "\n%s(): Reading file succeeded...", __FUNCTION__);
+	if (fread(Data, 1, MemoryAmount, DataFile) < filelen && ferror(DataFile)) {
+		ErrorMessage(__FUNCTION__, "\
+		FreedroidRPG was unable to read a given text file, that should be there and\n\
+		should be accessible.\n\
+		Filename: %s", PLEASE_INFORM, IS_FATAL, filename);
+	} else {
+		DebugPrintf(1, "\n%s(): Reading file succeeded...", __FUNCTION__);
+	}
 
 	if (fclose(DataFile) == EOF) {
 		fprintf(stderr, "\n\nfilename: '%s'\n", filename);
 		ErrorMessage(__FUNCTION__, "\
-		Freedroid was unable to close a given text file, that should be there and\n\
+		FreedroidRPG was unable to close a given text file, that should be there and\n\
 		should be accessible.\n\
 		This indicates a strange bug in this installation of Freedroid, that is\n\
 		very likely a problem with the file/directory permissions of the files\n\
-		belonging to Freedroid.", PLEASE_INFORM, IS_FATAL);
+		belonging to FreedroidRPG.", PLEASE_INFORM, IS_FATAL);
 	} else {
 		DebugPrintf(1, "\n%s(): file closed successfully...\n", __FUNCTION__);
 	}
@@ -318,7 +310,7 @@ This indicates a serious bug in this installation of Freedroid.", PLEASE_INFORM,
 			fprintf(stderr, "File_End_String: '%s'\n", File_End_String);
 
 			ErrorMessage(__FUNCTION__, "\
-		    Freedroid was unable to find the string, that should indicate the end of\n\
+		    FreedroidRPG was unable to find the string, that should indicate the end of\n\
 		    the given text file within this file.\n\
 		    This indicates a corrupt or outdated data or saved game file.", PLEASE_INFORM, IS_FATAL);
 		} else {
@@ -361,12 +353,14 @@ void
 ReadValueFromStringWithDefault(char *SearchBeginPointer, const char *ValuePreceedText, const char *FormatString,
 			       const char *DefaultValueString, void *TargetValue, char *EndOfSearchSectionPointer)
 {
-	char OldTerminaterCharValue;
+	char OldTerminaterCharValue = 0;
 	const char *SourceLocation;
 
 	// We shortly make a termination char into the string.
-	OldTerminaterCharValue = EndOfSearchSectionPointer[0];
-	EndOfSearchSectionPointer[0] = 0;
+	if (EndOfSearchSectionPointer) {
+		OldTerminaterCharValue = EndOfSearchSectionPointer[0];
+		EndOfSearchSectionPointer[0] = 0;
+	}
 
 	// Now we locate the spot, where we finally will find our value
 	SourceLocation = strstr(SearchBeginPointer, ValuePreceedText);
@@ -386,9 +380,10 @@ sscanf using a certain format string failed!\n\
 This indicates a corrupted or seriously outdated game data or saved game file.", PLEASE_INFORM, IS_FATAL);
 	}
 	// Now that we are done, we restore the given SearchArea to former glory
-	EndOfSearchSectionPointer[0] = OldTerminaterCharValue;
-
-};				// void ReadValueFromString( ... )
+	if (EndOfSearchSectionPointer) {
+		EndOfSearchSectionPointer[0] = OldTerminaterCharValue;
+	}
+}
 
 /**
  * This function should analyze a given passage of text, locate an 
@@ -428,6 +423,59 @@ This indicates a corrupted or seriously outdated game data or saved game file.",
 
 };				// void ReadValueFromString( ... )
 
+/** 
+ * This function is for reading in a range of positive values from a string with a default.
+ * 
+ * If there is no string, the default value is assigned and it returns 1, with no warning.
+ *
+ * If there is a string with garbled input (negative minimum value, or minimum>maximum), it warns,
+ * and assigns the default, and a 2 is returned.
+ *
+ * A valid string, with no errors means it returns a 0.
+*/
+
+int ReadRangeFromString(char *SearchString, const char *StartIndicationString, const char *EndIndicationString, int *min, int *max, int default_val)
+{
+	char *ptr;
+	int return_value = 0;
+
+	// First, see if we have a valid search string
+	char *search_ptr = ReadAndMallocStringFromDataOptional(SearchString, StartIndicationString, EndIndicationString);
+	if (!search_ptr) {
+		//Search string was not valid, set everything to default value.
+		*min = *max = default_val;
+		return 1;
+	}
+
+	*min = strtol(search_ptr, &ptr, 10);
+	if (*ptr == '-') {
+		ptr++;
+		*max = strtol(ptr, NULL, 10);
+	} else {
+		*max = *min;
+	}
+
+	//Handle corrupted values
+	if (*min < 0) {
+		ErrorMessage(__FUNCTION__, "\
+The value read in as a minimum (%d) is a negative number.\n\
+This most likely means corrupted data.\n\
+Setting both the maximum and minimum to the default value (%d).", NO_NEED_TO_INFORM, IS_WARNING_ONLY, *min, default_val);
+		*min = *max = default_val;
+		return_value = 2;
+	} else if (*max < *min) {
+		ErrorMessage(__FUNCTION__, "\
+The value read in as a maximum (%d) is less than the minimum (%d).\n\
+This most likely means corrupted data.\n\
+Setting both the maximum and minimum to the default value (%d).", NO_NEED_TO_INFORM, IS_WARNING_ONLY, *max, *min, default_val);
+		*min = *max = default_val;
+		return_value = 2;
+	}
+
+	free(search_ptr);
+	return return_value;
+}
+
 /**
  * This function does the rotation of a given vector by a given angle.
  * The vector is a vector.
@@ -488,7 +536,12 @@ int inflate_stream(FILE * DataFile, unsigned char **DataBuffer, int *size)
 	unsigned char *src = MyMalloc(filelen + 1);
 	int cursz = 1048576;	//start with 1MB
 	unsigned char *temp_dbuffer = malloc(cursz);
-	fread(src, filelen, 1, DataFile);
+	if (fread(src, filelen, 1, DataFile) != 1) {
+		ErrorMessage(__FUNCTION__, "Error reading compressed data stream.", PLEASE_INFORM, IS_WARNING_ONLY);
+		free(temp_dbuffer);
+		free(src);
+		return -1;
+	}
 
 	int ret;
 	z_stream strm;
@@ -504,11 +557,11 @@ int inflate_stream(FILE * DataFile, unsigned char **DataBuffer, int *size)
 	strm.avail_out = cursz;
 	strm.next_out = (Bytef *) temp_dbuffer;
 
-	ret = inflateInit(&strm);
+	ret = inflateInit2(&strm, 47);
 	if (ret != Z_OK) {
 		ErrorMessage(__FUNCTION__, "\
 		zlib was unable to start decompressing a stream.\n\
-		This indicates a serious bug in this installation of Freedroid.", PLEASE_INFORM, IS_WARNING_ONLY);
+		This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM, IS_WARNING_ONLY);
 		free(temp_dbuffer);
 		free(src);
 		return -1;
@@ -532,7 +585,7 @@ int inflate_stream(FILE * DataFile, unsigned char **DataBuffer, int *size)
 
 			ErrorMessage(__FUNCTION__, "\
 			zlib was unable to decompress a stream\n\
-			This indicates a serious bug in this installation of Freedroid.", PLEASE_INFORM, IS_WARNING_ONLY);
+			This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM, IS_WARNING_ONLY);
 			free(temp_dbuffer);
 			free(src);
 			return -1;
@@ -569,7 +622,7 @@ int deflate_to_stream(unsigned char *source_buffer, int size, FILE *dest)
 	strm.avail_in = size;
 	strm.next_in = source_buffer;
 
-    ret = deflateInit(&strm, Z_DEFAULT_COMPRESSION);
+    ret = deflateInit2(&strm, Z_DEFAULT_COMPRESSION, Z_DEFLATED, 31, 8, Z_DEFAULT_STRATEGY);
 	if (ret != Z_OK) {
 		ErrorMessage(__FUNCTION__, "\
 		zlib was unable to start compressing a string.", PLEASE_INFORM, IS_WARNING_ONLY);
